### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
#!/bin/sh
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
#

#
# Plesk script
#


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# Migration manager tables will be managed by plesk
migrate_pmm_management_db_into_core_1011()
{
	if ! db_test_table "PMM" || ! db_test_table "PMMDefault"; then
		$mysql < ${PRODUCT_BOOTSTRAPPER_DIR}/db/pmm_db.sql
		[ "$?" = "0" ] || die "install PMM tables during upgrade"
	fi
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:syntax=sh

generate_encryption_key()
{
	local key_file="/etc/psa/private/secret_key"
	local key_dir="`dirname $key_file`"
	local rc=0

	[ -d "$key_dir" ] || mkdir -p "$key_dir"

	if [ ! -e "$key_file" ]; then
		dd if=/dev/urandom of="$key_file" bs=16 count=1 2>/dev/null
	else
		rc=1
	fi
	fix_key_permissions
	return $rc
}

fix_key_permissions()
{
	local key_file="/etc/psa/private/secret_key"
	local key_dir="`dirname $key_file`"

	if [ -e "$key_file" ]; then
		chown psaadm:0 "$key_file"
		chmod 0600 "$key_file"
	fi

	if [ -d "$key_dir" ]; then
		chown psaadm:0 "$key_dir"
		chmod 0700 "$key_dir"
	fi
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#admin

set_admin_params()
{

	ADMIN_ROOT=$PRODUCT_ROOT_D/admin

	UTILS_DIR=$ADMIN_ROOT/sbin

	admin_user="${product}adm"
	admin_UID=8444
	admin_group="${product}adm"
	admin_GID=8444

	admin_httpsd_conf="$ADMIN_ROOT/conf/httpsd.conf"
	ADMIN_CERT="$ADMIN_ROOT/conf/httpsd.pem"

	admin_pid_file=$ADMIN_ROOT/logs/httpsd.pid
	admin_lock_file=$ADMIN_ROOT/logs/httpsd.lock

	define_start_script
}

lock_keyupdate()
{
	export PLESK_KEYUPDATE_DISABLED=installation
}

## @@constructor set_admin_params
## @@constructor lock_keyupdate

set_admin_perms()
{
   	#remove stale file
	rm -f "$ADMIN_ROOT/htdocs/index.php3"

#	if [ "X$machine" != "Xlinux" -o "X$linux_distr" = "Xdebian" ]; then
#		set_admin_additional_perms
#	fi


	available_certificates=`ls -1 $ADMIN_ROOT/conf/*.pem* 2>/dev/null`
	if [ "X${available_certificates}" != "X" ]; then
		for i in ${available_certificates}; do
			chown 0:0 $i || die
			chmod 400 $i || die
		done
	fi

	[ -d $ADMIN_ROOT/man ] &&
	set_ac 0 $admin_group 755 "$ADMIN_ROOT/man/*" &&
	for i in `find $ADMIN_ROOT/man -type d -print`; do
		set_ac root $admin_group 755 "$i"
		set_ac root $admin_group 644 "$i/*"
	done

	[ -d ${DUMP_D} ] &&
		set_ac $admin_user $admin_group 755 ${DUMP_D}

	#set locales permissions
	for dir in htdocs plib; do
		chmod -R 644 "$ADMIN_ROOT/$dir/locales"
		find "$ADMIN_ROOT/$dir/locales" -type d -exec chmod 755 {} \;
	done

	if [ "X$DEMO_VERSION" = "Xyes" ]; then
		set_ac root $admin_group 775 "$ADMIN_ROOT/htdocs/images/custom_buttons"
	else
		set_ac root $admin_group 4110 "$ADMIN_ROOT/sbin/wrapper"
		set_ac root $admin_group 4110 "$ADMIN_ROOT/sbin/mod_wrapper"
	fi
}

install_admin()
{
	p_echo

	p_echo "===> Installing Admin Server"

	# set default permissions on $ADMIN_ROOT and his contents
	set_admin_perms

	# installation
	cd "$PRODUCT_ROOT_D" || die

	get_pid $ADMIN_ROOT/bin/httpsd true root
	req_pid=$common_var
	if [ $req_pid -gt 1 ]; then
		inten="stop running Admin server"
		echo_try $inten
		kill_pids $ADMIN_ROOT/bin/httpsd root
		if [ -f "$admin_lock_file.*" ]; then
			rm -f "$admin_lock_file.*" >> $product_log 2>&1
		fi
		if [ -f "$admin_pid_file" ]; then
			rm -f "$admin_pid_file"
		fi
		suc
	fi


	if [ ! -s "$ADMIN_CERT" ] ;then
		inten="copy default SSL Certificate for admin server"
		echo_try $inten
		get_certificate "$ADMIN_CERT" || die $inten
		set_ac 0 0 0400 "$ADMIN_CERT"
		suc
	fi

	if [ "X$DEMO_VERSION" = "Xyes" ]; then
		chown 0:psaadm $ADMIN_CERT || die
		chmod 440 $ADMIN_CERT || die
	else
		chown 0:0 $ADMIN_CERT || die
		chmod 400 $ADMIN_CERT || die
	fi
}

get_certificate()
{
	local dst="$1"
	local src="${certificate_file}"
# Check or generate new default certificate
	[ -s "$src" ] || generate_default_certificate
 	set_ac 0 0 0400 "${src}"
	cp -fp $src $dst
}

generate_default_certificate()
{
	# Currently I have no will to accurately rewrite the stuff below
	# so I just add support for (optional) file parameter. Actually
	# one need to accurately handle temporary files, directory creation
	# etc. Probably next time...
	local cert_file

	cert_file="${certificate_file}"
	if [ -s "$cert_file" ]; then
		p_echo "default certificate already exists"
		return
	fi

	# This var also can safely be made local. It's not used outside
	local OPENSSL_CNF is_temp_cnf rc

	OPENSSL_CNF=${PRODUCT_ROOT_D}/admin/conf/openssl.cnf
	is_temp_cnf=0

	if [ ! -f "${OPENSSL_CNF}" ]; then
		# Well, base psa package isn't yet installed. 
		# let's form a temporary config file
		OPENSSL_CNF=$(mktemp /tmp/openssl.cnf.XXXXXX)
		[ $? != 0 ] && die "Unable to create temporary file"
		cat >>${OPENSSL_CNF} <<EOF
[ req ]
attributes=req_attributes
distinguished_name=req_distinguished_name

[ req_attributes ]
challengePassword               = A challenge password
unstructuredName                = An optional company name

[ req_distinguished_name ]
countryName             = Country Name
stateOrProvinceName     = State or Province Name
localityName            = Locality Name
organizationName        = Organisation Name
organizationalUnitName  = Organization Unit Name
commonName              = Common Name
emailAddress            = Email Address
EOF
		is_temp_cnf=1
	fi

	echo "US
Virginia
Herndon
Parallels
Parallels Panel
Parallels Panel
info@parallels.com


" | openssl req -x509 -nodes -days 365 -newkey rsa:2048 	-config ${OPENSSL_CNF} -set_serial `date +%s` -keyout "${cert_file}_" 	-out "${cert_file}_" >> $product_log 2>&1

	# preserve exit code
	rc=$?

	if [ $rc -eq 0 ]; then
		cat "${cert_file}_" | sed -e 's/^\(-----END.*\)/\1/' > $cert_file
		set_ac 0 0 0400 "${cert_file}"
	fi
	# cleanup temporary files
	rm -f "${cert_file}_" ||:
	if [ "x$is_temp_cnf" = "x1" ]; then
		rm -f ${OPENSSL_CNF} ||:
	fi
	return $rc
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

clean_panel_sessions()
{
	db_do "DELETE FROM sessions"
	rm -f $PRODUCT_ROOT_D/admin/sessions/*
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
set_apache_params()
{
	apache_user="www-data"
	apache_UID=80
	apache_group="www-data"
	apache_GID=80

	user_apxs="/usr/bin/apxs2"

	set_apache_params_linux

	apache_service="$apache_service_name"

	apache_httpd_conf="$HTTPD_CONF_D/httpd.conf"
	apache_httpd_conf2="$HTTPD_CONF_D/apache2.conf"
	apache_httpd_conf_in="$HTTPD_CONF_D/httpd.conf.in"

	apache_httpd_include="$HTTPD_INCLUDE_D/zz010_psa_httpd.conf"

	APACHE_CERT="$HTTPD_CONF_D/httpd.pem"
	APACHE_ROOT="/usr"

	min_suexec_UID=10000
	max_suexec_UID=16000
	min_suexec_GID=$min_suexec_UID
	max_suexec_GID=$max_suexec_UID

	suexec_storage=/usr/lib/plesk-9.0/suexec
	suexec=/usr/lib/apache2/suexec
	suexec_dir=/usr/lib/apache2
	suexec_file=suexec

	rpm_httpd_bin=/usr/sbin/httpd
}

set_apache_params_linux()
{
	apache_pid_file="$APACHE_ROOT/logs/httpd.pid"
	apache_lock_file="$APACHE_ROOT/logs/httpd.lock"
	product_lock_file="$HTTPD_CONF_D/cnf.lock"
	apache_service_name="apache2"

	apache_modules_d="/usr/lib/apache2/modules"
}

module_exists()
{
	local dir
	[ -n "$1" ] || return 1
	test -e "${HTTPD_CONF_D}/mods-available/$1.load" && return
	if [ -n "${additional_apache_modules_d}" ]; then
		for dir in $additional_apache_modules_d; do 
			if [ -f "${dir}/mod_$1.so"  ] ; then return ; fi
			if [ -n "$2" -a -f "${dir}/$2" ] ; then return ; fi
		done
	fi
	return 1
}

# Run before it
#        read_conf
#        # set_common_params
#        set_apache_params
add_apache_module()
{
	local module_name="$1"
	local module_soname="$2" # optional
	local config="$3" # optional

	if ! module_exists $module_name "$module_soname"; then
		# TODO : fix message error
		p_echo "Shared object mod_${module_name}.so not found in ${apache_modules_d}...skipped."
		return
	fi

		if [ -x /usr/sbin/a2enmod ]; then
			/usr/sbin/a2enmod $module_name >> $product_log
		else
			warn "a2enmod utilily not found! Module $module_name was NOT enabled."
		fi
}

true apache_status_linux_debian

apache_status_linux_debian()
{
	get_pid "/usr/sbin/apache2" false
	local pid=$common_var
	if test "$pid" -ne 1; then
# running
		return 0
	fi
	return 1
}

httpdmng_reconfigure()
{
	local httpdmng_tool="${PRODUCT_ROOT_D}/admin/sbin/httpdmng"
	local what="$1"
	shift

	case "$what" in
		all|server) : ;;
		*) simply_die "Unexpected argument '$what' to httpdmng_reconfigure()" ;;
	esac

	if [ ! -x "$httpdmng_tool" ]; then
		p_echo "Unable to do 'httpdmng --reconfigure-$what': the utility is not yet on its place. Maybe will attempt later."
		return 1
	fi

	! "$httpdmng_tool" "--reconfigure-$what" "$@" >> $product_log 2>&1 || return 0

	# There are 2 well known possible reasons of webserver templates reconfiguration failure:
	# 1. domain/server/webmail templates are well, but arbitrary domain unable to be configured due to  the some reasons,
	#	for example broken vhost structure
	# 2. broken domain/server/webmail templates


	# STEP 1. try to rollback to the default templates
	! httpdmng_reconfigure_rollback_to_default_templates "${what}" "$@" || return 0

	if [ "all" = "${what}" ]; then
	# STEP 2. ignore broken domains
		pp_echo "Warning: web server configuration is broken. We will try to repair it. This operation can take a lot of time, please do not interrupt the process."
		p_echo "Unable to rebuild web server configuration, possible there are broken domains"
		local inten="reconfigure web-server configurations skipping broken domains"
		echo_try "${inten}"
		if "$httpdmng_tool" "--reconfigure-$what" "-skip-broken" "$@" >> $product_log 2>&1; then
			suc
			return 0
		else
	# STEP 3. try to rollback to the default templates, broken domains are ignored (possibly problem are in the server/webmail configs)
			warn "${inten}"
			! httpdmng_reconfigure_rollback_to_default_templates "${what}" "-skip-broken" "$@" || return 0
		fi
	fi

	return 1
}


# NOTE: do not use function directly, it just helper for httpdmng_reconfigure !!!
# here we try to reconfigure webserver configs using default temnplates
httpdmng_reconfigure_rollback_to_default_templates()
{
	local template_d="${PRODUCT_ROOT_D}/admin/conf/templates"
	local inten="rebuild web server configs with default templates after reconfiguration failure"
	local tmp_d=
	local what="$1"
	shift

	[ -d "${template_d}/custom" ] || return 1
	p_echo "Unable to rebuild web server configuration with currently active default templates"
	echo_try "${inten}"

	if ! tmp_d=`mktemp -d "${template_d}/broken_templates_XXXXXXXX" 2>>$product_log`; then
		warn "${inten} - unable to create temporary directory for custom templates"
		return 1
	fi
	if ! mv -f "${template_d}/custom/"* "${tmp_d}"; then
		warn "${inten} - unable to move custom templates into ${tmp_d}"
		return 1
	fi
	rm -rf "${template_d}/custom"

	if ! "$httpdmng_tool" "--reconfigure-$what" "$@" >> $product_log 2>&1; then
		warn "${inten} - using default templates does not solve the problem, restore custom ones"
		mv -f "${tmp_d}" "${template_d}/custom"
		return 1
	fi

	suc
	pp_echo "Custom templates which are breaking web server reconfiguration were moved to ${tmp_d}"
	cat <<EOF >"${tmp_d}/README"
This directory contains broken custom webserver templates which are breaking web server reconfiguration.
Custom templates were moved here on `date` during product update or upgrade.
EOF
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
api_rpc_create_persistent_tables()
{
	inten="create persistent database storage"
	echo_try $inten
	$mysql < $PRODUCT_BOOTSTRAPPER_DIR/db/api-rpc/api-rpc_db.sql

	if [ "X$?" = "X0" ]; then
		suc
	else
		warn "database initialization"
	fi
}

#
# Upgrade triggers should be clever enough to recognize necessity of upgrade
#
upgrade_api_75()
{
	if db_test_table secret_keys; then
		# secret_keys was created - at least 8.0 version
		return
	fi

	$mysql < $PRODUCT_BOOTSTRAPPER_DIR/db/api-rpc/api-rpc_db_upgrade_750.sql
}

upgrade_api_800()
{
	if db_test_column_enum_value exp_event event_type exceeded; then
		# 'exceeded' was added in 8.0.1
		return
	fi

	$mysql < $PRODUCT_BOOTSTRAPPER_DIR/db/api-rpc/api-rpc_db_upgrade_800.sql
}

upgrade_api_801()
{
	if db_test_column_enum_value exp_event obj_class domain_alias; then
		# domain_alias was added in 8.0.1 update
		return
	fi

	$mysql < $PRODUCT_BOOTSTRAPPER_DIR/db/api-rpc/api-rpc_db_upgrade_801.sql
}

upgrade_api_830()
{
  $mysql < $PRODUCT_BOOTSTRAPPER_DIR/db/api-rpc/api-rpc_db_upgrade_830.sql
}

upgrade_api_840()
{
  $mysql < $PRODUCT_BOOTSTRAPPER_DIR/db/api-rpc/api-rpc_db_upgrade_840.sql
}

upgrade_api_860()
{
  $mysql < $PRODUCT_BOOTSTRAPPER_DIR/db/api-rpc/api-rpc_db_upgrade_860.sql
}

#
# End of upgrade triggers
#

api_rpc_upgrade_persistent_tables()
{
	inten="upgrade persistent database storage"
	echo_try $inten

	# As we don't know previous version of the package (for all packaging systems)
	# all available database upgrade scripts should be executed.
	upgrade_api_75 && 	upgrade_api_800 && 	upgrade_api_801 &&     upgrade_api_830 &&     upgrade_api_840 &&     upgrade_api_860

	if [ "X$?" = "X0" ]; then
		suc
	else
		warn "database upgrade"
	fi
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
apsc_import_data_1011()
{
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/aps-controller-data-import.php 2>>"$product_log"
	[ "$?" -eq 0 ] || 		warn "cannot import data to APS controller database. Please rerun $PRODUCT_ROOT_D/admin/plib/scripts/aps-controller-data-import.php manually"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
#-*- vim:syntax=sh

repeat_until_succeeds()
{
	local repeat_func="$1"
	local modify_func="$2"
	local max_times="$3"

	for i in `seq 1 "$max_times"`; do
		eval "$repeat_func" && return 0 || eval "$modify_func"
	done
	return 1
}

apsc_try_create_db()
{
	local mysql_db_name="mysql"
	echo "CREATE DATABASE $apsc_db_name" | mysql >> "$product_log" 2>&1
}

apsc_modify_db_name()
{
	apsc_db_name="${apsc_db_name_base}_`get_random_string 4 'a-z0-9'`"
}

apsc_try_create_db_user()
{
	local mysql_db_name="mysql"
	echo "CREATE USER '$apsc_db_user'@'localhost' IDENTIFIED BY '$apsc_db_passwd'" | mysql >> "$product_log" 2>&1 && 	echo "GRANT ALL PRIVILEGES ON $apsc_db_name.* TO '$apsc_db_user'@'localhost'" | mysql >> "$product_log" 2>&1
}

apsc_modify_db_user()
{
	apsc_db_user="${apsc_db_user_base}_`get_random_string 4 'a-z0-9'`"
}

apsc_init_db()
{
	local mysql_db_name="$apsc_db_name"
	local mysql_user="--user=$apsc_db_user"
	local mysql_passwd="$apsc_db_passwd"
	local sql_dump_file="$PRODUCT_BOOTSTRAPPER_DIR/db/apsc_mysql_db.sql"

	[ -r "$sql_dump_file" ] || return 1
	mysql < "$sql_dump_file" >> "$product_log" 2>&1
}

apsc_remove_db()
{
	[ -n "$apsc_db_name" ] || return
	local mysql_db_name="mysql"
	echo "DROP DATABASE $apsc_db_name" | mysql >> "$product_log" 2>&1
}

apsc_remove_db_user()
{
	[ -n "$apsc_db_user" ] || return
	local mysql_db_name="mysql"
	echo "DROP USER '$apsc_db_user'@'localhost'" | mysql >> "$product_log" 2>&1
}

set_apsc_params()
{
	odbc_config_bin="/usr/bin/odbc_config" # present at least on CentOS 5
	if [ -x "$odbc_config_bin" ]; then
		odbc_dsn_conf=`$odbc_config_bin --odbcini 2>> "$product_log"`
		odbc_drivers_conf=`$odbc_config_bin --odbcinstini 2>> "$product_log"`
	fi
	if [ -z "$odbc_dsn_conf" ]; then
		odbc_dsn_conf="/etc/odbc.ini"
	fi
	if [ -z "$odbc_drivers_conf" ]; then
		odbc_drivers_conf="/etc/odbcinst.ini"
	fi

	odbc_isql_bin="/usr/bin/isql"
	odbc_iusql_bin="/usr/bin/iusql"

	odbc_mysql_driver="/usr/lib/i386-linux-gnu/odbc/libmyodbc.so"

	apsc_driver_library="/usr/lib/libmysqlserver.so.2"
}

apsc_sanity_check_binaries()
{
	# To easily detect breakage on new OSes
	[ -f "$odbc_mysql_driver" ] || die "find MySQL ODBC driver"
	[ -f "$apsc_driver_library" ] || die "find MySQL platform driver library"
}

read_apsc_connection_param_db()
{
	local param="$1"
	db_select "SELECT val FROM misc WHERE param like 'aps_$param'"
	echo "$db_select_output"
}

read_apsc_connection_params_db()
{
	# Warning: password can be encrypted. Don't use it.
	apsc_db_name=`read_apsc_connection_param_db "database"`
	apsc_db_user=`read_apsc_connection_param_db "login"`
	apsc_db_passwd=`read_apsc_connection_param_db "password"`
	apsc_db_host=`read_apsc_connection_param_db "host"`
}

save_apsc_connection_params_db()
{
	local register_script="$PRODUCT_ROOT_D/admin/plib/scripts/register_apsc_database.php"
	[ -f "$register_script" ] || die 'can not find "register_apsc_database.php"'

	sw_engine_pleskrun "$register_script" --register 		-host 'localhost' 		-port 3306 		-database "$apsc_db_name" 		-login "$apsc_db_user" 		-password "$apsc_db_passwd" >> "$product_log" 2>&1
}

save_apsc_connection_params_conf()
{
	conf_setval "$prod_conf_t" APS_DB_DRIVER_LIBRARY "$apsc_driver_library"
}

save_apsc_connection_params()
{
	save_apsc_connection_params_conf || die "store driver library path for APS controller db connection"
	save_apsc_connection_params_db || die "store APS controller db connection parameters into product db"
}

check_ini_section_exists()
{
	local section="$1"
	local file="$2"
	grep -q "\[$section\]" "$file" >/dev/null 2>&1
}

check_odbc_driver_exists()
{
	# We could use odbcinst here, but it wouldn't save us much trouble
	local driver="$1"
	check_ini_section_exists "$driver" "$odbc_drivers_conf"
}

check_odbc_dsn_exists()
{
	local dsn="$1"
	check_ini_section_exists "$dsn" "$odbc_dsn_conf"
}

remove_ini_section()
{
	local section="$1"
	local file="$2"
	[ -r "$file" ] || return 0
	awk "/\[.*\]/ { del = 0 } /\[$section\]/ { del = 1 } ( del != 1 ) { print }" "$file" > "$file.new" && 	mv -f "$file.new" "$file"
}

remove_odbc_driver()
{
	local driver="$1"
	remove_ini_section "$driver" "$odbc_drivers_conf"
}

remove_odbc_dsn()
{
	local dsn="$1"
	remove_ini_section "$dsn" "$odbc_dsn_conf"
}

apsc_try_create_odbc_driver()
{
	check_odbc_driver_exists "$apsc_odbc_driver_name" && return 1

	local odbc_mysql_driver64=
	if echo "$odbc_mysql_driver" | grep -q lib64 2>/dev/null ; then
		odbc_mysql_driver64="$odbc_mysql_driver"
	fi

	cp -f "$odbc_drivers_conf" "$odbc_drivers_conf.new" && 	cat <<EOF >> "$odbc_drivers_conf.new" && mv -f "$odbc_drivers_conf.new" "$odbc_drivers_conf"
[$apsc_odbc_driver_name]
Description = MySQL driver for Plesk
Driver      = $odbc_mysql_driver
Setup       = 
FileUsage   = 1
Driver64    = $odbc_mysql_driver64
Setup64     = 
UsageCount  = 1

EOF
	local rc="$?"
	[ "$rc" -eq 0 ] || rm -f "$odbc_drivers_conf.new"
	return "$rc"
}

apsc_try_create_odbc_dsn()
{
	check_odbc_dsn_exists "$apsc_dsn_name" && return 1

	cp -f "$odbc_dsn_conf" "$odbc_dsn_conf.new" && 	cat <<EOF >> "$odbc_dsn_conf.new" && mv -f "$odbc_dsn_conf.new" "$odbc_dsn_conf"
[$apsc_dsn_name]
Description = MySQL APS controller database (temporary DSN)
Driver      = $apsc_odbc_driver_name
SERVER      = localhost
USER        = $apsc_db_user
PASSWORD    = $apsc_db_passwd
PORT        = 3306
DATABASE    = $apsc_db_name

EOF
	local rc="$?"
	[ "$rc" -eq 0 ] || rm -f "$odbc_dsn_conf.new"
	return "$rc"
}

apsc_modify_odbc_driver()
{
	# just remove the one that bothers us
	remove_odbc_driver "$apsc_odbc_driver_name"
}

apsc_modify_odbc_dsn()
{
	apsc_dsn_name="${apsc_dsn_name_base}_`get_random_string 4 'a-z0-9'`"
}

check_apsc_installed()
{
	local db_name=`read_apsc_connection_param_db "database"`

	[ -n "$db_name" ] || return 1
	db_test_database "$db_name"
}

odbc_test_connection_using_dsn()
{
	local dsn="$1"
	local isql_bin="$2"
	[ -n "$isql_bin" ] || return 1
	p_echo "Attempting to check APS db connection via ODBC, using $isql_bin (empty output is good)"
	if [ ! -x "$isql_bin" ]; then
		p_echo " Executable $isql_bin was not found, skipped."
		return 1
	fi

	echo "" | $isql_bin -b "$dsn" >> "$product_log" 2>&1
	local rc="$?"

	[ "$rc" -eq 0 ] || p_echo " Failed, rc = $rc."
	return $rc
}

odbc_test_connection()
{
	# This function should be reimplemented via sw-engine. Until then skip the check.
	p_echo "APS db accessibility check was skipped."
	return 0

	if [ ! -x "$odbc_isql_bin" -a ! -x "$odbc_iusql_bin" ]; then
		warn "isql utilities were not found. APS db accessibility check was skipped."
		return 0
	fi

	local apsc_dsn_name_base="apsc"
	local apsc_dsn_name="$apsc_dsn_name_base"

	[ -r "$odbc_dsn_conf" ] || die "find ODBC config"
	repeat_until_succeeds apsc_try_create_odbc_dsn apsc_modify_odbc_dsn 50 || die "create ODBC DSN"

	odbc_test_connection_using_dsn "$apsc_dsn_name" "$odbc_isql_bin" || 	odbc_test_connection_using_dsn "$apsc_dsn_name" "$odbc_iusql_bin"
	local rc="$?"

	remove_odbc_dsn "$apsc_dsn_name"

	return $rc
}

install_apsc()
{
	set_apsc_params

	if check_apsc_installed; then
		inten="upgrade APS controller database"
		echo_try "$inten"
		upgrade_apsc && 		suc || warn "$inten"
	else
		inten="set up APS controller database"
		echo_try "$inten"

		apsc_db_name_base="apsc"
		apsc_db_name="$apsc_db_name_base"
		apsc_db_user_base="apsc"
		apsc_db_user="$apsc_db_user_base"
		apsc_db_passwd=`get_random_string 12 'A-Za-z0-9_'`

		repeat_until_succeeds apsc_try_create_db      apsc_modify_db_name 50 && 		repeat_until_succeeds apsc_try_create_db_user apsc_modify_db_user 50 && 		apsc_init_db || die "create database for APS controller"

		apsc_odbc_driver_name="MySQL"

		[ -r "$odbc_drivers_conf" ] || touch "$odbc_drivers_conf"
		apsc_sanity_check_binaries
		repeat_until_succeeds apsc_try_create_odbc_driver apsc_modify_odbc_driver 50 || die "create ODBC driver configuration"

		if odbc_test_connection ; then
			save_apsc_current_db_version "$product_db_version"
			save_apsc_connection_params

			suc
		else
			p_echo "Failed to establish test connection. Cleaning up."

			remove_odbc_driver "$apsc_odbc_driver_name"
			apsc_remove_db_user
			apsc_remove_db

			die "establish test connection to APS database via ODBC"
		fi
	fi
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh:

# For upgrade we use admin credentials instead of apsc since password for the 
# latter might be encrypted.

upgrade_apsc_db()
{
	if check_apsc_installed; then
		local inten="upgrade APS controller database"
		echo_try "$inten"; pp_echo
		upgrade_apsc && suc || die "$inten"
	fi
}

apsc_fix_db_credentials()
{
	set_apsc_params

	check_apsc_installed || return 0

        echo_try "connect to apsc database"

        local check_script="$PRODUCT_ROOT_D/admin/plib/scripts/check_apsc_connection.php"
        str="`sw_engine_pleskrun "$check_script"`"
        [ "$str" = "connection ok" ] && suc && return 0 || p_echo "failed"

	read_apsc_connection_params_db
	apsc_db_passwd=`get_random_string 12 'A-Za-z0-9_'`
	reset_user_password "$apsc_db_user" "$apsc_db_passwd" "$apsc_db_host"
        save_apsc_connection_params_db

        str="`sw_engine_pleskrun "$check_script"`"
        [ "$str" = "connection ok" ] && suc && return 0 || p_echo "failed"
}

upgrade_apsc()
{
	read_apsc_connection_params_db

	local apsc_db_version=`read_apsc_current_db_version`
	if [ -z "$apsc_db_version" ] || ! expr "$apsc_db_version" + 0 >/dev/null 2>&1 ; then
		# db_version is not set or invalid. Create it.
		apsc_db_version="$prev_short_version"
		save_apsc_current_db_version "$apsc_db_version"
	fi

	local optional_repair_arg="${do_repair:+--repair}"
	local optional_repair_remark="${optional_repair_arg:+ and repair}"
	pp_echo "===> Cumulative APS controller database ($apsc_db_name) upgrade$optional_repair_remark has been started."
	cu_run_upgrader "apsc" apsc_upgrader 		--load-config-data apsc_cu_load_config_data --store-config-data apsc_cu_store_config_data 		--prepare apsc_upgrader_prepare --rollback apsc_upgrader_rollback --commit apsc_upgrader_commit 		--from-version "$apsc_db_version" --stage "db" $optional_repair_arg --update-version
	if [ "$?" -eq 0 ]; then
		pp_echo "===> Upgrade$optional_repair_remark of APS controller database has been completed."
	else
		pp_echo "===> APS controller database was not upgraded completely. See installation log for details."
	fi
}

create_apsc_meta_info_table()
{
	local mysql_db_name="$apsc_db_name"
	local query="
		CREATE TABLE IF NOT EXISTS meta_info (
		    param   VARCHAR(255) CHARACTER SET ascii COLLATE ascii_bin NOT NULL PRIMARY KEY
		,   value   VARCHAR(2000) CHARACTER SET binary NOT NULL
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;"
	
	echo "$query" | mysql >> "$product_log" 2>&1
}

read_apsc_current_db_version()
{
	local mysql_db_name="$apsc_db_name"
	db_test_table "meta_info" || return 0
	apsc_read_meta_info_attr 'db_version'
}

save_apsc_current_db_version()
{
	local apsc_db_version="$1"
	local mysql_db_name="$apsc_db_name"
	db_test_table "meta_info" || create_apsc_meta_info_table || die "create APSC meta_info DB table"
	db_do "REPLACE INTO meta_info (param, value) VALUES ('db_version', '$apsc_db_version')"
}

add_apsc_db_changes()
{
	local apsc_db_version="$1"
	local inten="upgrade APSC SQL database from $apsc_db_version"
	local upgrade_db_sql="$PRODUCT_BOOTSTRAPPER_DIR/db/apsc_mysql_db_upgrade_$apsc_db_version.sql"
	local sed_script="s/@@APSC_DATABASE@@/$apsc_db_name/g"

	local mysql_db_name="$apsc_db_name"
	sed -e "$sed_script" "$upgrade_db_sql" | mysql >> "$product_log" 2>&1 || die "$inten"
}

apsc_innodb_engine()
{
	local mysql_db_name="$apsc_db_name"
	db_select "SELECT TABLE_NAME FROM information_schema.TABLES where TABLE_SCHEMA = '$apsc_db_name' and ENGINE = 'MyISAM'"
	for tableName in $db_select_output; do
		db_do "ALTER TABLE $tableName ENGINE=InnoDB"
	done
}

apsc_read_meta_info_attr()
{
	db_select "SELECT value FROM meta_info WHERE param = '$1'"
	echo "$db_select_output"
}

apsc_cu_load_config_data()
{
	local upgrader_name="$1"
	local mysql_db_name="$apsc_db_name"

	[ "$upgrader_name" = "apsc" ] || return 0
	db_test_table "meta_info" || return 0

	cu_db_current_timestamp="`  apsc_read_meta_info_attr 'cu_current_timestamp'`"
	cu_db_continue_from="`      apsc_read_meta_info_attr 'cu_continue_from'`"
	cu_db_failed_actions_list="`apsc_read_meta_info_attr 'cu_failed_actions_list'`"
	cu_db_latest_timestamp="`   apsc_read_meta_info_attr 'cu_latest_timestamp'`"
}

apsc_cu_store_config_data()
{
	local upgrader_name="$1"
	local mysql_db_name="$apsc_db_name"

	[ "$upgrader_name" = "apsc" ] || return 0
	db_test_table "meta_info" || create_apsc_meta_info_table || die "create APSC meta_info DB table"

	db_do "REPLACE INTO meta_info (param, value) VALUES 		('cu_current_timestamp', '$cu_db_current_timestamp'), 		('cu_continue_from', '$cu_db_continue_from'), 		('cu_failed_actions_list', '$cu_db_failed_actions_list'), 		('cu_latest_timestamp', '$cu_db_latest_timestamp')"
}

apsc_upgrader_prepare()
{
	local apsc_db_name=`read_apsc_connection_param_db "database"`
	db_backup "mysql.preupgrade.apsc.${prev_version}-${product_version}" "$apsc_db_name"
	APSC_UPGRADER_DB_BACKUP="$dfile"
}

apsc_upgrader_rollback()
{
	[ -n "$APSC_UPGRADER_DB_BACKUP" ] || return

	pp_echo "===> Restoring APS controller database from backup '$APSC_UPGRADER_DB_BACKUP'"
	zcat "$APSC_UPGRADER_DB_BACKUP" | mysql_anydb || {
		pp_echo "     [ERROR] Database restoration failed. Try to restore database from '$APSC_UPGRADER_DB_BACKUP' manually."
		return
	}

	APSC_UPGRADER_DB_BACKUP=
}

apsc_upgrader_commit()
{
	save_apsc_current_db_version "$product_db_version"
}

apsc_upgrader()
{
	local version="cu_check_stage db cu_check_version"
	local ts="cu_check_timestamp"
	local db="cu_check_stage db"
	local do="cu_do"


	cu_start_upgrade_from_version 1101
	$db $ts '2013/11/27 14:22:20' $do apsc_innodb_engine
	$version $do APSC_DB_UPGRADE_FUNCTION_11_1_0
}

# ----- APSC upgrade routines start here -----

# For a function to be recognized as an APSC db upgrader, you should call it from apsc_upgrader()
# with appropriate "tags", e.g.:
#  $ts "2012/11/12 15:08:45" $do APSC_DB_UPGRADE_FUNCTION_NAME
# where "2012/11/12 15:08:45" is a timestamp that is produced by `date '+%Y/%m/%d %H:%M:%S'` 
# or `date '+%Y%m%d%H%M%S'` or any other sane format with the same order of format specifiers.
#
# Please note that to safely use '$ts' here, one should call 'cu_upgrader_conf_init "apsc" ...'
# somewhere during APSC DB installation.
# 
# Upgrade function names may be arbitrary, but let's prefix them with APSC_DB_UPGRADE_FUNCTION_.

APSC_DB_UPGRADE_FUNCTION_11_1_0()
{
	add_apsc_db_changes 11.1.0
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#awstats

set_awstats_params()
{
	AWSTATS_ROOT_D="$PRODUCT_ROOT_D/awstats"
}

## @@constructor set_awstats_params

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

update_statistics_path_in_awstats_confs()
{
	[ -z "$*" ] && return
	sed -i '{ /\/system\/[^/]\+\/statistics\//b; s:\(DirData=\)\"\(.*\)/\([^/]\+\)/statistics/\(.*\)\":\1\"\2/system/\3/statistics/\4\": }' "$@"
}

upgrade_domains_awstats_conf_11_5_24()
{
	update_statistics_path_in_awstats_confs `ls $PRODUCT_ROOT_D/etc/awstats/*.conf 2>/dev/null`
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# Backup manager tables will be managed by plesk

migrate_backup_management_db_into_core_1011()
{
# Remove backupmng from root crontab. Will be moved to system /etc/cron.d
	remove_from_crontab "$PRODUCT_ROOT_D/admin/sbin/backupmng"

	if ! db_test_table "BackupsScheduled"; then
# Perform installation of backup manager schema
		$mysql < ${PRODUCT_BOOTSTRAPPER_DIR}/db/pbm_db.sql
		[ "$?" = "0" ] || die "install backup manager tables during upgrade"
	else
# Perform upgrade of backup manager schema
		upgrade_pbm_80
		upgrade_pbm_801
		upgrade_pbm_900
	fi
# preserve number of simultaneously running backups
	db_select "SELECT val FROM misc WHERE param='max_bu_proc_number'"
	[ -n "$db_select_output" ] || 		db_do "INSERT INTO misc VALUES('max_bu_proc_number', 10)"

# remove BackupsRunning
	db_fix_drop_table "BackupsRunning"
}

upgrade_pbm_80()
{
	if db_test_column BackupsScheduled repository; then
		# repository appeared in 8.1
		return
	fi

	$mysql < ${PRODUCT_BOOTSTRAPPER_DIR}/db/pbm_db_upgrade_80.sql
}

upgrade_pbm_801()
{
	if db_test_column BackupsScheduled split_size; then
		# split_size appeared in 8.1
		return
	fi

	$mysql < ${PRODUCT_BOOTSTRAPPER_DIR}/db/pbm_db_upgrade_801.sql
}

upgrade_pbm_900()
{
	if db_test_column_enum_value BackupsScheduled obj_type reseller; then
		# resseller type appeared in 9.0
		return
	fi

	$mysql < ${PRODUCT_BOOTSTRAPPER_DIR}/db/pbm_db_upgrade_900.sql
}

pbm_create_certificate()
{
	local inten="generate certificate for backup signing"

	[ -x "$PRODUCT_ROOT_D/admin/sbin/backup_sign" ] || return 0

	if db_select "select val from misc where param='bu_cert_id'" && [ -n "$db_select_output"  ]; then
#	Nothing to be done
		return
	fi

	echo_try $inten

	if ! $PRODUCT_ROOT_D/admin/sbin/backup_sign generate >> $product_log 2>&1; then
		warn "backup certificate generation"
		return 1
	fi

	suc
}

install_cron_backup()
{
	local backup_file_before_10_5_24="/etc/cron.d/plesk-backup-manager"
	local backup_file="/etc/cron.d/plesk-backup-manager-task"

	if [ -s "${backup_file_before_10_5_24}" ]; then
		mv -f "${backup_file_before_10_5_24}" "${backup_file}"
	else
		rm -f "${backup_file_before_10_5_24}"
	fi

	[ ! -s "${backup_file}" ] || return
	minutes=`get_random_number 15`

	cat >"${backup_file}" <<-EOF
$minutes,`expr $minutes + 15`,`expr $minutes + 30`,`expr $minutes + 45`	*	*	*	*	root [ -x $PRODUCT_ROOT_D/admin/sbin/backupmng ] && $PRODUCT_ROOT_D/admin/sbin/backupmng >/dev/null 2>&1
EOF
}
#!/bin/sh
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

chroot_warning_prgtype()
{
	local prg type
	prg="$1"
	type="$2"

	p_echo "$prg: $type"
	p_echo "probably it will not work in chrooted accounts"
	warn "register $prg in chrooted environment"
}

install_chroot_program()
{
	local inten prg cp_update filetype libdir libdir64
	inten="register $1 in chroot"
	prg="$1"
	libdir="$2"
	libdir64="$3"

	filetype=`file -ib "$prg"`

	case "$filetype" in
	application/x-executable*statically' 'linked*) 	;;
	application/x-shellscript*|text/x-shellscript*) ;;
	application/x-executable*|application/x-sharedlib*)
		# sometimes 'file' say the executable is a shared library
		# newer OSes also say just "executable" without distibguishind between static and dynamic
		ldd "$prg" | awk '/not found/ { print; exit 1; }
						/not a dynamic executable/ { exit 0; }
						NR=4 && $1 !~ /^\// && $2=="=>" && $4 ~ /\(0x[[:xdigit:]]*\)/ {
							  if ($3 ~ /\/lib64\//) {
								dir = DIR64;
							  } else {
								dir = DIR;
							  }
							  if (system("cp -v -p " CP_UPDATE " " $3 " " dir "/" $1)!=0)
								{ exit 1; }
							}
					' DIR="$libdir" DIR64="$libdir64" CP_UPDATE="$cp_update" >> "$product_log" 2>&1 || report_problem "$inten"
		;;
	application/octet-strea*)
		;;
	*)
		chroot_warning_prgtype "$prg" "$filetype"
		;;
	esac
}

install_chroot_base()
{
	local libdir libdir64

	CHROOT_ROOT_D="$HTTPD_VHOSTS_D/chroot"
	CHROOT_PROGRAMS="bash cat chmod cp grep head id less ln ls mkdir more 			mv pwd rm rmdir sash sh tail touch du vi true false scp groups 			wget zip unzip tar gzip gunzip curl unrar"

	os=`uname -s`
	if [ "${os}" != "Linux" ]; then
		die "unknown system $os" ;
	fi

	cp_update=-u
	libdir="$CHROOT_ROOT_D/lib"
	libdir64="$CHROOT_ROOT_D/lib64"

	copy="cp -v -p $cp_update"
	echo_try "install chrooted environment"

	# NOTE: previously during chroot upgrade all chroot environments in vhosts were removed and then added again via chrootmng.
	# This behavior was removed to support remote SN. Not sure it is in fact needed.
	# See also http://plesk-process.parallels.com/TargetProcess2/Project/Planning/UserStory/View.aspx?UserStoryID=70752

	mkdir -m 755 -p "$CHROOT_ROOT_D"
	chown 0:0 "$CHROOT_ROOT_D"

	mkdir -m 755 -p "$CHROOT_ROOT_D/bin"
	mkdir -m 755 -p "$CHROOT_ROOT_D/dev"
	mkdir -m 755 -p "$CHROOT_ROOT_D/lib"
	[ -d /lib64 ] && mkdir -m 755 -p "$CHROOT_ROOT_D/lib64"
	mkdir -m 755 -p "$CHROOT_ROOT_D/usr"
	mkdir -m 755 -p "$CHROOT_ROOT_D/usr/lib"
	[ -d /usr/libexec ] && mkdir -m 755 -p "$CHROOT_ROOT_D/usr/libexec"
	[ -d /libexec ] && mkdir -m 755 -p "$CHROOT_ROOT_D/libexec"
	mkdir -m 755 -p "$CHROOT_ROOT_D/etc"
	mkdir -m 1777 -p "$CHROOT_ROOT_D/tmp"
	mkdir -m 755 -p "$CHROOT_ROOT_D/var"
	mkdir -m 1777 -p "$CHROOT_ROOT_D/var/tmp"

	[ -e "$CHROOT_ROOT_D/dev/null" ] || mknod -m 666 "$CHROOT_ROOT_D/dev/null" c 1 3
	chown root:root "$CHROOT_ROOT_D/dev/null"

	cd "$CHROOT_ROOT_D/bin"
	for f in $CHROOT_PROGRAMS; do
		path=`which "$f" 2>>"$product_log"`
		found=$?
		if test $found -eq 0 -a -x "$path"; then
			cp -v -p "$path" "$f" >> "$product_log" 2>&1
			install_chroot_program "$f" "$libdir" "$libdir64"
		fi
	done

	echo_try "install sftp-server binary into chroot environment"

	sftp_path="$(awk '/^Subsystem[[:space:]]sftp.*$/ {print $3}' < /etc/ssh/sshd_config)"
	sftp_dir="${sftp_path%/*}"

	if [ -z "$sftp_dir" ]; then
		echo "Warning: sftp-server binary not found" >> "$product_log"
		echo "+ sftp connections will not be available for chrooted accounts" >> "$product_log"
		warn "installation of sftp-server binary into chroot environment"
	else
		mkdir -p "$CHROOT_ROOT_D$sftp_dir" >> "$product_log" 2>&1
		cp -v -p "$sftp_path" "$CHROOT_ROOT_D$sftp_path" >> "$product_log" 2>&1
		suc
		install_chroot_program "$CHROOT_ROOT_D$sftp_path" "$libdir" "$libdir64"
	fi

	$copy /lib/ld-linux* /lib/libnss_*.so.2  "$CHROOT_ROOT_D/lib" >> "$product_log" 2>&1
	[ ! -d /lib64 ] || $copy /lib64/ld-linux* /lib64/libnss_*.so.2  "$CHROOT_ROOT_D/lib64" >> "$product_log" 2>&1

	if test -r /etc/termcap; then
		cp -v -p /etc/termcap "$CHROOT_ROOT_D/etc/termcap" >> "$product_log" 2>&1
	fi

	touch "$CHROOT_ROOT_D/etc/passwd"
	touch "$CHROOT_ROOT_D/etc/group"

	# /etc/resolv.conf may be a symlink on some systems (e.g., on Debian if resolvconf package is installed)
	if [ -f "/etc/resolv.conf" ]; then
		local hardlink_symlink_tgt="ln -v -f -T"
		# ln -L is not supported on some old OSes, such as CentOS/RedHat 5, Ubuntu 10.04. Hope nobody uses symlinks there.
		ln -L 2>&1 | grep -q invalid || hardlink_symlink_tgt="$hardlink_symlink_tgt -L"
		$hardlink_symlink_tgt "/etc/resolv.conf" "$CHROOT_ROOT_D/etc/resolv.conf" >> "$product_log" 2>&1
	fi

	if test -d "$PRODUCT_ROOT_D/chroot"; then
		cd "$PRODUCT_ROOT_D/chroot"

		for f in *; do
			tar cpf - -C "$f" . | tar xpf - -C "$CHROOT_ROOT_D" || report_problem "copy $f to chroot parttern"
			find $d -type f -perm +111 | while read p; do
				install_chroot_program "$p" "$libdir" "$libdir64"
			done
		done
	fi

	suc

	selinux_relabel_dir -F "$CHROOT_ROOT_D"
	selinux_relabel_dir -F "/etc/resolv.conf"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

# import certificate into database when installing
import_default_certificate()
{
	db_select "SELECT val FROM misc WHERE param = 'default_certificate_id'"

	local default_certificate_id="$db_select_output"

	if [ -z "$default_certificate_id" ]; then
# Update misc table if needed
		echo_try "import default certificate"
		import_certificate "$certificate_file"  "default certificate"

		db_do "REPLACE INTO misc set param='default_certificate_id', val='$last_certificate_id'"
		default_certificate_id="$last_certificate_id"
		suc
	fi

	db_select "SELECT val FROM misc WHERE param = 'cert_rep_id'"
	local admin_repository_id="$db_select_output"

	if [ -z "$admin_repository_id" ]; then
		echo_try "push default certificate into repository"
		db_select "REPLACE INTO Repository SET component_id='${default_certificate_id}';SELECT LAST_INSERT_ID();"
		admin_repository_id="$db_select_output"
		db_do "REPLACE INTO misc set param='cert_rep_id', val='${admin_repository_id}'"
	else
#	simple certificate insertion to repository
		db_do "REPLACE INTO Repository SET rep_id='${admin_repository_id}', component_id='${default_certificate_id}'"
	fi
}

# import certificate from a file to certificates table
# $1 - certoificate file path
# $2 - short description of certificate
# last_certificate_id variable will contain its identifier
import_certificate()
{
	local certificate_file certificate_comment inten

	certificate_file=$1
	certificate_comment=$2

	cert_pub_val=`cat ${certificate_file} |
		awk 'BEGIN { state=0 }
		{
			if (state == 0) {
				if (index($0, "-BEGIN CERTIFICATE-") != 0) state = 1;
			};
			if (state == 1) {
				print $0;
				if (index($0, "-END CERTIFICATE-") != 0) state = 2;
			}
		}' |
		perl -n -e 's/([^ A-Za-z0-9])/"%" . sprintf("%02X", unpack("C", $1))/ge; s/ /\+/g; print $_;'`

	cert_priv_val=`cat ${certificate_file} |
		awk 'BEGIN { state=0 }
		{
			if (state == 0) {
				if ((index($0, "-BEGIN ") != 0) &&
					(index($0, " PRIVATE KEY-") != 0)) state = 1;
			};

			if (state == 1) {
				print $0;
				if ((index($0, "-END ") != 0) &&
					(index($0, " PRIVATE KEY-") != 0)) state = 2;
			}
		}' |
		perl -n -e 's/([^ A-Za-z0-9])/"%" . sprintf("%02X", unpack("C", $1))/ge; s/ /\+/g; print $_;'`

	if [ ! -d "${PRODUCT_ROOT_D}"/var/certificates ]; then
		inten="creating directory for certificates"
		mkdir -p "${PRODUCT_ROOT_D}"/var/certificates || die ${inten}
		chown root:0 "${PRODUCT_ROOT_D}"/var/certificates || die ${inten}
		chmod 500 "${PRODUCT_ROOT_D}"/var/certificates || die ${inten}
	fi

	# copy certificate file to file repository
	newcertfile=`mktemp "${PRODUCT_ROOT_D}"/var/certificates/certXXXXXXX`

	cp "${certificate_file}" "${newcertfile}"  || die "place certificate to its directory"
	chown root:0 "${newcertfile}"
	chmod 400 "${newcertfile}"

	newcertfile=`basename ${newcertfile}`
	db_select "INSERT INTO certificates (pvt_key, cert, name, ca_cert, csr, cert_file, ca_file)
		VALUES ('${cert_priv_val}', '${cert_pub_val}', '${certificate_comment}', '', '',
			'${newcertfile}', '');
		SELECT LAST_INSERT_ID()"

	last_certificate_id="$db_select_output"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

drop_upgrade_flag()
{
        rm -f "${PRODUCT_ROOT_D}/version.upg"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
install_rbash()
{
	p_echo

	p_echo "===> Installing rbash"

	p_echo " Checking that $rbash_shell registered as login shell..."
	egrep "^$rbash_shell" /etc/shells >> $product_log 2>&1
	case $? in
		0)	p_echo " $rbash_shell is already registered as login shell"
			;;
		*)	
			if [ -f $bash_shell ]; then
				ln -sf $bash_shell $rbash_shell
				inten="add $rbash_shell to /etc/shells"
				echo_try $inten
				echo "$rbash_shell" >> /etc/shells && suc || die $inten
			fi
			;;
	esac
	p_echo
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
get_old_domain()
{
    if [ "X$domain" = "X" ]; then
		get_config_parameters
    fi
}

stop_before_upgrade()
{
	$START_SH stop 2>/dev/null || /usr/bin/killall -9 httpsd >> $product_log 2>&1
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
update_components_table()
{
	if [ "X${do_upgrade}" != "X1" ]; then
		return
	fi

	local inten="update Components"
	echo_try "$inten"

	db_do --inten "$inten" "DELETE FROM ServiceNodeEnvironment WHERE section = 'componentsPackages'"
	suc

	packagemng_set_dirty_flag
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh:

#courier-imap

set_courier_imap_params()
{
	COURIERIMAP_CONFDIR="/etc/courier-imap"
	IMAPD_CERT="/usr/share/imapd.pem"
	POP3D_CERT="/usr/share/pop3d.pem"

	# Certificate paths for Courier-IMAP <= 3.0.8
	OLD_IMAPD_CERT="/usr/share/courier-imap/imapd.pem"
	OLD_POP3D_CERT="/usr/share/courier-imap/pop3d.pem"

	courier_imapd_service="courier-imapd"
	courier_imaps_service="courier-imaps"
	courier_pop3d_service="courier-pop3d"
	courier_pop3s_service="courier-pop3s"
	courier_authdaemon_service="courier-authdaemon"

	# Service name for Courier-IMAP <= 3.0.8
	old_courier_service="courier-imap"
}

## @@constructor set_courier_imap_params

courier_imap_generate_certs()
{
	# Generates POP3 and IMAP certificates or upgrades their location (3.0.8 => 4.12.0 upgrade)
	local restart_needed=0

	set_courier_imap_params

	if [ ! -f "$IMAPD_CERT" ]; then
		if [ -f "$OLD_IMAPD_CERT" ]; then
			inten="move certificate for Courier-IMAP server for IMAP-SSL during upgrade"
			mv -f "$OLD_IMAPD_CERT" "$IMAPD_CERT" || die "$inten"
		else
			inten="generate Certificate for Courier-IMAP server for IMAP-SSL"
			echo_try $inten
			get_certificate "$IMAPD_CERT" || die $inten
			set_ac 0 0 0400 "$IMAPD_CERT"
			suc
		fi
		restart_needed=110
	fi

	if [ ! -f "$POP3D_CERT" ]; then
		if [ -f "$OLD_POP3D_CERT" ]; then
			inten="move certificate for Courier-IMAP server for POP3-SSL during upgrade"
			mv -f "$OLD_POP3D_CERT" "$POP3D_CERT" || die "$inten"
		else
			inten="copy default SSL Certificate for Courier-IMAP server for POP3-SSL"
			echo_try $inten
			cp -fp "$IMAPD_CERT" "$POP3D_CERT" || die $inten
			suc
		fi
		restart_needed=110
	fi

	return $restart_needed
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
upgrade_courier_imap_auth_1013()
{
	set_courier_imap_params
	local IMAP_CFG="$COURIERIMAP_CONFDIR/imapd"
	local POP3_CFG="$COURIERIMAP_CONFDIR/pop3d"
	local imap_pci_comp pop3_pci_comp

	ls $IMAP_CFG.pci.* 1>/dev/null 2>&1 && imap_pci_comp="on"
	ls $POP3_CFG.pci.* 1>/dev/null 2>&1 && pop3_pci_comp="on"

	# sed code is taken from 10.4.4/MU5/MU-imapd.sh
	if [ "$imap_pci_comp" != "on" ]; then
		if ! egrep -q '^IMAP_CAPABILITY="?[^"]*AUTH=[^"]*"?\s*$' $IMAP_CFG 2>/dev/null ; then
			cp $IMAP_CFG $IMAP_CFG.plesk.bak && 			sed -e "s/^IMAP_CAPABILITY=\(.*\) IDLE\\\"$/IMAP_CAPABILITY=\\1 AUTH=CRAM-MD5 AUTH=PLAIN IDLE\\\"/g" 				-e "s/^IMAP_CAPABILITY_ORIG=\(.*\) IDLE\\\"$/IMAP_CAPABILITY_ORIG=\\1 AUTH=PLAIN IDLE\\\"/g" 				-e "s/^IMAP_CAPABILITY_TLS=.*$/IMAP_CAPABILITY_TLS=\\\"\\\$IMAP_CAPABILITY\\\"/g" 				-e "s/^IMAP_CAPABILITY_TLS_ORIG=.*$/IMAP_CAPABILITY_TLS_ORIG=\\\"\\\$IMAP_CAPABILITY_ORIG\\\"/g" 				<$IMAP_CFG >$IMAP_CFG.plesk.tmp &&
			mv $IMAP_CFG.plesk.tmp $IMAP_CFG || rm -f $IMAP_CFG.plesk.{tmp,bak}
		fi
	fi

	if [ "$pop3_pci_comp" != "on" ]; then
		if egrep -q '^POP3AUTH(_TLS)?=("\s*")?\s*$' $POP3_CFG 2>/dev/null ; then
			cp $POP3_CFG $POP3_CFG.plesk.bak && 			sed -e "s/^POP3AUTH=.*$/POP3AUTH=\\\"LOGIN CRAM-MD5 PLAIN\\\"/g" 				-e "s/^POP3AUTH_ORIG=.*$/POP3AUTH_ORIG=\\\"LOGIN CRAM-MD5 PLAIN\\\"/g" 				-e "s/^POP3AUTH_TLS=.*$/POP3AUTH_TLS=\\\"LOGIN CRAM-MD5 PLAIN\\\"/g" 				-e "s/^POP3AUTH_TLS_ORIG=.*$/POP3AUTH_TLS_ORIG=\\\"LOGIN CRAM-MD5 PLAIN\\\"/g" 				<$POP3_CFG >$POP3_CFG.plesk.tmp && 			mv $POP3_CFG.plesk.tmp $POP3_CFG || rm -f $POP3_CFG.plesk.{tmp,bak}
		fi
	fi
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh:

set_poplock_params()
{
	set_mail_params

	PLESK_POSTFIX_DB_ROOT="/var/spool/postfix/plesk"
	PLESK_POSTFIX_POP_DB_ROOT="/var/spool/postfix/plesk-pop"
	PLESK_POSTFIX_POPLOCK_DB="hash:/var/spool/postfix/plesk-pop/poplock"
	POSTFIX_POPLOCK_DB_PATH="/var/spool/postfix/plesk-pop/poplock.db"
	POSTFIX_POPLOCK_DB_PERM="0460"
	POSTFIX_POPLOCK_DB_USER="postfix"
	POSTFIX_POPLOCK_DB_GROUP="$MAIL_USERGROUP"

	QMAIL_POPLOCK_DB_DIR="/var/lib/plesk/mail/poplock"
	QMAIL_POPLOCK_DB_PATH="/var/lib/plesk/mail/poplock/poplock.db"
	QMAIL_POPLOCK_DB_PERM="0660"
	QMAIL_POPLOCK_DB_USER="$MAIL_USERGROUP"
	QMAIL_POPLOCK_DB_GROUP="nofiles"
}

poplock_db_qmail_upgrade_to_courier_4_12()
{
	[ -d "$QMAIL_POPLOCK_DB_DIR" ] || return 0
	[ -f "$QMAIL_POPLOCK_DB_PATH" ] || return 0
	get_userID "$QMAIL_POPLOCK_DB_USER" && get_groupID "$QMAIL_POPLOCK_DB_GROUP" || return 0

	set_ac "$QMAIL_POPLOCK_DB_USER" "$QMAIL_POPLOCK_DB_GROUP" 0770 "$QMAIL_POPLOCK_DB_DIR"
	set_ac "$QMAIL_POPLOCK_DB_USER" "$QMAIL_POPLOCK_DB_GROUP" "$QMAIL_POPLOCK_DB_PERM" "$QMAIL_POPLOCK_DB_PATH"
}

poplock_db_postfix_upgrade_to_courier_4_12()
{
	[ -d "$PLESK_POSTFIX_DB_ROOT" -a -d "$PLESK_POSTFIX_POP_DB_ROOT" ] || return 0
	local old_poplock_db_path="$PLESK_POSTFIX_DB_ROOT/`basename $POSTFIX_POPLOCK_DB_PATH`"
	[ -f "$old_poplock_db_path" -a ! -L "$old_poplock_db_path" ] || return 0
	get_userID "$POSTFIX_POPLOCK_DB_USER" && get_groupID "$POSTFIX_POPLOCK_DB_GROUP" || return 0

	mv -f "$old_poplock_db_path" "$POSTFIX_POPLOCK_DB_PATH" 2>>"$product_log"
	set_ac "$POSTFIX_POPLOCK_DB_USER" "$POSTFIX_POPLOCK_DB_GROUP" "$POSTFIX_POPLOCK_DB_PERM" "$POSTFIX_POPLOCK_DB_PATH"
	ln -s "../`basename $PLESK_POSTFIX_POP_DB_ROOT`/`basename $POSTFIX_POPLOCK_DB_PATH`" "$old_poplock_db_path" 2>>"$product_log"

	# we should be able to skip reconfiguring Postfix thanks to compatibility symlink, but we'll do it nonetheless
	[ -s "/etc/postfix/main.cf" ] || return 0
	
	set_postfix_params

	local poplock_hash_basename="`basename $PLESK_POSTFIX_POPLOCK_DB`"
	local filtered_value="`read_postconf_value 'mynetworks' | sed -e 's|^.*/'$poplock_hash_basename'$|'$PLESK_POSTFIX_POPLOCK_DB'|'`"
	write_postconf_value 'mynetworks' "$filtered_value" || return 1
	pleskrc postfix reload
}

upgrade_courier_to_4_12_pop_before_smtp_11_5_23()
{
	set_poplock_params

	# poplock.db files for both QMail and Postfix may be present (e.g., if there was a mail provider switch before upgrade)
	# therefore we attempt to run both upgraders. Each of them just does nothing if not applicable, so that's OK.
	poplock_db_qmail_upgrade_to_courier_4_12 && poplock_db_postfix_upgrade_to_courier_4_12
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:syntax=sh

install_cron_somely()
{
	install_cron_somely_noop
}

install_cron_somely_noop()
{
	:
}

remove_from_crontab()
{
	program=$1

	echo_try "remove $program from crontab"

	cron_file=`mktemp /tmp/cronXXXXXX`
	$crontab -l 2>/dev/null | egrep -v "^[^#].*$program" > $cron_file
	$crontab $cron_file

	rm -f $cron_file
	suc
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
fix_crontab_109()
{
    remove_from_crontab "ttsmailparse.php"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

insert_fullhostname()
{
#	replace FullHostName
	inten="insert Hostname to the MySQL database"
	echo_try $inten

	query="select val from misc where param='FullHostName'"
	tmp="`echo \"$query\" | mysql`"

	if [ "X$tmp" = "X" ]; then
		query="replace into misc (param,val) values('FullHostName','$fullhost')"
		echo $query | mysql && suc || die $inten
	fi

	suc
}

setup_support_url()
{
	local support_url

	support_url="$1"
	db_do "REPLACE INTO misc (param,val) values('support_url', '$support_url')"
}


# add_db_changes [--fix] version
# Runs ${PRODNAME}_db_upgrade_${version}.sql
# (with --fix, runs ${PRODNAME}_db_upgrade_${version}_fix.sql)
add_db_changes()
{
	if [ "$db_fix_check_stage" = "yes" ]; then
		return
	fi

	local inten upgrade_db_sql fix
	fix=""
	eval `sh_get_args '--fix) fix="_fix";;'`
	inten="upgrade SQL database from $1"
	upgrade_db_sql="$PRODUCT_BOOTSTRAPPER_DIR/db/${PRODNAME}_db_upgrade_${1}${fix}.sql"
	mysql < $upgrade_db_sql >> "$product_log" 2>&1 || die "$inten"
}

# db_fix_add_column [--init def_value] table column type
# Adds a column of the specified type to the table, optionally
# initializing it with def_value, which is an SQL expression. If the
# column already exists, does nothing
db_fix_add_column()
{
	local def_value=""
	eval `sh_get_args '--init) def_value="$2"; shift;;'`
	local table="$1"
	local column="$2"
	local type="$3"

	if [ "$db_fix_check_stage" = "yes" ]; then
		if ! db_test "DESC $table" "\$1 == \"$column\""; then
			if [ -z "$db_added_columns" ]; then
				db_added_columns="$table.$column"
			else
				db_added_columns="$db_added_columns|$table.$column"
			fi
		fi
		return
	fi

	local query="ALTER TABLE \`$table\` ADD \`$column\` $type"
	if [ -n "$def_value" ]; then
		query="$query; UPDATE \`$table\` SET \`$column\`=$def_value"
	fi
	db_do_if_not "DESC $table" "\$1 == \"$column\"" "$query"
}

# db_fix_change_column [--init def_value] table column type
# Changes a column of the specified type to the table, optionally
# initializing it with def_value, which is an SQL expression.
# Changes a column from the table if it exists
db_fix_change_column()
{
	if [ "$db_fix_check_stage" = "yes" ]; then
		return
	fi

	local def_value=""
	eval `sh_get_args '--init) def_value="$2"; shift;;'`
	local table="$1"
	local old_column="$2"
	local new_column="$3"
	local type="$4"

	local query="ALTER TABLE \`$table\` CHANGE \`$old_column\` \`$new_column\` $type"
	if [ -n "$def_value" ]; then
		query="$query; UPDATE \`$table\` SET \`$new_column\`=$def_value"
	fi
	db_do_if "DESC \`$table\`" "\$1 == \"$old_column\"" "$query"
}

# db_fix_del_column table column
# Deletes a column from the table if it exists
db_fix_del_column()
{
	if [ "$db_fix_check_stage" = "yes" ]; then
		return
	fi

	local table="$1"
	local column="$2"

	db_do_if "DESC \`$table\`" "\$1 == \"$column\"" "ALTER TABLE \`$table\` DROP \`$column\`"
}

# db_fix_add_unique_key [--check-where condition] table column [column ...]
# Adds a unique [compound] key on the specified columns of the table.
# If the key already exists, or there is another key that substitutes
# this one, nothing is done. Existing keys that are substituted by the
# new one are dropped. If --check-where is specified, limit checking
# to rows that satisfy the given SQL condition
db_fix_add_unique_key()
{
	local check_where=""
	eval `sh_get_args '--check-where) check_where="--where \"$2\""; shift;;'`
	if [ "$db_fix_check_stage" = "yes" ]; then
		eval db_check_unique $check_where "$@"
		return
	fi

	local table="$1"
	local columns="$2"
	local esc_columns="$2"
	shift 2
	local i
	for i; do
		columns="$columns,$i"
		esc_columns="$esc_columns,\`$i\`"
	done
	local type name icolumns
	local found=no
	db_get_keys "$table" | (
		while read type name icolumns; do
			case "$icolumns" in
				"$columns"|"$columns",*)
					if [ "$type" != "KEY" ]; then
						found=yes
						break
					fi
					;;
			esac
			case "$columns" in
				"$icolumns"|"$icolumns",*)
					if [ "$type" != "PRIMARY" ]; then
						db_do "ALTER TABLE \`$table\` DROP KEY \`$name\`"
					fi
					;;
			esac
		done
		if [ "$found" = no ]; then
			db_do "ALTER TABLE \`$table\` ADD UNIQUE ($esc_columns)"
		fi
		exit 0
	)
	local status="$?"
	if [ "$status" -ne 0 ]; then
		exit "$status"
	fi
}

# db_fix_drop_unique_key table column [column ...]
# Drops an unique [compound] key on the specified columns of the
# table. If the key does not exist, nothing is done
db_fix_drop_unique_key()
{
	if [ "$db_fix_check_stage" = "yes" ]; then
		return
	fi

	local table="$1"
	local columns="$2"
	shift 2
	local i
	for i; do
		columns="$columns,$i"
	done
	local type name icolumns
	local found=no
	db_get_keys "$table" | (
		while read type name icolumns; do
			if [ "$icolumns" = "$columns" ] && [ "$type" = "UNIQUE" ]; then
				db_do "ALTER TABLE $table DROP KEY $name"
				break
			fi
		done
		exit 0
	)
	local status="$?"
	if [ "$status" -ne 0 ]; then
		exit "$status"
	fi
}

# db_fix_drop_table table_name
db_fix_drop_table()
{
	local table_name
	table_name="$1"

	db_do "DROP TABLE IF EXISTS \`$table_name\`"
}

# db_test test_query awk_script
# Runs test_query and processes it with awk_script. If the output is
# not empty, return 0, otherwise return 1. Hint: supply '1' for
# awk_script to test just for the presence of any output.
db_test()
{
	local any_db=
	eval `sh_get_args '--any-db) any_db=yes;;'`

	local test_query="$1"
	local awk_script="$2"

	if [ -n "$any_db" ]; then
		local output="`mysql_raw_anydb -e \"$test_query\" 2>>\"$product_log\"`"
	else
		local output="`mysql_raw -e \"$test_query\" 2>>\"$product_log\"`"
	fi
	local status=$?
	if [ "$status" -ne 0 ]; then
		p_echo "$output"
		die "run the following SQL query: $1"
	fi

	echo -n "$output" | awk -F '\t' -- "$awk_script" | test `wc -l` -ne 0
}

# db_do [--inten <inten>] query
# Runs query. If it fails, die
# the inten string describes the query reason (to make finding the bug simpler)
db_do()
{
	local desc="execute SQL query"
	eval `sh_get_args '--inten) desc=" (to $2)"; shift;;'`
	if [ "$db_fix_check_stage" = "yes" ]; then
		return
	fi

	local query="$1"

	mysql -e "$query" >>"$product_log" 2>&1 || die "$desc, the query was: $query"
}

# db_select <query>
# runs <query> via mysql_raw
# writes output to db_select_output
# if query fails, output errors and return 1
db_select()
{
	local desc="execute SQL query"
	local query="$1"
	local output="`mysql_raw -e \"$query\" 2>>\"$product_log\"`"
	local status="$?"
	if [ "$status" -ne "0" ]; then
		p_echo "$output"
		die "run the following SQL query: $query"
	fi

	db_select_output="$output"
	return 0
}

# db_do_if test_query awk_script query
# If db_test test_query awk_script returns zero (true), run the query.
# If any query fails, die
db_do_if()
{
	if [ "$db_fix_check_stage" = "yes" ]; then
		return
	fi

	local test_query="$1"
	local awk_script="$2"
	local query="$3"

	if db_test "$test_query" "$awk_script"; then
		db_do "$query"
	fi
}

# db_do_if_not test_query awk_script query
# If db_test test_query awk_script returns nonzero (false), run the
# query. If any query fails, die
db_do_if_not()
{
	if [ "$db_fix_check_stage" = "yes" ]; then
		return
	fi

	local test_query="$1"
	local awk_script="$2"
	local query="$3"

	if ! db_test "$test_query" "$awk_script"; then
		db_do "$query"
	fi
}

# db_check_not test_query awk_script message
# If db_test test_query awk_script returns zero (true), die with
# message
db_check_not()
{
	local test_query="$1"
	local awk_script="$2"
	local message="$3"

	if db_test "$test_query" "$awk_script"; then
		p_echo "$message"
		die
	fi
}

# db_check_unique [--where condition] table column [column ...]
# If the table contains duplicate tuples of values in the specified
# columns, die. If --where is specified, limit checking to rows that
# satisfy the given SQL condition
db_check_unique()
{
	local where=""
	eval `sh_get_args '--where) where="WHERE $2"; shift;;'`
	local table="$1"
	shift
	local columns="$1"
	shift
	local i
	for i; do
		if [ -n "$db_added_columns" ]; then
			eval "case $table.$i in $db_added_columns) return ;; esac"
		fi
		columns="$columns, \`$i\`"
	done
	db_check_not "SELECT COUNT(*) FROM \`$table\` $where GROUP BY $columns HAVING COUNT(*) > 1 LIMIT 1" 1 		"Database integrity check failed: duplicate values of ($columns) exist in table $table"
}

# db_get_keys table
# Outputs the list of keys for a table, one on a row, in the following format:
# name type columns
# Where type is one of PRIMARY, UNIQUE, and KEY, name is always
# PRIMARY for primary keys, and columns is a comma-separated list of
# columns without embedded spaces. All backtick characters are removed
db_get_keys()
{
	local table="$1"
	local query="SHOW CREATE TABLE \`$table\`"
	local output="`mysql_raw -e "$query"`"
	local status=$?
	if [ "$status" -ne 0 ]; then
		p_echo "$output"
		die "run the following SQL query: $1"
	fi

	echo "$output" | awk '
		$1 == "PRIMARY"	{ print "PRIMARY", "PRIMARY", $3 }
		$1 == "UNIQUE"	{ print "UNIQUE", $3, $4 }
		$1 == "KEY"		{ print "KEY", $2, $3 }' | 		sed -e 's/[`()]//g' -e 's/,[[:space:]]*$//g'
}

# db_test_database database
# Returns 0 if the database exists
db_test_database()
{
	local database="$1"

	local mysql_db_name="mysql"
	db_test "SHOW DATABASES" "\$1 == \"$database\""
}

# db_test_table table
# Returns 0 if the table exists
db_test_table()
{
	local table="$1"

	db_test "SHOW TABLES LIKE '$table'" 1
}

# db_test_column table column
# Returns 0 if table.column exists
db_test_column()
{
	local table="$1"
	local column="$2"

	db_test "DESC \`$table\`" "\$1 == \"$column\""
}

# db_test_column_enum_value <table> <column> <value awaited in enum>
# Returns O if table.column contains value in enum description
db_test_column_enum_value()
{
	local table="$1"
	local column="$2"
	local value="$3"

	db_test "DESC \`$table\` \`$column\`" "/[\'\"]$value[\'\"]/ { print 1 }"
}

db_create_default_templates()
{
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/create_default_templates.php 2>>"$product_log"
	[ "$?" -eq 0 ] || 		warn "default tempaltes cannot be created"
}

db_update_notifications()
{
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/update_notifications.php 2>>"$product_log"
	[ "$?" -eq 0 ] || 		warn "cannot update notifications. Please rerun $PRODUCT_ROOT_D/admin/plib/scripts/update_notifications.php manually"
}

db_update_plesk10()
{
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/db-update-plesk10.php 2>>"$product_log"
	[ "$?" -eq 0 ] || 		warn "cannot update db. Please rerun $PRODUCT_ROOT_D/admin/plib/scripts/db-update-plesk10.php manually"
}

smb_install_db()
{
	$PRODUCT_ROOT_D/admin/smb/application/utils/dbupgrade >>"$product_log" 2>&1
	[ "$?" -eq 0 ] || 		die "SMB database not created"

}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
db_upgrade_ip_addresses_1010()
{
    db_fix_add_column IP_Addresses main "ENUM('true', 'false') NOT NULL DEFAULT 'false'"
    db_fix_add_column IP_Addresses status "BIGINT UNSIGNED NOT NULL DEFAULT 0"
}

fix_ip_addresses_1010()
{
    echo_try "Set missing main property to IP addresses"
    sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_ip_addresses_1010.php && suc || warn "fail\nCannot update IP addresses information. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_ip_addresses_1010.php manually."
}

db_upgrade_dns_recs_t_1010()
{
    db_do_if_not         "SELECT id FROM dns_recs_t WHERE type IN ('AAAA', 'CNAME') AND host='ns.<domain>.'" 1         "REPLACE INTO dns_recs_t(displayHost, host, type, displayVal, val, time_stamp) VALUES('ns.<domain>.', 'ns.<domain>.', 'AAAA', '<ipv6>', '<ipv6>', NOW());"

    db_do_if_not         "SELECT id FROM dns_recs_t WHERE type IN ('AAAA', 'CNAME') AND host='<domain>.'" 1         "REPLACE INTO dns_recs_t(displayHost, host, type, displayVal, val, time_stamp) VALUES('<domain>.', '<domain>.', 'AAAA', '<ipv6>', '<ipv6>', NOW()+1);"

    db_do_if_not         "SELECT id FROM dns_recs_t WHERE type IN ('AAAA', 'CNAME') AND host='webmail.<domain>.'" 1         "REPLACE INTO dns_recs_t(displayHost, host, type, displayVal, val, time_stamp) VALUES('webmail.<domain>.', 'webmail.<domain>.', 'AAAA', '<ipv6>', '<ipv6>', NOW()+2);"

    db_do_if_not         "SELECT id FROM dns_recs_t WHERE type IN ('AAAA', 'CNAME') AND host='mail.<domain>.'" 1         "REPLACE INTO dns_recs_t(displayHost, host, type, displayVal, val, time_stamp) VALUES('mail.<domain>.', 'mail.<domain>.', 'AAAA', '<ipv6>', '<ipv6>', NOW()+3);"

    db_do_if_not         "SELECT id FROM dns_recs_t WHERE type IN ('A', 'CNAME') AND host='ipv4.<domain>.'" 1         "REPLACE INTO dns_recs_t(displayHost, host, type, displayVal, val, time_stamp) VALUES('ipv4.<domain>.', 'ipv4.<domain>.', 'A', '<ip>', '<ip>', NOW()+4);"

    db_do_if_not         "SELECT id FROM dns_recs_t WHERE type IN ('AAAA', 'CNAME') AND host='ipv6.<domain>.'" 1         "REPLACE INTO dns_recs_t(displayHost, host, type, displayVal, val, time_stamp) VALUES('ipv6.<domain>.', 'ipv6.<domain>.', 'AAAA', '<ipv6>', '<ipv6>', NOW()+5);"
}

db_upgrade_domain_services_1010()
{
     db_fix_add_column DomainServices ipCollectionId "INT UNSIGNED"
}

db_upgrade_web_domain_services_1010()
{
    echo_try "Create web domain services"
    sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_web_domain_services_1010.php && suc || warn "fail\nCannot update web domain services information. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_web_domain_services_1010.php manually."
# psa.hosting and psa.forwarding ip_address_id columns are dropped in db_upgrade_drop_ip_address_id_11_5_22
}

db_upgrade_hosting_1010()
{
     db_fix_add_column hosting certificate_id "INT UNSIGNED DEFAULT 0"
}

db_upgrade_subdomains_1010()
{
     db_fix_add_column subdomains certificate_id "INT UNSIGNED DEFAULT 0"
}

db_upgrade_check_local_sso_is_used()
{
	# if sso is not installed locally - it is remote
	[ -f /etc/sso/sso_config.ini ] || return 1

	# if sso_server hostname matches full hostname of this machine - it is local
	db_select "SELECT val FROM misc WHERE param like 'sso_server'"
	local sso_hostname=`echo "$db_select_output" | sed -ne 's|.*://\([^:/]*\).*|\1|p'`
	[ -n "$sso_hostname" ] || return 1
	[ -n "$fullhost" ] || {
		warn "Failed to determine whether SSO is local (full hostname not defined), assuming remote."
		return 1
	}
	if [ "$sso_hostname" = "$fullhost" ]; then
		return 0
	fi

	# check whether sso_server hostname resolves to one of our ip addresses
	local ipv4_sso_addrs=""
	local ipv6_sso_addrs=""
	local host_query=""
	host_query=`host -t A "$sso_hostname" 2>/dev/null`
	if [ "$?" -eq 0 ]; then
		ipv4_sso_addrs=`echo "$host_query" | awk '{ print $NF }'`
	fi
	host_query=`host -t AAAA "$sso_hostname" 2>/dev/null`
	if [ "$?" -eq 0 ]; then
		ipv6_sso_addrs=`echo "$host_query" | awk '{ print $NF }'`
	fi

	local sso_addrs_list=`echo $ipv4_sso_addrs $ipv6_sso_addrs $sso_hostname | 						  sed -e 's|\([^ ]*\)|"\1",|g' | 						  sed -e 's|^[ ,]*\(.*\)|\1|' | sed -e 's|\(.*\)[ ,]$|\1|'`
	if [ -z "$sso_addrs_list" ]; then
		sso_addrs_list="null"
	fi
	db_select "SELECT ip_address FROM IP_Addresses WHERE ip_address in ($sso_addrs_list)"
	if [ -n "$db_select_output" ]; then
		return 0
	fi

	# if it does not - sso is most probably remote
	return 1
}

db_upgrade_sso_port_1010()
{
	db_upgrade_check_local_sso_is_used || return 0
	db_do "UPDATE misc SET val=REPLACE(val, ':11443', ':8443') WHERE param like 'sso%'";
	db_do "UPDATE misc SET val=REPLACE(val, ':11444', ':8443') WHERE param like 'sso%'";
}

db_optimize_configurations_table_1010()
{
    db_do "ALTER TABLE Configurations CHANGE objectId objectId INT UNSIGNED";
    db_do_if_not "SHOW INDEX FROM Configurations" "\$3 == \"objectTypeAndId_index\"" "CREATE INDEX objectTypeAndId_index ON Configurations (objectType, objectId)"
}

db_cleanup_exp_event_1010()
{
	db_test_table "exp_event" || return
	db_do "DELETE FROM exp_event WHERE obj_class=''"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
db_upgrade_backups_scheduled_1011()
{
    db_fix_add_column BackupsScheduled content_type "ENUM('backup_content_all_at_domain', 'backup_content_vhost_only', 'backup_content_mail_only') DEFAULT 'backup_content_all_at_domain'"
}

db_upgrade_exp_event_obj_class_1011()
{
    db_fix_change_column "exp_event" "obj_class" "obj_class" "VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL"
}

db_upgrade_ssobranding_port_1011()
{
	db_upgrade_check_local_sso_is_used || return 0
	db_do "UPDATE SSOBranding SET idp_url=REPLACE(idp_url, ':11444', ':8443')"
}

db_upgrade_filesharing_1011()
{
	echo_try "Update filesharing data"
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_filesharing_1011.php && suc || warn "fail\nCannot update filesharing information. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_filesharing_1011.php manually."
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
db_upgrade_resp_attach_1012()
{
	db_fix_change_column "resp_attach" "filename" "filename" "VARCHAR(245) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL"
}

db_upgrade_permissions_1012()
{
	echo_try "Setting default values of new permissions in subscriptions and service plans"
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_permissions_1012.php && suc || warn "fail\nCannot set default values of new permissions in subscriptions and service plans. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_permissions_1012.php manually."
}

db_upgrade_add_simple_plan_1012()
{
	echo_try "create simple service plan"
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/create_default_simple_template.php && suc || warn "fail\nCannot create simple service plan. Please run $PRODUCT_ROOT_D/admin/plib/scripts/create_default_simple_template.php manually."
}

db_upgrade_dns_reverse_ptr_ipv6_1012()
{
	db_do_if_not "SHOW INDEX FROM dns_recs" "\$3 == \"host\"" "CREATE INDEX host ON dns_recs (host)"

	db_do_if_not 		"SELECT id FROM dns_recs_t WHERE type='PTR' AND host='<ipv6>'" 1 		"REPLACE INTO dns_recs_t(displayHost, host, type, displayVal, val, opt, time_stamp) VALUES('<ipv6>', '<ipv6>', 'PTR', '<domain>.', '<domain>.', 64, NOW())"
}

db_upgrade_php_settings_1012()
{
	echo_try "Update existing PHP settings information in service plans and hostings"
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_php_settings_1012.php && suc || warn "fail\nCannot convert PHP settings. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_php_settings_1012.php manually."
}

db_upgrade_mail_domain_services_1012()
{
	echo_try "Upgrade mail domain services"
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_mail_domain_services_1012.php && suc || warn "fail\nCannot upgrade mail domain services. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_mail_domain_services_1012.php manually."
}

db_upgrade_dns_wwwprefix_1012()
{
	echo_try "update www records for domain aliases"
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_dns_1012.php && suc || warn "fail\nnCannot upgrade dns for domain aliases. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_dns_1012.php manually."
}

db_upgrade_upgrade_history_1012()
{
	db_fix_change_column "upgrade_history" "db_version" "db_version" "VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin NOT NULL"
}

db_upgrade_parent_domain_id_1012()
{
	db_fix_add_column domains parentDomainId "INT UNSIGNED NOT NULL DEFAULT 0"
}

db_upgrade_subdomains_1012()
{
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_subdomains_1011.php && suc || warn "fail\nupgrade subdomains"
}

db_upgrade_service_nodes_1012()
{
	db_fix_add_column IP_Addresses serviceNodeId "INT UNSIGNED NOT NULL DEFAULT 1"
	db_fix_add_column Configurations serviceNodeId "INT UNSIGNED NOT NULL DEFAULT 1"
	db_fix_add_column sys_users serviceNodeId "INT UNSIGNED NOT NULL DEFAULT 0"
	db_do_if "SHOW INDEX FROM sys_users" "\$2 == \"0\" && \$3 == \"login\"" "
		ALTER TABLE sys_users DROP INDEX \`login\`;
		ALTER TABLE sys_users ADD UNIQUE (\`serviceNodeId\`, \`login\`);
		UPDATE \`sys_users\` set \`serviceNodeId\` = 1 WHERE 0 = \`serviceNodeId\`;
	"
}

db_upgrade_dns_schema_1012()
{
	db_fix_add_column dns_zone syncSoa "enum('true','false','skip') NOT NULL default 'skip'"
	db_fix_add_column dns_zone syncRecords "enum('true','false','skip') NOT NULL default 'skip'"
	db_fix_add_column dns_recs_t uid "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_bin DEFAULT NULL"
	db_do "update dns_zone set syncSoa='true', syncRecords='true' where id not in (select val from misc where param='default_dns_zone_id')"
}

db_upgrade_dns_data_1012()
{
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_dns_1011.php && suc || warn "fail\nupgrade dns"
}

db_upgrade_local_service_node_1012()
{
	db_do_if 		"DESC ServiceNodes" "\$1 == \"name\"" 		"REPLACE INTO ServiceNodes (\`id\`, \`name\`, \`transport\`) VALUES (1, 'local', 'local');"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
db_upgrade_configurations_1013()
{
	db_fix_add_column "Configurations" "active" "enum('false', 'true') NOT NULL DEFAULT 'true'"
}

db_upgrade_subdomains_zones_1013()
{
	echo_try "Upgrade mode of DNS zones for subdomains"
# Script named in compatibility with windows
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_subdomains_zones_1013.php && suc || warn "fail\nnCannot upgrade mode of DNS zones for subdomains. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_subdomains_zones_1013.php manually."
}

db_upgrade_dns_1013()
{
    db_do "delete from dns_refs where status = 'syn'"
    sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_dns_1011.php && suc || warn "fail\nfix upgrade dns"
    db_do "update dns_zone set syncSoa='skip', syncRecords='skip' where id not in (select val from misc where param='default_dns_zone_id')"
}

db_upgrade_secret_key_1013()
{
    db_fix_add_column "secret_keys" "client_id" "INT UNSIGNED DEFAULT NULL"
    db_do_if_not "SHOW INDEX FROM secret_keys" "\$3 == \"client_id\"" "ALTER TABLE secret_keys ADD INDEX (client_id)"
}

db_upgrade_secret_key_fix_1013()
{
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_secret_keys_1013.php && suc || warn "fail\nfix secret keys clients ids"
}

db_upgrade_admin_simple_plan_permissions_1013()
{
    echo_try "Upgrade permissions for Admin Simple Plan"
    sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_admin_simple_plan_permissions_1013.php && suc || warn "fail\nnCannot upgrade permissions for Admin Simple Plan. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_admin_simple_plan_permissions_1013.php manually."
}

db_upgrade_aps_settings_1013()
{
	db_do "delete from smb_apsSettings"
}

db_upgrade_sessions_1013()
{
    db_fix_add_column "sessions" "ip_dual_stack" "VARCHAR(39) CHARACTER SET ascii COLLATE ascii_general_ci"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
db_add_ftp_sys_users()
{
    db_fix_add_column sys_users mapped_to "INT UNSIGNED"
    db_do_if_not "SHOW INDEX FROM sys_users" "\$3 == \"mapped_to\"" "ALTER TABLE sys_users ADD INDEX (mapped_to)"
}

db_add_plans_schema()
{
    db_fix_add_column "clients" "vendor_id" "INT UNSIGNED"
    db_do "UPDATE clients SET vendor_id = parent_id WHERE ISNULL(vendor_id) AND !ISNULL(parent_id)"

    db_fix_add_column "domains" "vendor_id" "INT UNSIGNED"
    db_do "UPDATE domains SET vendor_id = cl_id WHERE ISNULL(vendor_id)"

    db_fix_del_column "Templates" "note_id"

    db_fix_add_column "Templates" "owner_id" "INT UNSIGNED"
    db_fix_add_column "Templates" "type" "ENUM('reseller', 'client', 'domain', 'domain_addon')"
    db_fix_add_column "Templates" "uuid" "VARCHAR(36)"
}

db_add_plans_data()
{
    script="$PRODUCT_ROOT_D/admin/plib/scripts/upgrade_plans.php"
    if [ -f "$script" ]; then
        sw_engine_pleskrun "$script" >> "$product_log" 2>&1
    fi

    db_do_if_not "SHOW INDEX FROM clients" "\$3 == \"vendor_id\"" "ALTER TABLE clients ADD INDEX (vendor_id)"

    db_fix_change_column "domains" "vendor_id" "vendor_id" "INT UNSIGNED NOT NULL"
    db_do_if_not "SHOW INDEX FROM domains" "\$3 == \"vendor_id\"" "ALTER TABLE domains ADD INDEX (vendor_id)"

    db_fix_change_column "Templates" "uuid" "uuid" "VARCHAR(36) NOT NULL"
    db_fix_change_column "Templates" "owner_id" "owner_id" "INT UNSIGNED NOT NULL"
    db_fix_change_column "Templates" "type" "type" "ENUM('reseller', 'client', 'domain', 'domain_addon') NOT NULL"
    db_do_if_not "SHOW INDEX FROM Templates" "\$3 == \"owner_id\"" "ALTER TABLE Templates ADD INDEX (owner_id)"
    db_do_if_not "SHOW INDEX FROM Templates" "\$3 == \"uuid\"" "ALTER TABLE Templates ADD UNIQUE(uuid)"

    script="$PRODUCT_ROOT_D/admin/plib/scripts/upgrade_plans_shared.php"
    if [ -f "$script" ]; then
		sw_engine_pleskrun "$script" >> "$product_log" 2>&1
    fi
}

db_add_webspaces()
{
# Add and populate www_root for domains
	if ! db_test_column "hosting"  "www_root"; then
		db_fix_add_column "hosting" "www_root" "TEXT CHARACTER SET ascii COLLATE ascii_bin NOT NULL"
		db_do 'UPDATE hosting, domains SET hosting.www_root=CONCAT("'$HTTPD_VHOSTS_D'/", domains.name, "/httpdocs") WHERE hosting.dom_id=domains.id'
	fi

# Add and populate www_root for subdomains
	db_fix_add_column "subdomains" "www_root" "TEXT CHARACTER SET ascii COLLATE ascii_bin NOT NULL"
	db_do 'UPDATE subdomains, domains SET subdomains.www_root=CONCAT("'$HTTPD_VHOSTS_D'/", domains.name, "/subdomains/", subdomains.name, "/httpdocs") WHERE subdomains.dom_id=domains.id'

# webspace_id for domains + index. 0 by default
	db_fix_add_column "domains"	"webspace_id" "INT UNSIGNED NOT NULL DEFAULT 0"
	db_do_if_not "SHOW INDEX FROM domains" "\$3 == \"webspace_id\"" "ALTER TABLE domains ADD INDEX (webspace_id)"

# webspace_status, if status !=0 it will be 2
	if ! db_test_column "domains" "webspace_status"; then
		db_fix_add_column "domains" "webspace_status" "BIGINT UNSIGNED NOT NULL DEFAULT 0"
		db_do "UPDATE domains SET webspace_status=status, status=2 WHERE status <> 0"
	fi

# Migrate 'native' users of subdomains to additional-ftp-accounts and drop 'sys_user_type' from subdomains
	if db_test_column "subdomains" "sys_user_type"; then
		sw_engine_pleskrun $PRODUCT_BOOTSTRAPPER_DIR/db/convert_sysusers_107.php "subdomains" && 		db_fix_del_column "subdomains" "sys_user_type" || warn "Cannot convert 'native' subdomain users to additional ftp accounts"
	fi

# Remove uniq(sys_user_id) from hosting
	db_do_if "SHOW INDEX FROM hosting" "\$3 == \"sys_user_id\"" "ALTER TABLE hosting DROP INDEX sys_user_id"

# Map all webuser's sysusers to main domain's ftp system users
# and chown vhost's dir data are owned by webusers
	sw_engine_pleskrun $PRODUCT_BOOTSTRAPPER_DIR/db/convert_sysusers_107.php "webusers" || warn "Cannot map webusers to domain ftp users"

	db_fix_add_column "disk_usage" "www_root" "BIGINT UNSIGNED DEFAULT 0"
}

db_add_domain_permissions()
{
    db_fix_add_column "domains" "permissions_id" "INT UNSIGNED NOT NULL DEFAULT 0"
}

db_migrate_sb_data_107()
{
	if ! db_test_table "SBSites" || ! db_test_table "SBConfig"; then
		return
	fi

	echo_try "save legacy sitebuilder url data"
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/save_sb_links.php && suc || warn "migration of legacy sitebuilder url data"
}

db_default_skin_107()
{
	echo_try "register default skin"
	db_do "DELETE from Skins"
	db_do "REPLACE INTO Skins (id, name, place) values (1, 'Default', 'default')"

# Update client, domains, mailnames
	db_do "UPDATE cl_param SET val=1 WHERE param='skin_id'"
	db_do "UPDATE dom_param SET val=1 WHERE param='skin_id'"
	db_do "UPDATE mn_param SET val=1 WHERE param='skin_id'"

# Update misc
	db_do "UPDATE misc SET val=1 WHERE param='admin_skin_id'"
	db_do "UPDATE misc SET val=1 WHERE param='def_skin_id'"

	suc
}

db_unlimited_client_expiration_107()
{
	echo_try "change client/reseller expiration to unlimited"
	db_do "UPDATE Limits SET value=-1 WHERE limit_name='expiration' AND id IN (SELECT limits_id FROM clients WHERE limits_id IS NOT NULL)"
	suc
}

db_clean_up_ip_pool_107()
{
	echo_try "clean up unused ip adresses from client ip pools"
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/ip_clean_107.php && suc || warn "fail\nCannot clean up clients ip pools"
}

db_billing_integration_107()
{
	db_fix_drop_table "BillingCache"
	db_fix_add_column "clients" "external_id" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_bin NOT NULL DEFAULT ''"
	db_fix_add_column "domains" "external_id" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_bin NOT NULL DEFAULT ''"
	db_fix_add_column "Templates" "external_id" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_bin NOT NULL DEFAULT ''"
}

db_upgrade_hp_mail_107()
{
	db_fix_add_column "mail" "userId" "INT UNSIGNED NOT NULL DEFAULT 0"
}

db_update_sessions_107()
{
	echo_try "Update sessions"
	db_do "DELETE from sessions"

	db_fix_add_column "sessions" "modified" "INT UNSIGNED"
	db_fix_add_column "sessions" "lifetime" "INT UNSIGNED"
	db_fix_add_column "sessions" "data" "TEXT CHARACTER SET binary"

	suc
}

db_migrate_sa_prefs_107()
{
	echo_try "Update SA preferences"

	if ! db_test_column "spamfilter" "reject_spam"; then
		pnnl_echo "already done"
		return
	fi

	if ! sw_engine_pleskrun $PRODUCT_BOOTSTRAPPER_DIR/db/migrate_sa_prefs_107.php; then
		die "Cannot migrate spamasassin preferences"
	fi

	db_fix_del_column "spamfilter" "reject_spam"
	suc
}

db_generate_guids_107()
{
	local make_guids_script="$PRODUCT_ROOT_D/admin/plib/make_guids.php"
	if [ -f "$make_guids_script" ]; then
		sw_engine_pleskrun "$make_guids_script" >> "$product_log" 2>&1
	fi
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
db_change_domain_admins_length_108()
{
	db_fix_change_column "clients" "login" "login" "VARCHAR(255)"
}

db_subscription_limits_default_values_108()
{
        local subscription_ids;
        local subscription_id;
        local last_insert_id;
        local limit_name="$1";
        local limit_default_value="$2";

        db_select "select d.id from domains d LEFT join Limits l on l.id = d.limits_id AND l.limit_name = '$limit_name' WHERE 0 = d.webspace_id AND l.value IS NULL";
        subscription_ids="$db_select_output";

        for subscription_id in $subscription_ids; do
            db_select "INSERT INTO Limits (id, limit_name, value) SELECT d.limits_id, '$limit_name', $limit_default_value FROM domains d WHERE d.id=$subscription_id;
                   SELECT last_insert_id()"
            last_insert_id="$db_select_output";
            db_select "SELECT d.limits_id FROM domains d WHERE d.id=$subscription_id";
            if [ "X${db_select_output}" = "X0" ]; then
                db_do "UPDATE domains SET limits_id=$last_insert_id WHERE id=$subscription_id";
            fi
        done
}

db_upgrade_subscription_limits_default_values_108()
{
	db_subscription_limits_default_values_108 'max_site_builder' -1
	db_subscription_limits_default_values_108 'max_unity_mobile_sites' -1
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
db_upgrade_clients_idn_109()
{
    db_fix_change_column "clients" "login" "login" "VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL"
}

db_upgrade_subscriptions_109()
{
    db_fix_add_column Subscriptions custom "ENUM('true', 'false') NOT NULL DEFAULT 'false'"
    db_fix_add_column Subscriptions uuid "VARCHAR(36) CHARACTER SET ascii COLLATE ascii_general_ci"
    
    echo_try "Assign missing UUIDs to subscriptions"

# Note, script is named *_108.php, because we change version of core 
# from 10.8.1 to 10.9.0. Renaming script is wrong idea, because it is
# executed on windows side
    sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_subscriptions_108.php && suc || warn "fail\nCannot update subscriptions information. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_subscriptions_108.php manually."
}

db_upgrade_app_pools_109()
{
    sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_app_pools_109.php && suc || warn "fail\nCannot update app pools information. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_app_pools_109.php manually."
}

db_upgrade_custom_buttons_109()
{
    db_fix_add_column custom_buttons plan_item_name "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci"
}

db_upgrade_maintenance_mode_109()
{
    db_fix_add_column hosting maintenance_mode "ENUM('false','true') NOT NULL DEFAULT 'false'"
    db_fix_add_column subdomains maintenance_mode "ENUM('false','true') NOT NULL DEFAULT 'false'"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_add_indexes_for_performance_11_1_11()
{
    db_do_if_not "SHOW INDEX FROM clients" "\$3 == \"type\"" "CREATE INDEX type ON clients (type)"
    db_do_if_not "SHOW INDEX FROM domains" "\$3 == \"external_id\"" "CREATE INDEX external_id ON domains (external_id)"
}

db_upgrade_change_sys_users_home_to_utf8()
{
	db_fix_change_column "sys_users" "home" "home" "VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_service_node_properties_move_property_to_configuration_11_1_12()
{
    local sourceName="$1"
    local destinationName="$3"
    local destinationSection="$2"
    db_select "SELECT \`serviceNodeId\` FROM \`ServiceNodeProperties\` WHERE \`name\` = '$sourceName'"
    for serviceNodeId in $db_select_output; do
        db_do_if_not             "SELECT \`name\` FROM \`ServiceNodeConfiguration\` WHERE \`serviceNodeId\` = $serviceNodeId AND \`section\` = '$destinationName' AND \`name\` = '$destinationSection'" 1             "INSERT INTO \`ServiceNodeConfiguration\` (\`serviceNodeId\`, \`section\`, \`name\`, \`value\`) SELECT '$serviceNodeId', '$destinationSection', '$destinationName', \`value\` FROM \`ServiceNodeProperties\` WHERE \`serviceNodeId\` = '$serviceNodeId' AND \`name\` = '$sourceName'"
    done
}

db_upgrade_service_node_properties_move_misc_to_configuration_11_1_12()
{
    local sourceName="$1"
    local destinationName="$3"
    local destinationSection="$2"
    db_select "SELECT \`id\` FROM \`ServiceNodes\` WHERE \`ipAddress\` = 'local'"
    for serviceNodeId in $db_select_output; do
        db_do_if_not             "SELECT \`name\` FROM \`ServiceNodeConfiguration\` WHERE \`serviceNodeId\` = $serviceNodeId AND \`section\` = '$destinationSection' AND \`name\` = '$destinationName'" 1             "INSERT INTO \`ServiceNodeConfiguration\` (\`serviceNodeId\`, \`section\`, \`name\`, \`value\`) SELECT '$serviceNodeId', '$destinationSection', '$destinationName', \`val\` FROM misc WHERE \`param\` = '$sourceName'"
    done
    db_do "DELETE FROM \`misc\` WHERE \`param\`= '$sourceName'"
}

db_upgrade_service_node_properties_11_1_12()
{
    if db_test_table 'ServiceNodeProperties'; then
        db_upgrade_service_node_properties_move_property_to_configuration_11_1_12 'server.nginx.enabled'             'webProxy' 'enabled'
        db_upgrade_service_node_properties_move_property_to_configuration_11_1_12 'server.nginx.backend-port'       'webProxy' 'backendPort'
        db_upgrade_service_node_properties_move_property_to_configuration_11_1_12 'server.nginx.ssl-backend-port'   'webProxy' 'sslBackendPort'
        db_upgrade_service_node_properties_move_property_to_configuration_11_1_12 'server.nginx.frontend-port'      'webProxy' 'frontendPort'
        db_upgrade_service_node_properties_move_property_to_configuration_11_1_12 'server.nginx.ssl-frontend-port'  'webProxy' 'sslFrontendPort'

        db_upgrade_service_node_properties_move_property_to_configuration_11_1_12 'server.webserver.httpPort'       'webServer' 'httpPort'
        db_upgrade_service_node_properties_move_property_to_configuration_11_1_12 'server.webserver.httpsPort'      'webServer' 'httpsPort'

        db_upgrade_service_node_properties_move_misc_to_configuration_11_1_12 'http_port' 'webServer' 'httpPort'
        db_upgrade_service_node_properties_move_misc_to_configuration_11_1_12 'https_port' 'webServer' 'httpsPort'

        db_select "SELECT SUBSTR(\`name\`, 26) FROM \`ServiceNodeProperties\` WHERE \`name\` LIKE 'coldfusionservice.config.%' GROUP BY \`name\`"
        for cfName in $db_select_output; do
            db_upgrade_service_node_properties_move_property_to_configuration_11_1_12 'coldfusionservice.config.$cfName' 'coldfusion' '$cfName'
        done

        db_upgrade_service_node_properties_move_property_to_configuration_11_1_12 'server.configuration.crontabSecureShell' 'scheduler' 'crontabSecureShell'

        db_fix_drop_table ServiceNodeProperties
    fi

    if db_test_table 'MailServerProperties'; then
        db_do "REPLACE INTO \`ServiceNodeConfiguration\` (\`serviceNodeId\`, \`section\`, \`name\`, \`value\`) SELECT \`serviceNodeId\`, 'mailServer', \`name\`, \`value\` FROM \`MailServerProperties\`";

        db_fix_drop_table 'MailServerProperties'
    fi
}

db_upgrade_custom_buttons_11_1_12()
{
    echo_try "Converting custom buttons from domain to client level"
    db_do "UPDATE custom_buttons cb, (SELECT cb.id, 4 AS level, d.cl_id AS level_id FROM custom_buttons cb INNER JOIN domains d on cb.level_id = d.id WHERE level = 8) sq SET cb.level = sq.level, cb.level_id = sq.level_id WHERE cb.id = sq.id"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_service_node_configuration_11_1_13()
{
    db_select "SELECT DISTINCT serviceNodeId FROM ServiceNodeConfiguration WHERE serviceNodeId NOT IN (SELECT serviceNodeId FROM ServiceNodeConfiguration WHERE section = 'WebProxy' AND name = 'enabled')"
    for serviceNodeId in $db_select_output; do

        p_echo "Fix web proxy settings for service node $serviceNodeId"

        db_select "SELECT value FROM ServiceNodeConfiguration WHERE section = 'WebProxy' AND name = 'backendPort' AND serviceNodeId = '$serviceNodeId'"
        local backendPort="$db_select_output"
        if [ -z "$backendPort" ]; then
            backendPort="7080"
        fi

        db_select "SELECT value FROM ServiceNodeConfiguration WHERE section = 'WebProxy' AND name = 'sslBackendPort' AND serviceNodeId = '$serviceNodeId'"
        local sslBackendPort="$db_select_output"
        if [ -z "$backendPort" ]; then
            backendPort="7081"
        fi

        local proxyEnabled="false"
        if db_test "SELECT * FROM ServiceNodeConfiguration WHERE name = 'webServer' AND (section = 'httpPort' AND value = '$backendPort' OR section = 'httpsPort' AND value = '$sslBackendPort') AND serviceNodeId = '$serviceNodeId'" 1; then
            proxyEnabled="true"
        fi

        p_echo "Frontend port = $frontendPort, backend port = $backendPort, proxy enabled = $proxyEnabled"

        db_do "INSERT INTO ServiceNodeConfiguration (serviceNodeId, section, name, value) VALUES ('$serviceNodeId', 'WebProxy', 'enabled', '$proxyEnabled')"

    done


    db_do "UPDATE IGNORE ServiceNodeConfiguration SET name = section, section = 'webServer' WHERE name = 'webServer'"
    db_do "DELETE FROM ServiceNodeConfiguration WHERE name = 'webServer'"

    db_do "UPDATE IGNORE ServiceNodeConfiguration SET name = section, section = 'webProxy' WHERE name = 'webProxy'"
    db_do "DELETE FROM ServiceNodeConfiguration WHERE name = 'webProxy'"

    db_do "UPDATE ServiceNodeConfiguration SET section = 'webProxy' WHERE section = 'WebProxy'"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_dns_templates_11_1_14()
{
	echo_try "drop rows from psa.dns_recs_t with empty values of column 'type'"
	db_do "DELETE from dns_recs_t WHERE \`type\`=''"
	suc
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_subscription_properties_11_1_15()
{
	echo_try "Create subscription records"
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_subscriptions_108.php && suc || warn "fail\nCannot update subscriptions information. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_subscriptions_108.php manually."

	echo_try "Move limits and permissions from main domain of subscription to subscription properties"
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_subscription_properties_11.1.16.php && suc || warn "fail\nCannot update subscription properties. Please run $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_subscription_properties_11.1.16.php manually."
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_forwarding_11_1_17()
{
    db_fix_add_column "forwarding" "http_code" "INT UNSIGNED NOT NULL DEFAULT 301"
}

db_upgrade_seo_safe_domain_aliases_11_1_17()
{
    db_fix_add_column "domain_aliases" "seoRedirect" "ENUM('false', 'true') NOT NULL DEFAULT 'false'"
}

db_upgrade_db_user_unique_key_11_1_17()
{
    db_fix_drop_unique_key "db_users" "db_id" "login"
    db_fix_drop_unique_key "db_users" "db_server_id" "login"
    db_fix_add_unique_key "db_users" "db_server_id" "db_id" "login"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_php_handler_id_11_1_18()
{
    db_fix_add_column --init php_handler_type hosting php_handler_id "varchar(255) NOT NULL DEFAULT 'module'"
    db_do "REPLACE INTO \`TmplData\` (\`tmpl_id\`, \`element\`, \`value\`) SELECT \`tmpl_id\`, 'php_handler_id', \`value\` FROM \`TmplData\` WHERE \`element\` = 'php_handler_type'"
    db_do "DELETE from \`TmplData\` WHERE \`element\` = 'php_handler_type'"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_drop_php_version_11_5_19()
{
    db_fix_del_column 'hosting' 'php_handler_type'
    db_fix_del_column 'hosting' 'php_version'
}

db_upgrade_webmails_11_5_19()
{
    db_fix_add_column Webmails serviceNodeId "int(10) unsigned NOT NULL default 1"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_enable_dns_connector_11_5_21()
{
	local connector=

	db_select "select p.val from cl_param p INNER JOIN clients c ON c.id = p.cl_id WHERE c.type = 'admin' AND p.param = 'dns-connector-class-name' LIMIT 1"
	case "$db_select_output" in
		Service_Dns_PoaConnector)
			connector="poa"
		;;
		Service_Dns_PleskConnector)
			connector="plesk"
		;;
	esac

	if [ -n "$connector" ]; then
		db_select "SELECT \`id\` FROM \`ServiceNodes\` WHERE \`ipAddress\` = 'local'"
		local nodes="$db_select_output"
		for serviceNodeId in $nodes; do
			db_do_if_not 				"SELECT \`value\` FROM \`ServiceNodeConfiguration\` WHERE \`serviceNodeId\` = $serviceNodeId AND \`section\` = 'dnsConnector' AND \`name\` = '$connector'" 1 				"INSERT INTO \`ServiceNodeConfiguration\` (\`serviceNodeId\`, \`section\`, \`name\`, \`value\`) values ($serviceNodeId, 'dnsConnector', '$connector', 'true')"
		done
	fi
	db_do "DELETE FROM cl_param WHERE param='dns-connector-class-name'"
}

db_upgrade_set_force_update_applications_notification_11_5_21()
{
	db_select "SELECT DISTINCT d.cl_id AS clientId FROM Subscriptions AS s INNER JOIN apsContexts AS a ON (s.id = a.subscriptionId) INNER JOIN domains AS d ON (s.object_type = 'domain' AND s.object_id = d.id) LEFT JOIN cl_param AS clp ON (d.cl_id = clp.cl_id AND clp.param = 'show_force_update_applications_notification') WHERE clp.cl_id IS NULL"
	for clientId in $db_select_output; do
		db_do "INSERT INTO \`cl_param\` (\`cl_id\`, \`param\`, \`val\`) values ($clientId, 'show_force_update_applications_notification', 'true')"
	done
}

db_add_public_ip_11_5_21()
{
	db_fix_add_column "IP_Addresses" "public_ip_address" "VARCHAR(39) CHARACTER SET ascii COLLATE ascii_general_ci  DEFAULT NULL"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_drop_ip_address_id_11_5_22()
{
    db_fix_del_column "hosting" "ip_address_id"
    db_fix_del_column "forwarding" "ip_address_id"
}

db_upgrade_set_property_unpaid_website_status_for_plans_11_5_22()
{
    db_select "SELECT t.id AS planId FROM Templates AS t LEFT JOIN TmplData AS td ON (t.id = td.tmpl_id AND td.element = 'unpaid_website_status') WHERE t.type='domain' AND td.value IS NULL"
    for planId in $db_select_output; do
        db_do "INSERT INTO \`TmplData\` (\`tmpl_id\`, \`element\`, \`value\`) values ($planId, 'unpaid_website_status', 'disabled')"
    done
}

db_upgrade_set_property_unpaid_website_status_for_subscriptions_11_5_22()
{
    db_select "SELECT s.id AS subscriptionId FROM Subscriptions AS s LEFT JOIN SubscriptionProperties AS sp ON (s.id = sp.subscription_id AND sp.name = 'unpaid_website_status') WHERE s.object_type='domain' AND sp.value IS NULL"
    for subscriptionId in $db_select_output; do
        db_do "INSERT INTO \`SubscriptionProperties\` (\`subscription_id\`, \`name\`, \`value\`) values ($subscriptionId, 'unpaid_website_status', 'disabled')"
    done
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_correcting_property_unpaid_website_status_for_plans_11_5_24()
{
	db_do "UPDATE \`TmplData\` SET \`element\` = 'unpaid_website_status' WHERE \`element\` = 'unpaidWebsiteStatus';"
}

db_upgrade_correcting_property_unpaid_website_status_for_subscriptions_11_5_24()
{
	db_do "UPDATE \`SubscriptionProperties\` SET \`name\` = 'unpaid_website_status' WHERE \`name\` = 'unpaidWebsiteStatus';"
}

db_upgrade_maintenance_mode_id_11_5_24()
{
	db_select "SELECT \`d\`.\`id\` AS \`domainId\`, \`d\`.\`status\` AS \`domainStatus\`, COALESCE(\`w\`.\`id\`, \`d\`.\`id\`) AS \`webspaceId\`, \`d\`.\`cl_id\` AS \`clientId\`, COALESCE(\`w\`.\`permissions_id\`, \`d\`.\`permissions_id\`) AS \`permissionsId\` FROM \`hosting\` AS \`h\` INNER JOIN \`domains\` AS \`d\` ON \`d\`.\`id\` = \`h\`.\`dom_id\` LEFT JOIN \`domains\` AS \`w\` ON \`w\`.\`id\` = \`d\`.\`webspace_id\` WHERE \`h\`.\`maintenance_mode\` = 'true' AND \`d\`.\`status\` = 0"
    test -z "$db_select_output" && return 0
	echo "$db_select_output" | while read row; do
		domainId=`echo $row | awk '{print $1}'`
		domainStatus=`echo $row | awk '{print $2}'`
		webspaceId=`echo $row | awk '{print $3}'`
		clientId=`echo $row | awk '{print $4}'`
		permissionsId=`echo $row | awk '{print $5}'`

		if db_test "SELECT \`p\`.\`value\` FROM \`Subscriptions\` AS  \`s\` LEFT JOIN \`SubscriptionProperties\` AS \`sp\` ON \`sp\`.\`subscription_id\` = \`s\`.\`id\` AND \`sp\`.\`name\` = 'permissionsId' INNER JOIN \`Permissions\` AS \`p\` ON \`p\`.\`id\` = COALESCE(\`sp\`.\`value\`, $permissionsId) AND \`p\`.\`permission\` = 'manage_website_maintenance' AND \`p\`.\`value\` = 'false' WHERE \`s\`.\`object_id\` = $webspaceId AND \`s\`.\`object_type\` = 'domain'" 1; then
			if db_test "SELECT \`p\`.\`type\` FROM \`clients\` as \`c\` INNER JOIN \`clients\` as \`p\` ON \`p\`.\`id\` = \`c\`.\`parent_id\` AND \`p\`.\`type\` = 'reseller' WHERE \`c\`.\`id\` = $clientId" 1; then
				domainStatus=$(( $domainStatus | 32 ))
			else
				domainStatus=$(( $domainStatus | 16 ))
			fi
		else
			domainStatus=$(( $domainStatus | 64 ))
		fi

		db_do "UPDATE \`domains\` SET \`status\` = $domainStatus WHERE \`id\` = $domainId"
		db_do_if_not 			"SELECT \`dom_id\` FROM \`dom_param\` WHERE \`param\` = 'turnOffAction' AND \`dom_id\` = $domainId" 1 			"INSERT INTO \`dom_param\` (\`dom_id\`, \`param\`, \`val\`) VALUES ($domainId, 'turnOffAction', 'suspend')"
		db_do_if_not 			"SELECT \`dom_id\` FROM \`dom_param\` WHERE \`param\` = 'selfTurnOffAction' AND \`dom_id\` = $domainId" 1 			"INSERT INTO \`dom_param\` (\`dom_id\`, \`param\`, \`val\`) VALUES ($domainId, 'selfTurnOffAction', 'suspend')"
	done
}

db_upgrade_service_node_configuration_11_5_24()
{
    db_fix_change_column "ServiceNodeEnvironment" "value" "value" "TEXT CHARACTER SET utf8 COLLATE utf8_general_ci"
    db_fix_change_column "ServiceNodeConfiguration" "value" "value" "TEXT CHARACTER SET utf8 COLLATE utf8_general_ci"
}

db_upgrade_secret_keys_11_5_24()
{
    db_fix_add_column "secret_keys" "lookup_id" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci DEFAULT NULL"
    db_do_if_not "SHOW INDEX FROM secret_keys" "\$3 == \"lookup_id\"" "ALTER TABLE secret_keys ADD INDEX (lookup_id)"
    sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_secret_keys_11.5.25.php && suc || warn "fail\nencryption of secret keys"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
db_upgrade_correcting_aps_tokens_11_5_29()
{
    db_fix_change_column "ApsTokens" "packageId" "applicationId" "VARCHAR(2000) CHARACTER SET binary NOT NULL"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

upgrade_cgi_bin_mode_defaults_11_5_30()
{
    local script="$PRODUCT_ROOT_D/admin/plib/scripts/upgrade_cgi_bin_mode.php"
    if [ -f "$script" ]; then
        sw_engine_pleskrun "$script" >> "$product_log" 2>&1
    fi
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
db_upgrade_heterogenous_1100()
{
    db_fix_add_column "hosting" "maintenance_mode" "ENUM('false','true') NOT NULL DEFAULT 'false'"

    db_fix_add_column "clients" "passwd" "VARCHAR(20) NOT NULL DEFAULT ''"
    db_fix_add_column "clients" "ownership" "ENUM('false','true') NOT NULL DEFAULT 'true'"

    db_fix_add_column "dom_level_usrs" "passwd" "VARCHAR(20) NOT NULL DEFAULT ''"
    db_fix_add_column "dom_level_usrs" "uid" "VARCHAR(255) DEFAULT NULL"
    db_fix_add_column "dom_level_usrs" "ownership" "ENUM('false','true') NOT NULL DEFAULT 'true'"

    db_fix_change_column "domains" "guid" "guid" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_bin NOT NULL DEFAULT '00000000-0000-0000-0000-000000000000'"

    db_fix_add_column "disk_usage" "mysql_dbases" "BIGINT(20) UNSIGNED NOT NULL DEFAULT '0'"
    db_fix_add_column "disk_usage" "mssql_dbases" "BIGINT(20) UNSIGNED NOT NULL DEFAULT '0'"

    db_fix_change_column "hosting" "fp_adm" "fp_adm" "VARCHAR(20) CHARACTER SET ascii COLLATE ascii_bin NOT NULL"
    db_fix_change_column "hosting" "webstat" "webstat" "VARCHAR(20) NOT NULL DEFAULT 'none'"
    db_fix_add_column "hosting" "fp_no_index" "ENUM('false','true') NOT NULL DEFAULT 'false'"
    db_fix_add_column "hosting" "ssi_html" "ENUM('false','true') NOT NULL DEFAULT 'false'"
    db_fix_add_column "hosting" "php_isapi" "ENUM('false','true') NOT NULL DEFAULT 'false'"
    db_fix_add_column "hosting" "php_version" "VARCHAR(255) NOT NULL DEFAULT '4'"
    db_fix_add_column "hosting" "managed_runtime_version" "VARCHAR(255) NOT NULL DEFAULT '1.1'"
    db_fix_add_column "hosting" "at_domains" "ENUM('false','true') NOT NULL DEFAULT 'false'"
    db_fix_add_column "hosting" "write_modify" "VARCHAR(255) NOT NULL DEFAULT ''"
    db_fix_add_column "hosting" "webdeploy" "ENUM('false','true') NOT NULL DEFAULT 'false'"

    db_fix_add_column "web_users" "ssi_html" "ENUM('false','true') NOT NULL DEFAULT 'false'"
    db_fix_add_column "web_users" "php_isapi" "ENUM('false','true') NOT NULL DEFAULT 'false'"
    db_fix_add_column "web_users" "managed_runtime_version" "VARCHAR(255) NOT NULL DEFAULT '1.1'"
    db_fix_add_column "web_users" "write_modify" "VARCHAR(255) DEFAULT NULL"

    db_fix_add_column "ftp_users" "permission" "INT UNSIGNED NOT NULL DEFAULT 3";

    db_fix_add_column "DatabaseServers" "parameters_id" "INT(10) UNSIGNED NOT NULL"
	db_fix_add_column "DatabaseServers" "type" "ENUM('mysql','postgresql','mssql') NOT NULL"
	db_fix_change_column "DatabaseServers" "type" "type" "ENUM('mysql','postgresql','mssql') NOT NULL"

    db_fix_add_column "db_users" "passwd" "VARCHAR(16) NOT NULL DEFAULT ''"
    db_fix_add_column "db_users" "status" "ENUM('normal','orphaned') NOT NULL DEFAULT 'normal'"

    db_fix_change_column "TmplData" "element" "element" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL"
    db_fix_change_column "longtaskparams" "param" "param" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL"

    db_fix_add_column "protected_dirs" "dom_type" "ENUM('domain','subdomain') NOT NULL default 'domain'"

    db_fix_change_column "hosting" "php_handler_type" "php_handler_type" "ENUM('cgi', 'fastcgi', 'module', 'isapi') NOT NULL default 'module'"
    db_fix_change_column "subdomains" "php_handler_type" "php_handler_type" "ENUM('cgi', 'fastcgi', 'module', 'isapi') NOT NULL default 'module'"

    db_fix_add_column "mail" "disk_usage" "BIGINT DEFAULT 0 NOT NULL"

	if ! db_test_table "domain_aliases"; then
        db_do "RENAME TABLE domainaliases TO domain_aliases"
    fi
}

db_upgrade_apsmail_1100()
{
    db_fix_add_column "mail" "apsServiceAccountInstanceId" "INT UNSIGNED DEFAULT 0 NOT NULL"
    db_fix_add_column "mail" "apsAntivirusServiceInstanceId" "INT UNSIGNED DEFAULT 0 NOT NULL"
    db_fix_add_column "mail_aliases" "apsServiceInstanceId"  "INT UNSIGNED DEFAULT 0 NOT NULL"
    db_fix_add_column "mail_resp" "apsServiceInstanceId"  "INT UNSIGNED DEFAULT 0 NOT NULL"
    db_fix_add_column "mail_redir" "apsServiceInstanceId" "INT UNSIGNED DEFAULT 0 NOT NULL"
    db_fix_add_column "mail" "apsAntispamServiceInstanceId" "INT UNSIGNED DEFAULT 0 NOT NULL"
}

db_upgrade_service_nodes_1100()
{
    db_fix_drop_unique_key "ServiceNodes" "name"
    db_fix_change_column "ServiceNodes" "name" "description" "VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ''"
    db_fix_change_column "ServiceNodes" "remoteIpAddress" "ipAddress" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL DEFAULT 'local'"
    db_fix_add_unique_key "ServiceNodes" "ipAddress"
    db_fix_add_column "ServiceNodes" "type" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL DEFAULT 'plesk'"
    db_do "UPDATE ServiceNodes SET ipAddress='local' WHERE ISNULL(ipAddress) OR ipAddress=''"
}

db_upgrade_webusers_miva_1100()
{
	db_fix_add_column "web_users" "miva" "ENUM('false','true') NOT NULL DEFAULT 'false'"
}

db_upgrade_mail_server_settings_1100()
{
    db_select "SELECT \`param\` FROM \`misc\` WHERE \`param\` IN (         'spf_enabled', 'spf_behavior', 'spf_guess', 'spf_rules', 'spf_exp',         'disable_pop_auth', 'disable_smtp_auth',         'message_submission', 'message_submission_ip',         'password_strength', 'poplock_time', 'rbl', 'rbl_server', 'spf_rule', 'relay',         'domain_keys_sign', 'domain_keys_verify', 'ml_size',         'spamfilter_enabled', 'spamfilter_use_mailuser_prefs', 'spamfilter_max_children', 'spamfilter_max_addr_list_length',         'virusfilter'     )";
    for propertyName in $db_select_output; do
            db_do "DELETE FROM \`MailServerProperties\` WHERE \`name\`='$propertyName' AND \`serviceNodeId\`=1";
            db_do "INSERT INTO \`MailServerProperties\` (\`serviceNodeId\`, \`name\`, \`value\`) SELECT 1, \`param\`, \`val\` FROM \`misc\` WHERE \`param\` = '$propertyName'";
            db_do "DELETE FROM \`misc\` WHERE \`param\`='$propertyName'";
    done;
	db_fix_add_column "badmailfrom" "serviceNodeId" "INT UNSIGNED NOT NULL DEFAULT 1"
	db_fix_add_column "smtp_poplocks" "serviceNodeId" "INT UNSIGNED NOT NULL DEFAULT 1"

    db_fix_add_column "spamfilter" "serviceNodeId" "INT UNSIGNED NOT NULL DEFAULT 1"
	db_do_if 		"SHOW KEYS FROM spamfilter WHERE Key_name = 'username' AND Column_name = 'username'" 1 		"ALTER TABLE spamfilter DROP KEY \`username\`"
	db_do_if_not 		"SHOW KEYS FROM spamfilter WHERE Key_name = 'spamfilter_username_serviceNodeId' AND Column_name = 'serviceNodeId'" 1 		"ALTER TABLE spamfilter ADD UNIQUE KEY spamfilter_username_serviceNodeId (\`username\`, serviceNodeId)"

    db_fix_add_column "GL_settings" "serviceNodeId" "INT UNSIGNED NOT NULL DEFAULT 1"
	db_do_if 		"SHOW KEYS FROM GL_settings WHERE Key_name = 'param' AND Column_name = 'param'" 1 		"ALTER TABLE GL_settings DROP KEY \`param\`"
	db_do_if 		"SHOW KEYS FROM GL_settings WHERE Key_name = 'PRIMARY' AND Column_name = 'param'" 1 		"ALTER TABLE GL_settings DROP PRIMARY KEY"
	db_do_if_not 		"SHOW KEYS FROM GL_settings WHERE Key_name = 'PRIMARY' AND Column_name = 'serviceNodeId'" 1 		"ALTER TABLE GL_settings ADD PRIMARY KEY (serviceNodeId, \`param\`)"

    db_fix_add_column "GL_remote_domains" "serviceNodeId" "INT UNSIGNED NOT NULL DEFAULT 1"
	db_do_if 		"SHOW KEYS FROM GL_remote_domains WHERE Key_name = 'domain' AND Column_name = 'domain'" 1 		"ALTER TABLE GL_remote_domains DROP KEY \`domain\`"
	db_do_if_not 		"SHOW KEYS FROM GL_remote_domains WHERE Key_name = 'PRIMARY' AND Column_name = 'serviceNodeId'" 1 		"ALTER TABLE GL_remote_domains ADD PRIMARY KEY (serviceNodeId, \`domain\`)"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_upgrade_longtasks_1101()
{
    db_fix_add_column "longtasks" "context" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci DEFAULT NULL"
    db_fix_change_column "longtasks" "status" "status" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL DEFAULT 'not_started'"
    db_fix_add_column "longtasks" "statusProcessed" "INT UNSIGNED NOT NULL DEFAULT 0"

    db_fix_change_column "longtasks" "progress_text" "progressStatus" "VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL DEFAULT 'queue'"
    db_fix_add_column "longtasks" "progressParams" "BLOB DEFAULT NULL"
    db_fix_change_column "longtasks" "progress_max" "progressMax" "INT UNSIGNED NOT NULL DEFAULT 1"
    db_fix_change_column "longtasks" "progress_value" "progressValue" "INT UNSIGNED NOT NULL DEFAULT 0"
    db_fix_change_column "longtasks" "start_time" "startTime" "TIMESTAMP"
    db_fix_change_column "longtasks" "finish_time" "finishTime" "TIMESTAMP"

    db_fix_del_column "longtasks" "title"
    db_fix_del_column "longtasks" "owner"
    db_fix_del_column "longtasks" "owner_type"

    db_fix_change_column "longtaskparams" "val" "val" "BLOB DEFAULT NULL"

    db_do "DELETE FROM \`longtasks\`";
    db_do "DELETE FROM \`longtaskparams\`";
}

db_upgrade_db_users_1101()
{
    db_fix_add_column "db_users" "dom_id" "INT UNSIGNED NOT NULL DEFAULT 0"
    db_fix_add_column "db_users" "db_server_id" "INT UNSIGNED NOT NULL DEFAULT 0"
    db_do "UPDATE db_users, data_bases SET db_users.dom_id = data_bases.dom_id, db_users.db_server_id = data_bases.db_server_id WHERE data_bases.id = db_users.db_id"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

db_pmm_dashboard_90()
{
	db_do "UPDATE DashboardPresetConfig SET uri='/plesk/server/migration/' WHERE uri='/plesk/server/pmm/'"
}

db_fix_90()
{
	clean_dashboard_86
    add_db_changes 900
	db_add_gl_filter
	db_add_spf_to_dns_template
}

db_add_gl_filter()
{
	db_fix_add_column domains gl_filter "ENUM('on', 'off') NOT NULL DEFAULT 'on'"
}
db_add_spf_to_dns_template()
{
	db_do_if_not 		"SELECT id FROM dns_recs_t WHERE val='v=spf1 +a +mx -all'" 1 		"REPLACE INTO dns_recs_t(displayHost, host, type, displayVal, val, time_stamp) VALUES('<domain>.', '<domain>.', 'TXT', 'v=spf1 +a +mx -all', 'v=spf1 +a +mx -all', NOW()+8);"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
db_update_dashboard_951()
{
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/update_dashboard_presets_9_5_1.php 2>>"$product_log"
	[ "$?" -eq 0 ] || 	    warn "Unable to update home page presets, see log for details. Please rerun $PRODUCT_ROOT_D/admin/plib/scripts/update_dashboard_presets_9_5_1.php manually"
}

db_update_exp_event()
{
	db_test_table "exp_event" || return
    
	local query

	query="DELETE FROM exp_event WHERE NOT (event_type='started' OR event_type='stopped'
		OR event_type='created' OR event_type='updated' OR event_type='deleted'
		OR event_type='status_changed' OR event_type='terminated' OR event_type='flushed'
		OR event_type='installed' OR event_type='uninstalled' OR event_type='siteapp_added'
		OR event_type='siteapp_removed' OR event_type='expired' OR event_type='exceeded'
		OR event_type='guid_changed')"

	db_do "$query"

	query="DELETE FROM exp_event WHERE NOT (obj_class='license' OR obj_class='service' 
		OR obj_class='ip_address' OR obj_class='admin_info' OR obj_class='siteapp'
		OR obj_class='session_preferences' OR obj_class='plesk_component' OR obj_class='client'
		OR obj_class='client_limits' OR obj_class='client_status' OR obj_class='client_prefs'
		OR obj_class='client_perms' OR obj_class='client_ip_pool' OR obj_class='client_limit_traffic'
		OR obj_class='client_limit_size' OR obj_class='domain' OR obj_class='domain_limits'
		OR obj_class='domain_user' OR obj_class='domain_alias' OR obj_class='domain_status'
		OR obj_class='phosting' OR obj_class='fhosting' OR obj_class='domain_limit_traffic'
		OR obj_class='domain_limit_size' OR obj_class='db_server' OR obj_class='subdomain'
		OR obj_class='mailname' OR obj_class='webuser' OR obj_class='maillist' 
		OR obj_class='dns_zone' OR obj_class='mailname_antivirus' OR obj_class='mailname_spamfilter'
		OR obj_class='mailname_mailgroup' OR obj_class='mailname_autoresponder' 
		OR obj_class='mailname_attachment' OR obj_class='remote_dns' OR obj_class='dashboard_preset' 
		OR obj_class='dashboard_preset_type' OR obj_class='dashboard_preset_name'
		OR obj_class='domain_perfomance' OR obj_class='template_domain' OR obj_class='template_client'
		OR obj_class='template_admin' OR obj_class='log_rotation' OR obj_class='server_ip'
		OR obj_class='db' OR obj_class='db_user' OR obj_class='reseller' OR obj_class='reseller_limits'
		OR obj_class='reseller_status' OR obj_class='reseller_prefs' OR obj_class='reseller_perms'
		OR obj_class='reseller_ip_pool' OR obj_class='reseller_limit_traffic'
		OR obj_class='reseller_limit_size')"

	db_do "$query"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
set_drweb_params()
{
	drweb_user="drweb"
	drweb_group="drweb"

	drweb_service="drwebd"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#proftpd

set_ftpd_params()
{
	anonftp_user="${product}ftp"
	anonftp_group="${product}ftp"

	PROFTPD_ROOT="$PRODUCT_ROOT_D/ftpd"
	PROFTPD_ETC_D="$PROFTPD_ROOT/etc"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
upgrade_proftpd_107()
{
	echo_try "upgrade anon_ftp quota to new format"
	local domain;
	for domain in $HTTPD_VHOSTS_D/*; do
		[ -e $domain/anon_ftp/incoming/quotadir ] || continue
		rm -f $domain/anon_ftp/incoming/quotadir/.undeletable 			$domain/anon_ftp/incoming/quotadir/.quota >> $product_log 2>&1
		rmdir $domain/anon_ftp/incoming/quotadir >> $product_log 2>&1
	done
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/upgrade_proftpd_107.php && suc || warn "fail\nupgrade anon_ftp quota to new format"
}

tomcat_reconfigure_server_safe()
{
	echo_try "reconfigure Tomcat services"
	local ports=

	# needed for proper work tomcatmng
	conf_setval "${prod_conf_t}" CATALINA_HOME "${CATALINA_HOME}"

	for name in warp coyote; do
		db_select "SELECT val FROM misc WHERE val = '${name}_connector_port'" 			|| {
				p_echo "Failed to get Tomcat parameter '${name}_connector_port' from the database"
				return 1
			}
		if [ -n "$db_select_output" ]; then
			ports="--${name}-connector-port $db_select_output"
		fi
	done

	"$UTILS_DIR/tomcatmng" --install-psa --replace-if-exists $ports 		|| {
			p_echo "Failed to configure Tomcat PSA service"
			return 1
		}

	db_select "SELECT d.name
				FROM domains d, DomainServices ds
				WHERE d.id = ds.dom_id
					AND type='tomcat'
					AND ds.status = 0" 		|| {
			p_echo "Failed to get active Tomcat domains"
			return 1
		}
	local domains="`echo $db_select_output | tr ' ' ','`"

	if [ -n "$domains" ]; then
		"$PLESK_LIBEXEC_DIR/tomcat_upgrade_helper" --reconfigure-domains "$domains" 			|| {
				p_echo "Failed to reconfigure Tomcat settings for active domains"
				return 1
			}
	fi

	suc
	return 0
}

tomcat_reconfigure_server()
{
	tomcat_reconfigure_server_safe || warn "Tomcat server reconfiguration"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:syntax=sh

set_swkey_params()
{
	SWKEY_REPOSITORY="/etc/sw/keys"
	KEYUPGRADE_PATH="/usr/lib/plesk-9.0/key-upgrade"
	KEYUPGRADE8_PATH="/usr/lib/plesk-9.0/key-upgrade8"
}

key_upgrade_cmd()
{
	set_swkey_params
	# set to run key upgrade script
	p_echo "Starting Plesk Key upgrade task"
	nohup /bin/sh -c "$KEYUPGRADE_PATH 11.5.30 2>&1 | logger -p daemon.err -t plesk-key-upgrader" 		>/dev/null 2>&1 &
}

license_upgrade_1013()
{
	sw_engine_pleskrun $PRODUCT_ROOT_D/admin/plib/scripts/license_upgrade_1013.php >> $product_log 2>&1
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh
# Usage:  pleskrc <service> <action>
pleskrc()
{
	[ 2 -le $# ] || die "Not enough arguments"

	local service_name=$1
	local action=$2
	local ret=0
	local inten
	local service_script=""
	shift
	shift

	# Now check redefined functions
	if test "$machine" = "linux" && is_function "${service_name}_${action}_${machine}_${linux_distr}"; then
		"${service_name}_${action}_${machine}_${linux_distr}" $@
		return $?
	elif is_function "${service_name}_${action}_${machine}"; then
		"${service_name}_${action}_${machine}" $@
		return $?
	elif is_function "${service_name}_${action}"; then
		"${service_name}_${action}" $@
		return $?
	fi

	# Not redefined - call default action
	eval "service=\$${service_name}_service"
	[ -n "$service" ] || die "$action $service_name service (Empty service name for '$service_name')"

	inten="$action service $service"
	[ "$action" = "status" -o "$action" = "exists" ] || echo_try "$inten"

	service_ctl $action $service $service_name

	ret="$?"
	if [ "$action" != "status" -a "${action}" != "exists" ]; then
		[ "$ret" -eq 0 ] && suc || warn $inten
	fi

	return $ret
}

# NOTE:
# 	Function service_ctl in both variations are just helper for pleskrc().
# 	Do not call it directly, use pleskrc()!!!
service_ctl()
{
	local action=$1
	local service=$2
	local service_name=$3

	local action_cmd=
	local service_script=

	if [ -x "$SYSTEM_RC_D/$service" ]; then
		service_script="$SYSTEM_RC_D/$service"
	elif [ ! -z "${PRODUCT_RC_D}" -a -x "$PRODUCT_RC_D/$service" ]; then
		service_script="$PRODUCT_RC_D/$service"
	fi

	if [ "$action" = "exists" ]; then 
		[ -n "$service_script" ] && return 0 || return 1
	elif [ -z "$service_script" ]; then
		warn "Unable to ${inten} - control script doesn't exist or isn't executable"
		return 1
	fi


	if [ -x "/usr/sbin/invoke-rc.d" ]; then
		action_cmd="/usr/sbin/invoke-rc.d $service"
	elif [ -x "/sbin/service" ]; then
		action_cmd="/sbin/service $service"
	elif [ -n ${service_script} ]; then
		action_cmd="$service_script"
	fi

	
	case "$action" in
		start)
			pleskrc $service_name status || $action_cmd $action
			;;
		stop)
			! pleskrc $service_name status || $action_cmd $action
			;;
		restart)
			if pleskrc $service_name status; then 
				$action_cmd $action
			else 
				$action_cmd start
			fi
			;;
		reload)
			! pleskrc $service_name status || $action_cmd $action
			;;
		status)
		    $action_cmd "status"
			;;
	    *)
		    $action_cmd $action
			;;
	esac >> $product_log 2>&1
}

is_function()
{
	local type_output="`type \"$1\" 2>/dev/null | head -n1 | awk '{print $NF}'`"
	test "X${type_output}" = "Xfunction"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
form_undotted_version()
{
# Backward compatible with versions < 11.1.11
	local major=`echo $1 | awk -F. '{ print $1 }'`
	local minor=`echo $1 | awk -F. '{ print $2 }'`
	local patch=`echo $1 | awk -F. '{ print $3 }'`
	if [ -z "$major" -o -z "$minor" -o -z "$patch" ]; then
		return 1
	fi

	if [ $major -lt 11 ] ; then
		echo "${major}${minor}"
	elif [ $major -eq 11 -a $minor -lt 1 ] || [ $major -eq 11 -a $minor -eq 1 -a $patch -lt 11 ]; then
		printf "%d%02d" $major $minor
	else
		printf "%03d%03d%03d" $major $minor $patch
	fi
}

set_core_version()
{
    local prod_version="$1"
    local version
    local core_version

    versions_list="10.0.0_10.8.0 10.0.1_10.9.0 10.1.0_10.10.0 10.1.1_10.10.1 10.2.0_10.11.0 10.3.1_10.12.0 10.4.4_10.13.4"

    for pair in $versions_list; do
        eval `echo "$pair" | awk -F '_' '{print "version="$1; print "core_version="$2}'`
        if [ "$version" = "$prod_version" ]; then
            prev_version="$core_version"
        fi
    done
}

get_prev_version()
{
	local oldest_known_product_version="90"
	local known_old_product_versions="90 92 93 95 100 101 102 103 104 107 108 109 1010 1011 1012 1013 1100 1101"

	# Debian package system reports previous version as script argument
	if [ "X${prev_version}" = "X" ]; then

		if [ -f "$1/core.version" ]; then
		# Check version from core
		    prev_version=`cat $1/core.version | awk '{ print $1 }'`
		    prev_build=`cat $1/core.version | awk '{ print $4 }'`
		elif [ -f $1/version ]; then
		# version file must exists in product directory
		    prev_version=`cat $1/version | awk '{ print $1 }'`
		    prev_build=`cat $1/version | awk '{ print $4 }'`
                    set_core_version "$prev_version"
		elif [ -f $1/version.upg ]; then
		# version file must exists in product directory
		    prev_version=`cat $1/version.upg | awk '{ print $1 }'`
		    prev_build=`cat $1/version.upg | awk '{ print $4 }'`
		else
		    p_echo "    Unable to find version file $1. "
		    p_echo "    Please, restore version file or remove old ${PRODUCT_NAME}, if unnecessary."
		    exit 1
		fi
	fi

	prev_short_version=`form_undotted_version ${prev_version}`
	if [ "X${prev_version}" = "X" -o "X${prev_build}" = "X" -o -z "$prev_short_version" ]; then
		p_echo "    Unable to determine version or build number of ${PRODUCT_NAME} installed. It is impossible"
		p_echo "    to upgrade ${PRODUCT_NAME} to version $product_version."
		p_echo "    To try to install a new version without saving data remove '$1' directory"
		p_echo "    manually. Please backup all data before removing it if necessary."
		exit 1
	fi

	# check, that it is known version of product
	if [ $prev_short_version -le '1101' ] ; then
		for i in $known_old_product_versions ; do
			if [ "X${prev_short_version}" = "X${i}" ]; then
				return 0
			fi
		done
	else
		# We support all (?!) versions >= 11.1.11 (for now).
		return 0
	fi

	# version is unknown for upgrader. let the user to know the propblem
	p_echo "    Unable to upgrade. You have $prev_version version installed,"
	p_echo "    but the upgrader is incompatible with this version."
	p_echo "    Please consider using migration instead of an upgrade."
	exit 1
}

check_product_root()
{
	# check product root
	p_echo
	pnnl_echo "===> Checking for previous installation ... "

	do_upgrade=0

	if [ ! -s "$PRODUCT_ROOT_D/version" -a ! -s "$PRODUCT_ROOT_D/version.upg" -a ! -s "$PRODUCT_ROOT_D/core.version" ]; then
		p_echo "not found."
	else
		p_echo "found."
		get_prev_version ${PRODUCT_ROOT_D}
		do_upgrade=1
	fi
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

check_localtime_file()
{
	[ -f /etc/localtime ] && return 0
	report_problem 		"There is no /etc/localtime file in system." 		"Run tzsetup as root to set up your timezone"
	return 1
}

check_tmp_permissions()
{
	local tmp_perm_mask=777
	[ $((0`stat -c %a /tmp` & 0$tmp_perm_mask)) -eq $((0$tmp_perm_mask)) ] && return 0

	if [ -f /tmp/pp-bootstrapper-ez-templates-prep-install.flag ]; then
		chmod 1777 /tmp
		p_echo " /tmp file access permissions were adjusted to 1777."
		p_echo
		return 0
	else
		report_problem 			"/tmp should have at least $tmp_perm_mask file access permissions." 			"Run 'chmod 1777 /tmp' as root to set up proper file access permissions."
		return 1
	fi
}

preinstall_checks()
{
	check_localtime_file || exit 1
	check_tmp_permissions || exit 1
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

print_congratulations()
{

	printf "\a\a\a"
	p_echo
	p_echo
	p_echo
	p_echo

	local upgrade

	local print_address=`echo $ipaddress| awk '{print $1}'`

	pp_echo
	if [ "X${minor_problem}" != "X1" ]; then
		pp_echo "                             Congratulations!"
	else
		pp_echo
	fi
	pp_echo "        All stages of the installation/upgrade have been finished."
	pp_echo "        ${PRODUCT_NAME} is now running on your system."
	pp_echo "        To complete the system configuration process, please proceed to URL:"
	pp_echo "        https://${fullhost}:8443/ or"
	pp_echo "        https://${print_address}:8443/"
	pp_echo "        Use 'root' and '<YOUR root PASSWORD>' as login name and password in case of fresh installation."
	pp_echo "        Use 'admin' and '<YOUR admin PASSWORD>' as login name and password in case of upgrade,"
	pp_echo "        or if Plesk is already configured."
	pp_echo
	pp_echo "        Further, use the following commands to start and stop the system:"
	pp_echo "        $START_SH start and"
	pp_echo "        $START_SH stop respectively."
	pp_echo

	if [ "X${minor_problem}" = "X1" ]; then
		pp_echo " *****************************************************************************"
		pp_echo " *                                                                           *"
		pp_echo " *   WARNING:    Some problems occured during upgrade. You need to           *"
		pp_echo " *            check log file                                                 *"
		pp_echo " *            ${product_log}       *"
		pp_echo " *            to fix these problems.                                         *"
		pp_echo " *                                                                           *"
		pp_echo " *****************************************************************************"
		pp_echo
	fi

	if test -n "$upgrade_default_key_installed"; then

		pp_echo "*****************************************************************************"
		pp_echo
		pp_echo " WARNING:    Automatic license upgrade was not performed during"
		pp_echo "             the upgrade process. The product is currently running"
		pp_echo "             on the default license with limited functionality"
		pp_echo "             until such time that an appropriate license is installed."
		pp_echo
		pp_echo "*****************************************************************************"

	fi
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

# echo message to product log and console (always visible)
pp_echo()
{
    if [ -n "$product_log" ] ; then
        echo "$@" >> "$product_log" 2>&1
    fi
    echo "$@"
}

# echo message to product log, unless debug
p_echo()
{
    if [ -n "$PLESK_INSTALLER_DEBUG" -o -n "$PLESK_INSTALLER_VERBOSE" -o -z "$product_log" ] ; then
        echo "$@"
    else
        echo "$@" >> "$product_log" 2>&1
    fi
}

# echo message to product log without new line, unless debug
pnnl_echo()
{
    if [ -n "$PLESK_INSTALLER_DEBUG" -o -n "$PLESK_INSTALLER_VERBOSE" -o -z "$product_log" ] ; then
        echo -n "$*"
    else
        echo -n "$*" >> "$product_log" 2>&1
    fi
}

die()
{
	PACKAGE_SCRIPT_FAILED="$*"

	printf "\a\a"
	report_problem 		"ERROR while trying to $*" 		"Check the error reason(see log file: ${product_log}), fix and try again"

	selinux_close

	exit 1
}

simply_die()
{
	report_problem "$@"
	exit 1
}

# This function needs for some print warning when some services cannot restart
# if called with arg, then we add arg to variable SR_WARNING and print warning
# message for arg, else print warning message for SR_WARNING
service_restart_warning()
{
	if [ "X$1" != "X" ]; then
		if [ "X${SR_WARNING}" = "X" ]; then
			SR_WARNING=$1
		else
			for i in ${SR_WARNING}; do
				if [ "$i" = "$1" ]; then
					break
				fi
			done
			if [ "$1" != "$i" ]; then
				SR_WARNING=$SR_WARNING" "$1
			fi
		fi
		p_echo "WARNING!"
		p_echo "Some problems occured when $1 restart, check it after installation/upgrade complete"
	        p_echo
	 	p_echo "Continue..."
		p_echo
	else
		if [ "X${SR_WARNING}" != "X" ]; then
			p_echo "WARNING!"
		        p_echo "Some problems occured when following services restart:"
			for i in ${SR_WARNING}; do
				p_echo $i
			done
			p_echo "Don't forget to check it after installation/upgrade complete"
        		p_echo
	       	        p_echo "More information can be found in log file: ${product_log}"
			p_echo
		fi
	fi
}

warn()
{
	local inten
	inten="$1"
	p_echo
	p_echo "WARNING!"
	pnnl_echo "Some problems are found during $inten"
	p_echo "(see log file: ${product_log})"
	p_echo
	p_echo "Continue..."
	p_echo

	product_log_tail | send_error_report_with_input "Warning: $inten"

	[ -n "$PLESK_INSTALLER_DEBUG" -o -n "$PLESK_INSTALLER_VERBOSE" ] || 	product_log_tail
}

# Use this function to report failed actions.
# Typical report should contain
# - reason or problem description (example: file copying failed)
# - how to resolve or investigate problem (example: check file permissions, free disk space)
# - how to re-run action (example: perform specific command, restart bootstrapper script, run installation again)
report_problem()
{
	[ -n "$product_problems_log" ] || product_problems_log="/dev/stderr"

	p_echo
	if [ "0$problems_occured" -eq 0 ]; then
		echo "***** $process problem report *****" >> "$product_problems_log" 2>&1
	fi
	for problem_message in "$@"; do
		p_echo "$problem_message"
		echo "$problem_message" >> "$product_problems_log" 2>&1
	done
	p_echo

	product_log_tail | send_error_report_with_input "Problem: $@"

	[ -n "$PLESK_INSTALLER_DEBUG" -o -n "$PLESK_INSTALLER_VERBOSE" ] || 		product_log_tail

	problems_occured=1
}

comm_err()
{
  p_echo "       Unfortunately, this situation has not yet been"
  p_echo "       resolved."
  p_echo "       Please, visit $support_contact for assistance."
  p_echo
  p_echo "Exiting..."
  p_echo

  exit 1
}

gid_err()
{
  gid="$1"

  p_echo
  p_echo "ERROR: It seems there is a group with GID=$gid"
  p_echo "       in this system, but $PRODUCT_NAME needs the same GID"
  p_echo "       for its operation."

  send_error_report "Error: group with GID=$gid already exists."
  comm_err
}

group_err()
{
  group="$1"
  gid="$2"

  p_echo
  p_echo "ERROR: It seems that there is group $group"
  p_echo "       in your system. $PRODUCT_NAME uses the same group"
  p_echo "       name but with another group ID ($gid)."

  send_error_report "Error: group $group exists, but it doesn't have GID=$gid."
  comm_err
}

uid_err()
{
  uid="$1"

  p_echo
  p_echo "ERROR: It seems there is a user with UID=$uid"
  p_echo "       in this system, but $PRODUCT_NAME needs the same UID"
  p_echo "       for its operation."

  send_error_report "Error: user with UID=$uid already exists."
  comm_err
}

logname_err()
{
  user="$1"
  uid="$2"

  p_echo
  p_echo "ERROR: It seems that there is a $user in your system"
  p_echo "       with a UID other than that used by $PRODUCT_NAME ($uid)"
  p_echo "       or the $user is in the primary group, but this"
  p_echo "       group is not the one that $PRODUCT_NAME uses."

  send_error_report "Error: user $user exists, but it doesn't have UID=$uid or it belongs to wrong primary group."
  comm_err
}

echo_try()
{
	msg="$*"
	pnnl_echo " Trying to $msg... "
}

suc()
{
	p_echo "done"
}

send_error_report()
{
	echo -n | send_error_report_with_input "$@"
}

# do not call it w/o input! Use send_error_report in these cases.
send_error_report_with_input()
{
	{
		echo $@
		echo ""
		if [ -n "$error_report_context" ]; then
			echo "Context: $error_report_context"
			echo ""
		fi
		cat -
	} | $PRODUCT_ROOT_D/admin/bin/send-error-report "install" >/dev/null 2>&1
}

set_error_report_context()
{
	error_report_context="$@"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
get_ip_address_list()
{
	local ipaddress=''

	if test -x /sbin/ip ; then
		ipaddress=`/sbin/ip addr | grep -v 'inet6' | grep -v '127\.[0-9]*\.[0-9]*\.[0-9]*' | sed -n -e 's/^ *inet \([0-9\.]*\).*$/\1/ p'`
	else
		ipaddress=`$ifconfig |grep "$inet_str" | grep -v "inet6" | sed -e "s/$inet_str.\([0-9.]*\).*/\1/g"|awk '{print $1}' - | grep -v '^127\.[0-9]*\.[0-9]*\.[0-9]*$'`
	fi
	
	echo -n $ipaddress
}

get_domain()
{
	host="`${get_hostname}`"
	local try_dot=`echo "$host" | sed -e 's/^[^.]*\..*$/\./'`

	case "$try_dot" in
			.)	domain="`echo "$host" | sed -e 's/^[^.]*\.//'`"
				host="`echo $host | sed -e 's/\..*//'`"
				;;
			*)	domain="`$get_domainname`"
				;;
	 esac
	 domain=`echo "$domain" | sed 's/\.$//'`

	 if [ "X$host" = "X" -o "X${domain}" = "X" -o "$domain" = "(none)" ]; then
		p_echo "Warning!"
		p_echo "Unable to detect the fully qualified domain name of the host."
		p_echo "Please make sure that FQDN of the host is correctly specified in /etc/hosts and goes right after the host's IP address"
		p_echo "(depending on your OS, you might need to set the host name in /etc/HOSTNAME or /etc/hostname)."
		p_echo ""
		p_echo "Setting it to localhost.localdomain"
		host="localhost"
		domain="localdomain"
	 fi

 	 fullhost="${host}.${domain}"
	 return 0;
}

get_config_parameters()
{
 admin_email=

#list of addresses
 local ipaddress_list="`get_ip_address_list`"

 get_domain

 if [ "X$PLESK_DISABLE_HOSTNAME_CHECKING" = "X" -a "X$PLESK_VZ" != "X1" ]; then
 	check_hostname "$fullhost" || die "check the hostname" 
 else
    ipaddress="`echo "$ipaddress_list" | head -1`"
 fi

 start_after_install=1;

 p_echo
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

# vim:ft=sh

mk_backup()
{
	target="$1"
	dup="$2"
	opts="$3"

	if [ -L "$target" ]; then
		rm "$target"
	elif [ -$opts "$target" ]; then
		if [ ! -$opts "$target.$product_suffo" ]; then
			case "$dup" in
				mv)
					mv -f $target $target.$product_suffo || die "mv -f $target $target.$product_suffo"
					;;
				cp)
					cp -fp $target $target.$product_suffo || die "cp -fp $target $target.$product_suffo"
					;;
				*)
					p_echo " mk_backup: wrong option -- must be 'cp' or 'mv'"
					die "mk_backup"
					;;
			esac
		else
			case "$dup" in
				mv)
					mv -f $target $target.$product_suff || die "mv -f $target $target.$product_suff"
					;;
				cp)
					cp -fp $target $target.$product_suff || die "cp -fp $target $target.$product_suff"
					;;
				*)
					p_echo " mk_backup: wrong option -- must be 'cp' or 'mv'"
					die "mk_backup"
					;;
			esac
		fi
	else
		case "$opts" in
			f|d)
				;;
			*)
				p_echo " mk_backup: wrong option -- must be 'f' or 'd'"
				die "mk_backup"
				;;
		esac
	fi
}

# accumulates chown and chmod
set_ac()
{
	u_owner="$1"
	g_owner="$2"
	perms="$3"
	node="$4"

	chown $u_owner:$g_owner $node || die "chown $u_owner:$g_owner $node"
	chmod $perms $node || die "chmod $perms $node"
}

start_product_now()
{
	define_start_script
	p_echo
	if [ "X$start_after_install" = "X1" ]; then
		inten="start ${PRODUCT_NAME}"
		echo_try $inten
		p_echo
		$START_SH stop
		/etc/init.d/sw-cp-server stop

# [dpkg etc issue, bug #138010]
		[ -e "${START_SH}.dpkg-dist" ] && mv -f "${START_SH}.dpkg-dist" "${START_SH}"

		$START_SH start
	fi
}

start_product_now_final()
{
	unset PLESK_KEYUPDATE_DISABLED
	start_product_now
}

set_default_locale()
{
	p_echo 'Setting the default locale'
	default_locale_file="$PRODUCT_ROOT_D/admin/htdocs/locales/default"
	if [ -f $default_locale_file ]; then
		test -r "$default_locale_file" || die "set up default locale: cannot read '$default_locale_file'"
		default_locale="`cat $default_locale_file | sed 's/^\([^:]*\):.*/\1/'`"
		default_locale_name="`cat $default_locale_file | sed 's/^[^:]*:\([^:]*\):.*/\1/'`"
		default_locale_country="`cat $default_locale_file | sed 's/^[^:]*:[^:]*:[^:]*:\([^:]*\);.*/\1/'`"
	else
		default_locale="en-US"
		default_locale_name="ENGLISH"
		default_locale_country="UNITED STATES"
	fi
	p_echo "The default locale is set to $default_locale ($default_locale_name, $default_locale_country)"
}

set_services_info()
{
	if [ "1" = "${PLESK_VZ}" ]; then
		p_echo 'Setting the services information on VZ mode.'
		cp -f ${PRODUCT_ETC_D}/service/vzmode/*.xml ${PRODUCT_ETC_D}/service
	fi
}

detect_vz()
{
	PLESK_VZ=0
	PLESK_VE_HW_NODE=0
	PLESK_VZ_TYPE=

	local issue_file="/etc/issue"
	local vzcheck_file="/proc/self/status"
	[ -f "$vzcheck_file" ] || return 1

	local env_id=`sed -ne 's|^envID\:[[:space:]]*\([[:digit:]]\+\)$|\1|p' "$vzcheck_file"`
	[ -n "$env_id" ] || return 1
	if [ "$env_id" = "0" ]; then
		# Either VZ/OpenVZ HW node or unjailed CloudLinux
		PLESK_VE_HW_NODE=1
		return 1
	fi

	if grep -q "CloudLinux" "$issue_file" >/dev/null 2>&1 ; then
		return 1
	fi

	if [ -f "/proc/vz/veredir" ]; then
		PLESK_VZ_TYPE="vz"
	elif [ -d "/proc/vz" ]; then
		PLESK_VZ_TYPE="openvz"
	fi

	PLESK_VZ=1
	return 0
}

call_optional_function()
{
	export LANG=C LC_MESSAGES=C LC_ALL=C
	local type_output="`type \"$1\" 2>/dev/null | head -n 1`"
	case "$type_output" in
		*function)
			"$@"
			;;
		*)
			return 0
			;;
	esac
}

add_string()
{
	local pattern string config
	pattern="$1"
	string="$2"
	config="$3"

	if ! grep -q "^$pattern\$" "$config"; then
		echo "$string" >>"$config"
	fi
}

sh_get_args()
{
	echo 'while true; do case "$1" in '"$1"'*) break;; esac; shift; done'
}

get_phpini_var()
{
	[ -z "$1" ] && return
	cat $php_ini | perl -n -e 'next if(/^;/);  print $5 if(/^(\s*)'$1'(\s*)=(\s*)("*)(.*?)("|\s|;)/);'
}

# see also: common/php/plib/PhpSettings/Config.php
phpini_value_is_boolean()
{
    value=`echo "$1" | tr "[:upper:]" "[:lower:]"`
    for i in '1' 'on' 'true' 'yes' '0' 'off' 'false' 'no' 'none'; do
        [ "$value" = "$i" ] && return 0
    done
    return 1
}

phpini_value_is_integer()
{
    expr match "$1" '[[:digit:]]\+[KMGT]\?$' 2>&1 > /dev/null
}

phpini_quote_value()
{
    if   phpini_value_is_boolean "$1" ; then
        echo $1 |  tr "[:upper:]" "[:lower:]"
    elif phpini_value_is_integer "$1" ||          expr match "$1" '\".*\"$' 2>&1 >/dev/null ||          test -z "$1" ; then
        echo $1
    else
        echo "\"$1\""
    fi
}

set_phpini_var()
{
	local variable=$1
	local value=`phpini_quote_value $2`

	if [ -z "$variable" ]; then
		warn "set_phpini_var() error: variable name is empty."
		return
	fi

	last_entry_line=`grep -n "^[^a-zA-Z]*$variable" $php_ini | tail -n 1 | awk -F ':' '{print $1}'`;
	if [ -z "$last_entry_line" -o 0$last_entry_line -eq 0 ]; then
		echo "$1 = $value" >> $php_ini
		result=$?
	else
		tmp_file=`mktemp /tmp/php-iniXXXXXX`
		$cp_preserve $php_ini $tmp_file
		cat $php_ini | awk -v variable="$variable" -v value="$value" '
		{
			line+=1;
			if ($0 ~ /^[ \t]*'$variable'[ \t]*=/) {
				printf("%s = %s\n",variable,value);
				done=1;
			} else {
				print $0;
			}
			if (!done && (line>'$last_entry_line')) {
				printf("%s = %s\n",variable,value);
				done=1;
			}
		}' > $tmp_file && mv -f $tmp_file $php_ini >> $product_log 2>&1
		result=$?
	fi

	if [ 0$result -eq 0 ]; then
		echo "$variable=$value has been added to $php_ini" >> $product_log 2>&1
	else
		warn "set_phpini_var() error: can't add $variable=$value to $php_ini";
	fi
}

### the function similar to awk -F'$fs' 'print $N'
get_narg_fs()
{
	local IFS="$2"
	get_narg $3 $1
}

get_narg()
{
	shift $1 2>/dev/null || return
	echo $1
}

get_random_number()
{
	perl -e '
	 my $res = int(rand($ARGV[0]));
	 if (exists $ARGV[1]) {
	  $res = $res + $ARGV[1];
	 }
	 print $res;
	' "$@"
}

get_random_string()
{
	local str_length="$1"
	local str_symbols="$2"
	if [ -x "$PRODUCT_ROOT_D/admin/sbin/random_str" -a -z "$str_symbols" ]; then
		"$PRODUCT_ROOT_D/admin/sbin/random_str" "$str_length"
	else
		# random_str utility may be unavailable in pre phase
		if [ -z "$str_length" ]; then
			str_length="14"
		fi
		if [ -z "$str_symbols" ]; then
			str_symbols="A-Za-z0-9_"
		fi

		< /dev/urandom tr -dc "$str_symbols" 2>/dev/null | head -c "$str_length" 2>/dev/null
	fi
}

fix_etc_hosts()
{
	if ! grep -q -E '^127\..*[[:space:]]localhost([[:space:]]|$)' /etc/hosts; then
		awk '
			BEGIN{ set = 0; }
			/^127\.0\.0\.1[[:space:]]/{ print $0, " localhost"; set = 1; next; }
			{ print }
			END{ if (set == 0) { print "127.0.0.1 localhost"; } }
		' /etc/hosts >/etc/hosts.tmp 		&& mv /etc/hosts.tmp /etc/hosts 		|| report_problem "insert localhost entry in /etc/hosts"
	fi
}

sequence()
{
	if type seq >/dev/null 2>&1; then
		seq $*
	elif type jot >/dev/null 2>&1; then
		jot $*
	else
		die "Unable to find seq or jot command"
	fi
}

get_ini_conf_var()
{
	local conf="$1"
	local section="$2"
	local param="$3"

	[ -n "$conf" -a -n "$param" ] || die "get_ini_conf_var(): required parameters missing"

	local section_empty=0
	[ -n "$section" ] || section_empty=1

	perl -n -e 'BEGIN { $insect='$section_empty' }
				next if (/^\s*;/);
				$insect=0 if (/^\s*\[.*\]/);
				$insect=1 if (/^\s*\['$section'\]/);
				$val = $2, $val =~ s/\s+$//, print $val . "\n" 
					if ($insect && /^\s*('$param')\s*=\s*([^;\n]*)(;.*)?$/);' $conf | head -n 1
}

sw_engine_pleskrun()
{
	if [ -x $PRODUCT_ROOT_D/bin/sw-engine-pleskrun ]; then
		$PRODUCT_ROOT_D/bin/sw-engine-pleskrun "$@"
	elif [ -x "/usr/bin/sw-engine" ]; then
		local args
		if [ -e "$PRODUCT_ROOT_D/admin/conf/php.ini" ]; then
			args="-c $PRODUCT_ROOT_D/admin/conf/php.ini"
		else
			args="-d psasem.semfile=\"$PRODUCT_ROOT_D/var/psasem.sem\""
		fi
		/usr/bin/sw-engine $args "$@"
	else
		echo "No Plesk PHP interpreter found"
		return 1
	fi
}

install_default_license_key()
{
	pnnl_echo "install default license key ..."

# key from 8.x?
	if 	db_test_table "key_history" && 		db_test "SELECT COUNT(*) FROM key_history WHERE filename = '/etc/psa/psa.key' LIMIT 1" '$1!="0"'; then
			p_echo "not needed, license from 8.x found"
			return;
	fi

# check OF-key
	if sw_engine_pleskrun -r 'exit(of_get_key_by_product("plesk-unix") === false ? 1 :0);'; then
			p_echo "not needed, OF license found"
			return
	fi

	local dest_dir="/etc/sw/keys/keys"
	local key=plesk-default-key.xml

    [ ! -s "$dest_dir/$key" ] || return

    cp -f "${PRODUCT_ROOT_D}/tmp/$key" "$dest_dir/"
    chown root:swkey-data "$dest_dir/$key"
    chmod 0660 "$dest_dir/$key"
    suc
}

register_sw_engine_service()
{
	register_service sw-engine
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
label_file_once()
{
	local _fname _mess _messLines _fileHead _tmpFile

	test $# -eq 2 || die 'label_file_once() not enough arguments'
	
	_fname="${1}"
	_mess="${2}"

	inten="add header to file ${_fname}"
	echo_try $inten

	if [ ! -f "${_fname}" ]; then
		p_echo "can not add header to file ${_fname} : file does not exist"
		return
	fi
	_messLines=`printf "${_mess}" | wc -l | awk '{print $NF}'`

	_tmpFile=`basename "${_fname}"`
	_tmpFile=`mktemp "/tmp/${_tmpFile}.XXXXXX"` || die "can not create temporary file"
	head -n ${_messLines} "${_fname}" > "${_tmpFile}"

	printf "${_mess}" | diff -q - "${_tmpFile}" 1>/dev/null
	if [ $? -eq 0 ]; then
		p_echo "file ${_fname} already contains required header"
		rm -f "${_tmpFile}"
		return
	fi
	rm -f "${_tmpFile}"

	# preserve permissions, not optimal, need to rework
	cp -p "${_fname}" "${_fname}.tmp"

	printf "${_mess}" | cat - "${_fname}" > "${_fname}.tmp"
	cp -f "${_fname}.tmp" "${_fname}"
	rm -f "${_fname}.tmp"
	p_echo "completed successfully"
}

lookup_command()
{
	test $# -eq 1 || die 'lookup_command() not enough arguments'
	local _path=/bin:/usr/bin:/usr/local/bin:/sbin:/usr/sbin:/usr/local/sbin:${PATH}
	
	local utility_path=`env PATH=${_path} which "$1" 2>/dev/null`
	if [ -z "$utility_path" ]; then
		p_echo "Unable to find utility '$1'in ${_path}"
		return
	fi
	echo $utility_path
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
set_super_server_params()
{
    case "${machine}_${linux_distr}" in
	linux_*)
	    superserver_mode=native
		superserver=xinetd
	    set_xinetd_params $superserver_mode
	;;
	*)
	    die "Unable to detect super server type"
	;;
    esac
}

set_xinetd_params()
{
    superserver_conf="/etc/xinetd.conf"
    superserver_service="xinetd"
    superserver_dir="/etc/xinetd.d"
}

superserver_reconfig()
{
    pleskrc superserver reload
}

configure_xinetd_compat()
{
	local inten="configure xinetd compatibility mode with $superserver"
	if [ "$linux_distr" = "debian" -o "$linux_distr" = "ubuntu" ]; then
		[ "$superserver_service" = "xinetd" ] || return 1
		[ -f "/etc/default/xinetd" ] || return 1
		grep -q 'XINETD_OPTS=.*-inetd_compat' /etc/default/xinetd && return 1
		echo_try $inten

		if ! grep -q '^\s*XINETD_OPTS' /etc/default/xinetd; then
			echo 'XINETD_OPTS="-inetd_compat"' >>/etc/default/xinetd
			suc
			return 0
		fi

		eval `grep '^\s*XINETD_OPTS' /etc/default/xinetd`
		XINETD_OPTS="$XINETD_OPTS -inetd_compat"
		local tmp_file=`mktemp /tmp/xinetdXXXXXX`
		sed -e "s/XINETD_OPTS.*/XINETD_OPTS=\"$XINETD_OPTS\"/g" /etc/default/xinetd > $tmp_file && mv -f $tmp_file /etc/default/xinetd
		suc
		return 0
	fi
	return 1
}

deconfigure_xinetd_compat()
{
	local inten="deconfigure xinetd from compatibility mode with $superserver"

	# non-zero value means no xinetd reload is required
	local xinetd_restart=1

	if [ "$linux_distr" = "debian" -o "$linux_distr" = "ubuntu" ]; then
		[ "$superserver_service" = "xinetd" ] || return 1
		[ -f "/etc/default/xinetd" ] || return 1

		local tmp_file
		grep -q '^[[:space:]]*INETD_COMPAT[[:space:]]*=.*$' /etc/default/xinetd
		if [ $? -eq 0 ]; then
			tmp_file=`mktemp /tmp/xinetdXXXXXX`
			sed -e "s/^\s*INETD_COMPAT\s*=.*$/INETD_COMPAT=No/g" /etc/default/xinetd > $tmp_file && mv -f $tmp_file /etc/default/xinetd

			# xinetd reload is necessary
			xinetd_restart=0
		fi

		grep -q 'XINETD_OPTS=.*-inetd_compat' /etc/default/xinetd || return $xinetd_restart
		echo_try $inten

		tmp_file=`mktemp /tmp/xinetdXXXXXX`
		sed -e "s/-inetd_compat//g" /etc/default/xinetd > $tmp_file && mv -f $tmp_file /etc/default/xinetd
		suc

		# xinetd reload is necessary
		return 0
	fi

	# xinetd reload is not necessary
	return 1
}

super_server_action()
{
    local in out
    local action="$1"
    local service="$2"
    local template="$3"

    inten="$action $service service record for $superserver_service daemon"

    case "$action" in
	remove) ;;
	register)
		[ -z "$template" ] && die "Template for super server $action was not defined"
		;;
	comment|disable)
		;; 
	configure)
		case "$superserver_mode" in
		    native)
			register_service $superserver_service defaults defaults
			deconfigure_xinetd_compat && pleskrc superserver restart
			;;
		    compat)
			configure_xinetd_compat && pleskrc superserver restart
			;;
		    *)
			die "Mode for $superserver_service was not defined"
			;;
		esac

		return 0
		;;
	*)
		die "Some arguments was not defined or defined incorrect for action with super server"
		;;
    esac 

    case "$superserver" in
	inetd)
		super_server_modify_inetd "$service" "$action" "$template"
		;;
	xinetd)
		super_server_modify_xinetd "$service" "$action" "$template"
		;;
	*)
		die "Unable to define super server type"
		;;	
    esac

    if [ $? -ne 0 ]; then
	die $inten 
    fi
}

super_server_modify_inetd()
{
	local service="$1"
	local action="$2"
	local template="$3"

	case "$action" in
		comment|disable)
			grep -q "^$service[[:space:]]" $superserver_conf || return 0
			sed -e "s|^$service|#$service|g" < "$superserver_conf" > "$superserver_conf.tmp" 			&& mv -f "$superserver_conf.tmp" "$superserver_conf" 			|| return 1
			;;
		remove)
			if [ -x /usr/sbin/update-inetd ]; then
				/usr/sbin/update-inetd --enable "$service"
				/usr/sbin/update-inetd --remove "$service"
			else
				grep -q "^$service[[:space:]]" $superserver_conf || return 0
				sed -e "s|^$service[[:space:]].*||g" < "$superserver_conf" > "$superserver_conf.tmp" 			    	&& mv -f "$superserver_conf.tmp" "$superserver_conf" 			    	|| return 1
			fi
			;;
		register)
			if [ -x /usr/sbin/update-inetd ]; then
				/usr/sbin/update-inetd --enable "$service"
				/usr/sbin/update-inetd --remove "$service"
				/usr/sbin/update-inetd --add "$template"
			else
				egrep -q "$template" $superserver_conf
				if [ "$?" -ne "0" ]; then 
					super_server_modify_inetd comment $service && 					echo "$template" >> $superserver_conf || return 1
				fi
			fi
			;;
	esac

	return 0
}

super_server_modify_xinetd()
{
	local file
	local service="$1"
	local action="$2"
	local template="$3"

	for file in $superserver_dir/*; do
		grep -q "$service" $file 1>>$product_log 2>&1 || continue

		case "$action" in
		    remove)
			    awk "/^[[:space:]]*(#|[[:space:]])*service[[:space:]]+$service($|[[:space:]]+)/,/.*}.*/ 				{next;} {print}
			    " <$file >$file.tmp 			    && mv -f $file.tmp $file || return 1
			    ;;
		    comment)
			    awk "/^[[:space:]]*service[[:space:]]+$service($|[[:space:]]+)/,/.*}.*/ 			       { print \"#\"\$0; next; }
			       {print}
			    " < $file > $file.tmp 			    && mv -f $file.tmp $file || return 1
			    ;;
		esac
	done

	case "$action" in
	    register)
		    echo "$template" > "$superserver_dir/${service}_psa" || return 1
		    label_file_once "$superserver_dir/${service}_psa" "${AUTOGENERATED_CONFIGS_UPGRADE}"
		    ;;
	    disable)
			[ -f "$superserver_dir/${service}_psa" ] && mv -f "$superserver_dir/${service}_psa" "$superserver_dir/${service}.psa"
		    ;;
	esac

	return 0
}

true superserver_status_linux_debian
superserver_status_linux_debian()
{
    get_pid "/usr/sbin/xinetd" false
    local pid=$common_var
    if test "$pid" -ne 1; then
# running
        return 0
    fi
    return 1
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
lockfile()
{
	local tempfile="$1.$$"
	local lockfile="$1.lock"

	echo $$ > $tempfile 2>/dev/null || {
		p_echo "Unable to create lock file in `dirname $tempfile`"
		return 1
	}

	ln $tempfile $lockfile >/dev/null 2>&1 && {
		rm -f $tempfile
		return 0
	}

	kill -0 `cat $lockfile` >/dev/null 2>&1 && {
		rm -f $tempfile
		return 1
	}

	p_echo "Removing stale lock file $lockfile"
	rm -f $lockfile
	ln $tempfile $lockfile >/dev/null 2>&1 && {
		rm -f $tempfile
		return 0
	}

	rm -f $tempfile
	return 1
}

unlockfile()
{
	rm -f "$1.lock"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
#-*- vim:syntax=sh

product_log_name_ex()
{
	local aux_descr="$1"
	local action="${CUSTOM_LOG_ACTION_NAME-installation}"

	if [ -n "$aux_descr" ]; then
		aux_descr="_${aux_descr}"
	fi

	if [ -n "$CUSTOM_LOG_NAME" ]; then
		echo "${CUSTOM_LOG_NAME}${action:+_$action}${aux_descr}.log"
	else
		echo "plesk_11.5.30${action:+_$action}${aux_descr}.log"
	fi
}

product_log_name()
{
	product_log_name_ex
}

product_problems_log_name()
{
	product_log_name_ex "problems"
}

problems_log_tail()
{
	[ -f "$product_problems_log" ] || return 0
	tac "$product_problems_log" | awk '/^START/ { exit } { print }' | tac
}

product_log_tail()
{
	[ -f "$product_log" ] || return 0
	tac "$product_log" | awk '/^START/ { exit } { print }' | tac
}

cleanup_problems_log()
{
	awk 'BEGIN 						{ st = "" } 
		 /^START/ 					{ st=$0; next } 
		 /^STOP/ && (st ~ /^START/) { st=""; next } 
		 (st != "") 				{ print st; st="" } 
		 							{ print }
		' "$product_problems_log" > "$product_problems_log.tmp" && 	mv -f "$product_problems_log.tmp" "$product_problems_log" || 	rm -f "$product_problems_log.tmp"
	
	if [ ! -s "$product_problems_log" ]; then 
		rm -f "$product_problems_log"
	fi
}

set_log_action_name()
{
	CUSTOM_LOG_ACTION_NAME="$1"
}

log_start()
{
	true product_log_name product_problems_log_name

	local title="$1"
	product_log="/tmp/`product_log_name`"
	product_problems_log="/tmp/`product_problems_log_name`"
	problems_occured=0

	echo "START $title" >>"$product_log" 	&& echo "START $title" >>"$product_problems_log" 	&& chmod 600 "$product_log" "$product_problems_log" 	|| {
		echo "Cannot write installation log $product_log, $product_problems_log" >&2
		exit 1
	}

	is_function profiler_setup && profiler_setup "$title" || :
}

log_stop()
{
	local title="$1"
	local subject="$2"
	if [ -z "$subject" ]; then
		subject="[${title}]"
	fi

	# check if problems are non-empty, check for problems_occured
	local status
	local problem_lines="`problems_log_tail | wc -l`"
	if [ "$problem_lines" -eq 0 ]; then
		status="completed successfully"
	else
		if [ $problems_occured -ne 0 ]; then
			status="failed"
		else
			status="completed with warnings"
		fi
	fi

	p_echo
	p_echo "**** $subject $status."
	p_echo

	if [ "$problem_lines" -ne 0 ]; then
		problems_log_tail >>"$product_log" 2>&1
		problems_log_tail
	fi

	echo "STOP $title" >>"$product_log"
	if [ $problems_occured -ne 0 ]; then
		echo "STOP $title: PROBLEMS FOUND" >>"$product_problems_log"
	else
		echo "STOP $title: OK" >>"$product_problems_log"
	fi

	if [ "X${PLESK_INSTALLER_KEEP_PROBLEMS_LOG}" = "X" ]; then
		cleanup_problems_log
	fi

	is_function profiler_stop && profiler_stop || :
}

report_context_action()
{
	local action
	get_product_versions
	if [ "X$do_upgrade" = "X1" -o "$product_prev_version" != "$product_this_version" ]; then
		action="Doing $product_name upgrade from $product_prev_version to $product_this_version"
	else
		action="Doing $product_name $product_this_version installation"
	fi
	echo "$action"
}

set_maintenance_mode_flag()
{
	touch /var/lock/parallels-panel-maintenance-mode.flag
}

reset_upgrade_failure_flag()
{
	rm -f /var/lock/parallels-panel-maintenance-mode.flag
	rm -f /var/lock/parallels-panel-upgrade-failure.flag
}

reset_maintenance_mode_flag()
{
	rm -f /var/lock/parallels-panel-maintenance-mode.flag
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

generate_password_file()
{
	local passwd_file="$1"
	local title="$2"
	local user="$3"
	local group="$4"
	local mode="$5"

	if [ "$#" -ne 5 ]; then 
		die "Some arguments was not defined for this function"
	fi

	local passwd
	local inten="generate random password for $title"

	echo_try $inten

	if [ ! -s "$passwd_file" ]; then 
		passwd=`get_random_string` || die $inten
		echo "$passwd" > "$passwd_file" || die $inten
		set_ac $user $group $mode $passwd_file || die $inten
	else
		set_ac $user $group $mode $passwd_file || die $inten
	fi
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

get_pid()
{
	local i

	local ex_f="$1"
	local opt="$2"
	local owner="$3"

	local min_num="1"

#	Use pidof by default, bug 121868, except for FreeBSD - 140182
	if type pidof >/dev/null 2>&1 && [ "$os" != "BSD" ]; then
		for pid in `pidof -o $$ -o $PPID -o %PPID -x $ex_f`; do
#	Check for owner
			[ "$opt" = "true" -a "$owner" != "`ps -p $pid -o ruser=`" ] && continue
			min_num=$pid
			break
		done
		common_var=$min_num
		return $min_num
	fi

	case "$opt" in
		false)
			for i in `$ps_long | grep $ex_f | grep -v grep | grep -v httpsdctl | grep -v apachectl | awk '{print $2}' -`; do
				min_num=$i
				break
			done
			;;
		true)
			for i in `$ps_long | grep $ex_f | grep -v grep | grep -v httpsdctl | grep -v apachectl | grep "$owner" | awk '{print $2}' -`; do
				min_num=$i
				break
			done
			;;
		*)
			p_echo "get_pid: wrong parameter"
			die "get_pid $ex_f $opt $owner"
			;;
	esac

	common_var=$min_num
	return $min_num
}

kill_pids()
{
	ex_f="$1"
	owner="$2"

	for i in `$ps_long | grep $ex_f | grep -v grep | grep -v httpsdctl | grep -v apachectl | grep $owner | awk '{print $2}' -`; do
		if [ $i -gt 1 ]; then
			$K_TERM $i >> $product_log 2>&1
		fi
	done
}

check_hostname()
{
    local hostname="$1"
    local output

	echo_try "resolve hostname '${hostname}' and validate its IP address"
	output="`perl -e '
		use strict;
		use Socket;
		use Errno;

		my $host = shift || "localhost";
		my $port = 715;

		my $proto = getprotobyname("tcp");
		if (!$proto) {
			print "System  error: $!";
			exit 1;
		}
		my $addr = inet_aton($host);
		if(!$addr) {
			print "Unable to resolve host name.\n";
			print "Please specify the host name in the /etc/hosts file or specify DNS configuration in the /etc/resolv.conf on your system.";
			exit 1;
		}

		if (!socket(SOCK, PF_INET, SOCK_STREAM, $proto)) {
			print "System error: $!";
			exit 1;
		}

		my $sin = sockaddr_in($port, $addr);

		if (!bind(SOCK, $sin) && $! != Errno::EADDRINUSE) {
			print "Can not bind a socket to address $host:$port - $!\n";
			print "Please, check your network configuration";
			exit 1;
		}

		printf inet_ntoa($addr);
		close(SOCK);
		' $hostname `"

		if [ $? != 0 ]; then
			p_echo
			p_echo "${output}"
			return 1
		fi

		ipaddress=${output}
		suc
		return 0
}

txtdb_set_params()
{
	db_file=""
	db_owner=""
	db_group=""
	db_perms="0644"
	db_delim=':'
	db_key_field="1"
	"$@"
	[ -n "$db_file" -a -n "$db_delim" ] || die "set txtdb parameters. Check arguments for 'txtdb_*' calls."
}

txtdb_fix_perms()
{
	[ -n "$db_owner" -a -n "$db_group" ] && chown "$db_owner":"$db_group" "$db_file"
	[ -n "$db_perms" ] && chmod "$db_perms" "$db_file"
	:
}

# Create new empty DB or replace an existing one
txtdb_truncate()
{
	local set_params_func="$1"
	local db_file db_owner db_group db_perms db_delim db_key_field
	txtdb_set_params "$set_params_func"

	rm -f "$db_file" && touch "$db_file" && txtdb_fix_perms
}

# Check whether DB file already exists and is writable
txtdb_exists()
{
	local set_params_func="$1"
	local db_file db_owner db_group db_perms db_delim db_key_field
	txtdb_set_params "$set_params_func"

	[ -w "$db_file" ]
}

txtdb_do_update()
{
	local set_params_func="$1"
	local awk_script="$2"
	local key_value="$3"
	local data_value="$4"
	local db_file db_owner db_group db_perms db_delim db_key_field
	txtdb_set_params "$set_params_func"

	awk -F "$db_delim" -v KEY="$key_value" -v DATA="$data_value" -v KEY_FIELD="$db_key_field" 		"$awk_script" "$db_file" > "$db_file.new" 2>> $product_log && 		mv -f "$db_file.new" "$db_file" && txtdb_fix_perms
}

txtdb_do_select()
{
	local set_params_func="$1"
	local awk_script="$2"
	local key_value="$3"
	local data_value="$4"
	local db_file db_owner db_group db_perms db_delim db_key_field
	txtdb_set_params "$set_params_func"

	awk -F "$db_delim" -v KEY="$key_value" -v DATA="$data_value" -v KEY_FIELD="$db_key_field" 		"$awk_script" "$db_file" 2>> $product_log
}

# Insert or replace existing record identified by key field
txtdb_replace()
{
	local db_file db_owner db_group db_perms db_delim db_key_field
	txtdb_set_params "$1"

	# Note that records order is not preserved
	local replace_with="$2"
	local key_to_replace=`get_narg_fs "$replace_with" "$db_delim" "$db_key_field"`
	txtdb_do_update "$1" 'END { print DATA } ( $KEY_FIELD != KEY )' "$key_to_replace" "$replace_with"
}

# Select entire record by key field
txtdb_select()
{
	local key_to_select="$2"
	txtdb_do_select "$1" '( $KEY_FIELD == KEY )' "$key_to_select"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#
# Unified per-component cumulative upgrade support.
#

# --- Upgrader utility functions ---

cu_get_continue_stage()
{
	local continue_from="$1"
	get_narg_fs "$continue_from" '-' 1
}

cu_get_continue_id()
{
	local continue_from="$1"
	echo "`  get_narg_fs "$continue_from" '-' 2  `-`  get_narg_fs "$continue_from" '-' 3  `"
}

cu_get_current_continue_from()
{
	echo "$CU_LAST_STAGE-$CU_LAST_ACTION_ID"
}

# Remove extraneous delimiters (usually spaces) between array elements
normalize_array()
{
	echo $@
}

# Check that first argument is in array (other arguments)
is_in_array()
{
	local needle="$1"
	shift # everything else is haystack
	for elem in $@; do
		[ "$elem" = "$needle" ] && return 0
	done
	return 1
}

# Remove all instances of first argument from array (other arguments). Echoes normalized array with space delimiters.
remove_from_array()
{
	local what="$1"
	shift # everything else is array
	local new_array=
	for elem in $@; do
		[ "$elem" = "$what" ] || new_array="$new_array $elem"
	done
	echo $new_array
}

array_length_helper()
{
	echo $#
}

# Get number of elements in a given array
array_length()
{
	array_length_helper $@
}

# --- Chained upgrade actions checks ---

# cu_do <upgrade action> [<arguments>]: Execute an action or echo information about it
cu_do()
{
	if [ -n "$CU_DO_GET_ACTION_NAME" ]; then
		echo "$1"
		return
	fi

	if [ -n "$CU_CONTINUE_FROM_ID" ]; then
		# Run only actions starting from CU_CONTINUE_FROM_ID
		# WARNING: this will not work properly if there is a subshell execution somewhere along the chain!
		[ "$CU_LAST_ACTION_ID" = "$CU_CONTINUE_FROM_ID" ] && CU_CONTINUE_FROM_ID= || return
	fi

	if "$@"; then
		cu_remove_failed_action "$CU_LAST_ACTION_ID"
		return 0
	else
		cu_add_failed_action "$CU_LAST_ACTION_ID"
		return 1
	fi
}

# Echo resulting action name (function name) or empty if skipped
cu_get_action_name()
{
	local CU_DO_GET_ACTION_NAME="yes"
	"$@"
}

# Execute action only if it wasn't applied before (by timestamp, e.g. use `date '+%Y%m%d%H%M%S'` or even `date '+%Y/%m/%d %H:%M:%S'`)
cu_check_timestamp()
{
	local timestamp="`echo \"$1\" | tr -dc '[:digit:]'`"
	shift

	true cu_get_action_name
	CU_LAST_ACTION_ID="$timestamp-`cu_get_action_name "$@"`"

	# Skip timestamp check if this action should be done again (during repair)
	if [ -z "$CU_FAILED_ACTIONS" ] || ! is_in_array "$CU_LAST_ACTION_ID" "$CU_FAILED_ACTIONS"; then
		[ "$timestamp" -gt "$CU_START_TIMESTAMP" ] || return
	fi

	if [ "$timestamp" -gt "$CU_LATEST_TIMESTAMP" ]; then
		CU_LATEST_TIMESTAMP="$timestamp"
	fi

	"$@"
}

# Execute action only if it wasn't applied before (by version)
cu_check_version()
{
	true cu_get_action_name
	CU_LAST_ACTION_ID="$CU_UPGRADE_VERSION_FROM-`cu_get_action_name "$@"`"

	# Skip version check if this action should be done again (during repair)
	if [ -z "$CU_FAILED_ACTIONS" ] || ! is_in_array "$CU_LAST_ACTION_ID" "$CU_FAILED_ACTIONS"; then
		[ "$CU_UPGRADE_VERSION_FROM" -ge "$CU_START_VERSION_FROM" ] || return
	fi

	"$@"
}

# Execute action only if it belongs to a current stage
cu_check_stage()
{
	local stage="$1"
	shift

	[ "$stage" = "$CU_STAGE" ] || return

	"$@"
}

# --- Upgrader bookkeeping functions ---

# Start a block of upgrade actions that should be executed when upgrading from a given version
cu_start_upgrade_from_version()
{
	CU_UPGRADE_VERSION_FROM="$1"
}

# Remember failed upgrade action (to be able to rerun it afterwards)
cu_add_failed_action()
{
	local action_id="$1"
	[ -n "$action_id" ] && CU_FAILED_ACTIONS="$CU_FAILED_ACTIONS $action_id" && CU_HAS_NEW_FAILED_ACTIONS="yes"
}

cu_remove_failed_action()
{
	local action_id="$1"
	[ -n "$CU_FAILED_ACTIONS" -a -n "$action_id" ] && CU_FAILED_ACTIONS="` remove_from_array "$action_id" "$CU_FAILED_ACTIONS" `"
}

# Store configuration data using supplied callback. Duplicate to default location if necessary.
cu_do_store_config_data()
{
	local upgrader_name="$1"
	local store_config_data_func="$2"

	$store_config_data_func "$upgrader_name"
	# Duplicate into default storage, if custom one was used. This is mostly for humans.
	[ "$store_config_data_func" = "cu_txtdb_store_config_data" ] || cu_txtdb_store_config_data "$upgrader_name"
}

# Handler for cases when critical action fails and there is no rollback procedure defined
cu_critical_failure_handler()
{
	[ -n "$CU_LAST_DB_UPGRADER_NAME" -a -z "$CU_LAST_DB_CONTINUE_FROM" ] || {
		p_echo "Can't handle upgrader critical failure - upgrader state is broken. Probably a programming error."
		return
	}

	p_echo "Updating '$CU_LAST_DB_UPGRADER_NAME' configuration status after a critical failure during upgrade stage '$CU_LAST_STAGE' on '$CU_LAST_ACTION_ID' action"

	local cu_db_current_timestamp cu_db_continue_from cu_db_failed_actions_list cu_db_latest_timestamp
	# Update only continue_from and failed_actions_list fields.
	cu_db_current_timestamp="$CU_LAST_DB_CURRENT_TIMESTAMP"
	cu_db_continue_from="`cu_get_current_continue_from`"
	cu_db_failed_actions_list="$CU_FAILED_ACTIONS"
	cu_db_latest_timestamp="$CU_LAST_DB_LATEST_TIMESTAMP"

	cu_do_store_config_data "$CU_LAST_DB_UPGRADER_NAME" "$CU_LAST_STORE_CONFIG_DATA_FUNC"

	CU_LAST_DB_UPGRADER_NAME=
}

# Check that following upgrader run will actually have any actions executed
cu_check_upgrade_required()
{
	local upgrader_func="$1"
	local CU_DO_GET_ACTION_NAME="yes"
	local upgrade_actions="`$upgrader_func`"
	[ -n "$upgrade_actions" ]
}

# --- Upgrader set up functions ---

# Set up installed (product/package) version and installed configuration timestamp
cu_set_initial_versions()
{
	CU_START_VERSION_FROM="${1:-starting_component_version_not_specified}"
	CU_START_TIMESTAMP="${2:-0}"
}

# Reset upgrader global variables
cu_set_globals()
{
	local upgrader_name="$1"
	local stage="$2"
	local store_config_data_func="$3"

	CU_DO_GET_ACTION_NAME=
	CU_LATEST_TIMESTAMP=0
	CU_LAST_ACTION_ID=

	# Initial DB record values. Used by cu_critical_failure_handler()
	CU_LAST_DB_UPGRADER_NAME="$upgrader_name"
	CU_LAST_DB_CURRENT_TIMESTAMP="$cu_db_current_timestamp"
	CU_LAST_DB_CONTINUE_FROM="$cu_db_continue_from"
	CU_LAST_DB_FAILED_ACTIONS_LIST="$cu_db_failed_actions_list"
	CU_LAST_DB_LATEST_TIMESTAMP="$cu_db_latest_timestamp"
	CU_LAST_STAGE="$stage"
	CU_LAST_STORE_CONFIG_DATA_FUNC="$store_config_data_func"
}

# --- Default implementations to load/store configuration data ---

cu_txtdb_config_data_params()
{
	db_file="$PLESK_DB_DIR/components-configuration-state.db"
	db_delim='#'
}

# Path to default upgrader configuration db changed in Plesk 11.5.26. This function moves existing db.
cu_txtdb_try_upgrade_location()
{
	local old_db_file="$PRODUCT_ROOT_D/var/components-configuration-state.db"
	local db_file db_owner db_group db_perms db_delim db_key_field
	txtdb_set_params cu_txtdb_config_data_params

	if [ -s "$old_db_file" -a ! -e "$db_file" ]; then
		mv -f "$old_db_file" "$db_file"
	fi
}

cu_txtdb_ensure_config_data_exists()
{
	cu_txtdb_try_upgrade_location
	txtdb_exists cu_txtdb_config_data_params >/dev/null 2>&1 || txtdb_truncate cu_txtdb_config_data_params
}

cu_txtdb_load_config_data()
{
	local upgrader_name="$1"

	cu_txtdb_ensure_config_data_exists
	local record="`txtdb_select cu_txtdb_config_data_params "$upgrader_name"`"

	cu_db_current_timestamp="`  get_narg_fs "$record" '#' 2`"
	cu_db_continue_from="`      get_narg_fs "$record" '#' 3`"
	cu_db_failed_actions_list="`get_narg_fs "$record" '#' 4`"
	cu_db_latest_timestamp="`   get_narg_fs "$record" '#' 5`"
}

cu_txtdb_store_config_data()
{
	local upgrader_name="$1"
	local record="$upgrader_name#$cu_db_current_timestamp#$cu_db_continue_from#$cu_db_failed_actions_list#$cu_db_latest_timestamp"
	cu_txtdb_ensure_config_data_exists
	txtdb_replace cu_txtdb_config_data_params "$record"
}

# No-op config loader. For use with cu_run_upgrader --init for the first stage.
cu_dummy_load_config_data()
{
	:
}

# --- Main code for generic cumulative upgrader ---

cu_run_upgrader()
{
	local opt_upgrader_name="$1"
	local opt_upgrader_func="$2"
	shift 2

	local opt_prepare_func=	opt_rollback_func= opt_commit_func= opt_start_version= opt_stage= opt_do_repair= opt_do_update_version_in_db= opt_do_init=
	local opt_load_config_data_func=cu_txtdb_load_config_data opt_store_config_data_func=cu_txtdb_store_config_data

	local inten="perform upgrade for $opt_upgrader_name"

	while [ "$#" -gt 0 ]; do
		case "$1" in
			--prepare)
				opt_prepare_func="$2"
				[ "$#" -ge 2 ] && shift 2 || break
				;;
			--rollback)
				opt_rollback_func="$2"
				[ "$#" -ge 2 ] && shift 2 || break
				;;
			--commit)
				opt_commit_func="$2"
				[ "$#" -ge 2 ] && shift 2 || break
				;;
			--load-config-data)
				opt_load_config_data_func="$2"
				[ "$#" -ge 2 ] && shift 2 || break
				;;
			--store-config-data)
				opt_store_config_data_func="$2"
				[ "$#" -ge 2 ] && shift 2 || break
				;;
			--from-version)
				opt_start_version="$2"
				[ "$#" -ge 2 ] && shift 2 || break
				;;
			--stage)
				opt_stage="$2"
				[ "$#" -ge 2 ] && shift 2 || break
				;;
			--repair)
				opt_do_repair="yes"
				shift
				;;
			--update-version)
				opt_do_update_version_in_db="yes"
				shift
				;;
			--init)
				opt_do_init="yes"
				shift
				;;
			*)
				die "$inten: unknown key '$1'"
				;;
		esac
	done

	# Sanity checks
	[ -n "$opt_upgrader_name"          ] || die "$inten: empty component upgrader name argument"
	[ -n "$opt_upgrader_func"          ] && is_function "$opt_upgrader_func"          || die "$inten: invalid component upgrader function argument"
	[ -n "$opt_load_config_data_func"  ] && is_function "$opt_load_config_data_func"  || die "$inten: invalid value for --load-config-data argument"
	[ -n "$opt_store_config_data_func" ] && is_function "$opt_store_config_data_func" || die "$inten: invalid value for --store-config-data argument"
	[ -z "$opt_prepare_func"           ] || is_function "$opt_prepare_func"           || die "$inten: invalid value for --prepare argument"
	[ -z "$opt_rollback_func"          ] || is_function "$opt_rollback_func"          || die "$inten: invalid value for --rollback argument"
	[ -z "$opt_commit_func"            ] || is_function "$opt_commit_func"            || die "$inten: invalid value for --commit argument"
	[ -n "$opt_do_init" -a \( -n "$opt_do_repair" -o -n "$opt_prepare_func" -o -n "$opt_rollback_func" -o -n "$opt_commit_func" \) ] && die "$inten: excessive options prohibited for use with --init"

	# Upgrader setup
	local CU_STAGE="$opt_stage"
	local cu_db_current_timestamp cu_db_continue_from cu_db_failed_actions_list cu_db_latest_timestamp
	$opt_load_config_data_func "$opt_upgrader_name"
	cu_set_initial_versions "$opt_start_version" "$cu_db_current_timestamp"
	cu_set_globals "$opt_upgrader_name" "$opt_stage" "$opt_store_config_data_func"

	local CU_CONTINUE_FROM_ID= CU_FAILED_ACTIONS= CU_HAS_NEW_FAILED_ACTIONS=
	if [ -z "$opt_do_init" ]; then
		if [ -n "$cu_db_continue_from" ]; then
			if [ -z "$opt_do_repair" ]; then
				pp_echo "Unable to upgrade '$opt_upgrader_name' to the latest version, since its configuration is in broken state. Run bootstrapper repair to fix."
				return 2
			fi

			local cu_continue_from_stage="` cu_get_continue_stage "$cu_db_continue_from" `"
			CU_CONTINUE_FROM_ID="` cu_get_continue_id "$cu_db_continue_from" `"
			if [ "$cu_continue_from_stage" != "$CU_STAGE" ]; then
				p_echo "Skipping stage '$CU_STAGE' of upgrade for '$opt_upgrader_name' as previous upgrade attempt was not finished on another stage."
				return 3
			fi
		elif [ -n "$cu_db_failed_actions_list" ]; then
			if [ -z "$opt_do_repair" ]; then
				p_echo "Before start of upgrade for '$opt_upgrader_name' (stage '$CU_STAGE') following actions are registered as failed: $cu_db_failed_actions_list."
			else
				CU_FAILED_ACTIONS="` normalize_array "$cu_db_failed_actions_list" `"
			fi
		fi

		if ! cu_check_upgrade_required "$opt_upgrader_func" ; then
			p_echo "Upgrade or repair for '$opt_upgrader_name' (stage '$CU_STAGE') is not required"
			[ -n "$opt_prepare_func" ] && $opt_prepare_func "$opt_upgrader_name" "$CU_STAGE" noop-stage
			[ -n "$opt_commit_func" ] && $opt_commit_func "$opt_upgrader_name" "$CU_STAGE" noop-stage
			return 0
		fi

		# Preparation step
		[ -n "$opt_prepare_func" ] && $opt_prepare_func "$opt_upgrader_name" "$CU_STAGE"

		# Register rollback actions.
		# Note that rollback scope currently spans to the end of upgrade procedure (this function).
		if [ -n "$opt_rollback_func" ]; then
			transaction_add_rollback_action "$opt_rollback_func"
			# Rollback function contract: everything changed in this upgrader pass will be 
			# rolled back. Component version should not change. Therefore there is no need 
			# to update anything in DB.
		else
			transaction_add_rollback_action "cu_critical_failure_handler"
		fi

		# Run upgrade actions
		$opt_upgrader_func

		# Commit (post) step - if we are here upgrade has completed (but there may be failed actions)
		[ -n "$opt_commit_func" ] && $opt_commit_func "$opt_upgrader_name" "$CU_STAGE"
	else
		# This is just an initialization run to set proper upgrader configuration on component installation
		p_echo "Initializing '$opt_upgrader_name' upgrader configuration (stage '$CU_STAGE')"
		local CU_DO_GET_ACTION_NAME="yes"
		$opt_upgrader_func >/dev/null 2>&1
	fi

	# Update DB record if there are changes. Update current_timestamp only if explicitly allowed (e.g. this is the last stage).
	cu_db_continue_from=

	if [ -n "$opt_do_repair" -o -z "$CU_LAST_DB_FAILED_ACTIONS_LIST" ]; then
		cu_db_failed_actions_list="$CU_FAILED_ACTIONS"
	else
		# Append failed actions from this stage. Don't care for duplicates - they will be removed when a given action succeeds.
		cu_db_failed_actions_list="` normalize_array "$CU_LAST_DB_FAILED_ACTIONS_LIST" "$CU_FAILED_ACTIONS" `"
	fi

	if [ -z "$cu_db_latest_timestamp" -a "$CU_LATEST_TIMESTAMP" -gt 0 ] || [ "0$cu_db_latest_timestamp" -lt "$CU_LATEST_TIMESTAMP" ]; then
		cu_db_latest_timestamp="$CU_LATEST_TIMESTAMP"
	fi

	[ -n "$opt_do_update_version_in_db" -a "0$cu_db_latest_timestamp" -gt 0 ] && cu_db_current_timestamp="$cu_db_latest_timestamp"

	if [    "$cu_db_current_timestamp" != "$CU_LAST_DB_CURRENT_TIMESTAMP" 		 -o "$cu_db_latest_timestamp" != "$CU_LAST_DB_LATEST_TIMESTAMP" 		 -o "$cu_db_failed_actions_list" != "$CU_LAST_DB_FAILED_ACTIONS_LIST" 		 -o "$cu_db_continue_from" != "$CU_LAST_DB_CONTINUE_FROM" ]; then
		cu_do_store_config_data "$opt_upgrader_name" "$opt_store_config_data_func"
	fi

	# If this is just an initialization, we don't need to disarm rollback, etc.
	[ -z "$opt_do_init" ] || return 0

	# Upgrade is complete - disarm rollback actions
	if [ -n "$opt_rollback_func" ]; then
		transaction_remove_rollback_action "$opt_rollback_func"
	else
		transaction_remove_rollback_action "cu_critical_failure_handler"
	fi

	if [ -n "$cu_db_failed_actions_list" ]; then
		p_echo "After end of upgrade for '$opt_upgrader_name' (stage '$CU_STAGE') following actions are registered as failed: $cu_db_failed_actions_list."
	fi

	[ ! -n "$CU_HAS_NEW_FAILED_ACTIONS" ]
}

cu_upgrader_conf_init()
{
	local opt_upgrader_name="$1"
	local opt_upgrader_func="$2"
	shift 2

	local opt_stages= opt_load_config_data_func= opt_store_config_data_func=

	local inten="initialize upgrader configuration for $opt_upgrader_name"

	while [ "$#" -gt 0 ]; do
		case "$1" in
			--stages)
				opt_stages="$2"
				[ "$#" -ge 2 ] && shift 2 || break
				;;
			--load-config-data)
				opt_load_config_data_func="$2"
				[ "$#" -ge 2 ] && shift 2 || break
				;;
			--store-config-data)
				opt_store_config_data_func="$2"
				[ "$#" -ge 2 ] && shift 2 || break
				;;
			*)
				die "$inten: unknown key '$1'"
				;;
		esac
	done

	# Sanity checks
	[ -n "$opt_upgrader_name"          ] || die "$inten: empty component upgrader name argument"
	[ -n "$opt_stages"                 ] || die "$inten: empty value for --stages argument, should be space separated list of stages"

	# Preparations
	local num_stages="`array_length $opt_stages`"
	local cur_stage_idx=1

	# Run all stages through cu_run_upgrader. First stage resets configuration, last one upgrades current timestamp.
	for stage in $opt_stages; do
		local store_option="${opt_store_config_data_func:+--store-config-data $opt_store_config_data_func}"

		local load_option="${opt_load_config_data_func:+--load-config-data $opt_load_config_data_func}"
		if [ "$cur_stage_idx" -eq 1 ]; then
			load_option="--load-config-data cu_dummy_load_config_data"
		fi

		local update_option=
		if [ "$cur_stage_idx" -eq "$num_stages" ]; then
			update_option="--update-version"
		fi

		cu_run_upgrader "$opt_upgrader_name" "$opt_upgrader_func" --init --from-version 0 --stage "$stage" $load_option $store_option $update_option

		cur_stage_idx=$(( cur_stage_idx + 1 ))
	done
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
get_userID()
{
# try to get UID
	common_var=`id -u "$1" 2>/dev/null` 

# if id returns 0 the all is ok
	test "$?" -eq "0" -a -n "$common_var"
}

get_groupID()
{
# try to get GID, id -g doesn't show groups without users
	common_var=`cat /etc/group | awk -F':' '$1 == "'$1'" {print $3}'` 

# We have non-empty value if success
	test -n "$common_var"
}

group_check()
{
	local group grp_res
	group="$1"

	grp_res=`groupmod "$group" 1>/dev/null 2>&1 ; echo $?`

	echo "$grp_res"
}

add_user_to_group()
{
	local user group grp_res existing newlist

	user="$1"
	group="$2"

	# check exists group
	p_echo " Checking for the group '$group'..."
	grp_res=`group_check "$group"`

	if [ "X$grp_res" != "X0" ]; then
			p_echo " Group '$group' not exists"
			p_echo " It is necessary to add group '$group'"
			err
	fi

	inten="add supplementary group '$group' for user '$user'"
	echo_try "$inten"

	existing=`id -Gn "$user"|sed 's|[[:space:]]\+|,|g'`
	if
		test "`id -gn "$user"`" = "$group" 		|| echo "$existing" | grep -q "\\<$group\\>"
	then
		p_echo " already there"
		return
	fi

	if test -z "$existing"; then
		newlist="$group"
	else
		newlist="$existing,$group"
	fi

	usermod -G "$newlist" "$user" 2>>"$product_log" && suc || die "$inten"
}

group_op()
{
	group="$1"
    gid="$2"
	id_force="$3"
	what_done=0
	local inten

    if [ "X$gid" = "Xsearch" ]; then
        gid=`get_user_group_id "$group" "gid" 30 400` 
    fi

	inten="add group '$group'"

	p_echo " Checking for the group '$group'..."
	grp_res=`group_check "$group"`

	case "$id_force" in
		true)
			case "$grp_res" in
				0)
				    case "$machine" in
				    *)
					etc_grp_res=`egrep "^$group:" /etc/group`
					case "$etc_grp_res" in
					*:*:*:*)
						chk_res=`echo "$etc_grp_res" | grep ":$gid:"`
						case "$chk_res" in
							*:*:*:*)
								p_echo " Group '$group' already exists"
								;;
							*) group_err "$group" "$gid"
								;;
						esac
						;;
					*)
						p_echo " Group '$group' already exists"
						;;
					esac
					;;

				    esac
				    ;;
				*)
					gid_res=`egrep "^[^:]*:[*x]*:$gid:" /etc/group | head -1`
					case "$gid_res" in
						*:*:*:*)
							gid_err "$gid"
							;;
						*)
							echo_try "$inten"
							groupadd -g "$gid" "$group" >> $product_log 2>&1 && suc ||
							die $inten
							what_done=1
							;;
					esac
					;;
			esac
			;;
		false)
			case "$grp_res" in
				0)
					p_echo " Group '$group' already exists"
					;;
				*)
					echo_try "$inten"
					groupadd "$group" >> $product_log 2>&1 && suc ||
					die $inten
					what_done=1
					;;
			esac
			;;
		*)
			p_echo "group_op: wrong last parameter -- must be 'true' or 'false'"
			p_echo
			p_echo "Exiting..."
			p_echo
			exit 1
			;;
	esac

	p_echo
	common_var=$what_done
	if [ "X$what_done" = "X1" ]; then
		groups_created="$groups_created $group";
	fi
	return $what_done
}

user_op()
{
	user="$1"
	uid="$2"
	group="$3"
	u_desc="$4"
	u_home="$5"
	u_shell="$6"
	id_force="$7"
	gid_force="$8"
	what_done=0

    if [ "X$uid" = "Xsearch" ]; then
        uid=`get_user_group_id "$user" "uid" 30 400` 
    fi

	p_echo " Checking for the user '$user'..."
	id_res=`id "$user" 1>/dev/null 2>&1 ; echo $?`

	case "$id_force" in
		true)
			case "$id_res" in
				0)
					chk_res=`id "$user" 2>&1 | egrep '(^uid=.*gid=.*)|(^id.*user)'`
					if [ "X${gid_force}" = "Xfalse" ]; then
						chk_uid=`echo "$chk_res" | egrep "uid=""$uid""\(""$user""\)"`
					else
						chk_uid=`echo "$chk_res" | egrep "uid=""$uid""\(""$user""\)" | egrep "gid=[0-9]*\(""$group""\)"`
					fi
					case "$chk_uid" in
						uid=*gid=*)
							p_echo " User '$user' already exists"
							;;
						*) logname_err "$user" "$uid"
							;;
					esac
					chsh -s "$u_shell" "$user"
					;;
				*)
					uid_res=`egrep "^[^:]*:[^:]*:""$uid"":" /etc/passwd`
					case "$uid_res" in
						*:*:*:*)
							uid_err "$uid"
							;;
						*)
							pnnl_echo " Trying to add user '$user'... "
							useradd $useradd_options -u "$uid" -g "$group" -d "$u_home" -s "$u_shell" -c "$u_desc" "$user" >> $product_log 2>&1 &&
							suc || die "add user '$user'"
							what_done=1
							;;
					esac
					;;
			esac
			;;
		false)
			case "$id_res" in
				0)
					p_echo " User '$user' already exists"
					;;
				*)
					pnnl_echo " Trying to add user '$user'... "
					useradd $useradd_options -g "$group" -d "$u_home" -s "$u_shell" -c "$u_desc" "$user" >> $product_log 2>&1 &&
					suc || die "add user '$user'"
					what_done=1
					;;
			esac
			;;
		*)
			p_echo "user_op: wrong last parameter -- must be 'true' or 'false'"
			p_echo
			p_echo "Exiting..."
			p_echo
			exit 1
			;;
	esac

	p_echo
	common_var=$what_done
	if [ "X$what_done" = "X1" ]; then
		users_created="$users_created $user";
	fi
	return $what_done
}

get_user_group_id()
{
    local name="$1"
    local src_name="$2"
    local min_num="$3"
    local max_num="$4"
    local count="$min_num"

    case "$src_name" in
       uid)
            local src="/etc/passwd"
            local type="users"

            if get_userID "$name"; then
               echo "$common_var"
               return 0
            fi
       ;;
       gid)
            local src="/etc/group"
            local type="groups"
            if get_groupID "$name"; then
               echo "$common_var"
               return 0
            fi
       ;;
       *)
            p_echo "$src_name for $name was not set"
            inten="detect uid/gid. Source file was not detect."
            die "$inten"
       ;;
    esac

    if [ ! -f "$src" ]; then
	p_echo
	p_echo "ERROR!"
	p_echo "$src_file: no such file"
	p_echo "Exiting..."
	exit 1
    fi

    if [ $min_num -le 0 -o $min_num -ge $max_num ]; then 
		p_echo "get_user_group_id(): wrong parameters"
		exit 1
    fi 

	nums=`cat $src | awk -F: '{print $3}' - | xargs echo -n " "` 

    while [ "$count" -le "$max_num" ]; do
         echo " $nums " | grep " $count " > /dev/null 2>&1 || break
         count=`expr $count + 1`

         if [ "$count" -ge "$max_num" ]; then 
            p_echo "$src_name for $name was not set"
			inten="get free $src_name. Free $src_name not found."
			die "$inten"
         fi
    done  
    
    echo "$count"
    return 0
}

add_login_shell()
{
	local shell

	shell="$1"

	p_echo " Checking that $shell registered as login shell..."

	case $linux_distr in 
	    debian)
		add-shell $shell
	    ;;
	    *)
		if grep -q "^$shell\$" /etc/shells; then
			p_echo "$shell already registered as a login shell"
		else
			echo "$shell" >> /etc/shells 2>> $product_log || die "register login shell $shell"
		fi
	    ;;
	esac

	p_echo
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# prep_install

prep_install()
{
	mk_backup $inetd_conf cp f
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh

initial_conf()
{
	DEMO_VERSION="no"
	PRODNAME="psa"
	PRODUCT_NAME="psa"
	product_full="Plesk"
	product=${PRODNAME}
	PRODUCT_FULL_NAME="Plesk"

	product_etc="/etc/${PRODNAME}"
	prod_conf_t="/etc/psa/psa.conf"
	prodkey="$product_etc/$PRODNAME.key"

	minimal_changes=""

	EXTERNAL_PACKAGES=""
	EXTERNAL_PACKAGES_DIR=""

	BUILDER_UID="10007"

	PERL5LIB=/opt/psa/local/share/perl/5.14.2:/opt/psa/local/lib/perl/5.14.2
	export PERL5LIB

        support_contact="http://www.parallels.com/support"
	sales_email="sales@parallels.com"

	product_version="11.5.30"
	product_db_version="011005030"
	conceived_os_vendor=Ubuntu
	conceived_os_version="12.04"
	osrels="ubuntu12.04"

	prev_product="plesk"
	prev_clients_group="${prev_product}cln"

        clients_group="${product}cln"
        clients_GID=10001

        services_group="psaserv"
        services_GID=10003

        product_suff="saved_by_${product}".`date "+%m.%d;%H:%M"`
        product_suffo="saved_by_${product}"

	PREV_PRODUCT_ROOT_D="/usr/local/${prev_product}"

	# plesk default password
	if [ "X$DEMO_VERSION" = "Xyes" ]; then
		PRODUCT_DEFAULT_PASSWORD="plesk"
	else
		PRODUCT_DEFAULT_PASSWORD="setup"
	fi
}

read_conf()
{
	[ -n "$prod_conf_t" ] || prod_conf_t=/etc/psa/psa.conf

	if [ -s $prod_conf_t ]; then
		tmp_var=`perl -e 'undef $/; $_=<>; s/#.*$//gm;
				s/^\s*(\S+)\s*/$1=/mg;
				print' $prod_conf_t`
		eval $tmp_var
	else
		if [ "X$do_upgrade" = "X1" ]; then
			[ 0$ignore_miss_conf -ne 1 ] && p_echo "Unable to find product configuration file: $prod_conf_t"
			return 1
		fi
	fi
	return 0
}

get_my_cnf_param()
{
	local r=

	local my_cnf
	find_my_cnf "non-fatal" && 	r=`perl -e '$p="'"$1"'";
	undef $/; $_=<>; s/#.*$//gm;
	/\[mysqld\](.*?)\[/sg;
	$_=substr($1, rindex $1,"$p") and
	/$p\s*=(.*)/m and print $1
	' ${my_cnf}`
	echo $r
}

get_mysql_socket()
{
	mysql_socket="/var/run/mysqld/mysqld.sock"

	local mysqlsock=`get_my_cnf_param  socket`
	local MYSQL_SOCKETS="/var/lib/mysql/mysql.sock /tmp/mysql.sock /var/run/mysqld/mysqld.sock"

	for i in $mysql_socket $mysqlsock $MYSQL_SOCKETS; do
	    if [ -S "$i" ]; then
		    MYSQL_UNIX_PORT=$i
		    export MYSQL_UNIX_PORT
		    mysql_socket="$i"
		    break
	    fi
	done
}

conf_update()
{
	local def
	def="$1.default"
	umask 0022

	[ -s "$def" ] || return 1

	if [ ! -s "$1" ]; then
#		cp -f "$def" "$1"
#		return 0
		echo '#' > "$1"
	fi



perl -e '

open (my $config, $ARGV[0]) or die "Cannot open config $ARGV[0]\n";
while(<$config>){
s/#.*$//;
next if /^\s*$/;
/(\w+)\s+(.+)$/;
$cf{$1}=$2;
}
close $config;

$cf{PRODUCT_ROOT_D}=$ENV{PRODUCT_ROOT_D} if $ENV{PRODUCT_ROOT_D};

open (my $default, $ARGV[1]) or die "Cannot open default config $ARGV[1]\n";
while(<$default>){
 s/^\s*(\w+)(\s+)(.+)$/"$1$2".( (exists($cf{$1}))? $cf{$1}:$3 )/e;
 print;}
close $default;

' "$1" "$def" > "$1.new"

	if [ -s "$1.new" ]; then
	  if cmp -s $1.new $1; then
  		rm -f $1.new
		return 1
	  else
		rm -f $1.old
		ln $1  $1.old
		mv -f $1.new $1
		chmod 644 $1    # bug 64313, 65043
		p_echo "config updated"
		return 0
	  fi
	fi
	return 2
}

# setup new value for parameter
# $1 config file name $2 paramater name, $3 parameter value
conf_setval()
{
	cat $1 | awk -v varname="$2" -v varvalue="$3" 'BEGIN { f = 0 }
{ if ($1 == varname) { f = 1; print varname "\t" varvalue } else { print $0 } }
END { if (f == 0) { print "\n" varname "\t" varvalue } }' 		> $1.new

	if [ -s "$1.new" ]; then
	  if cmp -s $1.new $1; then
  		rm -f $1.new
		return 1
	  else
		rm -f $1.old
		ln $1  $1.old
		mv -f $1.new $1
		rm -f $1.old
		chmod 644 "$1"
		return 0
	  fi
	fi
	return 2
}

# A set of functions for work with config variables
# $1 is config file name, $2 is variable name, $3 is variable value

conf_getvar()
{
	cat $1 | perl -n -e '$p="'$2'"; print $1 if m/^$p\s+(.*)/'
}

conf_rmvar()
{
	cat $1 | perl -n -e '$p="'$2'"; print $_ if(! m/^$p\s+(.*)/)' >$1.new
	if cmp -s $1.new $1; then
		rm -f $1.new
		return 0
	fi
	rm -f $1.old
	ln $1  $1.old
	mv -f $1.new $1
	chmod 644 "$1"
}

conf_setvar()
{
	conf_setval "$@"
}

 #default values

product_default_conf()
{

PRODUCT_ROOT_D=/opt/psa
PRODUCT_RC_D=/etc/init.d
PRODUCT_ETC_D=/opt/psa/etc
PLESK_LIBEXEC_DIR=/usr/lib/plesk-9.0
HTTPD_VHOSTS_D=/var/www/vhosts
HTTPD_CONF_D=/etc/apache2
HTTPD_INCLUDE_D=/etc/apache2/conf.d
HTTPD_BIN=/usr/sbin/apache2
HTTPD_LOG_D=/var/log/apache2
HTTPD_SERVICE=apache2
QMAIL_ROOT_D=/var/qmail
PLESK_MAILNAMES_D=/var/qmail/mailnames
RBLSMTPD=/usr/sbin/rblsmtpd
FTPD_CONF=/etc/proftpd.conf
FTPD_CONF_INC=/etc/proftpd.include
FTPD_BIN_D=/usr/bin
FTPD_VAR_D=/var/run/proftpd
FTPD_SCOREBOARD=/var/run/proftpd_scoreboard
NAMED_RUN_ROOT_D=/var/named/run-root
NAMED_OPTIONS_CONF=
NAMED_ZONES_CONF=
WEB_STAT=/usr/bin/webalizer
MYSQL_VAR_D=/var/lib/mysql
MYSQL_BIN_D=/usr/bin
MYSQL_SOCKET=/var/run/mysqld/mysqld.sock
PGSQL_DATA_D=/var/lib/postgresql//main
PGSQL_CONF_D=/etc/postgresql//main
PGSQL_BIN_D=/usr/lib/postgresql/9.1/bin
DUMP_D=/var/lib/psa/dumps
DUMP_TMP_D=/tmp
MAILMAN_ROOT_D=/usr/lib/mailman
MAILMAN_VAR_D=/var/lib/mailman
PYTHON_BIN=/usr/bin/python
CATALINA_HOME=/var/lib/tomcat6
DRWEB_ROOT_D=/opt/drweb
DRWEB_ETC_D=/etc/drweb
GPG_BIN=/usr/bin/gpg
TAR_BIN=/bin/tar
AWSTATS_ETC_D=/etc/awstats
AWSTATS_BIN_D=/usr/lib/cgi-bin
AWSTATS_TOOLS_D=/usr/share/awstats/tools
AWSTATS_DOC_D=/usr/share/awstats
OPENSSL_BIN=/usr/bin/openssl
LIB_SSL_PATH=/lib/libssl.so
LIB_CRYPTO_PATH=/lib/libcrypto.so
CLIENT_PHP_BIN=/opt/psa/bin/php-cli
SNI_SUPPORT=true
APS_DB_DRIVER_LIBRARY=/usr/lib/libmysqlserver.so.2
IPv6_DISABLED=false
SA_MAX_MAIL_SIZE=256000

}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#-*- vim:ft=sh

register_service(){

	[ -n "$1" ] || die "register_service: service name not specified"
	local inten="register service $1"
	echo_try "$inten"

	{
		# update-rc.d for Debian/Ubuntu (not SuSE)
		/usr/sbin/update-rc.d "$1" defaults
	} >> $product_log 2>&1

	suc
}

unregister_service(){

	[ -n "$1" ] || die "unregister_service: service name not specified"
	local inten="unregister service $1"
	echo_try $inten

	{
		/usr/sbin/update-rc.d -f "$1" remove
	} >> $product_log 2>&1

	suc
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh

selinux_is_active()
{
	if [ -z "$SELINUX_ENFORCE" ]; then
		selinux_getenforce
	fi

	case "$SELINUX_ENFORCE" in
	Enforcing|Permissive) return 0;;
	*) return 1;;
	esac
}

selinux_get_mount_dir()
{
	unset SELINUX_MOUNT_DIR

	if awk '$2 == "/selinux"{exit(1)}' /proc/mounts && mkdir -p /selinux; then
		SELINUX_MOUNT_DIR=/selinux
	else
		SELINUX_MOUNT_DIR="`mktemp -d /tmp/selinuxXXXXXX`"
	fi >>"$product_log" 2>&1
}

selinux_getenforce()
{
	if [ "$1" = "--check" -a -n "$SELINUX_ENFORCE" ]; then
		return
	fi
	unset SELINUX_ENFORCE

	if ! ( command -v selinuxenabled >/dev/null 2>&1 && selinuxenabled ); then
		SELINUX_ENFORCE=Disabled
		return
	fi

	if awk '$3 == "selinuxfs"{exit(1)}' /proc/mounts; then
		selinux_get_mount_dir
		mount -t selinuxfs none "$SELINUX_MOUNT_DIR"
	fi

	if ! command -v getenforce >/dev/null 2>&1; then
		SELINUX_ENFORCE=Disabled
		return
	fi

	SELINUX_ENFORCE="`getenforce`"
	if test $? -ne 0; then
		SELINUX_ENFORCE=Disabled
		return
	fi
}

selinux_init()
{
	unset SELINUX_ENFORCE
	if selinux_is_active; then
		setenforce Permissive
	else
		unset SELINUX_ENFORCE
	fi
}

selinux_close()
{
	if [ -z "$SELINUX_ENFORCE" -o "$SELINUX_ENFORCE" = "Disabled" ]; then
		return
	fi

	setenforce "$SELINUX_ENFORCE"
}

selinux_fix_booleans()
{

	:
}


relabel_plesk_directories()
{
	local verbose_mode=
	if [ "$1" = "--verbose" ]; then
		verbose_mode="yes"
		shift
	fi

	selinux_params
	set_horde_params
	if selinux_is_active; then
		for dir in "$PRODUCT_ROOT_D" "$horde_logdir" "$NAMED_RUN_ROOT_D" 					"$QMAIL_ROOT_D/alias" "$QMAIL_ROOT_D/bin" "$QMAIL_ROOT_D/boot" "$QMAIL_ROOT_D/control" 					"$QMAIL_ROOT_D/plugins" "$QMAIL_ROOT_D/popuser" "$QMAIL_ROOT_D/queue" "$QMAIL_ROOT_D/users" 					/var/db/kav /var/db/Quarantine "$PLESK_DB_DIR" 					"$POSTFIX_LIBEXEC_DIR" 					/var/drweb "$DRWEB_ROOT_D" "$PLESK_LIBEXEC_DIR" 					"/etc/nginx" "/usr/sbin/nginx" /var/lib/nginx 					"/var/log/nginx" "/var/run/nginx.pid" 					"/var/lib/plesk/mail" /opt/kav "/usr/lib/php/modules" "/var/run" 					"$@"; do
			if [ -d "$dir" ]; then
				[ -z "$verbose_mode" ] || pp_echo "  Restoring SELinux context on '$dir'"
				selinux_relabel_dir -e "$NAMED_RUN_ROOT_D/proc" "$dir"
			fi
		done
	fi
}

selinux_relabel_dir()
{
	if ! command -v restorecon >/dev/null 2>&1; then
		return
	fi
	if [ "$1" = "-F" ]; then
		selinux_set_force_arg
		shift
	else
		SELINUX_FORGE_ARG=
	fi

	if [ "$1" = "-e" ]; then
		selinux_set_exclude_arg "$2"
		shift 2
	else
		SELINUX_EXCLUDE_ARG=
	fi

	restorecon $SELINUX_FORGE_ARG $SELINUX_EXCLUDE_ARG -R "$@" >>"$product_log" 2>&1 || report_problem "Error while setting SELinux types for '$dir'"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

selinux_params()
{
	:
}

selinux_set_force_arg()
{
	SELINUX_FORGE_ARG="" 
}

selinux_set_exclude_arg()
{
	selinux_set_exclude_arg_pass "$1"
}

selinux_set_exclude_arg_pass()
{
	SELINUX_EXCLUDE_ARG=""
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

necessary_system_accounts_prep_install_mysql()
{
# for mysql
	p_echo " Checking for the system groups and users necessary for MySQL..."
	group_op "$mysql_group" "$mysql_GID" false
	user_op "$mysql_user" "$mysql_UID" "$mysql_group" "MySQL Server" "/" "$dummy_shell" false
}

necessary_system_accounts_prep_install_psaadm()
{
# for admin
	p_echo " Checking for the system groups and users necessary for admin server..."
	group_op "$admin_group" "search" false
	user_op "$admin_user" "search" "$admin_group" "Admin Server" "$dummy_home" "$dummy_shell" false
}

necessary_system_accounts_prep_install_psaserv()
{
# for psaservices (only after creating apache and psaftp)
	group_op "$services_group" "$services_GID" false
	add_user_to_group "$admin_user" "$services_group"
}

necessary_system_accounts_prep_install_psacln()
{
# clients
	found=`get_user_group_id "clients" "gid" $clients_GID $max_suexec_GID`
	group_op $clients_group $found false
}

necessary_system_accounts_prep_install()
{
	p_echo
	p_echo "===> Checking for the necessary system accounts"
	common_var=0

	necessary_system_accounts_prep_install_mysql
	necessary_system_accounts_prep_install_psaadm
	necessary_system_accounts_prep_install_psaserv
	necessary_system_accounts_prep_install_psacln
}

necessary_system_accounts_post_install()
{
	# these users may not exist before packages isntallation
	add_user_to_group sw-cp-server "$admin_group"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#set_params

set_common_params()
{
	common_var=0

	PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/sbin:/usr/local/bin
	LANG=C
	export PATH LANG
	umask 022
	ulimit -n 65535 2>/dev/null

	K_HUP="/bin/kill -HUP"
	K_KILL="/bin/kill -KILL"
	K_TERM="/bin/kill -TERM"
	K_USR2="/bin/kill -USR2"
	K_TEST="/bin/kill -0"

	users_created=""
	groups_created=""

	certificate_file="$PRODUCT_ETC_D/httpsd.pem"
	services="/etc/services"
	mtab="/etc/mtab"
	get_hostname="hostname"
	get_domainname="domainname"

	#VZP used to determine that we're inside SVE
	vza_file="/var/vzagent"

	#default parameters
	tar="tar"
	crontab="/usr/bin/crontab"

	cp_preserve="cp -p"
	SYSTEM_RC_D=/etc/init.d
	PLESK_LIBEXEC_DIR="/usr/lib/plesk-9.0"
	PLESK_DB_DIR="/var/lib/plesk"
	POSTFIX_LIBEXEC_DIR="/usr/lib/postfix"
	PRODUCT_BOOTSTRAPPER_DIR="/opt/psa/bootstrapper/pp11.5.30-bootstrapper"
	AUTOGENERATED_CONFIGS="#ATTENTION!\n#\n#DO NOT MODIFY THIS FILE BECAUSE IT WAS GENERATED AUTOMATICALLY,\n#SO ALL YOUR CHANGES WILL BE LOST THE NEXT TIME THE FILE IS GENERATED.\n"
	AUTOGENERATED_CONFIGS_UPGRADE="#ATTENTION!\n#\n#DO NOT MODIFY THIS FILE BECAUSE IT WAS GENERATED AUTOMATICALLY,\n#SO ALL YOUR CHANGES WILL BE LOST AFTER YOU UPGRADE PARALLELS PLESK PANEL.\n"

	set_common_params_linux 

	detect_vz
}

set_common_params_linux()
{
	get_hostname="hostname -f"
	fstab="/etc/fstab"
	cp_preserve="cp --preserve=all --remove-destination"
	machine="linux"
	sendmail="/usr/sbin/sendmail"
	ps="ps axw"
	ps_long="ps axuw"
	false_shell="/bin/false"
	dummy_home="/"
	compress="gzip -9 -c"
	uncompress="gunzip -c"
	uudecode="uudecode -o /dev/stdout"
	ifconfig="/sbin/ifconfig -a"
	inet_str="inet addr"

	if [ -f /etc/slackware-version ]; then
	    linux_distr="slackware"
	    useradd_options=""
	    sndml_ini="/etc/rc.d/init.d/sendmail"
	    mail_local="/usr/libexec/mail.local"
	    dummy_shell=""
	    named_osrelease=0
	else
	    useradd_options="-M"
	    if [ -f /etc/mandrake-release ]; then
		linux_distr="mandrake"
	    elif [ -f /etc/fedora-release ]; then
		linux_distr="fedora"
	    elif [ -f /etc/SuSE-release ]; then
		linux_distr="suse"
		useradd_options="-r"
	    elif [ -f /etc/debian_version ]; then
		linux_distr="debian"
		get_domainname="dnsdomainname"
		useradd_options=""
	    else
		linux_distr="redhat"
	    fi

	    sndml_ini="/etc/init.d/sendmail"
	    mail_local="/usr/libexec/mail.local"
	    if [ -x /sbin/nologin ]; then
		dummy_shell="/sbin/nologin"
	    else
		dummy_shell="/bin/false"
	    fi
	    bash_shell="/bin/bash"
	    rbash_shell="/bin/rbash"
	    uudecode_full="/usr/bin/uudecode"
	    named_osrelease=`cat /proc/sys/kernel/osrelease | perl -F"/[.-]/" -n -a  -e 'printf "%02u%02u%02u\n", $F[0],$F[1],$F[2]'`
	fi

	return 0
}

define_start_script()
{
	if [ -f "$PRODUCT_RC_D/${product}.sh" ]; then
		START_SH="$PRODUCT_RC_D/${product}.sh"
	else
		START_SH="$PRODUCT_RC_D/${product}"
	fi
}

set_local_params()
{
	for i in set_admin_params lock_keyupdate set_apache_params set_awstats_params set_courier_imap_params set_syslog_params set_logrotate_params set_mysqld_params set_qmail_params set_webalizer_params ; do
		$i
		local res="$?"
		if [ "0$res" -gt 0 ]; then
			return "$res"
		fi
	done
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
set_swcpserver_params()
{
	swcpserver_service=sw-cp-server
	swcpserver_binary=/usr/sbin/sw-cp-serverd
	swcpserver_pidfile=/var/run/sw-cp-server.pid

	true swcpserver_status
}

swcpserver_status()
{
	get_pid "$swcpserver_binary" false
	local pid=$common_var
	if [ "$pid" -ne 1 ]; then
		return 0
	fi
	return 1
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# -*- vim:syntax=sh

set_syslog_params()
{
	syslog_service="sysklogd"
	syslog_conf_ng="/etc/syslog-ng/syslog-ng.conf"

	syslog_conf=""
	for config in rsyslog.conf rsyslog.early.conf syslog.conf; do
		[ -f "/etc/$config" ] && syslog_conf="$syslog_conf /etc/$config"
	done

	# FIXME: quick fix of 142129
	if [ "${machine}" = "linux" -a ! -x "$PRODUCT_RC_D/$syslog_service" -a -x "$PRODUCT_RC_D/rsyslog" ]; then
		syslog_service="rsyslog"
		syslog_binary="/sbin/rsyslogd"
	elif [ "${machine}" = "linux" -a ! -x "$PRODUCT_RC_D/$syslog_service" -a -x "$PRODUCT_RC_D/syslog-ng" ]; then
		syslog_service="syslog-ng"
		syslog_binary="/sbin/syslog-ng"
	else
		syslog_binary="/sbin/syslogd"
	fi
}
## @@constructor set_syslog_params

true syslog_status_linux_debian
syslog_status_linux_debian()
{
	get_pid "$syslog_binary" false
	local pid=$common_var
	if test "$pid" -ne 1; then
		# running
		return 0
	fi
	return 1
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

update_history()
{
	inten="insert upgrade info into history table"

	get_product_versions

	query="SELECT * FROM upgrade_history 
		WHERE version_info='${product_prev_version}' OR db_version='${product_prev_version}'
		LIMIT 1"
	if ! db_test "$query" '1'; then
		query="INSERT INTO upgrade_history (upgrade_date, version_info, 
		db_version) VALUES (now()-1,
			'${product_this_version}', '${product_prev_version}')"
		db_do --inten "$inten" "$query"
	fi

	query="INSERT INTO upgrade_history (upgrade_date, version_info, db_version) 
		VALUES (now(), '${product_this_version}', 
		'${product_this_version}${product_this_version_tag:+ (${product_this_version_tag})}')"
	db_do --inten "$inten" "$query"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# -*- vim:syntax=sh

packagemng_set_dirty_flag()
{
	if [ -f ${PRODUCT_ROOT_D}/admin/bin/packagemng ]; then
		${PRODUCT_ROOT_D}/admin/bin/packagemng --set-dirty-flag 1>>$product_log 2>&1
	fi
}

get_product_versions()
{
	local prod_root_d="/opt/psa"
	
	product_name="psa"
	product_this_version="11.5.30"
	product_this_version_tag=""
	if [ -z "$product_prev_version" ]; then
		if [ -r "$prod_root_d/version.upg" ]; then
			product_prev_version=`cat "$prod_root_d/version.upg" | awk '{ print $1 }'`
		elif [ -r "$prod_root_d/version" ]; then
			product_prev_version=`cat "$prod_root_d/version" | awk '{ print $1 }'`
		else
			product_prev_version="$product_this_version"
		fi
	fi
}

# Product management node is already installed (at least files are in place, maybe scripts were not finished yet)
is_management_node_installed()
{
	[ -s "$PRODUCT_ROOT_D/version" -o -s "$PRODUCT_ROOT_D/version.upg" -o -s "$PRODUCT_ROOT_D/core.version" -o -s "$PRODUCT_ROOT_D/.revision" ]
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
set_logrotate_params()
{
	# primary Plesk logrotate config file
	logrotate_conf="${product_etc}/logrotate.conf"

	# directory for logrotate configs of virtual hosts
	logrotate_include_d="${PRODUCT_ETC_D}/logrotate.d"
}

set_module_params()
{
	case $machine in
	    BSD*)
		packname="psa-module-$module"
	    ;;
	    *)
		packname="psa-$module"
	    ;;
	esac

	scriptname="psa-$module"

	# This is the version like 10203
	module_version_n=$((`echo "$module_version" | sed -e 's/^\(.*\)\.\(.*\)\.\(.*\)$/\1*10000+\2*100+\3/'`))

	build="115131206.11"

	product_rc_d="/etc/init.d"

	module_db_sql="$PRODUCT_ETC_D/modules/$module/${module}_db.sql"

	module_var="$PRODUCT_ROOT_D/var/modules/$module"

	sedflags="-r"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
upgrade_modules_permissions_1013()
{
	chown psaadm:0 ${PRODUCT_ROOT_D}/var/modules
}
### Copyright 1999-2013. Parallels IP Holdings GmbH. All Rights Reserved.

enable_innodb_engine()
{
	local have_innodb="$1"

	if [ -z "${have_innodb}" ]; then
		local mysql_db_name=mysql
		local db_select_output=
		db_select "SHOW VARIABLES LIKE 'have_innodb'"
		have_innodb=`get_narg 2 ${db_select_output}`
		[ -n "$have_innodb" ] || have_innodb="NO"
	fi

	echo_try "allow INNODB engine"
	if [ "${have_innodb}" = "YES" ]; then
		p_echo "Innodb is already enabled - nothing to be done"
		return 0
	fi

	# Tables are not converted back to INNODB, only engine support is switched on!
	find_my_cnf
	! grep -q '^skip-innodb' "$my_cnf" || sed -i -e '/^skip-innodb/ d' "$my_cnf"
	suc

	pnnl_echo "Restarting mysql"
	pleskrc mysql restart || die "restart MySQL"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# -*- vim:ft=sh

# mysql

set_mysqld_params()
{
	# FIXME: mysql_user is also used in set_mysql_auth() to store another value
	mysql_user="mysql"
	mysql_UID=3306
	mysql_group="mysql"
	mysql_GID=3306

	product_db_sql="$PRODUCT_BOOTSTRAPPER_DIR/db/${PRODNAME}_db.sql"

	set_mysql_server_params
	set_mysql_client_params
}

## @@constructor set_mysqld_params

set_mysql_server_params()
{
	MYSQL_ROOT="$PRODUCT_ROOT_D/mysql"
	mysql_bindir="$MYSQL_BIN_D"

	get_mysql_socket

	if [ -x "${PRODUCT_RC_D}/mysql" ]; then
	    mysql_service="mysql"
	elif [ -x "${PRODUCT_RC_D}/mysql.sh" ]; then
	    mysql_service="mysql.sh"
	elif [ -x "${PRODUCT_RC_D}/mysqld" ]; then
	    mysql_service="mysqld"
	elif [ -x "${PRODUCT_RC_D}/mysql" ]; then
	    mysql_service="mysql"
	elif [ "X$DEMO_VERSION" = "Xyes" ]; then
	    mysql_service="mysqld"
	else
	    die "detect MySQL service name"
	fi
}

true mysql_stop mysql_status

set_mysql_client_params()
{
	mysql_client="$MYSQL_BIN_D/mysql"

#	Override these variables as needed
	mysql_db_name="$PRODNAME"
	mysql_passwd_file="$product_etc/.${PRODNAME}.shadow"
	prev_mysql_passwd_file="$PREV_PRODUCT_ROOT_D/admin/conf/admin.conf"

	# FIXME: improve logic related to mysql_unconfigured flag - make it more intuitive/rework workflow.
	# Currently if the flag is non-empty, it means "skip setting mysql authentication parameters 
	# and testing connection because MySQL is probably not configured yet".
 	if [ -z "$mysql_unconfigured" ];then
#	Need for mysql_status related function
		set_mysql_server_params	
		set_mysql_auth
	fi

	mysql_args="-N"
	mysql_args_raw="-Br"

	# For backward compatibility only, should use mysql() and mysql_raw() instead
	mysql=mysql
	mysql_raw=mysql_raw
}

generate_mysql_credentials()
{
	args=$*
	mysql_defaults=`echo $args | cut -d: -f 1`

	mysql_user=`echo $args | cut -d: -f 2`
	[ -n "$mysql_user" ] && mysql_user="--user=$mysql_user"

	mysql_passwd=`echo $args | cut -d: -f 3-`
	[ -n "$mysql_passwd" ] && mysql_passwd="$mysql_passwd"
}

# This function must be called *ONLY* for psa installation
setup_admin_login()
{
	unset MYSQL_PWD
	local mysql_db_name="mysql"
	local sl
	local inten="define valid mysql credentials"
	local i

	echo_try "$inten"
	get_admin_passwd

# --no-defaults:admin:$admin_passwd		admin with pass
# --no-defaults:root:					root without pass
# :root:								root with pass in defaults
# --no-defaults:admin:					admin withiyt pass
# --no-defaults:admin:$PRODUCT_DEFAULT_PASSWORD		admin with default pass
# :admin:								admin with pass in defaults
# ::									user and pass in defaults
# --defaults-file=/root/.my.cnf			hspc defaults (paranoid)
#
	for sl in `sequence 20`; do
		for i in	"--no-defaults:admin:$admin_passwd" 					"--no-defaults:root:" 					":root:" 					"--no-defaults:admin:" 					"--no-defaults:admin:$PRODUCT_DEFAULT_PASSWORD" 					":admin:" 					"::" 					"--defaults-file=/root/.my.cnf::"; do
			generate_mysql_credentials "$i"
			if mysql_test_connection; then
				suc
# 	create/Update admin superuser
				echo "grant all privileges on *.* to 'admin'@'localhost' identified by '`mysql_quote_string $admin_passwd`' with grant option;flush privileges" | mysql_direct mysql
				update_psa_shadow
				set_mysql_auth
#	backup private my.cnf if it contains password (vz/hspc related)
				rm -f /.my.cnf
				grep -q '\s*password\s*=' /root/.my.cnf 2>/dev/null && mv -f /root/.my.cnf /root/.my.cnf.bak
				return 0
			fi
		done
		p_echo "One more attempt to connect"
		sleep "$sl"
	done
	die "$inten"
}

update_psa_shadow()
{
	echo "$admin_passwd" > $mysql_passwd_file || die
	chown $admin_user:$admin_group ${mysql_passwd_file} || die
	chmod 600 ${mysql_passwd_file} || die
}

encrypt_psa_shadow()
{
	local init_conf_bin="$PRODUCT_ROOT_D/bin/init_conf"
	[ -x "$init_conf_bin" ] && 	PSA_PASSWORD="$admin_passwd" "$init_conf_bin" --set-admin-password -passwd '' -encrypted-password >> "$product_log" 2>&1
	if [ "$?" -eq 0 ]; then
		admin_passwd=
		set_mysql_auth
	else
		warn "encrypting admin password"
	fi
}

# iF YOUR package want to use mysql db you must call this function directly
set_mysql_auth()
{
	local inten="set up mysql authentification"
	get_admin_passwd

	pleskrc mysql start

	mysql_user="--user=admin"
	mysql_passwd="$admin_passwd"
	unset mysql_defaults

	if [ -z "$MYSQL_AUTH_SKIP_CONNECTION_CHECK" ]; then
		mysql_test_connection 60 || die "$inten"
	fi
	suc
}

get_admin_passwd()
{
    [ -z "$admin_passwd" ] || return 0

    if [ -f "$mysql_passwd_file" ]; then
		admin_passwd=`cat "$mysql_passwd_file"`
		return 0
    fi

	admin_passwd=`get_random_string 12 'A-Za-z0-9_'`

	[ -n "$admin_passwd" ] || admin_passwd="$PRODUCT_DEFAULT_PASSWORD"
}

true mysql_quote_string

mysql_quote_string()
{
    local string="$1"
    quoted_string="`echo $string | sed -e 's|\([\\]\)|\\\\\1|g'`"
    echo -n "$quoted_string"
}

reset_admin_passwd()
{
	[ -n "$admin_passwd" ] || die "admin_passwd is not set yet"
	# I bet someone somewhere replaces 'localhost' with '%' (for weird automation purposes), so be nice
	echo "SET PASSWORD FOR 'admin'@'localhost' = PASSWORD('`mysql_quote_string $admin_passwd`')" | mysql_direct mysql || 	echo "SET PASSWORD FOR 'admin'@'%'         = PASSWORD('`mysql_quote_string $admin_passwd`')" | mysql_direct mysql
}

#Invoke mysql
mysql()
{
	mysql_anydb -D$mysql_db_name "$@"
}

mysql_anydb()
{
	(
		export MYSQL_PWD="$mysql_passwd"
		$mysql_client $mysql_user $mysql_args "$@" 2>>"$product_log"
		local status=$?

		if [ $status -gt 0 ]; then
			$mysql_client $mysql_user $mysql_args -D$mysql_db_name $mysql_args_raw -e "SHOW ENGINE innodb status" >>"$product_log" 2>&1
		fi
		unset MYSQL_PWD
		return $status
	)
}

# Invoke mysql without any wrapper or something else
mysql_direct()
{
	# a bit dirty but it works properly for passwords with spaces, quotes and double quotes:
	if [ -n "$mysql_passwd" ]; then
		(
			export MYSQL_PWD="$mysql_passwd"
			$mysql_client $mysql_defaults $mysql_user $mysql_args "$@" 2>>"$product_log"
			rc=$?
			unset MYSQL_PWD
			return $rc
		)
	else
		$mysql_client $mysql_defaults $mysql_user $mysql_args "$@" 2>>"$product_log"
	fi
}

# Invoke mysql in raw mode
mysql_raw()
{
	mysql $mysql_args_raw "$@"
}

mysql_raw_anydb()
{
	mysql_anydb $mysql_args_raw "$@"
}

mysql_start_linux_suse_ex()
{
	if [ -x "/sbin/service" ]; then
		action_cmd="/sbin/service mysql"
	else
		action_cmd="$SYSTEM_RC_D/$mysql_service"
	fi

	$action_cmd start >> $product_log 2>&1
}

true mysql_start_linux_suse
mysql_start_linux_suse()
{
	inten="start service mysql"
	echo_try "$inten"

	mysql_start_linux_suse_ex
	local rc="$?"

# bug 52690. MySQL init script reports failure if protected mysqld is running (true for SuSE >= 11.3)
	if [ "$rc" -ne 0 ]; then
		local mysqld_bin="/usr/sbin/mysqld"
		killall -TERM mysqld >> $product_log 2>&1
		if [ -x "$mysqld_bin" ]; then
			for i in 2 4 8 16 32; do
				get_pid "$mysqld_bin" false
				local pid="$common_var"
				if test "$pid" -eq 1; then
					break
				fi
				killall -TERM mysqld >> $product_log 2>&1
				sleep $i
			done
		fi
		mysql_start_linux_suse_ex
		rc="$?"
	fi

	[ "$rc" -eq 0 ] && suc || warn "$inten"
	return $rc
}

mysql_test_connection()
{
	inten="establish test connection"
	echo_try $inten
	attempts=${1:-1}	
	for i in `sequence $attempts`; do
		echo "" | mysql_direct mysql >> "$product_log" 2>&1
		if [ "$?" -eq "0" ]; then
			p_echo "connected"
			return 0
		fi
		[ "$attempts" -eq "1" ] || sleep 2
	done

	p_echo "failed"
	return 1
}

###	FIXME: probably need var service_restart warn
mysql_stop()
{
	local op_result i

	inten="stop MySQL server"
	echo_try $inten

	$PRODUCT_RC_D/$mysql_service stop >> $product_log 2>&1

	op_result=$?

	if [ "X$linux_distr" = "Xdebian" ]; then
# Debian has well designed mysql stopping code
		[ "$op_result" -eq 0 ] || die $inten
		suc
		return 0
	fi

	for i in 2 4 6 8 16; do
		if ! mysql_status ; then
			suc
			return 0
		fi

		# I just want to be sure that mysql really stopped
		killall -TERM mysqld mysql safe_mysqld mysqld_safe >> $product_log 2>&1

		sleep $i
	done

	die "$inten"
}

mysql_status()
{
	local file

    #Check with native script first
	#debian script always return 0. bug #111825
	[ "X$linux_distr" = "Xdebian" ] && msqld_status_supported="no"
	
	if [ -z "$msqld_status_supported" ]; then
# MySQL AB packages doesn't know about status command
		if LC_MESSAGES=C $PRODUCT_RC_D/$mysql_service 2>&1 | grep -q "status"; then
			msqld_status_supported="yes"
		else
			msqld_status_supported="no"
		fi
	fi

	if [ "$msqld_status_supported" = "yes" ]; then
# Common RPM mysql's and FreeBSD
	$PRODUCT_RC_D/$mysql_service status >> $product_log 2>&1 		&& return 0
	fi

	if [  "$msqld_status_supported" = "no" ]; then
# MySQL AB packages
		file="/usr/sbin/mysqld"
	fi

    if [ -x "$file" ]; then
		#standard build and debian
		get_pid "$file" false
		pid=$common_var
		if test "$pid" -ne 1; then
			echo "$file (pid $pid) is running..." >>$product_log 2>&1
			return 0
		else
			echo "$file is stopped" >>$product_log 2>&1
			return 1
		fi
	fi

	return 1
}

check_db()
{
	local myOK db_connected db_password

	inten="find $PRODNAME database"
	echo_try $inten

#	If database not exists
	if ! db_test --any-db "SHOW DATABASES" '/^psa$/'; then
		p_echo "not found"
		return 1
	fi
#	Database found, but version undefined
	if ! db_select "select val from misc where param='version'" || [ -z "$db_select_output" ]; then
		p_echo "DATABASE ERROR!!!"
		p_echo "Database $PRODNAME found, but version undefined"
		die $inten
	fi

	prev_db_version="$db_select_output"

# Now check psa database version
	p_echo "version is $prev_db_version"
	if [ "$prev_db_version" -le "011010" ] ; then
		prev_db_short_version=`expr ${prev_db_version} / 10`
	else
		# support upgrade on patch version
		prev_db_short_version="$prev_db_version"
	fi

	if [ "0$prev_db_short_version" -ne "0$prev_short_version" -a 		 "0$product_db_version" -ne "0$prev_db_version" ]; then
		p_echo "DATABASE ERROR!!!"
		p_echo "Previous product version is ${prev_version}, but previous database"
		p_echo "version is ${prev_db_version}. In most of cases it is result of"
		p_echo "previous upgrade try failure. Please, restore previous version"
		p_echo "from backup, and try again or contact technical support."
		die "check database version"
    fi

	# this message for situation, where psa was deleted, but database not
    if [ "X$prev_db_version" = "X$product_db_version" ]; then
	    p_echo "Version is OK, no need to create $PRODNAME database."
    fi

	return 0;
}

product_install_db()
{
	p_echo "===> Installing $PRODNAME database"

	get_mysql_socket

	inten="check ${PRODNAME} database"
	echo_try $inten
	if check_db; then
	    # product database OK, no need to create it
	    return 0
	fi

	inten="install initial $PRODUCT_NAME database"
	echo_try $inten
	[ -f "$product_db_sql" ] || die "$inten: file not found: $product_db_sql"
	mysql_anydb < $product_db_sql && suc || die $inten

	db_do "replace into misc (param,val) values('def_locale', '$default_locale')"
	db_do "$query replace into locales(id) values('$default_locale')"
	db_do "$query alter table clients alter locale set default '$default_locale'"

	if [ "$PLESK_VZ" = "1" ]; then
		db_do "replace into ServiceNodeConfiguration (\`serviceNodeId\`, \`section\`, \`name\`, \`value\`) values (1, 'mailServer', 'spamfilter_max_children', '1')" # default is 5
	fi

	db_init_updater_state

	set_db_version
	db_do "REPLACE INTO misc(param, val) VALUES('mysql41_compatible', 'true')"
}

db_init_updater_state()
{
	local disable_updater="false"
	$mysql <<EOF || die "init updater state"
		REPLACE INTO misc(param, val) VALUES('disable_updater', '$disable_updater');
EOF
}

set_db_version()
{
	#install db version
	echo_try "set psa database version to $product_db_version"
	db_do "replace into misc (param,val) values('version', '$product_db_version')"
	suc
}

configure_mysql_innodb()
{
	local my_cnf awk_script
	my_cnf=$1

	if ! $mysql_client --version | grep -q '3\.23'; then
		return
	fi

	if ! (test -f $my_cnf && grep -q '^\[mysqld\]' "$my_cnf"); then
		echo "[mysqld]" >> $my_cnf
		echo "innodb_data_file_path=ibdata1:10M:autoextend" >> $my_cnf
		echo "safe-show-database" >> $my_cnf
		return
	fi

	awk_script='/^\[mysqld\]$/{print; print "innodb_data_file_path=ibdata1:10M:autoextend"; next;} {print}'
	if ! grep -q '^innodb_data_file_path' "$my_cnf"; then
		awk "$awk_script" "$my_cnf" >"$my_cnf.tmp" && mv "$my_cnf.tmp" "$my_cnf" || die "edit $my_cnf"
	fi

	awk_script='/^\[mysqld\]$/{print; print "safe-show-database"; next;} {print}'
	if ! grep -q '^safe-show-database' "$my_cnf"; then
		awk "$awk_script" "$my_cnf" >"$my_cnf.tmp" && mv "$my_cnf.tmp" "$my_cnf" || die "edit $my_cnf"
	fi
}

#bug #46837. Disable "LOAD DATA LOCAL INFILE" command.
configure_mysql_infile()
{
	local my_cnf awk_script sys
	my_cnf=$1

	if ! (test -f $my_cnf && grep -q '^\[mysqld\]' "$my_cnf"); then
		echo "[mysqld]" >> $my_cnf
		echo "local-infile=0" >> $my_cnf
		return
	fi

	awk_script='/^\[mysqld\]$/{print; print "local-infile=0"; next;} {print}'
	if ! grep -q '^local-infile' "$my_cnf"; then
		awk "$awk_script" "$my_cnf" >"$my_cnf.tmp" && mv "$my_cnf.tmp" "$my_cnf" || die "edit $my_cnf"
	fi
}

configure_mysql_nonlocal()
{
	local my_cnf
	my_cnf=$1

	if grep -q '^bind-address' "$my_cnf"; then
		sed -e "s|^\(bind-address[[:space:]]*=[[:space:]]*127\.0\.0\.1.*\)|# \1|g" 			"$my_cnf" >"$my_cnf.tmp" && mv "$my_cnf.tmp" "$my_cnf" || die "edit $my_cnf"
	fi
}

configure_mysql_disable_old_passwords()
{
	local my_cnf="$1"
	local awk_script='/^[[:space:]]*old_passwords/ { print "# Forced OLD_PASSWORD format is turned OFF by Plesk\n#" $0; next } { print }'

	mysql_disable_old_passwords_conf_changed=0
	if awk "$awk_script" "$my_cnf" > "$my_cnf.tmp" || die "edit $my_cnf"; then
		if diff -q "$my_cnf" "$my_cnf.tmp" 1>/dev/null ; then
			rm -f "$my_cnf.tmp"
		else
			mysql_disable_old_passwords_conf_changed=1
			mv "$my_cnf.tmp" "$my_cnf" || die "disable old_passwords in $my_cnf"
		fi
	fi
}

find_my_cnf()
{
	local non_fatal="$1"
	local cnf_files="/etc/my.cnf /etc/mysql/my.cnf /var/db/mysql/my.cnf"

	for my_cnf in $cnf_files; do
		if [ -f ${my_cnf} ]; then
			break
		fi
	done

	[ -f "$my_cnf" -o -n "$non_fatal" ] || die "find MySQL server configuration file. " 		"If you use thirdparty MySQL server build, make sure you don't use implicit default configuration " 		"and have MySQL configuration file in one of the following locations: $cnf_files"
	[ -f "$my_cnf" ]
}

run_configure_mysql_funcs()
{
	local funcs="$@"

	local my_cnf
	find_my_cnf

	for func in $funcs; do
		eval "$func $my_cnf"
	done

	return 0
}

mysql_install_init()
{
	local my_cnf
	find_my_cnf

	configure_mysql_innodb "$my_cnf"
	configure_mysql_infile "$my_cnf"
	configure_mysql_nonlocal "$my_cnf"
	configure_mysql_disable_old_passwords "$my_cnf"


#	Start mysql without test connection mysql isn't initialize yet
	pleskrc mysql start

	setup_admin_login
	mysql_fix_remove_bad_users
	tune_vz_mysql "$my_cnf"
	pleskrc mysql restart

}

mysql_fix_remove_bad_users()
{
	local mysql_db_name="mysql"
	db_do "drop database if exists test"
	db_do "delete from user where User=''"
	db_do "delete from user where User='root'"
	db_do "flush privileges"
}

dump_db()
{
	local databases
	for db in "$@"; do
		if db_test_database "$db"; then
			databases="$databases $db"
		else
			p_echo "Warning: Not dumping MySQL database '$db' as it doesn't exist"
		fi
	done

	[ -n "$databases" ] || return 1
	(
		export MYSQL_PWD="$mysql_passwd"
		${MYSQL_BIN_D}/mysqldump $mysql_user --quote-names --databases $databases
		rc=$?
		unset MYSQL_PWD
		return $rc
	)
}

requires_configured_mysql()
{
	# Initialize and configure local MySQL server if it is not usable yet.
	# This function is for use in components that require MySQL, particularly on SN.

	local mysql_unconfigured=1
	set_mysqld_params

	if [ ! -s "$mysql_passwd_file" ]; then
		mysql_install_init
	fi
}

uses_mysql_client()
{
	# Initialize local MySQL authentication parameters and test connection.
	# After that db_*() and mysql*() functions may be used.
	# Any MySQL-specific overrides (e.g., local mysql_db_name="whatever") should appear
	# *after* calling this function.
	# This function is for use in component installers that require access to MySQL DB.

	local mysql_unconfigured=
	set_mysql_client_params
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# -- start of mysqlnd old passwords upgrade --

# The reason why this code is not in the db/upgrade_db_1100.sh file is the following:
# It turned out that during the upgrade from 8.6 to 11.0 sw-engine may be installed to
# satisfy bootstrapper's dependency; new sw-engine with the mysqlnd extenstion refused to work 
# with the passwords in the old format. So passwords upgrade should be done prior to the database upgrade.
# This is vital as there are PHP-scripts (working with the database) invoked during the upgrade.

upgrade_mysqlnd_old_passwd()
{
	# It's probably better to extract all child functions into .in file
	run_configure_mysql_funcs configure_mysql_disable_old_passwords
	if [ "$mysql_disable_old_passwords_conf_changed" = "1" ]; then
		pleskrc mysql restart
	fi

	db_test "SELECT length(PASSWORD('test'))" '$1 == 41' || die "disable old_passwords=1 in my.cnf"
	check_mysql_users_with_old_passwd_format || return 0

	inten="upgrade MySQL password format"
	reset_admin_passwd || die "$inten for admin"
	reset_sitebuilder_passwd || die "$inten for Sitebuilder user"
	reset_billing_passwd || die "$inten for Billing user"
	reset_apsc_passwd || die "$inten for APS controller user"
	reset_horde_passwd || die "$inten for Horde webmail user"
	reset_atmail_passwd || die "$inten for AtMail webmail user"
	reset_mysql_dbusers_passwd || die "$inten for Panel DB users"
	reset_phpmyadmin_passwd || die "$inten for phpMyAdmin control user"
	reset_siteeditor_passwd || die "$inten for SiteEditor user"

	check_mysql_users_with_old_passwd_format && warn_mysql_old_passwd_users && return 1 || :
}

check_mysql_users_with_old_passwd_format()
{
	local mysql_db_name=mysql
	local user="$1"
	local query="SELECT User FROM user WHERE length(Password) != 41"
	if [ -n "$user" ]; then
		query="${query} AND User = '$user'"
	fi
	db_test "${query} LIMIT 1" '1'
}

reset_mysql_user_password()
{
        local user="$1"
        local host="$2"
        local passwd="$3"
        local inten="reset database user password for '$user'@'$host'"

        echo_try "$inten"

        local mysql_db_name=mysql
        local query="SELECT count(*) FROM user WHERE User='$user' AND Host='$host'"
        if db_test "${query}" '$1 == 0'; then
                p_echo "user does not exists." 
                return 1
        fi

        echo "SET PASSWORD FOR '$user'@'$host' = PASSWORD('`mysql_quote_string $passwd`')" | mysql_direct mysql && suc && return 0

        warn "$inten"
        return 1
}

reset_user_password()
{
        local user="$1"
        local passwd="$2"
        local host="${3:-localhost}"

        reset_mysql_user_password "$user" "$host" "$passwd" && return 0

        [ "$host" = "127.0.0.1" -o "$host" = "::1" ]                 && host="localhost"                 && reset_mysql_user_password "$user" "$host" "$passwd"                 && return 0

        reset_mysql_user_password "$user" "%" "$passwd" && return 0

        return 1
}

is_local_mysql_instance()
{
	local host="$1"
	[ -z "$host" -o "$host" = "localhost" -o "$host" = "127.0.0.1" -o "$host" = "::1" ]
}

check_db_accessible()
{
	local mysql_db_name="$1"
	local mysql_user="--user=$2"
	local mysql_passwd="$3"

	db_do --inten "check that DB $1 is accessible to user $2" ""
}

reset_sitebuilder_passwd()
{
	local sb_conf="/usr/local/sb/config"
	[ -f "$sb_conf" ] || return 0

	local sb_host=`    get_ini_conf_var "$sb_conf" "database" "host"`
	local sb_username=`get_ini_conf_var "$sb_conf" "database" "username"`
	local sb_password=`get_ini_conf_var "$sb_conf" "database" "password"`
	local sb_dbname=`  get_ini_conf_var "$sb_conf" "database" "dbname"`

	is_local_mysql_instance "$sb_host" || return 0

        if reset_user_password "$sb_username" "$sb_password" "$sb_host"; then
                check_db_accessible "$sb_dbname" "$sb_username" "$sb_password" || return 1
        fi

	return 0
}

reset_siteeditor_passwd()
{
	local se_conf="/usr/local/siteeditor/config"
	[ -f "${se_conf}" ] || return 0

	local se_host=`    get_ini_conf_var "${se_conf}" "database" "host"`
	local se_username=`get_ini_conf_var "${se_conf}" "database" "username"`
	local se_password=`get_ini_conf_var "${se_conf}" "database" "password"`
	local se_dbname=`  get_ini_conf_var "${se_conf}" "database" "dbname"`

	is_local_mysql_instance "$se_host" || return 0

        if reset_user_password "${se_username}" "${se_password}" "${se_host}"; then
                check_db_accessible "$se_dbname" "$se_username" "$se_password" || return 1
        fi

        return 0
}

reset_billing_passwd()
{
	local billing_db_bin="/usr/share/plesk-billing/billing-db"
	local bil_host bil_user bil_passwd bil_dbname bil_mysql_local bil_info

	[ -x "$billing_db_bin" ] || return 0
	bil_mysql_local=`$billing_db_bin --is-local 2>/dev/null`
	if [ "$?" -eq 0 ]; then
		# New billing-db version is shell-friendly
		bil_dbname=`$billing_db_bin --get name`
		bil_user=`  $billing_db_bin --get user`
		bil_host=`  $billing_db_bin --get host`
	else
		# Slightly older ones provide --info option
		bil_info=`$billing_db_bin --info 2>/dev/null`
		if [ "$?" -eq 0 ]; then
			bil_dbname=`echo "$bil_info" | sed -ne 's/Database Name:\s*\(.*\)/\1/p'`
			bil_user=`  echo "$bil_info" | sed -ne 's/Database User:\s*\(.*\)/\1/p'`
			bil_host=`  echo "$bil_info" | sed -ne 's/Database Hostname:\s*\(.*\)/\1/p'`
			bil_mysql_local=
		fi
	fi
	if [ -z "$bil_dbname" -o -z "$bil_user" ]; then
		# And finally, if invoked w/o arguments, billing-db switches into MySQL prompt
		bil_info=`echo 'select database() \G select current_user() \G' | $billing_db_bin`
		[ "$?" -eq 0 ] || return 1
		local bil_auth=`echo "$bil_info" | sed -ne 's/current_user():\s*\(.*\)/\1/p'`
		bil_dbname=`    echo "$bil_info" | sed -ne 's/database():\s*\(.*\)/\1/p'`
		bil_user=`get_narg_fs "$bil_auth" '@' 1`
		bil_host=`get_narg_fs "$bil_auth" '@' 2`
		bil_mysql_local=
	fi
	bil_passwd=`get_random_string 12 'A-Za-z0-9_'`

	is_local_mysql_instance "$bil_host" || return 0
	[ -z "$bil_mysql_local" -o "$bil_mysql_local" = "yes" ] || return 0

	reset_user_password "$bil_user" "$bil_passwd" "$bil_host" && 
	{
		(
			export BILLING_DB_PASSWORD="$bil_passwd"
			$billing_db_bin --set --with-password >/dev/null 2>&1
		) || 		$billing_db_bin --set --password "$bil_passwd"
	} &&
	check_db_accessible "$bil_dbname" "$bil_user" "$bil_passwd" && 
	{
		$billing_db_bin --is-configured 2>/dev/null | grep -q yes || 		$billing_db_bin --is_configured 2>/dev/null | grep -q yes 
	}
}

reset_apsc_passwd()
{
	check_apsc_installed || return 0
	read_apsc_connection_params_db
	check_mysql_users_with_old_passwd_format "$apsc_db_user" || return 0

	is_local_mysql_instance "$apsc_db_host" || return 0

	if reset_user_password "$apsc_db_user" "$apsc_db_passwd" "$apsc_db_host"; then
		check_db_accessible "$apsc_db_name" "$apsc_db_user" "$apsc_db_passwd" || return 1
	fi

	return 0
}

get_webmail_config_param()
{
	local webmail_config="$1"
	local config_var="$2"
	local param="$3"
	local default_val="$4"
	local result=`sw_engine_pleskrun -n -r 'include "'$webmail_config'"; echo $'${config_var}${param}';' 2>/dev/null`
	[ -n "$result" ] || result="$default_val"
	echo "$result"
}

get_horde_config_param()
{
	get_webmail_config_param "/usr/share/psa-horde/config/conf.php" "conf" "$1" "$2"
}

get_atmail_config_param()
{
	get_webmail_config_param "/var/www/atmail/libs/Atmail/Config.php" "pref" "$1" "$2"
}

reset_horde_passwd()
{
	set_horde_params

	local horde_shadow="$horde_passwd_file"
	[ -f "$horde_shadow" ] || horde_shadow="/etc/psa/.webmail.shadow"
	local hor_host=`  get_horde_config_param "['sql']['hostspec']" "localhost"`
	local hor_user=`  get_horde_config_param "['sql']['username']" "horde"`
	local hor_dbname=`get_horde_config_param "['sql']['database']" "horde"`
	local hor_passwd=`get_horde_config_param "['sql']['password']"`
	[ -n "$hor_passwd" ] || hor_passwd=`cat "$horde_shadow" 2>/dev/null`
	[ -n "$hor_passwd" -a -n "$hor_user" -a -n "$hor_dbname" ] || return 0

	is_local_mysql_instance "$hor_host" || return 0

	if reset_user_password "$hor_user" "$hor_passwd" "$hor_host"; then
		check_db_accessible "$hor_dbname" "$hor_user" "$hor_passwd" || return 1
	fi

	return 0
}

reset_atmail_passwd()
{
	local atmail_shadow="/etc/psa-webmail/atmail/.atmail.shadow"
	local at_host=`  get_atmail_config_param "['sql_host']" "localhost"`
	local at_user=`  get_atmail_config_param "['sql_user']" "atmail"`
	local at_dbname=`get_atmail_config_param "['sql_table']" "atmail"`
	local at_passwd=`get_atmail_config_param "['sql_pass']"`
	[ -n "$at_passwd" ] || at_passwd=`cat "$atmail_shadow" 2>/dev/null`
	[ -n "$at_passwd" -a -n "$at_user" -a -n "$at_dbname" ] || return 0

	is_local_mysql_instance "$at_host" || return 0

	if reset_user_password "$at_user" "$at_passwd" "$at_host"; then
		check_db_accessible "$at_dbname" "$at_user" "$at_passwd" || return 1
	fi

	return 0
}

reset_mysql_dbusers_passwd()
{
	local mysql_db_name=mysql
	db_do --inten "Convert MySQL DB users passwords to new format" 		"UPDATE mysql.user m 
				INNER JOIN psa.db_users u ON (m.User = u.login) 
				INNER JOIN psa.accounts a ON (u.account_id = a.id) 
				INNER JOIN psa.data_bases d ON (u.db_id = d.id) 
		SET m.Password = PASSWORD(a.password) 
		WHERE d.type = 'mysql' AND a.type = 'plain'; 
		FLUSH PRIVILEGES;"
}

reset_phpmyadmin_passwd()
{
	local pmahost="localhost"
	local controlpass pmadb
	set_phpmyadmin_params

	local pma_config="$PHPMYADMIN_CONFIG.save"
	[ -f "$pma_config" ] || pma_config="$PHPMYADMIN_CONFIG"
	pma__set_values_from_config "$pma_config" 'controluser' 'controlpass' 'pmadb'

	[ -n "$controlpass" -a -n "$controluser" -a -n "$pmadb" ] || return 0

	if reset_user_password "$controluser" "$controlpass" "$pmahost"; then
		check_db_accessible "$pmadb" "$controluser" "$controlpass"
	fi

	return 0
}

warn_mysql_old_passwd_users()
{
	local mysql_db_name=mysql
	local query="SELECT concat(User, '@', Host) FROM user WHERE length(Password) != 41"

	if [ -n "$controluser" ]; then
		# There is an error in old phpMyAdmin upgrade routine: DB was recreated every time
		query="$query AND (User not like 'pma\_%' OR User = '$controluser')"
	fi

	query="$query AND (User != 'debian-sys-maint')"

	db_select "$query"
	local problem_users="$db_select_output"

	[ -n "$problem_users" ] || return 0
	report_problem "Passwords for some MySQL users were not upgraded." 		"All MySQL DB users that may be used by Panel MUST have passwords in new 41-character format in order for Panel to function properly." 		"Currently following users have passwords in old format: `echo -n $problem_users | sed -e 's/ /, /g'`." 		"Please ensure that MySQL server is not forced to use old password hashing algorithm and upgrade password format manually for users from the list above. Consult with MySQL server manual on detailed steps." 		"You may skip certain users if you are sure they weren't created and aren't used by Panel."
}

# -- end of mysqlnd old passwords upgrade --
#!/bin/sh
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

__tune_vz_mysql_sig_handler() {
    local rc=$?
    trap - EXIT
    rm -f -- "$1"
    exit $rc
}

__tune_vz_mysql()
{
    local myCnf myCnfTemp ||:

    myCnf=$1
    if test -z "$myCnf"; then
	warn "E:usage: $PROG /path/to/my.cnf"
	return 1
    fi

    if ! test -s "$myCnf" ; then
	warn "E: $myCnf doesn't exist or empty"
	return 100
    fi
    
    myCnfTemp=`mktemp ${myCnf}.XXXXXXX`
    if test $? -ne 0 ; then
	warn "E:Unable to create a temporary configuration file"
	return 2
    fi
    
    $cp_preserve $myCnf $myCnfTemp || {
	warn "E:Unable to create a temporary configuration file"
	return 2
    }

    trap "__tune_vz_mysql_sig_handler \"$myCnfTemp\""  HUP PIPE INT TERM EXIT

    local rc ||:

    local db_select_output
    local mysql_db_name=mysql
    db_select "SHOW VARIABLES LIKE 'have_innodb'"
    local have_innodb=`get_narg 2 ${db_select_output}`
    [ -n "$have_innodb" ] || have_innodb="NO"

    db_select "SHOW VARIABLES LIKE 'have_bdb'"
    local have_bdb=`get_narg 2 ${db_select_output}`
    [ -n "$have_bdb" ] || have_bdb="NO"

    cat "$myCnf" | gawk 		-v have_innodb="${have_innodb}" 		-v have_bdb="${have_bdb}" 		'
function getsize(sizestr)
{
  if (match(sizestr, "[^[:digit:]]")) {
    size=substr(sizestr, 0, RSTART);
    mult=substr(sizestr, RSTART, 1);
    if ("K" == mult) {
      mult=1024;
    } else if ("M" == mult) {
      mult=1024*1024;
    } else if ("G" == mult) {
      mult=1024*1024*1024;
    }
    size = size * mult;
  } else {
    size = sizestr;
  }
  return size;
}

# add unspecified variables
function finalize_section()
{
  if (have_bdb && 0 == skipbdb) {
    printf("skip-bdb\n\n");
    changed=1;
  }
  if (have_innodb && 0 == ibp_size) {
    print "innodb_buffer_pool_size=2M"
    changed=1;
  }
  if (have_innodb && 0 == iamp_size) {
    print "innodb_additional_mem_pool_size=500K"
    changed=1;
  }
  if (have_innodb && 0 == ilb_size) {
    print "innodb_log_buffer_size=500K"
    changed=1;
  }
  if (have_innodb && 0 == thread_conc_set) {
    print "innodb_thread_concurrency=2"
    changed=1;
  }
}

# decrease value of "varname" if it exceed the upper bound limit.
# If value fit (less or equal), leave it unmodified.
function adjustSetting(varstr, varname, minsize, sizestr)
{
  if (match(varstr,"^[[:space:]]*(set-variable[[:space:]]*=[[:space:]]*)?" varname "[[:space:]]*=[[:space:]]*")) {
    if (have_innodb) {
      size = getsize(substr(varstr, RSTART+RLENGTH));
      if (minsize < size) {
        changed = 1;
        print "#", $0;
        print varname "=" sizestr;
      } else {
        print $0
      }
    } else {
      # we do not have innodb so we need to disable this setting
      print "#", $0;
      changed = 1;
    }
    return 1;
  }
  return 0;
}

function reset_variables()
{
  skipbdb=0;   # already have skip-bdb
  ibp_size=0;  # already set innodb_buffer_pool_size
  iamp_size=0; # already set innodb_additional_mem_pool_size
  ilb_size=0;  # already set innodb_log_buffer_size
  thread_conc_set=0; # -*- innodb_thread_concurrency
}

BEGIN {
  insect=0;    # within one of server sections
  changed=0;   # already changed smth
  have_bdb = (tolower(have_bdb) == "yes") ;
  have_innodb = (tolower(have_innodb) == "yes") ;
  reset_variables()
}
/^[[:space:]]*\[.+\]/ {
  if (0 != insect) {
# need to close a previous section
    finalize_section();
    # section is ended. Reset variables
    reset_variables()
    insect    = 0;
  }
  if ( match($0, "^[[:space:]]*\\[(mysqld|server|mysqld_safe|safe_mysqld)\\]")) {
    insect = 1;
#    print "# in server section";
  }
}

(match($0, "^[[:space:]]*(set-variable[[:space:]]*=)?[[:space:]]*") && 0 != insect) {
  varstr=substr($0, RSTART+RLENGTH);
  if (adjustSetting(varstr, "innodb_buffer_pool_size", 2 * 1024 * 1024, "2M")) { ibp_size=1; next; }
  if (adjustSetting(varstr, "innodb_additional_mem_pool_size", 500 * 1024, "500K")) { iamp_size=1; next; }
  if (adjustSetting(varstr, "innodb_log_buffer_size", 500 * 1024, "500K")) { ilb_size=1; next; }
  if (adjustSetting(varstr, "innodb_thread_concurrency", 2, "2")) { thread_conc_set=1; next; }
}

(adjustSetting($0, "innodb_buffer_pool_size", 2 * 1024 * 1024, "2M")) { ibp_size=1; next; }
(adjustSetting($0, "innodb_additional_mem_pool_size", 500 * 1024, "500K")) { iamp_size=1; next; }
(adjustSetting($0, "innodb_log_buffer_size", 500 * 1024, "500K")) { ilb_size=1; next; }
(adjustSetting($0, "innodb_thread_concurrency", 2, "2")) { thread_conc_set=1; next; }

/^[[:space:]]*skip-bdb[[:space:]]*(#.*)?$/ { skipbdb=1; }

{ print $0; }

END {
  if (0 != insect) {
    finalize_section();
  }

  if (0 == changed) {
    exit 110;
  }
}
' > "$myCnfTemp"
    rc=$?

    if test $rc -eq 110; then
	# nothing was changed
	p_echo "I: nothing to change in $myCnf"
	return 0
    fi

    if test $rc -ne 0; then
	warn "E: unable to process ${myCnf}: rc=$rc"
	return 2
    fi
    
    local myCnfBak ||:
    myCnfBak="${myCnf}.bak"

    if ! ln -f "$myCnf" "$myCnfBak" ; then
	warn "E: Unable to create a backup file"
	return 3
    fi

    if ! mv -f "$myCnfTemp" "$myCnf"; then
	warn "E: Unable to overwrite the configuration file"
	return 4
    fi
    
    trap - EXIT
    return 0
}

tune_vz_mysql()
{
    # check if we in VZ environment
    test "x$PLESK_VZ" = "x1" || return 0;
    # run in subshell to isolate TRAP usage
    ( __tune_vz_mysql "$1" )
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
#-*- vim:syntax=sh

set_named_params()
{
	# set up default values
	bind_UID=53
	bind_GID=53
	bind_user="bind";
	bind_group="bind";

	# get UID of named user, if exists
	if get_userID $bind_user; then
	    bind_UID=$common_var;
	fi

	# get GID of named group, if exists
	if get_groupID $bind_group; then
	    bind_GID=$common_var;
	fi

	# path to directory of internal named
	NAMED_ROOT_D="${PRODUCT_ROOT_D:?'PRODUCT_ROOT_D is undefined'}/named"

	# path to directory of named pid file
	bind_run="${NAMED_RUN_ROOT_D:?'NAMED_RUN_ROOT_D is undefined'}/var/run/named"

	named_service="bind9"
	named_log="/dev/null"
	
	# path to named config file
	named_conf="/etc/named.conf"
	rndc_conf="/etc/rndc.conf"
	rndc_namedb_conf="/etc/namedb/rndc.conf"
	rndc_bind_conf="/etc/bind/rndc.conf"

	#140025. Restrict CPU cores for Bind
	bind_number_of_workers=2
}

sysconfig_named_debian()
{
	local named_sysconf_file config_rebuild

	named_sysconf_file=/etc/default/bind9

	config_rebuild=0

	if [ -f "${named_sysconf_file}" ]; then

		# check presence of required run-root option
		cat ${named_sysconf_file} | sed -e 's|#.*$||g' | grep -E "^[[:space:]]*OPTIONS\=.*[[:space:]]\-t[[:space:]]*${NAMED_RUN_ROOT_D}" > /dev/null 2>&1
		[ $? -eq 0 ] || config_rebuild=1

		# check presence of required config file option
		cat ${named_sysconf_file} | sed -e 's|#.*$||g' | grep -E "^[[:space:]]*OPTIONS\=.*[[:space:]]\-c[[:space:]]*${named_conf}" > /dev/null 2>&1
		[ $? -eq 0 ] || config_rebuild=1

		# check presence of bind user option 
		cat ${named_sysconf_file} | sed -e 's|#.*$||g' | grep -E "^[[:space:]]*OPTIONS\=.*[[:space:]]\-u[[:space:]]*${bind_user}" > /dev/null 2>&1
		[ $? -eq 0 ] || config_rebuild=1

		# check presence of workers number
		cat ${named_sysconf_file} | sed -e 's|#.*$||g' | grep -E "^[[:space:]]*OPTIONS\=.*[[:space:]]\-n[[:space:]]" > /dev/null 2>&1
		[ $? -eq 0 ] || config_rebuild=1

	else
		config_rebuild=1
	fi

	if [ 0${config_rebuild} -gt 0 ]; then

		[ ! -f ${named_sysconf_file} ] || mk_backup ${named_sysconf_file} mv f
		
		echo "OPTIONS=\"\${OPTIONS} -t ${NAMED_RUN_ROOT_D}  -c ${named_conf} -u ${bind_user} -n ${bind_number_of_workers}\"" > ${named_sysconf_file}
	fi
}

sysconfig_named()
{
	register_service "$named_service"
	sysconfig_named_debian

	return 0
}


true named_status_linux_debian
named_status_linux_debian()
{
    get_pid "/usr/sbin/named" false
    local pid=$common_var
    if test "$pid" -ne 1; then
# running
		return 0
    fi
    return 1
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh

upgrade_named_restrict_number_of_workers_11_5_29()
{
	set_named_params
	sysconfig_named
	! pleskrc named status || pleskrc named restart
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
pam_params()
{
   local calltype
   calltype=$1
   initial_conf
   local product="psa"
   pam_config_dir="/opt/psa/pam_plesk_config"
   [ "x" = "x1" ] && pam_extra_args="-m /lib/security/pam_plesk.so"
   if [ "x$calltype" != "x1" ]; then # install, not uninstall
       [ "x" = "x1" ] && pam_extra_args="$pam_extra_args --fixup-ftpd"
       [ "x" = "x1" ] && pam_extra_args="$pam_extra_args --fixup-des"
   fi
   pam_inst_script="pam_plesk_install"
   pam_uninst_script="pam_plesk_remove"

   return 0
}

install_pam()
{
  inten="install PAM module"
  pam_params
  echo_try "$inten"

  "$pam_config_dir/$pam_inst_script" -qq 	--install-log "$product_log" 	--problem-log "$product_problems_log" $pam_extra_args
 
  if [ "X$?" = "X0" ]; then
     suc
  else
     warn "install PAM module not successed" 
  fi

  return 0
}

install_phpmyadmin()
{
	p_echo "===> Installing phpMyAdmin"
	set_phpmyadmin_params
	rename_old_configs
	pma__configure
	suc
}

set_phpmyadmin_params()
{
	PHPMYADMIN_D="$PRODUCT_ROOT_D/admin/htdocs/domains/databases/phpMyAdmin"
	PHPMYADMIN_CONFIG="$PHPMYADMIN_D/libraries/config.default.php"
}

rename_old_configs()
{
	for i in $PHPMYADMIN_D/config.*; do
		[ -f "$i" -a "`echo $i | sed 's|config.sample.inc.php||'`" = "$i" ] && mv $i $i.old
	done;
}

pma__read_values_from_config_backup()
{
	if [ -f "$PHPMYADMIN_CONFIG.save" ]; then
		# looking for values of following variables in config.default.php.save
		pma__set_values_from_config "$PHPMYADMIN_CONFIG.save" 									'controluser' 'controlpass' 'pmadb' 									'bookmarktable' 'column_info' 'history' 'designer_coords' 'pdf_pages' 'recent' 'relation' 'table_coords' 'table_info' 'table_uiprefs' 'tracking' 'userconfig'
	fi
}

pma__write_values_to_config()
{
	# write new config.inc.php
	sed -e "`pma__sed_script`" < "$PHPMYADMIN_CONFIG.tpl" > "$PHPMYADMIN_CONFIG"
}

pma__configure()
{
	local controluser controlpass pmadb
	local bookmarktable column_info history designer_coords pdf_pages recent relation table_coords table_info table_uiprefs tracking userconfig

	controluser=''

	pma__set_default_values
	pma__set_random_values
	pma__read_values_from_config_backup

	# Make sure that phpmyadmin script is idempotent, when phpmyadmin upgrade
	pma__create_pmadb

	pma__write_values_to_config
}

pma__set_values_from_config()
{
	eval `pma__read_values "$@"`
}

pma__read_values()
{
	local PHPMYADMIN_CONFIG="$1"
	shift

	[ -f $PHPMYADMIN_CONFIG ] && sw_engine_pleskrun -r '
			try {
				error_reporting(0);
				$GLOBALS["db_host"] = "localhost";
				class vz {
					public static function in_vzcp_proxy() { return false; }
				}
				function in_vzcp_proxy() { return false; }
				require_once("'$PHPMYADMIN_CONFIG'");
				for ($i = 1; $i < $argc; $i++)
					if (isset($cfg["Servers"][1][$argv[$i]]))
						echo $argv[$i] . "=\"" . $cfg["Servers"][1][$argv[$i]] . "\"\n";
			} catch (Exception $e) {}' -- "$@" 2>/dev/null || true
}

pma__set_default_values()
{
	controluser="pma"
#	controlpass=""

	pmadb="phpmyadmin"
	####
	# WARNING:
	# If you will modify the values you have to add templates (@@PMA_BOOKMARK@@, etc.)
	# in create_tables.sql and create_tables_mysql_4_1_2+.sql.
	####
	bookmarktable="pma_bookmark"
	column_info="pma_column_info"
	designer_coords="pma_designer_coords"
	history="pma_history"
	pdf_pages="pma_pdf_pages"
	recent="pma_recent"
	relation="pma_relation"
	table_coords="pma_table_coords"
	table_info="pma_table_info"
	table_uiprefs="pma_table_uiprefs"
	tracking="pma_tracking"
	userconfig="pma_userconfig"
}

pma__set_random_values()
{
	local pma__random_suffix

	# generate random controluser and pmadb
	while true; do
		pma__random_suffix=`get_random_string 12`
		controluser="pma_$pma__random_suffix" # maximum length is 16 characters
		pmadb="phpmyadmin_$pma__random_suffix"
		# check controluser existance
		# if user already exists then continue
		! db_test "SELECT COUNT(*) FROM mysql.user WHERE User='$controluser' LIMIT 1;" '$1!="0"' || continue
		# check pmadb existance
		# if db already exists then continue
		! db_test_database "$pmadb" || continue
		break
	done
	# generate random controlpass
	controlpass=`get_random_string 16`
}

pma__create_pmadb()
{
	local create_tables_sql
	local inten
	inten="create pmadb"
	echo_try $inten
	create_tables_sql="$PHPMYADMIN_D/examples/create_tables.sql"
	# create pmadb
	sed -e "`pma__sed_script`" < "$create_tables_sql" | mysql_direct >> "$product_log" 2>&1 || warn "$inten"
	suc
}

pma__sed_script()
{
	echo "	s/@@CONTROLUSER@@/$controluser/g
	s/@@CONTROLPASS@@/$controlpass/g
	s/@@PMADB@@/$pmadb/g
	s/@@PMA_BOOKMARK@@/$bookmarktable/g
	s/@@PMA_COLUMN_INFO@@/$column_info/g
	s/@@PMA_DESIGNER_COORDS@@/$designer_coords/g
	s/@@PMA_HISTORY@@/$history/g
	s/@@PMA_PDF_PAGES@@/$pdf_pages/g
	s/@@PMA_RECENT@@/$recent/g
	s/@@PMA_RELATION@@/$relation/g
	s/@@PMA_TABLE_COORDS@@/$table_coords/g
	s/@@PMA_TABLE_INFO@@/$table_info/g
	s/@@PMA_TABLE_UIPREFS@@/$table_uiprefs/g
	s/@@PMA_TRACKING@@/$tracking/g
	s/@@PMA_USERCONFIG@@/$userconfig/g	"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

# vim:ft=sh

mail_restore_all()
{
	# temporary solution to close #143977 and #144076: three keys to turn on
	if [ ! -e "$PRODUCT_ROOT_D/version" -a ! -e "$PRODUCT_ROOT_D/core.version" ]; then
		p_echo "$PRODUCT_ROOT_D/version isn't yet on its place. Deferring mail_restore execution to a later stage" >&2
		return
	fi
	if [ ! -x "/usr/lib/plesk-9.0/mailsrv_entities_dump" ]; then
		p_echo "/usr/lib/plesk-9.0/mailsrv_entities_dump isn't yet on its place. Deferring mail_restore execution to a later stage" >&2
		return
	fi
	if [ ! -x "/usr/lib/plesk-9.0/mail_restore" ]; then
		p_echo "/usr/lib/plesk-9.0/mail_restore isn't yet on its place. Deferring mail_restore execution to a later stage" >&2
		return
	fi

	local args="--fast"
	if [ "$1" = "full" ]; then
		args="--with-spam"
	fi

	# FIXME: There's a possible race during upgrade procedure. Noone
	# managed to catch the bug in debug mode, so be stupid:
	sync; sync; sleep 3

	inten="execute mail_restore to synchronize mail server settings and Plesk Database"
	echo_try $inten
	/usr/lib/plesk-9.0/mail_restore $args >>$product_log 2>&1
	suc
}

true exim_status_linux_debian
exim_status_linux_debian()
{
	get_pid /usr/lib/exim/exim3 false
	local pid=$common_var

	if test "$pid" -ne 1; then
		#running
		return 0;
	fi
	return 1
}

install_postfix_smtpd_sasl_config()
{
	local smtpd_sasl_conf smtpd_sasl_conf_bak saslauthd_path saslauthd_path_relative

	saslauthd_path_relative=/private/plesk_saslauthd

	saslauthd_path=$saslauthd_path_relative
	
	smtpd_sasl_conf='/etc/postfix/sasl/smtpd.conf'

	if [ -f "$smtpd_sasl_conf" ]; then
	   smtpd_sasl_conf_bak="${smtpd_sasl_conf}.saved_by_plesk.`date +%s`"
	   rm -f "$smtpd_sasl_conf_bak" ||:
	   if ! ln "$smtpd_sasl_conf" "$smtpd_sasl_conf_bak" ; then
	      simply_die "unable to preserve $smtpd_sasl_conf to $smtpd_sasl_conf_bak"
	   fi
	fi
	mkdir -p "$(dirname $smtpd_sasl_conf)"
	cat >"$smtpd_sasl_conf" <<EOF
pwcheck_method: auxprop saslauthd
auxprop_plugin: plesk
saslauthd_path: $saslauthd_path
mech_list: DIGEST-MD5 CRAM-MD5 PLAIN LOGIN
auto_transition: yes
sql_engine: intentionally disabled
log_level: 4
EOF
}

set_postfix_mta()
{
	:
}

qmail_admin_alias()
{
	local userdata="`getent passwd $1`"
	local uid="`echo $userdata | awk -F':' '{print $3}'`"
	local gid="`echo $userdata | awk -F':' '{print $4}'`"
	local home="`echo $userdata | awk -F':' '{print $6}'`"

	if [ -z "$uid" -o -z "$gid" -o ! -d "$home" ]; then
		report_problem "Userinfo for $1 is incomplete. Qmail alias for this user will not be created" 			"Manually create .qmail file in $1 home directory"
		return 1
	fi

	[ ! -s "$home/.qmail" ] || return
	echo "&root" > "$home/.qmail"
	chmod 0600 "$home/.qmail"
	chown $uid:$gid "$home/.qmail"
}

start_postfix()
{
	local context="$1"

	[ "$context" = "post" ] || return

	pleskrc postfix start
	pleskrc pc_remote start
}

stop_handlers_tmpfs_1012()
{
	[ ! -x "/usr/lib/plesk-9.0/handlers-tmpfs" ] || /usr/lib/plesk-9.0/handlers-tmpfs stop
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

set_mail_params()
{
	HANDLERS_USER="mhandlers-user"
	MAIL_USERGROUP="popuser"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

mailname_to_lowercase_11_1_15()
{
	local res=0

	echo_try "rename mailboxes to lower case format"

	$PRODUCT_ROOT_D/admin/sbin/mailmng-service --stop-service
	if ! $PRODUCT_ROOT_D/bin/repair --convert-mailboxes-to-lowercase -no-mchk; then
		warn "rename mailboxes to lower case format. Please run $PRODUCT_ROOT_D/bin/repair --convert-mailboxes-to-lowercase manually."
		res=1
	else
		suc
	fi

	deferred_mail_restore_all

	return $res
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
## vim:filetype=sh

upgrade_drweb_storage()
{
	# should we move files from old quarantine into new?
	[ -e "/etc/drweb/drweb_handler.conf" ] || return 0
	sed -i 		-e 's|^\s*Spool\s*=.*|Spool = /var/spool/drweb/spool|g' 		-e 's|^\s*SpoolFilesMode\s*=.*|SpoolFilesMode = 0664|g' 		-e 's|^\s*Quarantine\s*=.*|Quarantine = /var/spool/drweb/infected|g' 		-e 's|^\s*QuarantineFilesMode\s*=.*|QuarantineFilesMode = 0660|g' 		/etc/drweb/drweb_handler.conf || warn "update drweb handler config"

	# as far as we change dir location and config, drweb daemon reload is required
	if [ -x "/opt/psa/admin/sbin/drwebmng" ] ; then
		/opt/psa/admin/sbin/drwebmng --reload || service_restart_warning drweb
	fi
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
## vim:filetype=sh

upgrade_greylisting_permissions()
{
	local gl_db_dir="/var/lib/plesk/mail/greylist"

	[ -d "$gl_db_dir" ] || return
	chown -R root:popuser ${gl_db_dir} || pp_echo 'Failed to set correct ownership for grey listing databases'
	chmod 770 ${gl_db_dir} || pp_echo 'Failed to set correct grey listing databases directory permissions'
	chmod 660 ${gl_db_dir}/* || pp_echo 'Failed to set correct grey listing databases permissions'

	#may be call restore if there is(are) failures?
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
## vim:filetype=sh

upgrade_quota_handler_address_11_1_13()
{
	local common_mess="update check-quota handler recipients address column in handlers DB"
	echo_try ${common_mess}

	handler_db_path=/var/lib/plesk/mail/handlers/handlers.db

	if [ ! -w "${handler_db_path}" ]; then
		# This is more probably an upgrade from earlier versions than a broken machine
		p_echo "Unable to ${common_mess} - missing DB '${handler_db_path}'. Will call mail_restore later."
		deferred_mail_restore_all
		return 0
	fi

	local sqlite3_binary=`lookup_command sqlite3`
	if [ -z "${sqlite3_binary}" ]; then
		pp_echo "Unable to ${common_mess} - missing sqlite3 binary"
		return 1
	fi

	local query='UPDATE `handlers` SET `address`="all-recipients" WHERE `name`="check-quota"'
	${sqlite3_binary} "${handler_db_path}" "$query"
	if [ $? -ne 0 ]; then
		pp_echo "Unable to ${common_mess} - execution of DB query '$query' failed"
		return 1
	fi

	suc
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
#qmail

set_qmail_params()
{
    QMAIL_DIR="/var/qmail"
    QMAIL_USERS="alias:2021:nofiles:false:alias  qmaild:2020:nofiles:false:  qmaill:2022:nofiles:false:  qmailp:2023:nofiles:false:  qmailq:2520:qmail:  qmailr:2521:qmail:  qmails:2522:qmail:"
    QMAIL_GROUPS="qmail:2520  nofiles:2020"

    #variable from psa.conf but used in mail-qc-driver
    #so we need define this before install psa base package
    if [ -z "$QMAIL_ROOT_D" ]; then 
	    QMAIL_ROOT_D="/var/qmail"
    fi

    mm_mailgroup=12

    qmail_service="qmail"
    qmail_extra_cmd=""
    qmail_log="/var/log/maillog"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

upgrade_qmail_reduce_suids_11_1_12()
{
	if ! /opt/psa/admin/sbin/mailmng-server --features | egrep -q "SMTP_Server_package.*qmail" 2>/dev/null; then
		p_echo "Skip qmail specific upgrade actions"
		return
	fi

	upgrade_qmail_permissions
	drop_unused_mailman_wrapper
}

upgrade_qmail_permissions()
{
	local inten="setup correct owner and permissions on poplock and mail authentication DB"
	echo_try "$inten"

	local res=0

	local perm_err="set correct permissions on"
	local ownership_err="set correct ownership for"
	
	if [ -d "/var/lib/plesk/mail/poplock" ]; then
		if ! chown -R root:nofiles /var/lib/plesk/mail/poplock; then
			warn "${ownership_err} poplock DB"
			res=1
		fi

		if ! chmod 770 /var/lib/plesk/mail/poplock; then
			warn "${perm_err} poplock DB directory"
			res=1
		fi

		if ! chmod 660 /var/lib/plesk/mail/poplock/poplock.db; then
			warn "${perm_err} poplock DB file"
			res=1
		fi
	fi

	if [ -d "/var/lib/plesk/mail/auth" ]; then
		if ! chown -R root:nofiles /var/lib/plesk/mail/auth; then
			warn "${ownership_err} auth DB"
			res=1
		fi

		if ! chmod 750 /var/lib/plesk/mail/auth; then
			warn "${perm_err} auth DB directory"
			res=1
		fi

		if ! chmod 0640 /var/lib/plesk/mail/auth/passwd.db; then
			warn "${perm_err} auth DB file"
			res=1
		fi

		if ! chmod 440 /var/lib/plesk/mail/auth/passwd_db_key; then
			res=1
			warn "${perm_err} auth passwd file "
		fi
	fi

	if [ $res = 0 ]; then
		suc
	else
		warn "$inten"
	fi
}

drop_unused_mailman_wrapper()
{
	[ -x /usr/lib/plesk-9.0/mailman_lists_dump ] && /usr/lib/plesk-9.0/mailman_lists_dump
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

# db_backup [ dump_prefix [ db1 [ db2 [...] ] ] ]
db_backup()
{
	local timestamp=`date +%Y%m%d-%H%M%S`
	local dfile_prefix="${1:-mysql.preupgrade.${prev_version}-${product_version}}"
	dfile="$DUMP_D/${dfile_prefix}.${timestamp}.dump.gz"

	local databases
	[ -n "$*" ] && shift
	if [ -n "$*" ]; then
		databases="$@"
	else
		local apsc_db_name=`read_apsc_connection_param_db "database"`
		databases="mysql $PRODNAME $apsc_db_name horde atmail"
	fi

	if test -f $dfile; then
		# dump names should not collide, but better be safe than sorry
		local i
		i=1
		while test -f "$dfile.$i"; do
			i=$(($i+1))
		done
		mv -v "$dfile" "$dfile.$i" >> "$product_log" || die "save previous backup $dfile"
	fi

	inten="backup MySQL database"
	echo_try "$inten"
	if ! test -d "$DUMP_D"; then
		mkdir -p "$DUMP_D"
	fi
	touch "$dfile" || die "$inten"
	chown 0:0 "$dfile" || die "$inten"
	chmod 600 "$dfile" || die "$inten"
	dump_db $databases | gzip > "$dfile" || die "$inten"
	suc
	p_echo " MySQL databases are dumped to $dfile"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
safe_prep_db_upgrade()
{
	p_echo
	p_echo "===> Upgrading database"
	p_echo

	[ -s "$PRODUCT_ROOT_D/version" -o -s "$PRODUCT_ROOT_D/core.version" ] && get_prev_version "$PRODUCT_ROOT_D"

	mysql_unconfigured=
	set_mysqld_params
	check_db

	safe_prep_db_backup_all

	upgrade_apsc_db
	core_run_upgrade_prep_stage
}

safe_prep_db_backup_all()
{
	# backup database and register rollback action
	db_backup
	SAFE_PREP_DB_BACKUP="$dfile"
	transaction_add_rollback_action "safe_prep_db_upgrade_rollback"
}

safe_prep_db_upgrade_rollback()
{
	if [ -n "$SAFE_PREP_DB_BACKUP" ]; then
		p_echo
		p_echo "===> Restoring database from backup $SAFE_PREP_DB_BACKUP"
		p_echo
		zcat "$SAFE_PREP_DB_BACKUP" | mysql_anydb
		if [ $? -ne 0 ]; then
			p_echo
			p_echo "    [ERROR] Database restoring failed"
			p_echo "    Try to restore database from $SAFE_PREP_DB_BACKUP manually"
			p_echo
			return
		fi
		SAFE_PREP_DB_BACKUP=
	fi
}

safe_prep_db_install()
{
	mysql_unconfigured=1
	set_default_locale
	mysql_install_init
	product_install_db
	insert_fullhostname
}

safe_prep_db()
{
	p_echo
	p_echo "===> Performing safe prep-install database actions"
	p_echo

	# perform database-related prep actions
	if [ "$do_upgrade" -ne 0 ]; then
		safe_prep_db_upgrade
	else
		safe_prep_db_install
	fi
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
transaction_begin()
{
	[ -n "$TRANSACTION_STARTED" ] && die "Another transaction in progress!"
	TRANSACTION_STARTED="true"
	TRANSACTION_ROLLBACK_FUNCS=
	TRANSACTION_COMMIT_FUNCS=
	trap "transaction_rollback" HUP PIPE INT QUIT TERM EXIT
}

transaction_rollback()
{
	[ -z "$TRANSACTION_STARTED" ] && die "Transaction is not started!"
	# perform rollback actions
	local f
	for f in ${TRANSACTION_ROLLBACK_FUNCS}; do
		"$f"
	done
	TRANSACTION_STARTED=
	TRANSACTION_ROLLBACK_FUNCS=
	TRANSACTION_COMMIT_FUNCS=
	trap - HUP PIPE INT QUIT TERM EXIT
	exit 1
}

transaction_commit()
{
	[ -z "$TRANSACTION_STARTED" ] && die "Transaction is not started!"
	# perform commit actions
	local f
	for f in ${TRANSACTION_COMMIT_FUNCS}; do
		"$f"
	done
	TRANSACTION_STARTED=
	TRANSACTION_ROLLBACK_FUNCS=
	TRANSACTION_COMMIT_FUNCS=
	trap - HUP PIPE INT QUIT TERM EXIT
}

transaction_add_rollback_action()
{
	[ -z "$TRANSACTION_STARTED" ] && die "Transaction is not started!"
	# LIFO rollback order
	[ -z "$TRANSACTION_ROLLBACK_FUNCS" ] 		&& TRANSACTION_ROLLBACK_FUNCS="$1" 		|| TRANSACTION_ROLLBACK_FUNCS="$1 $TRANSACTION_ROLLBACK_FUNCS"
}

transaction_remove_rollback_action()
{
	[ -z "$TRANSACTION_STARTED" ] && die "Transaction is not started!"
	# Preserves order of remaining records
	TRANSACTION_ROLLBACK_FUNCS=`remove_from_array "$1" "$TRANSACTION_ROLLBACK_FUNCS"`
}

transaction_add_commit_action()
{
	[ -z "$TRANSACTION_STARTED" ] && die "Transaction is not started!"
	# FIFO commit order
	[ -z "$TRANSACTION_COMMIT_FUNCS" ] 		&& TRANSACTION_COMMIT_FUNCS="$1" 		|| TRANSACTION_COMMIT_FUNCS="$TRANSACTION_COMMIT_FUNCS $1"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

core_upgrader_repair_arg()
{
	if [ -n "$1" -o -n "$do_repair" ]; then
		echo "--repair"
	fi
}

core_upgrader_action_name()
{
	if [ -n "`core_upgrader_repair_arg $1`" ]; then
		echo "upgrade and repair"
	else
		echo "upgrade"
	fi
}

core_run_upgrade_prep_stage()
{
	local action_name="`core_upgrader_action_name $1`"
	local optional_repair_arg="`core_upgrader_repair_arg $1`"
	pp_echo "===> Cumulative Plesk database $action_name (revertable stage) has been started."
	cu_run_upgrader "core" core_upgrader 		--load-config-data core_cu_load_config_data --store-config-data core_cu_store_config_data 		--prepare core_upgrader_prepare --rollback core_upgrader_rollback --commit core_upgrader_commit 		--from-version "$prev_db_short_version" --stage "prep" $optional_repair_arg
	if [ "$?" -eq 0 ]; then
		pp_echo "===> Cumulative $action_name of Plesk database (revertable stage) has been completed."
	else
		pp_echo "===> Plesk database was not upgraded completely. See installation log for details."
	fi
}

core_run_upgrade_post_stage()
{
	local action_name="`core_upgrader_action_name $1`"
	local optional_repair_arg="`core_upgrader_repair_arg $1`"
	pp_echo "===> Cumulative Plesk $action_name (final stage) has been started."
	cu_run_upgrader "core" core_upgrader 		--load-config-data core_cu_load_config_data --store-config-data core_cu_store_config_data 		--prepare core_upgrader_prepare --rollback core_upgrader_rollback --commit core_upgrader_commit 		--from-version "$prev_short_version" --stage "post" $optional_repair_arg --update-version
	if [ "$?" -eq 0 ]; then
		pp_echo "===> Cumulative $action_name of Plesk (final stage) has been completed."
	else
		pp_echo "===> Plesk was not upgraded completely. See installation log for details."
	fi
}

core_init_upgrader_conf()
{
	echo_try "initialize Plesk DB and files upgrader configuration"
	cu_upgrader_conf_init "core" core_upgrader 		--load-config-data core_cu_load_config_data --store-config-data core_cu_store_config_data 		--stages "prep post"
	suc
}

core_read_misc_attr()
{
	db_select "SELECT val FROM misc WHERE param = '$1'"
	echo "$db_select_output"
}

core_cu_load_config_data()
{
	local upgrader_name="$1"

	[ "$upgrader_name" = "core" ] || return 0
	db_test_table "misc" || return 0

	cu_db_current_timestamp="`  core_read_misc_attr 'upgrader_config_current_timestamp'`"
	cu_db_continue_from="`      core_read_misc_attr 'upgrader_config_continue_from'`"
	cu_db_failed_actions_list="`core_read_misc_attr 'upgrader_config_failed_actions_list'`"
	cu_db_latest_timestamp="`   core_read_misc_attr 'upgrader_config_latest_timestamp'`"
}

core_cu_store_config_data()
{
	local upgrader_name="$1"

	[ "$upgrader_name" = "core" ] || return 0
	db_test_table "misc" || die "save core upgrader state to 'misc' table"

	db_do "REPLACE INTO misc (param, val) VALUES 		('upgrader_config_current_timestamp', '$cu_db_current_timestamp'), 		('upgrader_config_continue_from', '$cu_db_continue_from'), 		('upgrader_config_failed_actions_list', '$cu_db_failed_actions_list'), 		('upgrader_config_latest_timestamp', '$cu_db_latest_timestamp')"
}

core_upgrader_prepare()
{
	local upgrader_name="$1"
	local upgrader_stage="$2"
	local upgrader_aux="$3"

	# DB is backed up earlier via safe_prep_db_backup_all()

	if [ "$upgrader_stage" = "prep" ]; then
		pp_echo "===> Preparing Plesk database upgrade (revertable stage)."

		get_old_domain
	else
		pp_echo "===> Preparing Plesk upgrade (final stage)."
	fi

	db_added_columns=
	[ "$upgrader_aux" = "noop-stage" ] || stop_before_upgrade
}

core_upgrader_commit()
{
	local upgrader_name="$1"
	local upgrader_stage="$2"

	if [ "$upgrader_stage" = "prep" ]; then
		set_db_version
	else
		[ -n "$do_repair" ] || update_history
	fi
}

core_upgrader_rollback()
{
	# Empty rollback function is required to suppress default upgrader behavior
	# (switching configuration into broken state). Actual rollback will be done
	# in safe_prep_db_upgrade_rollback().
	:
}

core_upgrader()
{
	local db="cu_check_stage prep"
	local files="cu_check_stage post"
	local ver="cu_check_version"
	local ts="cu_check_timestamp"
	local do="cu_do"

	# Some upgrade actions might rely on MySQL passwords being upgraded. Note that rollback won't work for Billing.
	$db    $ts '2011/11/15 17:13:14' $do upgrade_mysqlnd_old_passwd

	cu_start_upgrade_from_version 90
	$db    $ver $do db_pmm_dashboard_90
	$db    $ver $do db_fix_90

	cu_start_upgrade_from_version 92
	$db    $ver $do add_db_changes 920

	cu_start_upgrade_from_version 93

	cu_start_upgrade_from_version 95
	$db    $ver $do db_update_exp_event
	$files $ver $do db_update_dashboard_951

	cu_start_upgrade_from_version 100
	cu_start_upgrade_from_version 101

# db_upgrade_subscriptions_109 should be run before db_update_plesk10
# due to usage 'custom' field of 'Subscriptions' somewhere in db_update_plesk10
	cu_start_upgrade_from_version 109
	$files $ver $do db_upgrade_subscriptions_109

	cu_start_upgrade_from_version 107
	$db    $ver $do add_db_changes 1070
	$db    $ver $do db_update_sessions_107
	$db    $ver $do db_add_ftp_sys_users
	$db    $ver $do db_add_plans_schema
	$files $ver $do db_add_plans_data
	$db    $ver $do db_add_webspaces
	$db    $ver $do db_add_domain_permissions
	$db    $ver $do db_billing_integration_107
	$db    $ver $do db_migrate_sa_prefs_107
	$db    $ver $do db_upgrade_hp_mail_107
	$db    $ver $do db_default_skin_107
	$files $ver $do db_migrate_sb_data_107
	$files $ver $do db_unlimited_client_expiration_107
	$files $ver $do db_clean_up_ip_pool_107
	$files $ver $do db_generate_guids_107
	$files $ver $do db_create_default_templates
	$files $ver $do db_update_plesk10
	$files $ver $do set_swkey_params
	$files $ver $do key_upgrade_cmd
	$files $ver $do deferred_httpd_reconfigure

	cu_start_upgrade_from_version 108
	$db    $ver $do db_change_domain_admins_length_108
	$db    $ver $do db_upgrade_subscription_limits_default_values_108
	$files $ver $do upgrade_postfix_enforce_tls_108
	$files $ver $do upgrade_postfix_chroot_108

	cu_start_upgrade_from_version 109
	$db    $ver $do add_db_changes 1090
	$db    $ver $do db_upgrade_clients_idn_109
	$db    $ver $do db_upgrade_custom_buttons_109
	$db    $ver $do db_upgrade_maintenance_mode_109
	$files $ver $do fix_crontab_109
	$files $ver $do db_upgrade_app_pools_109
	$files $ver $do upgrade_vhosts_109

	cu_start_upgrade_from_version 1010
	$db    $ver $do api_rpc_create_persistent_tables
	$db    $ver $do api_rpc_upgrade_persistent_tables
	$db    $ver $do add_db_changes 10100
	$db    $ver $do db_upgrade_ip_addresses_1010
	$db    $ver $do db_upgrade_dns_recs_t_1010
	$db    $ver $do db_upgrade_domain_services_1010
	$files $ver $do db_upgrade_web_domain_services_1010
	$db    $ver $do db_upgrade_hosting_1010
	$db    $ver $do db_upgrade_subdomains_1010
	$db    $ver $do db_upgrade_sso_port_1010
	$db    $ver $do db_optimize_configurations_table_1010
	$db    $ver $do db_cleanup_exp_event_1010
	$files $ver $do fix_ip_addresses_1010

	cu_start_upgrade_from_version 1011
	$db    $ver $do add_db_changes 10110
	$db    $ver $do migrate_backup_management_db_into_core_1011
	$db    $ver $do migrate_pmm_management_db_into_core_1011
	$db    $ver $do db_upgrade_backups_scheduled_1011
	$db    $ver $do db_upgrade_exp_event_obj_class_1011
	$db    $ver $do db_upgrade_ssobranding_port_1011
	$files $ver $do apsc_import_data_1011
	$files $ver $do db_upgrade_filesharing_1011

	cu_start_upgrade_from_version 1012
	$db    $ver $do add_db_changes 10120
	$db    $ver $do db_upgrade_parent_domain_id_1012
	$db    $ver $do db_upgrade_service_nodes_1012
	$db    $ver $do db_upgrade_resp_attach_1012
	$db    $ver $do db_upgrade_dns_reverse_ptr_ipv6_1012
	$db    $ver $do db_upgrade_upgrade_history_1012
	$db    $ver $do db_upgrade_local_service_node_1012
	$db    $ver $do db_upgrade_dns_schema_1012
	$files $ver $do db_upgrade_dns_data_1012
	$files $ver $do db_upgrade_subdomains_1012
	$files $ver $do db_upgrade_permissions_1012
	$files $ver $do db_upgrade_add_simple_plan_1012
	$files $ver $do upgrade_postfix_1012
	$files $ver $do upgrade_mailname_domain_dirs_perms_1012
	$files $ver $do db_upgrade_php_settings_1012
	$files $ver $do db_upgrade_mail_domain_services_1012
	$files $ver $do db_upgrade_dns_wwwprefix_1012
	$files $ver $do stop_handlers_tmpfs_1012

	cu_start_upgrade_from_version 1013
	$db    $ver $do add_db_changes 10130
	$files $ver $do upgrade_courier_imap_auth_1013
	$files $ver $do db_upgrade_subdomains_zones_1013
	$db    $ver $do db_upgrade_configurations_1013
	$files $ver $do generate_encryption_key
	$files $ver $do upgrade_postfix_sasl_1013
	$files $ver $do db_upgrade_dns_1013
	$files $ver $do upgrade_modules_permissions_1013
	$files $ver $do db_upgrade_aps_settings_1013 # This function is executed on post stage only due to absence of table in early versions of plesk
	$db    $ver $do db_upgrade_secret_key_1013
	$files $ver $do db_upgrade_secret_key_fix_1013
	$files $ver $do db_upgrade_admin_simple_plan_permissions_1013
	$db    $ver $do db_upgrade_sessions_1013
	$files $ver $do license_upgrade_1013

	cu_start_upgrade_from_version 1100
	$db    $ver $do add_db_changes 11000
	$db    $ver $do db_upgrade_heterogenous_1100
	$db    $ver $do db_upgrade_apsmail_1100
	$db    $ver $do db_upgrade_service_nodes_1100
	$db    $ver $do db_upgrade_mail_server_settings_1100
	$db    $ver $do db_upgrade_webusers_miva_1100
	$files $ver $do pbm_create_certificate # We should generate certificate on upgrade since backup was merged into core
	$files $ver $do upgrade_adminlogs_permissions_1100

	cu_start_upgrade_from_version 1101
	$db    $ver $do db_upgrade_longtasks_1101
	$db    $ver $do db_upgrade_db_users_1101

	cu_start_upgrade_from_version 011001011
	$db    $ver $do db_upgrade_add_indexes_for_performance_11_1_11
	$db    $ver $do db_upgrade_change_sys_users_home_to_utf8
	$files $ver $do deferred_mail_restore_all

	cu_start_upgrade_from_version 011001012
	$db    $ver $do add_db_changes 11.1.12
	$db    $ver $do db_upgrade_service_node_properties_11_1_12
	$db    $ver $do db_upgrade_custom_buttons_11_1_12
	$files $ver $do upgrade_greylisting_permissions
	$files $ver $do upgrade_drweb_storage
	$files $ver $do upgrade_qmail_reduce_suids_11_1_12

	cu_start_upgrade_from_version 011001013
	$db    $ver $do db_upgrade_service_node_configuration_11_1_13
	$files $ver $do upgrade_quota_handler_address_11_1_13

	cu_start_upgrade_from_version 011001014
	$db    $ver $do db_upgrade_dns_templates_11_1_14

	cu_start_upgrade_from_version 011001015
	$files $ts '2012/12/06 13:03:47' $do db_upgrade_subscription_properties_11_1_15
	$files $ts '2013/07/09 17:45:00' $do mailname_to_lowercase_11_1_15
	$files $ts '2012/12/12 14:44:47' $do register_sw_engine_service
	$files $ts '2012/12/14 09:26:01' $do upgrade_pci_compliance_11_1_15

	cu_start_upgrade_from_version 011001016
	$db    $ts '2012/12/19 18:57:31' $do add_db_changes 11.1.16

	cu_start_upgrade_from_version 011001017
	$db    $ts '2013/01/15 17:41:18' $do db_upgrade_forwarding_11_1_17
	$db    $ts '2013/01/21 16:34:00' $do db_upgrade_seo_safe_domain_aliases_11_1_17
	$files $ts '2013/01/21 17:11:13' $do upgrade_postfix_allow_authenticated_dnsbled_clients_11_1_17
	$db    $ts '2013/11/26 09:37:00' $do db_upgrade_db_user_unique_key_11_1_17

	cu_start_upgrade_from_version 011001018
	$db    $ts '2013/01/29 15:00:00' $do add_db_changes 11.1.18
	$db    $ts '2013/02/04 19:10:05' $do db_upgrade_php_handler_id_11_1_18
	$files $ts '2013/05/24 18:13:37' $do upgrade_skel_remove_statistics_11_1_18

	cu_start_upgrade_from_version 011005019
	$db    $ts '2013/02/22 09:43:00' $do add_db_changes 11.5.19
	$db    $ts '2013/02/14 17:51:00' $do db_upgrade_drop_php_version_11_5_19
	$db    $ts '2013/02/25 14:43:00' $do db_upgrade_webmails_11_5_19

	cu_start_upgrade_from_version 011005021
	$db    $ts '2013/03/07 12:57:01' $do db_upgrade_enable_dns_connector_11_5_21
	$db    $ts '2013/03/19 15:19:00' $do add_db_changes 11.5.21
	$db    $ts '2013/03/25 14:24:01' $do db_upgrade_set_force_update_applications_notification_11_5_21
	$db    $ts '2013/03/28 09:40:01' $do db_add_public_ip_11_5_21

	cu_start_upgrade_from_version 011005022
	$files $ts '2013/04/01 17:38:44' $do db_upgrade_drop_ip_address_id_11_5_22
	$files $ts '2013/06/05 18:07:46' $do upgrade_vhost_directories_11_1_18
	$db    $ts '2013/04/08 18:18:44' $do db_upgrade_set_property_unpaid_website_status_for_plans_11_5_22
	$db    $ts '2013/04/08 18:18:44' $do db_upgrade_set_property_unpaid_website_status_for_subscriptions_11_5_22

	cu_start_upgrade_from_version 107
	# Moved from 107 and 1012 because of the BL (#80977) and backend (#136372) dependencies. Should be after upgrade_vhost_directories_11_1_18.
	$files $ver $do upgrade_proftpd_107

	cu_start_upgrade_from_version 011005023
	$files $ts '2013/04/18 15:25:35' $do fix_file_sharing_permissions_from_10_0_0
	$files $ts '2013/04/18 19:44:02' $do upgrade_courier_to_4_12_pop_before_smtp_11_5_23

	cu_start_upgrade_from_version 011005024
	$db    $ts '2013/04/26 14:00:44' $do db_upgrade_correcting_property_unpaid_website_status_for_plans_11_5_24
	$db    $ts '2013/04/26 14:00:44' $do db_upgrade_correcting_property_unpaid_website_status_for_subscriptions_11_5_24
	$db    $ts '2013/04/26 16:58:00' $do db_upgrade_maintenance_mode_id_11_5_24
	$db    $ts '2013/04/30 13:45:00' $do db_upgrade_service_node_configuration_11_5_24
	$files $ts '2013/04/30 12:37:00' $do db_upgrade_secret_keys_11_5_24
	$files $ts '2013/05/14 14:11:38' $do upgrade_domains_awstats_conf_11_5_24

	cu_start_upgrade_from_version 011005029
	$db    $ts '2013/06/20 10:30:00' $do add_db_changes 11.5.29
	$db    $ts '2013/06/25 13:39:00' $do db_upgrade_correcting_aps_tokens_11_5_29
	$files $ts '2013/07/22 14:44:38' $do upgrade_named_restrict_number_of_workers_11_5_29
	$files $ts '2013/07/22 14:45:00' $do apsc_fix_db_credentials
	$files $ver $do upgrade_cgi_bin_mode_defaults_11_5_30

	cu_start_upgrade_from_version 011005030
	$db    $ts '2013/07/22 15:01:00' $do db_upgrade_db_user_unique_key_11_1_17
}

###
### HOW TO ADD UPGRADE ACTION
###
### Short version:
###  To the end of core_upgrader() function (above) add a line in the format:
###   STAGE $ts 'TIMESTAMP' $do UPGRADE_ACTION
###  Where:
###   STAGE is either '$db' or '$files' without quotes. If your upgrade action affects only Plesk DB and doesn't require any
###       new files (e.g. PHP scripts) to run, then use '$db'. Otherwise use '$files'.
###   TIMESTAMP is a current (at the moment you write your code) timestamp in the format 'YYYY/MM/DD hh:mm:ss'.
###       You can use output of `date '+%Y/%m/%d %H:%M:%S'` command.
###   UPGRADE_ACTION is a call to the function that implements your upgrade action. While you can supply arguments here, it is
###       advised not to. Function name should reflect what you are upgrading and *from* which version. Usually if you use
###       '$db' as STAGE then function name should start with 'db_upgrade_', otherwise with 'upgrade_'.
###
###  If you use 'add_db_changes <VERSION>' as UPGRADE_ACTION add your update SQL to plesk-install/psa_db_upgrade_<VERSION>.sql,
###  don't forget to add the aforementioned file to plesk-install/Makefile.
###
###
### Slightly longer version:
###  'cu_start_upgrade_from_version <version>' calls mark groups of upgrade actions for upgrade from a given version. They
###      affect only upgrade actions with '$ver' filter, but they also serve a role of self-documentation and therefore
###      should be added for any product version change.
###  '$db' marks upgrade actions that will only be executed in prep-install bootstrapper stage. In case of critical failure
###      all changes made to Plesk DB here will be automatically reverted. You should do only changes to Plesk DB during
###      this stage; don't use anything not packaged with bootstrapper here (you may use db_*() and add_db_changes() functions).
###  '$files' marks upgrade actions that will only be executed in post-install bootstrapper stage. During this stage all
###      required '$db' actions have already been completed (unless some of them were optional), all new packaged files
###      are already in place and there are no old removed packaged files. You can do pretty much everything at this stage.
###  '$ver' marks that whether or not this upgrade action should be called will be checked based on previous DB or product
###      version (for use with '$db' and '$files' respectively). This requires cu_start_upgrade_from_version() call somewhere
###      above.
###  '$ts TIMESTAMP' marks that whether or not this upgrade action should be called will be checked based on core component
###      configuration version (which is a timestamp). Any upgrade actions with a TIMESTAMP newer than configuration version
###      will be executed. This approach allowes smooth upgrade between builds within a same product version. Use it.
###      TIMESTAMP is a string with a current timestamp in any sensible format which allows sane ordering when punctuation is
###      stripped. Output of the following command is recommended for use as TIMESTAMP: `date '+%Y/%m/%d %H:%M:%S'`.
###      Usually timestamps will increase from previous upgrade action to the next, but it is not a requirement.
###  '$do' is just a required word that should preceed upgrade action call. The call itself may include parameters, but it's
###      advised not to, since it's much better to have upgrade actions uniquely identifiable by name.
###
###  Upgrade action should be more or less self-contained. Never do several unrelated things in a single upgrade action.
###  Never separate related upgrade actons across several calls in core_upgrader() unless there is a good reason to.
###
###  Upgrade actions can be either critical or optional. Optional upgrade actions should indicate failure with non-zero return
###  code. Critical upgrade actions should indicate failure with die() or similar calls (that terminate execution). Note that
###  using most of db_*() functions makes an upgrade action a critical one. Critical failure of a '$db' upgrade action will
###  trigger DB rollback. Critical failure of a '$files' upgrade action will switch core configuration into a broken unusable
###  state, from which one can recover only by successfully passing all required upgrade actions during bootstrapper repair
###  procedure. "Optional" failure of any upgrade action will add it to the list of failed ones and continue with upgrade.
###  All failed actions are rerun during bootstrapper repair procedure.
###

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#vhosts

install_vhosts()
{
	set_common_params
	read_conf

	inten="create Virtual Hosts directory"
# Paranoia
	echo_try $inten
	mkdir -p "$HTTPD_VHOSTS_D/default"
	mkdir -p "$HTTPD_VHOSTS_D/.skel"
	suc

	inten="sync default virtual host"
	echo_try $inten

	rsync -rlptgoD --ignore-existing /usr/share/plesk-vhost/ $HTTPD_VHOSTS_D/default/
	suc
# Dir check for awoid overwrite in future
	if [ ! -d "$HTTPD_VHOSTS_D/.skel/0" -a -x "$PRODUCT_ROOT_D/admin/sbin/skelmng" ]; then
		$PRODUCT_ROOT_D/admin/sbin/skelmng --install-default --id 0 >> "$product_log" 2>&1
	fi
}

chown_webstat()
{
	# select only domains with hosting
	query="select d.id from domains d, hosting h where d.id=h.dom_id";

	for dom_id in `echo $query | $mysql`; do

		query1="select s.login from sys_users s, hosting h where s.id=h.sys_user_id and h.dom_id='$dom_id'";
		login=`echo $query1 | $mysql`;

		query1="select name from domains where id='$dom_id'";
		dom_name=`echo $query1 | $mysql`;

		if [ -d "$HTTPD_VHOSTS_D/$dom_name/httpdocs/webstat" ]; then
#			chown $login:$clients_group `get_ls full "$HTTPD_VHOSTS_D/$dom_name/httpdocs/webstat"`
			find "$HTTPD_VHOSTS_D/$dom_name/httpdocs/webstat" -exec chown $login:$clients_group {} \;
		fi

	done >> "$product_log" 2>&1
}

vhosts_install_error_doc()
{
	local document dir file default_skeleton wrkdir sample_doc sample_file
	sample_doc="not_found.html"
	document="$1"
	default_skeleton="$PRODUCT_ROOT_D/tmp/default_skeleton.tgz"

	echo_try " Installing error document $document"

	wrkdir=`mktemp -d /tmp/skelXXXXXXXX`
	if test $? -ne 0; then
		p_echo " $wrkdir"
		die "create temporary directory";
	fi
	$tar xzf "$default_skeleton" -C "$wrkdir" "./error_docs/$document" >>"$product_log" 2>&1 	|| die "extract $document from default skeleton"

	find "$HTTPD_VHOSTS_D/.skel" "$HTTPD_VHOSTS_D" 			-maxdepth 2 -type d -name error_docs 	| while read dir; do
		file="$dir/$document"
		sample_file="$dir/$sample_doc"
		if ! test -e "$file"; then
			local doc_owner doc_group doc_perms
			doc_owner=0
			doc_group=0
			doc_perms=644
			if test -e "$sample_file"; then
				doc_owner=`stat -c %u "$sample_file"`
				doc_group=`stat -c %g "$sample_file"`
				doc_perms=`stat -c %a "$sample_file"`
			else
				warn "Unable to determine correct owner, group and access rights for $file. Please check them manually"
			fi
			install -o $doc_owner -g $doc_group -m $doc_perms "$wrkdir/error_docs/$document" "$file" >>"$product_log" 2>&1 			|| die "install document $document in $dir"

			domain_name=`echo "$dir" | awk -F/ '{ print $(NF-1) }'`
			sed -i -e "s/@domain_name@/$domain_name/g" "$file"
		fi
	done
	if test $? -ne 0; then exit 1; fi

	rm -rf "$wrkdir" >> "$product_log" 2>&1

	suc
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
upgrade_vhosts_109()
{
	vhosts_install_error_doc maintenance.html
}

### Copyright 1999-2013. Parallels IP Holdings GmbH. All Rights Reserved.

upgrade_vhost_directories_11_1_18()
{
	echo_try "upgrade vhost directories for domains"
	$PRODUCT_ROOT_D/bin/repair --update-vhosts-structure -no-reconfigure && suc || warn "fail\nCannot upgrade vhost directories for domains. Please run $PRODUCT_ROOT_D/bin/repair --update-vhosts-structure manually."
	deferred_httpd_reconfigure_all
}

upgrade_skel_remove_statistics_11_1_18()
{
	echo_try "drop unused 'statistics' directory from vhost skeletons"
	local unmodified_skel_stat_md5sum="3f5517860e8adfa4b05c9ea6268b38eb"
	local skel_stat_md5sums="`find -L $HTTPD_VHOSTS_D/.skel/*/statistics -type f 2>/dev/null | xargs --no-run-if-empty md5sum | cut -d ' ' -f 1 | sort | uniq`"
	if [ -z "$skel_stat_md5sums" -o "$skel_stat_md5sums" = "$unmodified_skel_stat_md5sum" ]; then
		rm -rf $HTTPD_VHOSTS_D/.skel/*/statistics >>"$product_log" 2>&1
	else
		pnnl_echo " 'statistics' directory seems customized, so it was preserved. Please remove it later manually. "
	fi
	suc
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#webalizer

set_webalizer_params()
{
	WEBALIZER_ROOT_D="$PRODUCT_ROOT_D/webalizer"
}

## @@constructor set_webalizer_params

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
wd_adapt()
{
    if [ -x $PRODUCT_ROOT_D/admin/bin/modules/watchdog/wd ] ; then
        $PRODUCT_ROOT_D/admin/bin/modules/watchdog/wd --adapt >>$product_log 2>&1
    fi
}

wd_configure()
{
	if [ -x $PRODUCT_ROOT_D/libexec/modules/watchdog/configure/configure ] ; then
		$PRODUCT_ROOT_D/libexec/modules/watchdog/configure/configure >>$product_log 2>&1
	fi
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh

read_postconf_value()
{
	local param="$1"
	/usr/sbin/postconf -h "$param" 2>>"$product_log" | sed -e 's/\s*,\s*/\n/g'
}

write_postconf_value()
{
	local param="$1"
	local value="$2"
	local conf_value="`echo "$value" | awk 'BEGIN { ORS=""; PORS="" } { print PORS; print; PORS=", " }'`"
	/usr/sbin/postconf -e "$param=$conf_value" >>"$product_log" 2>&1
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

set_postfix_params()
{
	postfix_service="postfix"
	pc_remote_service="pc-remote"
}

true postfix_status
postfix_status()
{
	# here be dragons.
	# the practical experience shows that simple checking of status of
	# Postfix "master" process is not enough. So we read Postfix master
	# process pid file if any, then try to look for a process with
	# name ``qmgr'' and parent pid being equal to
	# the pid read from the pidfile. If pgrep finds such a process
	# it returns 0, if not its exit status is non-zero.
	# pgrep is portable enough to prefer it to "hand-made" alternatives
	# such as famous ``ps | grep $name | grep -v grep...'' pipes
	# bug 147822. do not interrupt installation for FreeBSD

	[ -f "/var/spool/postfix/pid/master.pid" ] || return 1

	local ppid

	read ppid </var/spool/postfix/pid/master.pid 2>/dev/null
	if [ $? -ne 0 -o -z "$ppid" ]; then
		# not found or other error
		return 1;
	fi
	pgrep -P $ppid qmgr >/dev/null 2>/dev/null
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
## vim:filetype=sh

upgrade_postfix_1012()
{
	# Remove excessive smtp proxies from master.cf, as we are going to use milter
	local master_cf_path=/etc/postfix/master.cf
	[ -s "$master_cf_path" ] || return 0
	perl -ne '
		next if ($_ =~ /^.?127.0.0.1:1002[567]\s+/);
		if ($_ =~ /^(.?(smtp|smtps|submission)\s+.*)-o\s+smtpd_proxy_filter=127.0.0.1:10025(.*)$/) { print "$1$3\n"; next; }
		if ($_ =~ /^(.?pickup\s+.*)-o\s+content_filter=smtp:127.0.0.1:10027(.*)$/) { print "$1$2\n"; next; }
		print $_;
	' < $master_cf_path > $master_cf_path.tmp && mv -f $master_cf_path.tmp $master_cf_path
}

upgrade_mailname_domain_dirs_perms_1012()
{
	# this is for mailname folders that were created when Postfix was in charge
	local popuser="popuser"
	for domain_mail_dir in /var/qmail/mailnames/* ; do
		[ ! -d "$domain_mail_dir" ] || chown $popuser:$popuser "$domain_mail_dir"
	done
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
upgrade_postfix_sasl_1013()
{
	local smtpd_sasl_conf='/etc/postfix/sasl/smtpd.conf'
	[ -f "$smtpd_sasl_conf" ] || return 0
	install_postfix_smtpd_sasl_config
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
## vim:filetype=sh

upgrade_postfix_enforce_tls_108()
{
	local master_cf_path
	master_cf_path=/etc/postfix/master.cf

	[ -f $master_cf_path ] || return 0

	sed -e '
		/submission[[:space:]]\+inet/ {
			/smtpd_tls_security_level/ !{
				s/-o[[:space:]]\+smtpd_enforce_tls=yes/& -o smtpd_tls_security_level=encrypt/
			}
		}
	' < $master_cf_path > $master_cf_path.tmp && 	mv -f $master_cf_path.tmp $master_cf_path
}

upgrade_postfix_chroot_108()
{
	return 0
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh

upgrade_postfix_allow_authenticated_dnsbled_clients_11_1_17()
{
	[ -s "/etc/postfix/main.cf" ] || return 0

	set_postfix_params

	local filtered_value="`read_postconf_value 'smtpd_client_restrictions' | sed -e '/^permit_mynetworks$/d' -e '/^permit_sasl_authenticated$/d'`"
	local extended_value="permit_mynetworks
permit_sasl_authenticated
$filtered_value"

	write_postconf_value 'smtpd_client_restrictions' "$extended_value" || return 1
	pleskrc postfix reload
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
register_webmail()
{
		local name="$1"
		local inten="register webmail '$1'"
		
		[ -x "$webmail_tool" ] || return
		[ -z "$name" ] && die $inten

		$webmail_tool --install --name="$name" || warn $inten
		deferred_httpd_reconfigure
}

install_db_file()
{
	local file="$1"
	local mysql_db_name="$2"
	local inten="$3"
	echo_try $inten
	[ -f "$file" ] || die "Unable to $inten: $file not found"
	cat $file | mysql >>"$product_log" 2>&1 || die "Unable to $inten, see $product log for details"
	suc
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
set_webmail_params()
{
	webmail_conf_d="/etc/psa-webmail"
	webmail_tool="${PRODUCT_ROOT_D}/admin/bin/webmailmng"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
set_horde_params()
{
	horde_datadir="/usr/share/psa-horde"
	horde_confdir="/etc/psa-webmail/horde"
	horde_sysconfd="$horde_confdir/horde"
	horde_logdir="/var/log/psa-horde"
	horde_passwd_file="/etc/psa-webmail/horde/.horde.shadow"
	horde_title="Horde Web Based mail client"

	imp_datadir="$horde_datadir/imp"
	imp_sysconfd="$horde_confdir/imp"
	
	turba_datadir="$horde_datadir/turba"
	turba_sysconfd="$horde_confdir/turba"

	kronolith_datadir="$horde_datadir/kronolith"
	kronolith_sysconfd="$horde_confdir/kronolith"

	ingo_datadir="$horde_datadir/ingo"
	ingo_sysconfd="$horde_confdir/ingo"

	mnemo_datadir="$horde_datadir/mnemo"
	mnemo_sysconfd="$horde_confdir/mnemo"
	
	passwd_datadir="$horde_datadir/passwd"
	passwd_sysconfd="$horde_confdir/passwd"

	horde_user="horde_sysuser"
	horde_group="horde_sysgroup"

	horde_php_ini="/etc/psa-webmail/horde/horde/php.ini"
}


horde_run_migration_script()
{
	local app=$1
	local migrate_cmd="${psa_pear_dir}/pear/horde-db-migrate --config=${psa_pear_conf} ${app}"
	local horde_conf=${horde_sysconfd}/conf.php

	if [ -z "${app}" ]; then
		#we can not exec script if cache driver is sql, cause cache table may be not installed yet
		#if app is specified it means that framework is already configured, so cache tables is already created
		sed -i -e "s|\$conf\['cache'\]\['driver'\].*|\$conf['cache']['driver'] = 'File';|g" ${horde_conf}
	fi

	horde_run_php_script ${migrate_cmd} || warn "install or upgrade horde database, please re-run ${migrate_cmd}"

	if [ -z "${app}" ]; then
		sed -i -e "s|\$conf\['cache'\]\['driver'\].*|\$conf['cache']['driver'] = 'Sql';|g" ${horde_conf}
	fi
}

horde_run_php_script()
{
	env PHP_PEAR_SYSCONF_DIR=$psa_pear_dir sw-engine -d "include_path=${psa_pear_include_dir}:." $@ >> $product_log 2>&1
}

horde_convert_shares_to_sql()
{
	local app_name=$1
	local script_name=${psa_pear_dir}/pear/${app_name}-convert-datatree-shares-to-sql

	if ! db_test_table horde_datatree; then
		p_echo "Skip convert datatree to mysql - table horde_datatree is missed"
		return
	fi

	db_select "SELECT datatree_id FROM horde_datatree WHERE group_uid = 'horde.shares.${app_name}'"
	[ -n "${db_select_output}" ] || return 0


	if [ ! -f "${script_name}" ]; then
		warn "convert datatree shares to sql - missed convertion script"
		return
	fi

	# NOTE: using echo instead of printf is breaks correct execution on DEB-like OSes
	printf '1\ny' | horde_run_php_script "${script_name}" || warn "execute '${script_name}'"
}

upgrade_horde_DB_to_latest_h3()
{
	local mysql_db_name="horde"

	#H5 or above detect
	! db_test_table horde_core_schema_info || return 0

	if ! db_test_table horde_syncml_map; then
        	install_db_file "$horde_datadir/.upgrade/3.1_to_3.2.mysql.sql"                 	$mysql_db_name                 	"upgrade horde database 3.1 -> 3.2"
	fi

	local inten="upgrade horde db sequences"
	horde_run_php_script "$horde_datadir/.upgrade/2008-08-29_fix_mdb2_sequences.php" && suc || warn "$inten"

	if db_test_column horde_signups signup_email; then
		install_db_file "$horde_datadir/.upgrade/3.3_to_3.3.5.sql" 					$mysql_db_name                 	"upgrade horde database 3.3 -> 3.3.5" 
	fi
}

upgrade_imp_DB_to_latest_h3()
{
	local mysql_db_name="horde"

	#H5 or above detect
	! db_test_table imp_schema_info || return 0

	if ! db_test_table imp_sentmail; then
		install_db_file "$imp_datadir/.upgrade/imp.sql" 			$mysql_db_name                 	"upgrade imp database 4.1.x -> 4.2.x"
	fi
}

upgrade_ingo_DB_to_latest_h3()
{
	local mysql_db_name="horde"

	#H5 or above detect
	! db_test_table ingo_schema_info || return 0

	if ! db_test_table ingo_shares_users; then
		install_db_file "$ingo_datadir/.upgrade/ingo.sql" 			$mysql_db_name                 	"upgrade ingo database 1.1.x -> 1.2"
	fi

	if db_test_table ingo_shares_users; then
		install_db_file "$ingo_datadir/.upgrade/1.2_to_1.2.1.sql" 			$mysql_db_name                 	"upgrade ingo database 1.2 -> 1.2.1"
	fi

	if db_test_column ingo_shares_groups group_uid; then
		install_db_file "$ingo_datadir/.upgrade/1.2.1_to_1.2.2.sql" 			$mysql_db_name                 	"upgrade ingo database 1.2.1 -> 1.2.2"
	fi
}

upgrade_kronolith_DB_to_latest_h3()
{
	local mysql_db_name="horde"

	#H5 or above detect
	! db_test_table kronolith_schema_info || return 0

	if ! db_test_table kronolith_shares_groups; then
		install_db_file "$kronolith_datadir/.upgrade/2.1_to_2.2.sql" 			$mysql_db_name                 	"upgrade kronolith database 2.1 -> 2.2"
	fi

	if db_test_column kronolith_shares share_owner; then
		install_db_file "$kronolith_datadir/.upgrade/2.2_to_2.3.sql" 			$mysql_db_name                 	"upgrade kronolith database 2.2 -> 2.3"
	fi
}

upgrade_mnemo_DB_to_latest_h3()
{
	local mysql_db_name="horde"

	#H5 or above detect
	! db_test_table mnemo_schema_info || return 0

	if ! db_test_table mnemo_shares; then
		install_db_file "$mnemo_datadir/.upgrade/2.1_to_2.2.sql" 			$mysql_db_name                 	"upgrade mnemo database 2.1 -> 2.2"
	fi

	if db_test_column mnemo_shares share_owner; then
		install_db_file "$mnemo_datadir/.upgrade/2.2_to_2.2.1.sql" 			$mysql_db_name                 	"upgrade mnemo database 2.2 -> 2.2.1"
	fi

	if db_test_column mnemo_shares_groups group_uid; then
		install_db_file "$mnemo_datadir/.upgrade/2.2.1_to_2.2.2.sql" 			$mysql_db_name                 	"upgrade mnemo database 2.2.1 -> 2.2.2"
	fi

}


upgrade_turba_DB_to_latest_h3()
{
	local mysql_db_name="horde"

	#H5 or above detect
	! db_test_table turba_schema_info || return 0

	if ! db_test_table turba_shares; then
		install_db_file "$turba_datadir/.upgrade/2.1_to_2.2_add_sql_share_tables.sql" 			$mysql_db_name                 	"upgrade turba database 2.1 -> 2.2"
	fi

	if ! db_test_column turba_objects object_firstname; then
		local inten="change turba address book scheme"
		horde_run_php_script "$turba_datadir/.upgrade/2.1_to_2.2_sql_schema.php" && suc || warn "$inten"
	fi

	if db_test_column turba_shares share_owner; then
		install_db_file "$turba_datadir/.upgrade/2.2.1_to_2.3.sql" 		$mysql_db_name                 "upgrade turba database 2.2.1 -> 2.3"
	fi

}

generate_horde_password()
{
	generate_password_file "$horde_passwd_file" "$horde_title" root "$horde_group" 640
	horde_password=`cat $horde_passwd_file` || die $inten
}

generate_horde_php_ini()
{
	${PRODUCT_ROOT_D}/admin/sbin/phpinimng --type=cgi ${horde_php_ini} < ${horde_php_ini}.tpl
}

roundcube_fix_perms_on_private_files()
{
	# update perms on private roundcube files (for upgrade)
	set_ac root "$roundcube_group" 0640 ${roundcube_db_conf}
	set_ac root "$roundcube_group" 0640 ${roundcube_main_conf}
	set_ac root root 0640 ${roundcube_passwd_file}

	# remove redundant files from previous roundcube versions:
	rm -f "${roundcube_php_ini}.tpl"

	# fix permissions on log and temp dirs (at least for debian-like oses):
	set_ac "$roundcube_user" "root" 0750 "$roundcube_logs_d"
	set_ac "$roundcube_user" "root" 0750 "$roundcube_temp_d"

	# fix permissions on log files
	for l in `ls "${roundcube_logs_d}"`; do
		chown ${roundcube_user}:${roundcube_group} "${roundcube_logs_d}/$l"
	done
}

roundcube_fix_db_grants()
{
	local mysql_db_name='mysql'
	db_do "GRANT SELECT, INSERT, UPDATE, DELETE, CREATE, DROP, INDEX, ALTER 
			ON \`roundcubemail\`.* TO 'roundcube'@'localhost'; 
			FLUSH PRIVILEGES;"
}

roundcube_generate_php_ini()
{
	${PRODUCT_ROOT_D}/admin/sbin/phpinimng --type=cgi "${roundcube_php_ini}" < "${roundcube_php_ini_tpl}"
}

roundcube_get_current_files_version()
{
	local version_file="$roundcube_home/program/include/iniset.php"
	sed -ne 's/^\s*define(.RCMAIL_VERSION.,\s*.\([0-9.]\+[a-z-]*\).);.*$/\1/p' "$version_file" 2>>"$product_log"
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:set ft=sh:

set_roundcube_params()
{
	roundcube_passwd_file="/usr/share/psa-roundcube/config//.roundcube.shadow"
	roundcube_title="Roundcube Web Based mail client"
	roundcube_db_conf="/usr/share/psa-roundcube/config//db.inc.php"
	roundcube_main_conf="/usr/share/psa-roundcube/config//main.inc.php"
	roundcube_sql_d="/usr/share/psa-roundcube//SQL"
	roundcube_user="roundcube_sysuser"
	roundcube_group="roundcube_sysgroup"
	roundcube_home="/usr/share/psa-roundcube/"
	roundcube_logs_d="/var/log/plesk-roundcube/"
	roundcube_temp_d="/var/tmp/plesk-roundcube/"

	roundcube_php_ini="/etc/psa-webmail/roundcube/php.ini"
	roundcube_php_ini_tpl="/usr/share/psa-roundcube/config/php.ini.tpl"

	roundcube_preupgrade_version_file="/etc/psa-webmail/roundcube//version.upg"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

fix_file_sharing_permissions_from_10_0_0()
{
	set_apache_params

	local file_sharing_passwd_file="${HTTPD_VHOSTS_D}/fs-passwd/dav.passwd"

	[ -f "${file_sharing_passwd_file}" ] || return 0

	chown psaadm:${apache_group} "${file_sharing_passwd_file}" 
	chmod 640 "${file_sharing_passwd_file}"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
upgrade_adminlogs_permissions_1100()
{
    chown psaadm:0 ${PRODUCT_ROOT_D}/admin/logs
    chmod 750 ${PRODUCT_ROOT_D}/admin/logs
}
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

deferred_httpd_reconfigure()
{
	touch "/var/lock/plesk_deferred_httpd_reconfigure"
}

deferred_httpd_reconfigure_all()
{
	touch "/var/lock/plesk_deferred_httpd_reconfigure_all"
}

deferred_watchdog_reconfigure()
{
	touch "/var/lock/plesk_deferred_watchdog_reconfigure"
}

deferred_mail_restore_all()
{
	touch "/var/lock/plesk_deferred_mail_restore_all"
}

deferred_mail_spam_restore_full()
{
	touch "/var/lock/plesk_deferred_mail_spam_restore_full"
}

deferred_webserver_apache_configure()
{
	touch "/var/lock/plesk_deferred_webserver_apache_configure"
}

run_now_webserver_apache_configure()
{
	[ ! -x "$PLESK_LIBEXEC_DIR/services/webserver.apache" ] || $PLESK_LIBEXEC_DIR/services/webserver.apache configure
}

deferred_ftp_proftpd_configure()
{
	touch "/var/lock/plesk_deferred_ftp_proftpd_configure"
}

run_now_ftp_proftpd_configure()
{
	[ ! -x "$PLESK_LIBEXEC_DIR/services/ftp.proftpd" ] || $PLESK_LIBEXEC_DIR/services/ftp.proftpd configure
}

deferred_awstats_configure()
{
	touch "/var/lock/plesk_deferred_awstats_configure"
}

run_now_awstats_configure()
{
	[ ! -x "$PLESK_LIBEXEC_DIR/statistics/awstats" ] || $PLESK_LIBEXEC_DIR/statistics/awstats configure
}

deferred_super_server_action_configure()
{
	touch "/var/lock/plesk_deferred_super_server_action_configure"
}

run_now_courier_mailserver_change_notification()
{
	set_courier_imap_params
	pleskrc courier_authdaemon condrestart
}

### perform actions :
perform_deferred_actions()
{
	local flag_files="relabel_plesk_directories 		super_server_action_configure 		watchdog_configure 		watchdog_reconfigure 		ftp_proftpd_configure 		webserver_apache_configure 		httpd_reconfigure_all 		httpd_reconfigure 		mail_restore_all 		mail_spam_restore_full 		courier_mailserver_change_notification 		apache_restart 		awstats_configure 	"
	local res=0
	for flag in $flag_files; do
		if [ -e "/var/lock/plesk_deferred_$flag" ]; then
			local local_res=""
			case $flag in
				watchdog_configure)
					wd_configure && wd_adapt
				;;
				watchdog_reconfigure)
					wd_adapt	
				;;
				ftp_proftpd_configure)
					run_now_ftp_proftpd_configure
				;;
				webserver_apache_configure)
					run_now_webserver_apache_configure
				;;
				awstats_configure)
					run_now_awstats_configure
				;;
				httpd_reconfigure_all)
					httpdmng_reconfigure all
				;;
				httpd_reconfigure)
					httpdmng_reconfigure server
				;;
				relabel_plesk_directories)
					if [ -s "/opt/psa/version.upg" ]; then
						relabel_plesk_directories
					else
						relabel_plesk_directories "$HTTPD_VHOSTS_D" "$PLESK_MAILNAMES_D"
					fi
				;;
				mail_restore_all)
					mail_restore_all
				;;
				mail_spam_restore_full)
					local mail_spam_restore="/usr/lib/plesk-9.0/mail_spam_restore"
					[ ! -x "$mail_spam_restore" ] || "$mail_spam_restore" >>"$product_log" 2>&1
				;;
				courier_mailserver_change_notification)
					run_now_courier_mailserver_change_notification
				;;
				apache_restart)
					set_apache_params
					pleskrc apache restart
				;;
				super_server_action_configure)
					set_super_server_params
					super_server_action configure
				;;
				*)
				;;
			esac

			local temp_res=$?
			if [ -z "$local_res" ]; then
				local_res="$temp_res"
			fi
			if [ "0" = "$local_res" ]; then
				remove_deferred_action_flag $flag
			else
				res=1
				p_echo "Warning : failed to perform deferred action : $flag"
			fi
		fi
	done

	drop_upgrade_flag	
	print_completion_message
	
	return $res
}

remove_deferred_action_flag()
{
	local flag="$1"
	rm -f "/var/lock/plesk_deferred_$flag"

	# check for depending actions:
	case "$flag" in
		httpd_reconfigure_all)
			remove_deferred_action_flag httpd_reconfigure
		;;
		httpd_reconfigure)
			remove_deferred_action_flag apache_restart
		;;
		watchdog_configure)
			remove_deferred_action_flag watchdog_reconfigure
		;;
		*)
		;;
	esac
}

dump_incomplete_deferred_actions()
{
	ls -1 /var/lock/plesk_deferred_* 2>/dev/null || :
}

print_completion_message()
{
	# Congratulation message should be shown after all other actions done.
    if [ -z "$do_repair" ]; then
        define_start_script
        get_config_parameters
        print_congratulations
    fi
}

# vim:ft=sh:
### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
upgrade_pci_compliance_11_1_15()
{
	[ -s "$PRODUCT_ROOT_D/conf/cipher.lst" ] || return 0
	[ ! -e "/etc/sw-cp-server/conf.d/pci-compliance.conf" ] || return 0

	{
		echo -n "ssl_ciphers "
		cat $PRODUCT_ROOT_D/conf/cipher.lst | tr -d '\n'
		echo ";"
		echo "ssl_prefer_server_ciphers on;"
	} > /etc/sw-cp-server/conf.d/pci-compliance.conf
	/etc/init.d/sw-cp-server reload || :
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

set_common_pear_params()
{
	psa_pear_dir="/usr/share/psa-pear"
	psa_pear_conf="/usr/share/psa-pear/pear.conf"
	psa_pear_bin="${psa_pear_dir}/pear/pear"
	psa_pear_include_dir="/usr/share/psa-pear/pear/php"
}

pear_register_packages()
{
	local action=$1
	local pkg_descr_dir=$2

	[ -n "${pkg_descr_dir}" -a -d ${pkg_descr_dir} ] || die "act on PEAR packages - description directory is not specified or does not exist"

	case "${action}" in
		install)
			action=upgrade
		;;
		uninstall)
		;;
		*)
			die "act on PEAR packages - unknown command '${action}'"
		;;
	esac

	for i in ${pkg_descr_dir}/*.xml; do
		local pkg_name
		if [ "${action}" = "upgrade" ]; then
			#on install/upgrade we specify only full path for package xml description, for example /usr/share/psa-horde/.xml/Horde_Data.xml
			pkg_name=${i}
		else
			#on uninstall we should use channel/app-name, for example - horde/Horde_Data
			pkg_name="horde/"`basename "${i}" | cut -d '.' -f 1`
		fi

		if ! ${psa_pear_bin} -c "$psa_pear_conf" "${action}" --offline --nodeps --register-only "${pkg_name}" >> $product_log 2>&1; then
			warn "${action} package ${i} in PEAR"
			return 1
		fi
	done
}

bootstrapper_log_title()
{
	local title="Bootstrapper $product_version $script_action"
	if [ -n "$script_target" ]; then
		title="$title for $script_target"
	fi
	title="$title AT `date`"
	echo "$title"
}

bootstrapper_log_subject()
{
	local subject="Product $script_action"
	if [ -n "$script_target" ]; then 
		subject="$subject for $script_target"
	fi
	echo "$subject"
}

lock_bootstrapper()
{
	lockfile "/var/lock/parallels-panel-bootstrapper-running" || {
		p_echo "Another instance of bootstrapper is already running. Please wait until it finishes."
		p_echo "If you are completely sure there is no other instance running, "
		p_echo "just remove /var/lock/parallels-panel-bootstrapper-running.lock file and retry."
		p_echo "Note: running more than one instance at once may badly damage your system."

		exit 1
	}
}

unlock_bootstrapper()
{
	unlockfile "/var/lock/parallels-panel-bootstrapper-running"
}

bootstrapper_transaction_begin()
{
	lock_bootstrapper
	log_start "`bootstrapper_log_title`"
	set_error_report_context "`bootstrapper_log_title`. `report_context_action`"

	set_common_params
	read_conf

	transaction_begin

	# LIFO rollback order
	transaction_add_rollback_action "unlock_bootstrapper"
	transaction_add_rollback_action "show_report_after_rollback"
	p_echo
	p_echo "**** Product $script_action started."

	# Detect upgrade (most of the scripts depend on it)
	check_product_root
}

bootstrapper_transaction_end()
{
	# FIFO commit order
	transaction_add_commit_action "show_report_after_commit"
	transaction_add_commit_action "unlock_bootstrapper"
	transaction_commit
}

show_report_after_rollback()
{
	# Report fatal error
	log_stop "`bootstrapper_log_title`" "`bootstrapper_log_subject`"

	# Show manual restart recomendation for bootstrapped post-install action only.
	if [ -f /tmp/pp-bootstrapper-mode.flag -a "X$script_action" = "Xpost-install" ]; then
		p_echo "To complete product installation one should run bootstrapper script:"
		p_echo "      $PRODUCT_BOOTSTRAPPER_DIR/bootstrapper.sh repair"
		p_echo
	fi
}

show_report_after_commit()
{
	# show possible warnings in problem report
	log_stop "`bootstrapper_log_title`" "`bootstrapper_log_subject`"
}

# include base and optional bootstrapper actions, define set_component_pkgname_list() (see build.mk)
set_component_pkgname_list()
{
	COMPONENT_PKGNAME_base=plesk-base
	COMPONENT_PKGNAME_core=plesk-core
	COMPONENT_PKGNAME_libpamplesk=libpam-plesk
	COMPONENT_PKGNAME_psavhost=psa-vhost
	COMPONENT_PKGNAME_managementnode=plesk-management-node
	COMPONENT_PKGNAME_l10n=plesk-l10n
	COMPONENT_PKGNAME_horde=psa-horde
	COMPONENT_PKGNAME_imp=psa-imp
	COMPONENT_PKGNAME_ingo=psa-ingo
	COMPONENT_PKGNAME_kronolith=psa-kronolith
	COMPONENT_PKGNAME_turba=psa-turba
	COMPONENT_PKGNAME_mnemo=psa-mnemo
	COMPONENT_PKGNAME_passwd=psa-passwd
	COMPONENT_PKGNAME_roundcube=plesk-roundcube
	COMPONENT_PKGNAME_mailpcdriver=plesk-mail-pc-driver
	COMPONENT_PKGNAME_spammng=psa-spamassassin
	COMPONENT_PKGNAME_courierimap=psa-courier-imap
	COMPONENT_PKGNAME_drwebconfigurator=psa-drweb-configurator
	COMPONENT_PKGNAME_firewall=psa-firewall
	COMPONENT_PKGNAME_agent=plesk-service-node-agent
	COMPONENT_PKGNAME_phpmyadmin=psa-phpmyadmin
	COMPONENT_PKGNAME_dnsbinddriver=plesk-dns-bind-driver
	COMPONENT_PKGNAME_psaupdates=psa-updates
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:syntax=sh

declare_legacy_variables()
{
	echo_try "got legacy variables"
	[ -n "$prod_conf_t" ] || prod_conf_t="/etc/psa/psa.conf"

	if [ ! -s "$prod_conf_t" ]; then
		p_echo "product conf not exists, nothing to be done"
		return
	fi

	local var
	PLESK_LEGACY_VARS="$*"
	for var in $PLESK_LEGACY_VARS; do
		unset $var
		eval $var=`conf_getvar $prod_conf_t $var` # save the value
	done
	suc
}

dump_legacy_variables()
{
	echo_try "save legacy variables"
	[ -n "$prod_conf_t" ] || prod_conf_t="/etc/psa/psa.conf"

	if [ ! -s "$prod_conf_t" ]; then
		p_echo "product conf not exists, cannot dump anything"
		return
	fi

	local var
	local value
	for var in $PLESK_LEGACY_VARS; do
		value="`eval echo \\$${var}`"
 	[ -n "$value" -a -z "`conf_getvar $prod_conf_t $var`" ] || continue # variable is not defined or already in psa.conf
		echo "$var $value" >> $prod_conf_t
	done
	suc
}

base_rollback_product_config()
{
	if [ -n "$prod_conf_t_bak" -a -f "$prod_conf_t_bak" ]; then
		mv -f "$prod_conf_t_bak" "$prod_conf_t"
		unset prod_conf_t_bak
	fi
}

base_commit_product_config()
{
	if [ -n "$prod_conf_t_bak" -a -f "$prod_conf_t_bak" ]; then
		rm -f "$prod_conf_t_bak"
		unset prod_conf_t_bak
	fi
}

base_rollback_product_config_default()
{
	if [ -n "$prod_conf_default_t_bak" -a -f "$prod_conf_default_t_bak" ]; then
		mv -f "$prod_conf_default_t_bak" /etc/psa/psa.conf.default
		unset prod_conf_default_t_bak
	fi
}

base_commit_product_config_default()
{
	if [ -n "$prod_conf_default_t_bak" -a -f "$prod_conf_default_t_bak" ]; then
		rm -f "$prod_conf_default_t_bak"
		unset prod_conf_default_t_bak
	fi
}

base_bootstrapper_prep_install()
{

	# Create users and groups (rollback is not required)
	p_echo "Create user 'psaadm' and group 'psaadm'"
	group_op "psaadm" "search" false
	user_op "psaadm" "search" "psaadm" "psa user" "${PRODUCT_ROOT_D}/admin" "$dummy_shell" false

	p_echo "Create group swkey-data"
	group_op "swkey-data" "search" false
	add_user_to_group "psaadm" "swkey-data"

	set_qmail_params
	set_mail_params

	p_echo "Create Mail accounts"
	group_op "$MAIL_USERGROUP" "search" true
	user_op "$MAIL_USERGROUP" "search" "$MAIL_USERGROUP" "POP3 service user" "$QMAIL_DIR/$MAIL_USERGROUP" "$dummy_shell" true false
	user_op "$HANDLERS_USER" "search" "$MAIL_USERGROUP" "mail handlers user" "$dummy_home" "$dummy_shell" true false

	# Create or update product config
	prod_conf_t="/etc/psa/psa.conf"
	if [ -f "$prod_conf_t" ]; then
		prod_conf_t_bak="${prod_conf_t}.bak"
		cp -pf "$prod_conf_t" "$prod_conf_t_bak"
		transaction_add_rollback_action "base_rollback_product_config"
		transaction_add_commit_action "base_commit_product_config"
	fi

	# For backward compatibility
	[ -d "/etc/psa" ] || mkdir -p /etc/psa
	if [ -f "/etc/psa/psa.conf.default" ]; then
		cp -pf /etc/psa/psa.conf.default /etc/psa/psa.conf.default.bak
		prod_conf_default_t_bak="/etc/psa/psa.conf.default.bak"
		transaction_add_rollback_action "base_rollback_product_config_default"
		transaction_add_commit_action "base_commit_product_config_default"
	fi
	cp -f $PRODUCT_BOOTSTRAPPER_DIR/psa.conf.default /etc/psa/psa.conf.default
	chmod 755 /etc/psa

	declare_legacy_variables QMAIL_MAILNAMES_D CURL_CA_BUNDLE_FILE HTTPD_BIN_D LOGROTATE CGI_PHP_BIN
	# QMAIL_MAILNAMES_D -> PLESK_MAILNAMES_D
	[ ! -s "$prod_conf_t" ] || PLESK_MAILNAMES_D_SAVED="`conf_getvar $prod_conf_t QMAIL_MAILNAMES_D`"
	conf_update $prod_conf_t && p_echo "===> Updated $prod_conf_t"
	dump_legacy_variables
	[ -z "$PLESK_MAILNAMES_D_SAVED" ] || conf_setvar "$prod_conf_t" "PLESK_MAILNAMES_D" "$PLESK_MAILNAMES_D_SAVED"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh:

base_install_common()
{
	inten="generate default ${PRODUCT_NAME} SSL Certificate"
	echo_try $inten
	generate_default_certificate || die
	suc

# work around https://bugzilla.sw.ru/show_bug.cgi?id=435433
    chown root:swkey-data /etc/sw/keys/info /etc/sw/keys/lock/repository /etc/sw/keys/lock/ka
}

base_bootstrapper_post_install()
{
	base_install_common
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:syntax=sh

clean_python_modules()
{
	echo_try "clean up compiled python modules"
	find "/opt/psa/admin/lib/python" -name '*.pyc' -delete
	suc
}

core_groupadd_f()
{
    grep -q "^$1:" /etc/group || /usr/sbin/groupadd "$1"
}

stop_removed_httpsd()
{
	# cannot use rc.d script because it can be removed
	pidfile="$PRODUCT_ROOT_D/admin/logs/httpsd.pid"
	if test -f "$pidfile"; then
		httpsd_pid=`cat "$pidfile"`
		if test "$httpsd_pid" -gt 0 			&& ps -o command -p "$httpsd_pid" | grep -q httpsd
		then
			kill -TERM "$httpsd_pid"
		fi
	fi
}

maintainer_main_pre()
{
	ignore_miss_conf=1
	interactive_allowed=0
	mysql_unconfigured=1

	set_local_params
	set_ftpd_params

	get_config_parameters

	# debian will try upgrade unconfigured package
	# it is not an upgrade, actually
	if [ "X$do_upgrade" = "X1" ]; then
		if [ ! -s ${PRODUCT_ROOT_D}/version ]; then
			do_upgrade=0
		fi
	fi

# Upgrade psa -> plesk-core
	if [ "X$do_upgrade" = "X0" -a -s ${PRODUCT_ROOT_D}/version ]; then
			do_upgrade=1
	fi

	preinstall_checks

	if [ "X$do_upgrade" = "X1" ]; then

		set_mysqld_params

		#bug 137737
		enable_innodb_engine

		if [ -s ${PRODUCT_ROOT_D}/version ]; then
			cp ${PRODUCT_ROOT_D}/version ${PRODUCT_ROOT_D}/version.upg
		fi

	fi

	necessary_system_accounts_prep_install
	core_groupadd_f psacln
	add_login_shell "$PRODUCT_ROOT_D/bin/chrootsh"
}

core_start_product_on_rollback()
{
	start_after_install=1 # for start_product_now
	start_product_now
}

sso_port_change_prep()
{
	# SSO port is changed in sw-sso 2.6, i.e. on upgrade to 10.2.0
	[ "X$do_upgrade" = "X1" ] || return 0
	[ ! -f /etc/sw-cp-server/applications.d/00-sso-cpserver.conf ] || return 0

	# signal to SSO that we upgrade plesk.conf in 10.2.0
	touch /tmp/pp-bootstrapper-upgrade-cpserver-configs.flag

	# signal to Billing the need to perform upgrade actions
	touch /tmp/pp-bootstrapper-change-sso-port.flag

	# actual upgrade for Plesk is done in db_upgrade_sso_port_1010
}

core_bootstrapper_prep_install()
{
	selinux_init
	maintainer_main_pre

	# stop product
	define_start_script
	if [ -f $START_SH ]; then
		# schedule product start on rollback if product is started
		$START_SH status && transaction_add_rollback_action "core_start_product_on_rollback"
		stop_before_upgrade
	else
		stop_removed_httpsd

	fi

	# Perform safe prep-install db actions
	safe_prep_db

	sso_port_change_prep
	selinux_fix_booleans

	selinux_close
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

# mode: shell-script
#-*- vim:syntax=sh

PRODUCT_ROOT_D=/opt/psa
export PRODUCT_ROOT_D

set_relay_opts()
{
	local controls="$PRODUCT_ROOT_D/var/root.controls.lock"
	if [ ! -s "$controls" ]; then
		echo "/server/mail.php3:relay_open" > $controls
	fi
}

clean_backend_cache()
{
	db_do "DELETE FROM BackendCache"
	rm -f "$PRODUCT_ROOT_D/tmp/*.cache"
}

clean_upgrade_info()
{
	db_do "TRUNCATE smb_productUpgrades; TRUNCATE smb_componentUpdates;"
    db_do "DELETE FROM misc WHERE param = 'available_patch'"
}

smb_init_log()
{
	touch $PRODUCT_ROOT_D/admin/logs/panel.log
	chmod 0640 $PRODUCT_ROOT_D/admin/logs/panel.log
	chown psaadm:0 $PRODUCT_ROOT_D/admin/logs/panel.log
}

install_site_isolation_config()
{
	echo_try "install site isolation settings"
	if [ -e "$PRODUCT_ROOT_D/admin/conf/site_isolation_settings.ini" ]; then
		pnnl_echo "config found. Nothing to be done"
		return
	fi
	cp -f $PRODUCT_ROOT_D/admin/conf/site_isolation_settings.ini.default $PRODUCT_ROOT_D/admin/conf/site_isolation_settings.ini
	suc
}

configure_apache_for_filesharing()
{
	add_apache_module "dav"
	add_apache_module "dav_fs"
	add_apache_module "auth_digest"
    add_apache_module "authz_user"
	add_apache_module "dav_lock"
}

vz_disable_admin_apc()
{
	[ "$PLESK_VZ" = "1" ] || return
	local php_ini="$PRODUCT_ROOT_D/admin/conf/php.ini"
	local enabled=`get_phpini_var 'apc.enabled'`

	[ "$enabled" = "0" ] && return
	set_phpini_var 'apc.enabled' '0'
}

package_main()
{
	set_super_server_params
	deferred_super_server_action_configure
	mysql_unconfigured=1
	interactive_allowed=0

	fix_etc_hosts

	set_default_locale
	set_services_info

	selinux_init

	if [ "X$do_upgrade" = "X0" ]; then
		# check for psa-to-core upgrade situation
		check_product_root
	fi

	get_config_parameters

	if [ "X$do_upgrade" = "X1" ]; then
		get_prev_version $PRODUCT_ROOT_D
	fi

	install_rbash

	setup_admin_login
	if [ "X$do_upgrade" = "X0" ]; then
		generate_encryption_key
	fi

	install_cron_somely
	install_admin
	import_default_certificate
	install_chroot_base

	smb_install_db

	# APS controller setup. Should not be after cumultative upgrade. APSC requires INNODB. Should not be before smb_install_db.
	/usr/lib/plesk-9.0/vps_optimized_aspects/mysql-innodb off >> $product_log 2>&1
	install_apsc

	if [ "X$do_upgrade" = "X1" ]; then
		core_run_upgrade_post_stage
		clean_python_modules
	fi

	install_site_isolation_config
	packagemng_set_dirty_flag
	set_relay_opts
	update_components_table	# clear ServiceNodeEnvironment table and update information

	if [ "XRELEASE" = "XCTP" ]; then
		setup_support_url "http://swdn.swsoft.com/en/feedback/ctp"
	fi
	

	label_file_once "${PRODUCT_ETC_D}/webalizer.conf" "${AUTOGENERATED_CONFIGS}"
	# this routine requires 5.0.0 database version, which always come after upgrader
	chown_webstat

	create_version_file
	vz_disable_admin_apc

	if [ "X$do_upgrade" != "X1" ]; then
		core_init_upgrader_conf
# require version file
		db_create_default_templates
		db_update_notifications
# create guid for admin
		db_generate_guids_107
# cert for pbm
		pbm_create_certificate
	fi

	install_cron_backup
	configure_apache_for_filesharing

	clean_backend_cache

	if [ "X$do_upgrade" = "X1" ]; then
		if [ -e "$CATALINA_HOME/conf/server.xml" ]; then
			tomcat_reconfigure_server
        fi
	fi

	smb_init_log
	
	if [ "X$do_upgrade" != "X1" ]; then
		encrypt_psa_shadow
	fi



	# en_US.UTF-8 requred for Backup/Restore
	setup_utf8_locale

	clean_upgrade_info

	service_restart_warning
	selinux_close
}


setup_utf8_locale()
{


	if [ ! -x /usr/bin/locale -o ! -x /usr/sbin/locale-gen ]; then
		pnnl_echo "Warning: unable to allow UTF8 locale - package 'locales' is not installed!"
		return
	fi

	/usr/bin/locale -a | grep -q en_US.utf8
	if [ $? -eq 0 ]; then
		pnnl_echo "UTF8 locale is already allowed"
		return
	fi


	/usr/sbin/locale-gen "en_US.UTF-8"

}


maintainer_main_post()
{

	package_main &&
	register_service psa defaults defaults
	[ "X$do_upgrade" = "X1" ] || register_sw_engine_service
}

create_version_file()
{
    echo ${product_version} ${conceived_os_vendor} ${conceived_os_version} 115131206.11 > ${PRODUCT_ROOT_D}/core.version
}

sso_port_change_post()
{
	[ -f /tmp/pp-bootstrapper-upgrade-cpserver-configs.flag ] && rm -rf /tmp/pp-bootstrapper-upgrade-cpserver-configs.flag
	[ -f /tmp/pp-bootstrapper-change-sso-port.flag ] && rm -f /tmp/pp-bootstrapper-change-sso-port.flag

	# actual upgrade for Plesk is done in db_upgrade_sso_port_1010
}

core_bootstrapper_post_install()
{
	set_local_params
	necessary_system_accounts_post_install
	sso_port_change_post
	maintainer_main_post
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

libpamplesk_bootstrapper_post_install()
{
	install_pam
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#-*- vim:syntax=sh

psavhost_bootstrapper_post_install()
{
	install_vhosts
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
managementnode_create_version_file()
{
	# Make core happy on upgrade
	if [ ! -s "${PRODUCT_ROOT_D}/core.version" -a -s "${PRODUCT_ROOT_D}/version" ]; then
		cp -f "${PRODUCT_ROOT_D}/version" "${PRODUCT_ROOT_D}/core.version"
	fi
	echo 11.5.30 ${conceived_os_vendor} ${conceived_os_version} 115131206.11 > ${PRODUCT_ROOT_D}/version

	rm -f "${PRODUCT_ROOT_D}/version.rpm*"
}

managementnode_bootstrapper_post_install()
{
	start_after_install=1 # for start_product_now_final
	check_product_root
	install_vhosts
	managementnode_create_version_file
	install_default_license_key
	start_product_now_final
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# mode: shell-script
# vim:syntax=sh

set_locales_params()
{
	active_locales_filename='/tmp/psa_active_locales'
}

set_active_locales()
{
	local locales="$1"
	local loc_list=""
	local glue=""

	for i in $locales; do
		loc_list="$loc_list$glue'$i'"
		glue=", "
	done

	db_do "UPDATE locales SET active = 'true' WHERE id IN ($loc_list)"
	db_do "UPDATE locales SET active = 'false' WHERE id NOT IN ($loc_list)"
}

save_active_locales()
{
	db_select "SELECT id FROM locales WHERE active = 'true'"
	if [ -z "$db_select_output" ]; then 
		rm -f "$active_locales_filename"
	else
		echo "$db_select_output" > "$active_locales_filename"
	fi
}

restore_active_locales()
{
	[ -f "$active_locales_filename" ] || return 1

	local locales="`cat $active_locales_filename`"
	rm -f "$active_locales_filename"
	[ -n "$locales" ] || return 1

	set_active_locales "$locales"
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# mode: shell-script
# vim:syntax=sh

l10n_bootstrapper_prep_install()
{
	if [ "X$do_upgrade" = "X1" ]; then
		set_locales_params
		save_active_locales
	fi
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# mode: shell-script
# vim:syntax=sh

l10n_register()
{
	if [ ! -x "$PRODUCT_ROOT_D/admin/sbin/localemng" ]; then
		p_echo "Localemng not found - locales cannot be registered"
		return
	fi

	# We cannot use filepp because of bootstrapper
	for locale in de-DE es-ES fr-FR it-IT ja-JP nl-NL ru-RU zh-CN zh-TW pl-PL pt-BR ko-KR; do
		$PRODUCT_ROOT_D/admin/sbin/localemng --register --locale="$locale"
	done
}

l10n_bootstrapper_post_install()
{
    l10n_register

	if [ "X$do_upgrade" = "X1" ]; then
		set_locales_params
		restore_active_locales || p_echo "Failed to restore active locales"
	fi
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh:

horde_bootstrapper_prep_install()
{
	requires_configured_mysql
}


horde_install_framework()
{
	pear_register_packages install "${horde_datadir}/.xml" || warn 'install Horde5 framework'
}

create_horde_user()
{
	local mysql_db_name="mysql"
	db_do "REPLACE INTO user (host, user, password) VALUES('localhost', 'horde', PASSWORD('$horde_password'))"
	db_do "CREATE DATABASE IF NOT EXISTS horde"
	db_do "REPLACE INTO db (host, db, user, select_priv, insert_priv, update_priv, delete_priv, create_priv, drop_priv, index_priv, alter_priv) 		VALUES ( 			'localhost', 			'horde', 			'horde', 			'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y');"
	db_do "FLUSH PRIVILEGES"

}

horde_upg_convert_passwd_file()
{
	local pass_file
	for pass_file in "/etc/psa/.webmail.shadow" "/etc/psa/webmail/horde/.horde.shadow"; do
		[ -s "$pass_file" ] || continue
		mv -f "$pass_file" $horde_passwd_file
		break
	done
}

horde_upg_remove_vhost_conf()
{
	#we should drop old horde's apache config, cause zzz_horde_vhost.conf includes nor existent directory from old package
	rm -f "${HTTPD_INCLUDE_D}/zzz_horde_vhost.conf"
}

horde_upg_fix_perms_on_private_files()
{
	[ -d "$horde_logdir" ] && set_ac "$horde_user" root 750 "$horde_logdir"
	[ -e "$horde_logdir/psa-horde.log" ] && set_ac "$horde_user" "$horde_group" 640 "$horde_logdir/psa-horde.log"
}

horde_cu_upgrade_only_actions()
{
	[ "$HORDE_CLEAN_INSTALL_FLAG" = "yes" ] && return

	"$@"
}

horde_upgrader()
{
	local install_or_upgrade="cu_check_stage files"
	local upgrade_only="cu_check_stage files horde_cu_upgrade_only_actions"
	local ts="cu_check_timestamp"
	local do="cu_do"

	cu_start_upgrade_from_version 011005023
	$upgrade_only $ts '2013/05/31 12:05:27' $do horde_upg_convert_passwd_file
	$upgrade_only $ts '2013/05/31 12:05:29' $do horde_upg_remove_vhost_conf

	$install_or_upgrade $ts '2013/05/31 12:06:37' $do generate_horde_password
	$install_or_upgrade $ts '2013/05/31 12:06:39' $do create_horde_user

	#1. register application/framework in local PEAR
	$install_or_upgrade $ts '2013/05/31 12:06:37' $do echo_try "register horde5 and its application in local PEAR"
	$install_or_upgrade $ts '2013/05/31 12:06:39' $do horde_install_framework
	$install_or_upgrade $ts '2013/05/31 12:06:41' $do imp_install_application
	$install_or_upgrade $ts '2013/05/31 12:06:43' $do turba_install_application
	$install_or_upgrade $ts '2013/05/31 12:06:45' $do ingo_install_application
	$install_or_upgrade $ts '2013/05/31 12:06:47' $do mnemo_install_application
	$install_or_upgrade $ts '2013/05/31 12:06:50' $do kronolith_install_application
	$install_or_upgrade $ts '2013/05/31 12:06:52' $do passwd_install_application
	$install_or_upgrade $ts '2013/05/31 12:06:54' $do suc

	#2. upgrade framework/application DB to latest H3 (actual only for upgrade from PPP < 11.5.23)
	$upgrade_only $ts '2013/05/31 12:13:37' $do echo_try "upgrade horde <= 3 to latest 3"
	$upgrade_only $ts '2013/05/31 12:13:39' $do upgrade_horde_DB_to_latest_h3
	$upgrade_only $ts '2013/05/31 12:13:41' $do upgrade_imp_DB_to_latest_h3
	$upgrade_only $ts '2013/05/31 12:13:45' $do upgrade_turba_DB_to_latest_h3
	$upgrade_only $ts '2013/05/31 12:13:53' $do upgrade_ingo_DB_to_latest_h3
	$upgrade_only $ts '2013/05/31 12:14:17' $do upgrade_mnemo_DB_to_latest_h3
	$upgrade_only $ts '2013/05/31 12:14:23' $do upgrade_kronolith_DB_to_latest_h3
	$upgrade_only $ts '2013/05/31 12:14:25' $do suc

	#3. run migrations scripts for framework/application
	$install_or_upgrade $ts '2013/05/31 12:28:34' $do echo_try "execute horde5 upgrade scripts"
	$install_or_upgrade $ts '2013/05/31 12:28:35' $do horde_run_migration_script
	$install_or_upgrade $ts '2013/05/31 12:28:36' $do horde_run_migration_script imp
	$install_or_upgrade $ts '2013/05/31 12:28:38' $do horde_run_migration_script turba
	$install_or_upgrade $ts '2013/05/31 12:28:39' $do horde_run_migration_script ingo
	$install_or_upgrade $ts '2013/05/31 12:28:41' $do horde_run_migration_script mnemo
	$install_or_upgrade $ts '2013/05/31 12:28:43' $do horde_run_migration_script kronolith
	$install_or_upgrade $ts '2013/05/31 12:28:46' $do suc

	# 4.
	$upgrade_only $ts '2013/05/31 12:30:44' $do echo_try "convert datatree shares to SQL engine"
	$upgrade_only $ts '2013/05/31 12:30:46' $do horde_convert_shares_to_sql kronolith
	$upgrade_only $ts '2013/05/31 12:30:47' $do horde_convert_shares_to_sql mnemo
	$upgrade_only $ts '2013/05/31 12:30:49' $do horde_convert_shares_to_sql turba
	$upgrade_only $ts '2013/05/31 12:30:51' $do suc

	# 5. Run horde via fastcgi
	cu_start_upgrade_from_version 011005027
	# permissions for $horde_passwd_file are fixed earlier in generate_horde_password.

	$upgrade_only $ts '2013/06/07 11:53:28' $do horde_upg_fix_perms_on_private_files
	$upgrade_only $ts '2013/05/31 12:56:49' $do deferred_httpd_reconfigure

        # 6. Upgrade Horde from 5.0.4 to 5.1.4 during MU15 installation
	$install_or_upgrade $ts '2013/09/13 12:06:37' $do echo_try "register horde5.1.4 and its application in local PEAR"
	$install_or_upgrade $ts '2013/09/13 12:06:39' $do horde_install_framework
	$install_or_upgrade $ts '2013/09/13 12:06:41' $do imp_install_application
	$install_or_upgrade $ts '2013/09/13 12:06:43' $do turba_install_application
	$install_or_upgrade $ts '2013/09/13 12:06:45' $do ingo_install_application
	$install_or_upgrade $ts '2013/09/13 12:06:47' $do mnemo_install_application
	$install_or_upgrade $ts '2013/09/13 12:06:50' $do kronolith_install_application
	$install_or_upgrade $ts '2013/09/13 12:06:52' $do passwd_install_application
	$install_or_upgrade $ts '2013/09/13 12:06:54' $do suc

	$install_or_upgrade $ts '2013/09/13 12:28:34' $do echo_try "execute horde5.1.4 upgrade scripts"
	$install_or_upgrade $ts '2013/09/13 12:28:35' $do horde_run_migration_script
	$install_or_upgrade $ts '2013/09/13 12:28:36' $do horde_run_migration_script imp
	$install_or_upgrade $ts '2013/09/13 12:28:38' $do horde_run_migration_script turba
	$install_or_upgrade $ts '2013/09/13 12:28:39' $do horde_run_migration_script ingo
	$install_or_upgrade $ts '2013/09/13 12:28:41' $do horde_run_migration_script mnemo
	$install_or_upgrade $ts '2013/09/13 12:28:43' $do horde_run_migration_script kronolith
	$install_or_upgrade $ts '2013/09/13 12:28:46' $do suc

	# WARNING: all $install_or_upgrade actions will be executed on clean installation as well!
}

horde_run_upgrade()
{
	local optional_repair_arg=
	[ -z "$1" -a -z "$do_repair" ] || optional_repair_arg="--repair"

	local inten="upgrade${optional_repair_arg:+ and repair} Horde webmail configuration"
	echo_try "$inten"
	cu_run_upgrader "horde" horde_upgrader --stage "files" $optional_repair_arg --update-version && suc || warn "$inten"
}

horde_run_repair()
{
	set_apache_params
	set_horde_params
	set_common_pear_params
	local mysql_db_name="horde"

	horde_run_upgrade repair
}

horde_bootstrapper_post_install()
{
	uses_mysql_client

	set_webmail_params
	set_apache_params
	set_horde_params
	set_common_pear_params

	local mysql_db_name="horde"
	
	local HORDE_CLEAN_INSTALL_FLAG=""
	db_select "SHOW TABLES"
	[ -z "${db_select_output}" ] && HORDE_CLEAN_INSTALL_FLAG="yes"

	horde_run_upgrade

	register_webmail horde

	#TODO: should we clean cache?

	generate_horde_php_ini
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#used by horde's bootstrapper
imp_install_application()
{
	pear_register_packages install "${imp_datadir}/.xml" || warn 'install H5-application imp'
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#used by horde's bootstrapper
ingo_install_application()
{
	pear_register_packages install "${ingo_datadir}/.xml" || warn 'install H5-application ingo'
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#used by horde's bootstrapper
kronolith_install_application()
{	
	pear_register_packages install "${kronolith_datadir}/.xml" || warn 'install H5-application kronolith'
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#used by horde's bootstrapper
turba_install_application()
{
	pear_register_packages install "${turba_datadir}/.xml" || warn 'install H5-application turba'
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#used by horde's bootstrapper
mnemo_install_application()
{
	pear_register_packages install "${mnemo_datadir}/.xml" || warn 'install H5-application mnemo'
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#used by horde's bootstrapper
passwd_install_application()
{
        pear_register_packages install "${passwd_datadir}/.xml" || warn 'install H5-application passwd'
}

roundcube_bootstrapper_prep_install()
{
	requires_configured_mysql
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

roundcube_run_native_upgrader()
{
	# It should be safe to run native Roundcube upgrade script each time a package is updated
	# The script will update main configuration files and DB to the current version

	local upgrade_script="$roundcube_home/bin/update.sh"

	local old_version="`cat $roundcube_preupgrade_version_file 2>/dev/null`"
	local new_version="`roundcube_get_current_files_version`"
	if [ -z "$old_version" ]; then
		p_echo "Upgrade of $roundcube_title configs and DB is not required - no previous version"
		return 0
	fi

	if [ ! -x "$upgrade_script" ]; then
		p_echo "Upgrade of $roundcube_title configs and DB is not possible: '$upgrade_script' is not executable"
		return 1
	fi

	local inten="upgrade $roundcube_title configs and DB from '$old_version' to '$new_version'"
	echo_try "$inten"
	if yes 2>/dev/null | "$upgrade_script" "--version=$old_version" >>"$product_log" 2>&1 ; then
		suc
		rm -f "$roundcube_preupgrade_version_file"
		return 0
	else
		warn "$inten. Run $upgrade_script manually to complete upgrade."
		return 1
	fi
}

roundcube_upgrader()
{
	local files="cu_check_stage files"
	local ts="cu_check_timestamp"
	local do="cu_do"

	cu_start_upgrade_from_version 011005027
	$files $ts '2013/06/18 15:08:09' $do roundcube_fix_perms_on_private_files
	$files $ts '2013/06/18 15:09:09' $do deferred_httpd_reconfigure

	cu_start_upgrade_from_version 011006002	# Upgrade Roundcube from 0.8.6 to 0.9.4
	$files $ts '2013/10/08 11:31:58' $do roundcube_fix_db_grants
	$files $ts '2013/10/08 11:32:00' $do deferred_httpd_reconfigure

	call_optional_function dev_roundcube_upgrader
}

roundcube_run_upgrade()
{
	local optional_repair_arg=
	[ -z "$1" -a -z "$do_repair" ] || optional_repair_arg="--repair"

	local inten="upgrade${optional_repair_arg:+ and repair} Roundcube webmail configuration"
	echo_try "$inten"
	cu_run_upgrader "roundcube" roundcube_upgrader --stage "files" $optional_repair_arg --update-version && suc || warn "$inten"
}

roundcube_run_repair()
{
	set_apache_params
	set_roundcube_params

	roundcube_run_upgrade repair
	roundcube_run_native_upgrader
}

roundcube_bootstrapper_post_install()
{
	set_webmail_params
	set_roundcube_params

	roundcube_run_native_upgrader
	register_webmail roundcube
	roundcube_generate_php_ini
}

init_postfix_on_sn_hack()
{
	# As the name suggests, this is essentially a hack for SN installation/upgrade 
	# and as such should be removed when possible (e.g. mail_restore for SN is implemented).
	local gl_data_db="/var/lib/plesk/mail/greylist/data.db"
	local passwd_db="/var/spool/postfix/plesk/passwd.db"

	if ! is_management_node_installed; then
		[ -f "$passwd_db"  ] || "$PLESK_LIBEXEC_DIR/mailsrv_conf_init"
		[ -f "$gl_data_db" ] || "$PLESK_LIBEXEC_DIR/mail_grey_restore"
	fi
}

mailpcdriver_bootstrapper_post_install()
{
	# See bugs #53084 and #69422.
	[ -x "${PRODUCT_RC_D}/postfix" -a ! -f "${PRODUCT_RC_D}/qmail" ] || return 0
	set_postfix_params
	init_postfix_on_sn_hack
	set_postfix_mta
#	set_postfix_auth
	start_postfix "bootstrapper-post"
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
#-*- vim:ft=sh:

set_spammng_params()
{
	old_plesk_spamassassin_service='psa-spamassassin'
	old_spamd_pid="/var/run/spamd/spamd_full.pid"

	spamassassin_service='spamassassin'
}

spammng_enable_services()
{
	local service
	for service in spamassassin spamass-milter spamas-milter; do
		[ -f $SYSTEM_RC_D/$service ] || continue
		if [ -x "/usr/sbin/dpkg-divert" ]; then
			[ -n "`/usr/sbin/dpkg-divert --list $SYSTEM_RC_D/$service`" ] || continue
			rm -f $SYSTEM_RC_D/$service
			dpkg-divert --package psa-spamassassin --remove --rename $SYSTEM_RC_D/$service
		else
			chmod +x $SYSTEM_RC_D/$service
			register_service $service
		fi
	done
}

spammng_cleanup_old_psa_spamassassin_service()
{
	# the service was already stopped and deregistered in preinstall, now just cleanup potential leftovers (config files)
	rm -rf "/etc/psa-spamassassin"
    # Fake init script for fixing bug 132781; it is removed in bootstrapper_post_install.
    create_spamassasin_fake_init_script "$SYSTEM_RC_D/$old_plesk_spamassassin_service" 
	if [ -n "$old_plesk_spamassassin_unit" -a -f "$old_plesk_spamassassin_unit" ]; then
		rm -f "$old_plesk_spamassassin_unit"
	fi
	:
}

spamassassin_upgrader()
{
	local files="cu_check_stage files"
	local ts="cu_check_timestamp"
	local do="cu_do"

	$files $ts '2013/02/21 15:14:41' $do conf_rmvar "${prod_conf_t}" SPAMASSASSIN_MAX_CHILDREN

	cu_start_upgrade_from_version 011005019
	$files $ts '2013/02/21 15:15:00' $do spammng_cleanup_old_psa_spamassassin_service
	$files $ts '2013/02/21 15:15:07' $do spammng_enable_services
	$files $ts '2013/02/21 15:42:41' $do deferred_mail_spam_restore_full
}

create_spamassasin_fake_init_script()
{
    echo "#!/bin/sh" > $1
    echo "exit 0;" >> $1
    chmod 755 $1
}

spamassassin_run_upgrade()
{
	local optional_repair_arg=
	[ -z "$1" -a -z "$do_repair" ] || optional_repair_arg="--repair"

	local inten="upgrade${optional_repair_arg:+ and repair} SpamAssassin service configuration"
	echo_try "$inten"
	cu_run_upgrader "spamassassin" spamassassin_upgrader --stage "files" $optional_repair_arg --update-version && suc || warn "$inten"
}

spamassassin_run_repair()
{
	set_admin_params
	set_spammng_params
	set_mail_params

	spamassassin_run_upgrade repair
}

spammng_bootstrapper_post_install()
{
	set_spammng_params
    rm -f "$SYSTEM_RC_D/$old_plesk_spamassassin_service" 
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

#-*- vim:ft=sh

stop_and_unregister_courier_imap_lt_4()
{
	# stops old Courier-IMAP services (<= 3.0.8)
	pleskrc old_courier stop
	unregister_service "$old_courier_service" >>"$product_log" 2>&1
	:
}

cleanup_configuration_courier_imap_lt_4()
{
	# Old init script may not be removed during package upgrade on deb-based systems
	rm -f "$SYSTEM_RC_D/$old_courier_service" >>"$product_log" 2>&1
	# Remove any old .dist files so they are replaced for sure
	for conf in imapd imapd-ssl pop3d pop3d-ssl; do
		rm -f "$COURIERIMAP_CONFDIR/$conf.dist" >>"$product_log" 2>&1
	done
	# Move out files used by mkimapdcert (just a precaution)
	for conf in imapd.cnf pop3d.cnf; do
		mv -f "$COURIERIMAP_CONFDIR/$conf" "$COURIERIMAP_CONFDIR/$conf.saved_by_plesk" >>"$product_log" 2>&1
	done
	:
}

courier_imap_upgrade_certs()
{
	courier_imap_generate_certs
	:
}

courier_imap_sysconftool_upgrade()
{
	local sysconftool="/usr/lib/courier-authlib/sysconftool"
	local tool_ver="0.16"
	local inten="upgrade Courier-IMAP services configuration files via sysconftool"
	echo_try "$inten"; p_echo
	$sysconftool -r "$tool_ver" 		"$COURIERIMAP_CONFDIR/imapd.dist" "$COURIERIMAP_CONFDIR/imapd-ssl.dist" 		"$COURIERIMAP_CONFDIR/pop3d.dist" "$COURIERIMAP_CONFDIR/pop3d-ssl.dist" >>"$product_log" 2>&1 && suc
}

courier_imap_disable_pci_compliance_for_upgrade()
{
	local courier_pci_comp=
	{
		# Consider PCI compliance enabled if at least one file matching these globs exists
		ls $COURIERIMAP_CONFDIR/imapd.pci* || ls $COURIERIMAP_CONFDIR/imapd-ssl.pci* || 		ls $COURIERIMAP_CONFDIR/pop3d.pci* || ls $COURIERIMAP_CONFDIR/pop3d-ssl.pci* 
	} 1>/dev/null 2>&1 && 		courier_pci_comp="on"

	[ "$courier_pci_comp" = "on" -a -x "$PRODUCT_ROOT_D/admin/bin/pci_compliance_resolver" ] || return 0

	echo_try "disable PCI compliance for Courier-IMAP before upgrade"
	touch "$COURIERIMAP_CONFDIR/plesk-pci-compliance-was-enabled.flag"
	"$PRODUCT_ROOT_D/admin/bin/pci_compliance_resolver" --disable courier >>"$product_log" 2>&1 && suc || p_echo "failed"
	# This action is always successfull. Otherwise we would have to be sure courier_imap_reenable_pci_compliance() would fail too.
}

courier_imap_reenable_pci_compliance()
{
	[ -f "$COURIERIMAP_CONFDIR/plesk-pci-compliance-was-enabled.flag" ] || return 0

	local inten="reenable PCI compliance for Courier-IMAP after upgrade"
	echo_try "$inten"
	rm -f "$COURIERIMAP_CONFDIR/plesk-pci-compliance-was-enabled.flag"
	"$PRODUCT_ROOT_D/admin/bin/pci_compliance_resolver" --enable courier >>"$product_log" 2>&1 && suc || warn "$inten"
	# This action is always successfull. Reenabling already enabled courier will produce utility error.
}

courier_imap_upgrader()
{
	local prep="cu_check_stage prep"
	local post="cu_check_stage post"
	local bpos="cu_check_stage bootstrapper-post"
	local ts="cu_check_timestamp"
	local do="cu_do"

	# Upgrade from 3.0.8 to 4.12.0
	$prep $ts '2013/05/07 15:17:03' $do courier_imap_disable_pci_compliance_for_upgrade # should be in bootstrapper, but will work this way too
	$prep $ts '2013/05/06 16:03:58' $do stop_and_unregister_courier_imap_lt_4
	$prep $ts '2013/05/06 16:25:32' $do courier_imap_upgrade_certs
	$prep $ts '2013/05/22 15:28:58' $do cleanup_configuration_courier_imap_lt_4
	$post $ts '2013/05/06 17:33:42' $do courier_imap_sysconftool_upgrade
	$post $ts '2013/05/13 12:12:13' $do courier_imap_services_register
	$bpos $ts '2013/05/07 15:17:41' $do courier_imap_reenable_pci_compliance
	$post $ts '2013/05/07 15:30:49' $do deferred_mail_restore_all
	$bpos $ts '2013/05/13 17:57:44' $do courier_imap_finish_upgrade_to_4
# Note, when we upgrade courier from 3.0.8, old courier removes pid files. So we should restart
# courier in scope of bootstrapper *ONLY*, or courier initscripts will hangs on next stop, after transaction ends.
# Most probably, we can restart courier anywhere when upgrade from 9.5.x will be removed
	$bpos $ts '2013/05/14 15:39:23' $do courier_imap_services_action restart
}

courier_imap_run_upgrade()
{
	local stage="$1"
	[ "$stage" = "prep" -o "$stage" = "post" -o "$stage" = "bootstrapper-post" ] || die "courier_imap_run_upgrade(): upgrade stage is not specified or invalid"

	local optional_repair_arg=
	[ -z "$2" -a -z "$do_repair" ] || optional_repair_arg="--repair"

	local update_version_arg=
	[ "$stage" = "bootstrapper-post" ] && update_version_arg="--update-version"

	local inten="upgrade${optional_repair_arg:+ and repair} Courier-IMAP services configuration ($stage stage)"
	echo_try "$inten"
	cu_run_upgrader "courier_imap" courier_imap_upgrader --stage "$stage" $optional_repair_arg $update_version_arg && suc || warn "$inten"
}

courier_imap_run_repair()
{
	set_courier_imap_params

	courier_imap_run_upgrade prep repair
	courier_imap_run_upgrade post repair
	courier_imap_run_upgrade bootstrapper-post repair

	courier_imap_services_action condrestart
}

courier_imap_services_action()
{
	local action="$1"
	pleskrc courier_imapd "$action"
	pleskrc courier_imaps "$action"
	pleskrc courier_pop3d "$action"
	pleskrc courier_pop3s "$action"
}

courier_imap_services_register()
{
	register_service "$courier_imapd_service"
	register_service "$courier_imaps_service"
	register_service "$courier_pop3d_service"
	register_service "$courier_pop3s_service"
}

courier_imap_finish_upgrade_to_4()
{
	rm -f "$COURIERIMAP_CONFDIR/plesk-courier-imap-3.0.8-upgrade.flag" >>"$product_log" 2>&1
}

courierimap_bootstrapper_post_install()
{
	set_courier_imap_params

	courier_imap_run_upgrade bootstrapper-post
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh:

drweb_force_installing_new_drweb32ini()
{
	# New DrWeb 6.0.2 packages will install new drweb32.ini only if it doesn't exist yet
	mv -f "$DRWEB_ETC_D/drweb32.ini" "$DRWEB_ETC_D/drweb32.ini.old_drweb5" || :
}

drweb_reinstall_bases()
{
	# Old drweb-bases 5.0.1 packages execute 'rm -f /var/drweb/bases/*.vdb /var/drweb/updates/*'
	# unconditionally in pre-uninstall (i.e., even on upgrade). Therefore we need to replicate 
	# bases installation. The code is taken as-is from drweb-bases post-install.
	(
		if [ ! -f /var/drweb/updates/timestamp ] ; then
			cd /var/drweb/updates && 			gzip -dc /opt/drweb/bases/bases.tar.gz | tar -xpf - && 			rm -f /var/drweb/bases/*.vdb /var/drweb/lib/drweb32.dll && 			cp -p -f /var/drweb/updates/*.vdb /var/drweb/bases/ && 			cp -p -f /var/drweb/updates/drweb32.dll /var/drweb/lib/ && 			chown drweb /var/drweb/bases/*.vdb && 			chgrp drweb /var/drweb/bases/*.vdb && 			chown drweb /var/drweb/lib/drweb32.dll && 			chgrp drweb /var/drweb/lib/drweb32.dll && 			rm -f /var/drweb/updates/*.vdb /var/drweb/updates/drweb32.dll
		fi
	) >>"$product_log" 2>&1
}

drweb_restart_service()
{
	# Since DrWeb may be dead after upgrade to 6.0.2, we need to restart it
	pleskrc drweb restart || :
}

drweb_upgrader()
{
	local bprep="cu_check_stage bootstrapper-prep"
	local bpost="cu_check_stage bootstrapper-post"
	local ts="cu_check_timestamp"
	local do="cu_do"

	# Upgrade from 5.0.1 to 6.0.2
	$bprep $ts '2013/07/09 12:30:16' $do drweb_force_installing_new_drweb32ini
	$bpost $ts '2013/07/09 12:30:32' $do drweb_reinstall_bases
	$bpost $ts '2013/07/09 12:30:48' $do selinux_relabel_dir "/var/drweb" "$DRWEB_ROOT_D"
	$bpost $ts '2013/07/09 12:31:00' $do drweb_restart_service
}

drweb_run_upgrade()
{
	local stage="$1"
	[ "$stage" = "bootstrapper-prep" -o "$stage" = "bootstrapper-post" ] || die "drweb_run_upgrade(): upgrade stage is not specified or invalid"

	local optional_repair_arg=
	[ -z "$2" -a -z "$do_repair" ] || optional_repair_arg="--repair"

	local update_version_arg=
	[ "$stage" = "bootstrapper-post" ] && update_version_arg="--update-version"

	local inten="upgrade${optional_repair_arg:+ and repair} Parallels Premium antivirus (DrWeb) service configuration ($stage stage)"
	echo_try "$inten"
	cu_run_upgrader "drweb" drweb_upgrader --stage "$stage" $optional_repair_arg $update_version_arg && suc || warn "$inten"
}

drweb_run_repair()
{
	set_drweb_params

	drweb_run_upgrade bootstrapper-prep repair
	drweb_run_upgrade bootstrapper-post repair
}

drwebconfigurator_bootstrapper_prep_install()
{
	set_drweb_params

	[ -d "$DRWEB_ETC_D" ] || return 0
	drweb_run_upgrade bootstrapper-prep
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
#-*- vim:ft=sh:

config_drweb_key_expiration_notifications()
{
	local inten="Turn key expiration notifications for 2 days in drweb config files"
	echo_try "$inten"

	# Add New parameters for DrWeb upgrade

	add_string '[[:space:]]*NotifyPeriod[[:space:]]*=.*' 		"NotifyPeriod = 2" "$DRWEB_ETC_D/drweb32.ini"

	add_string '[[:space:]]*NotifyType[[:space:]]*=.*' 		"NotifyType = Everyday" "$DRWEB_ETC_D/drweb32.ini"

	# Change Default values for clear Drweb installation
	NotifyPeriod=2
	NotifyType=Everyday

	sed -e 's/^[[:space:]]*NotifyPeriod[[:space:]]*=[[:space:]]*14/NotifyPeriod = '$NotifyPeriod'/g' 		-e 's/^[[:space:]]*NotifyType[[:space:]]*=[[:space:]]*Ever[[:space:]]*$/NotifyType = '$NotifyType'/g' 	< $DRWEB_ETC_D/drweb32.ini > $DRWEB_ETC_D/drweb32.ini.tmp && 	mv -f $DRWEB_ETC_D/drweb32.ini.tmp $DRWEB_ETC_D/drweb32.ini || die "$inten"

	set_ac drweb drweb 644 $DRWEB_ETC_D/drweb32.ini || die "$inten"		

}

create_update_cron_task()
{
	# creates daily task for update DrWeb databases

	local task_file="/etc/cron.daily/drweb-update"

	echo_try "create daily cron task for drweb bases update"

	if [ ! -f "${task_file}" ]; then
	cat <<EOF > "${task_file}"
#!/bin/sh
exec /opt/drweb/update.pl
EOF
	set_ac 0 0 755 "${task_file}"
	fi

	suc
}

drweb_actions_for_install_or_upgrade()
{
	# FIXME: move some of these actions to drweb_upgrader since executing them one time should be enough
	config_drweb_key_expiration_notifications

	# register drweb handler for existing mailnames
	/usr/lib/plesk-9.0/mail_drweb_restore >> "$product_log" 2>&1
	qmail_admin_alias "$drweb_user"

	deferred_watchdog_reconfigure
	create_update_cron_task

	selinux_relabel_dir /opt/psa/handlers
}

drwebconfigurator_bootstrapper_post_install()
{
	set_drweb_params

	drweb_actions_for_install_or_upgrade

	drweb_run_upgrade bootstrapper-post
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh:

set_module_params_firewall()
{
	module="firewall"
	module_full="Firewall"

	module_version="1.0.2"

	iptables_bin_d=/sbin
	
	iptables_bin="$iptables_bin_d/iptables"
	ip6tables_bin="$iptables_bin_d/ip6tables"
    
	set_module_params
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh:

firewall_has_state_match()
{
	local iptables="$1"
	local safeact="$PRODUCT_ROOT_D/admin/sbin/modules/$module/safeact"

	[ -x "$iptables" -a -x "$safeact" ] || return 1
	$safeact --try-enable-features --iptables="$iptables" --rule="-m state --state NEW,ESTABLISHED,RELATED,INVALID" >/dev/null 2>&1
}

firewall_update_default_input_policy()
{
	# Sometimes state match is not available in iptables (e.g., VZ). In this case INPUT should ACCEPT connections by default.
	local default_policy="deny"
	if [ -x "$iptables_bin" ]; then
		firewall_has_state_match "$iptables_bin" || default_policy="allow"
	fi
	if [ -x "$ip6tables_bin" ]; then
		firewall_has_state_match "$ip6tables_bin" || default_policy="allow"
	fi

	# Previously default policy was always 'deny'
	[ "$default_policy" != "deny" ] || return 0
	echo_try "switch default firewall INPUT policy to ACCEPT since 'state' match is not available"

	local new_rule='a:4:{s:4:"type";s:8:"catchall";s:5:"class";s:8:"catchall";s:9:"direction";s:5:"input";s:6:"action";s:5:"allow";}'
	local old_rule_pat='a:%:{s:4:"type";s:8:"catchall";s:5:"class";s:8:"catchall";s:9:"direction";s:5:"input";s:6:"action";s:4:"deny";%'
	local input_dir="0"
	db_do "UPDATE module_${module}_rules SET object='$new_rule' WHERE object LIKE '$old_rule_pat' AND direction=$input_dir" && suc
}

firewall_read_object()
{
	db_select "SELECT object FROM module_firewall_objects WHERE tag = '$1'"
	echo "$db_select_output"
}

firewall_cu_load_config_data()
{
	local upgrader_name="$1"
	local table="module_${module}_objects"

	[ "$upgrader_name" = "firewall" ] || return 0
	db_test_table "$table" || return 0

	cu_db_current_timestamp="`  firewall_read_object 'upgrader_config_current_timestamp'`"
	cu_db_continue_from="`      firewall_read_object 'upgrader_config_continue_from'`"
	cu_db_failed_actions_list="`firewall_read_object 'upgrader_config_failed_actions_list'`"
	cu_db_latest_timestamp="`   firewall_read_object 'upgrader_config_latest_timestamp'`"
}

firewall_cu_store_config_data()
{
	local upgrader_name="$1"
	local table="module_${module}_objects"

	[ "$upgrader_name" = "firewall" ] || return 0
	db_test_table "$table" || die "save $module_full upgrader state to '$table' table"

	db_do "REPLACE INTO $table (tag, object) VALUES 		('upgrader_config_current_timestamp', '$cu_db_current_timestamp'), 		('upgrader_config_continue_from', '$cu_db_continue_from'), 		('upgrader_config_failed_actions_list', '$cu_db_failed_actions_list'), 		('upgrader_config_latest_timestamp', '$cu_db_latest_timestamp')"
}

firewall_upgrader()
{
	local post="cu_check_stage post"
	local ts="cu_check_timestamp"
	local do="cu_do"

	# Upgrade to Plesk 11.5.30, Firewall 1.0.2
	$post $ts '2013/07/15 16:12:16' $do firewall_update_default_input_policy
}

firewall_run_upgrade()
{
	local optional_repair_arg=
	[ -z "$1" -a -z "$do_repair" ] || optional_repair_arg="--repair"

	local inten="upgrade${optional_repair_arg:+ and repair} $module_full service configuration"
	echo_try "$inten"
	cu_run_upgrader "firewall" firewall_upgrader 		--load-config-data firewall_cu_load_config_data --store-config-data firewall_cu_store_config_data 		--stage "post" $optional_repair_arg --update-version && suc || warn "$inten"
}

firewall_run_repair()
{
	set_module_params_firewall

	firewall_run_upgrade repair
}


### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:syntax=sh

agent_bootstrapper_prep_install()
{
	set_admin_params
	necessary_system_accounts_prep_install_psaadm
	necessary_system_accounts_prep_install_psaserv
}

agent_bootstrapper_post_install()
{
	set_admin_params
	necessary_system_accounts_post_install

	# if we're not in bootstrapper context or 'core' is not installed
	if ! is_function is_component_installed || ! is_component_installed core ; then
		set_swcpserver_params
		pleskrc swcpserver restart
	fi
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

phpmyadmin_bootstrapper_post_install()
{
	install_phpmyadmin
}

dnsbinddriver_bootstrapper_post_install()
{
	uses_mysql_client

	echo_try "create reverse DNS zone"

	local psa_configured
	psa_configured=`core_read_misc_attr psa_configured`
	[ $? -eq 0 ] || die 'detect that panel is configured'

	if [ "true" = "${psa_configured}" ]; then
		suc
		return
	fi

	local util_name="${PRODUCT_ROOT_D}/admin/sbin/mk_reverse_zone"
	if [ -x "$util_name" ]; then
		if ! "${util_name}" >> "$product_log" 2>&1; then
			warn "create reverse DNS zone - there is error during utility '${util_name}' execution, please fix the problem and run it again."
		else 
			suc
		fi
	else
		die "locate utility '${util_name}'"
	fi
}

### Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.
# vim:ft=sh:

psaupdates_run_repair()
{
	uses_mysql_client
	apsc_fix_db_credentials
}

need_disable_skipbdb_mysql()
{
    local my_cnf="$1"

    local top_version=50
    local version=0

    [ -f $my_cnf ] || return 1

    if which rpm >/dev/null 2>&1; then
        if rpm -qi mysql-server 1>/dev/null 2>&1; then
            version="`rpm -q --qf '%{VERSION}' mysql-server | awk -F '.' '{print $1$2}'`"
        fi
        if rpm -qi mariadb 1>/dev/null 2>&1; then
            version="`rpm -q --qf '%{VERSION}' mariadb | awk -F '.' '{print $1$2}'`"
        fi
        if rpm -qi mysql-community-server 1>/dev/null 2>&1; then
            version="`rpm -q --qf '%{VERSION}' mysql-community-server | awk -F '.' '{print $1$2}'`"
        fi
    fi

    if which dpkg >/dev/null 2>&1; then
        installed="`dpkg --get-selections mysql-common 2>/dev/null | awk '{if ($2 == \"install\") {print $1}}'`"
        if [ -n "$installed" ]; then
            version="`dpkg-query -W -f='${Version}' mysql-common | awk -F '.' '{print $1$2}'`"
        fi
    fi

    str="`awk '/^[[:space:]]*skip-bdb/ {print}' < $my_cnf`"

    [ "$version" -gt "$top_version" -a -n "$str" ] && return 0

    return 1
}

need_disable_grant_tables_mysql()
{
    local my_cnf="$1"

    [ -f "$my_cnf" ] || return 1

    str="`awk '/^[[:space:]]*skip-grant-tables/ {print}' < $my_cnf`"

    [ -z "$str" ] && return 1

    return 0
}

update_plesk_completion_rules()
{
	local update_plesk_completion_bin="$PRODUCT_ROOT_D/admin/sbin/update_plesk_completion"
	[ -x "$update_plesk_completion_bin" ] || return 0
	# the utility is quite noisy, filter out useless messages
	"$update_plesk_completion_bin" 2>&1 | grep -v '^Usage:' >>$product_log
}

component_base_prep_install()
{
        local my_cnf
	local need_mysql_restart=0
	find_my_cnf "non_fatal"

        # Disable skip-bdb option in my.cnf for mysql >= 5.1
        cp -f ${my_cnf} ${my_cnf}.prep
        if need_disable_skipbdb_mysql $my_cnf; then
                echo_try "comment out skip-bdb option in ${my_cnf}"
                awk '/^[[:space:]]*skip-bdb[[:space:]]*(#.*)?$/ {
                        print "# skip-bdb option is turned OFF by Plesk";
                        print "#"$0;
                        next;
                    }{print;}' < $my_cnf > ${my_cnf}.new
                mv -f ${my_cnf}.new $my_cnf
		need_mysql_restart=1
                suc
        fi

        # Disable skip-grant-tables option in my.cnf
	if need_disable_grant_tables_mysql $my_cnf; then
                echo_try "comment out skip-grant-tables option in ${my_cnf}"
		awk '/^[[:space:]]*skip-grant-tables[[:space:]]*(#.*)?$/ {
                        print "# skip-grant-tables option is turned OFF by Plesk";
                        print "#"$0;
                        next;
                    }{print;}' < $my_cnf > ${my_cnf}.new
		mv -f ${my_cnf}.new $my_cnf
		need_mysql_restart=1
                suc
	fi

	if [ $need_mysql_restart -eq 1 -a "X$do_upgrade" = "X1" ]; then
		set_mysqld_params
		pleskrc mysql restart
	fi




	base_bootstrapper_prep_install
	core_bootstrapper_prep_install
	l10n_bootstrapper_prep_install
	if [ "X$PREP_INSTALL_ROLLBACK_CHECK" = "XYes" ]; then
		simply_die "We die to check bootstrapper rollback"
	fi
}

component_base_post_install()
{
	base_bootstrapper_post_install
	libpamplesk_bootstrapper_post_install
	core_bootstrapper_post_install
	managementnode_bootstrapper_post_install
	l10n_bootstrapper_post_install
	phpmyadmin_bootstrapper_post_install

	update_plesk_completion_rules
}


set_optional_component_params()
{
	set_component_pkgname_list

	optional_components="
		horde
		roundcube
		mail-pc-driver
		agent
		psa-vhost
		dns-bind-driver
		spammng
		courier-imap
		drweb-configurator
		psa-updates
	"

	# hack to forbid sh_optimizer to wipe bootstrapper instructions out
	true horde_bootstrapper_prep_install horde_bootstrapper_post_install
	true roundcube_bootstrapper_prep_install roundcube_bootstrapper_post_install
	true mailpcdriver_bootstrapper_post_install
	true agent_bootstrapper_prep_install agent_bootstrapper_post_install
	true psavhost_bootstrapper_post_install
	true dnsbinddriver_bootstrapper_post_install
	true spammng_bootstrapper_post_install
	true courierimap_bootstrapper_post_install
	true drwebconfigurator_bootstrapper_prep_install drwebconfigurator_bootstrapper_post_install
	true psaupdates_bootstrapper_post_install
}

is_package_installed_apt()
{
	dpkg -l $1 >/dev/null 2>&1
}

is_component_installed()
{
	local component=$1
	local package=$(eval "echo \$$(echo COMPONENT_PKGNAME_`echo $component | tr -d '-'`)")

	[ -n "$package" ] || return 1

	is_package_installed_apt $package
}

component_optional_action()
{
	local action=$1
	local component=$2

	if [ "X${component}" = "XBASE" ] && is_function "component_base_${action}"; then 
		"component_base_${action}"
	elif is_function "`echo $component | tr -d '-'`_bootstrapper_${action}"; then
		"`echo $component | tr -d '-'`_bootstrapper_${action}"
	else
		p_echo "Warning: there are no bootstrapper ${action} actions for component '$component'"
	fi
}

component_optional_prep_install()
{
	component_optional_action prep_install $1

	if [ "X$PREP_INSTALL_ROLLBACK_CHECK" = "XYes" ]; then
		simply_die "We die to check bootstrapper rollback"
	fi
}

component_optional_post_install()
{
	component_optional_action post_install $1
}

configure_local_components()
{
	deferred_webserver_apache_configure
	deferred_ftp_proftpd_configure

	deferred_awstats_configure
}

prepare_repair_actions()
{
	[ -s "$PRODUCT_ROOT_D/version" -o -s "$PRODUCT_ROOT_D/core.version" ] && get_prev_version "$PRODUCT_ROOT_D"
	mysql_unconfigured=
	set_mysqld_params
	check_db
	safe_prep_db_backup_all
}

run_repair_actions()
{
	pp_echo "Started bootstrapper repair procedure${PLESK_INSTALLER_FAST_REPAIR:+ (fast mode)}. This may take a while."
	pp_echo "Certain actions may be skipped if not applicable."
	pp_echo

	prepare_repair_actions
	set_optional_component_params

	pp_echo " Finishing up upgrade procedures and rerunning previously failed upgrade actions..."
	core_run_upgrade_prep_stage repair
	core_run_upgrade_post_stage repair
	install_apsc
	! is_component_installed 'spammng' || spamassassin_run_repair
	! is_component_installed 'courier-imap' || courier_imap_run_repair
	! is_component_installed 'roundcube' || roundcube_run_repair
	! is_component_installed 'horde' || horde_run_repair
	! is_component_installed 'drweb-configurator' || drweb_run_repair
	! is_component_installed 'firewall' || firewall_run_repair
	! is_component_installed 'psa-updates' || psaupdates_run_repair

	pp_echo " Reconfiguring mail subsystem..."
	mail_restore_all ${PLESK_INSTALLER_FAST_REPAIR:-full}

	pp_echo " Reconfiguring Apache web server..."
	run_now_webserver_apache_configure

	pp_echo " Reconfiguring ProFTPD FTP server..."
	run_now_ftp_proftpd_configure

	pp_echo " Reconfiguring AWStats web statistics..."
	run_now_awstats_configure

	pp_echo " Reconfiguring WatchDog..."
	wd_adapt

	pp_echo " Restoring SELinux contexts..."
	if [ -n "$PLESK_INSTALLER_FAST_REPAIR" ]; then
		relabel_plesk_directories --verbose
	else
		relabel_plesk_directories --verbose "$HTTPD_VHOSTS_D" "$PLESK_MAILNAMES_D"
	fi

	pp_echo " Regenerating web servers' configuration files..."
	if [ -z "$PLESK_INSTALLER_FAST_REPAIR" ]; then
		httpdmng_reconfigure all
	fi

	pp_echo " Cleaning active Panel sessions..."
	clean_panel_sessions

	pp_echo
	pp_echo "Bootstrapper repair finished."
	pp_echo "If problems persist, please check installer logs ('$product_log' and '$product_problems_log') for errors."
	pp_echo "If you can't resolve the issue on your own, please address Parallels support."
}

usage()
{
	local prog="`basename $0`"
	echo ""
	echo "Bootstrap setup actions for ${PRODUCT_FULL_NAME} ${product_version}"
	echo ""
	echo "Usage: $prog <prep-install|post-install|rerun|perform-deferred-actions> [component|BASE]"
	echo "       $prog <repair|perform-deferred-actions>"
	echo ""
	echo "Running '$prog repair' with PLESK_INSTALLER_FAST_REPAIR=1 environment"
	echo "variable set will skip some of the most time-consuming restore actions."
	echo ""
}

if [ "X${PLESK_INSTALLER_DEBUG}" != "X" ]; then
    set -x
fi

export LANG=C LC_MESSAGES=C LC_ALL=C
umask 022

product_build="ubuntu12.04.build115131206.11"
product_default_conf
initial_conf

script_action="$1"
script_target="$2"
case $script_action in 
	prep-install)
		repair_action="$script_action"
		bootstrapper_transaction_begin
		if [ -z "$script_target" ]; then
			component_base_prep_install
			# We don't know here whether a certain optional component is going to be installed.
			# So do not execute prep-install actions for it here.
		else
			set_optional_component_params
			component_optional_prep_install $script_target
		fi
		bootstrapper_transaction_end
	;;
	post-install)
		repair_action="repair"
		bootstrapper_transaction_begin
		set_optional_component_params
		if [ -z "$script_target" ]; then
			component_base_post_install
			for comp in $optional_components; do
				is_component_installed $comp && component_optional_post_install $comp;
			done
			configure_local_components
			perform_deferred_actions
		else
			component_optional_post_install $script_target
		fi
		bootstrapper_transaction_end
	;;
	rerun)
		repair_action="rerun"
		do_repair=1
		set_log_action_name "installation_rerun"
		bootstrapper_transaction_begin
		transaction_add_commit_action "reset_upgrade_failure_flag"
		transaction_add_rollback_action "reset_maintenance_mode_flag"
		set_maintenance_mode_flag
		set_optional_component_params
		if [ -z "$script_target" ]; then
			component_base_prep_install
			for comp in $optional_components; do 
				is_component_installed $comp && component_optional_prep_install $comp;
			done
			for comp in $optional_components; do
				is_component_installed $comp && component_optional_post_install $comp;
			done
			component_base_post_install
			configure_local_components
			perform_deferred_actions
		else
			component_optional_prep_install $script_target
			component_optional_post_install $script_target
		fi
		bootstrapper_transaction_end
	;;
	repair)
		repair_action="repair"
		do_repair=1
		set_log_action_name "repair"
		bootstrapper_transaction_begin
		transaction_add_commit_action "reset_upgrade_failure_flag"
		transaction_add_rollback_action "reset_maintenance_mode_flag"
		set_maintenance_mode_flag
		run_repair_actions
		perform_deferred_actions
		bootstrapper_transaction_end
	;;
	perform-deferred-actions)
		bootstrapper_transaction_begin
		perform_deferred_actions
		bootstrapper_transaction_end
	;;
	dump-incomplete-deferred-actions)
		dump_incomplete_deferred_actions
	;;
	*)
		usage
		exit 1
	;;
esac
